@@ -20,19 +20,22 @@
 #include <config-X11.h>
 
 #include "input.h"
-#include "triggers/triggers.h"
 #include "shortcuts_handler.h"
 #include "windows_handler.h"
 #include "khotkeysglobal.h"
 
+
 #include <X11/X.h>
 // #include <X11/Xutil.h>
-#include <QtGui/qx11info_x11.h>
+#include <QX11Info>
 #include <fixx11h.h>
 
-#include <KAction>
 #include <KDebug>
+#include <KGlobalAccel>
+#include <KShortcut>
 
+#include <QAction>
+#include <QtCore/QUuid>
 #include <kkeyserver.h>
 
 namespace KHotKeys {
@@ -41,9 +44,9 @@ namespace KHotKeys {
 ShortcutsHandler::ShortcutsHandler( HandlerType type, QObject *parent )
         : QObject(parent)
          ,_type(type)
-         ,_actions(new KActionCollection(this))
+         ,_actions(new KActionCollection(this, QStringLiteral("khotkeys")))
     {
-    _actions->setComponentData(KComponentData("khotkeys"));
+    connect(KGlobalAccel::self(), &KGlobalAccel::globalShortcutChanged, this, &ShortcutsHandler::shortcutChanged);
     }
 
 
@@ -54,7 +57,7 @@ ShortcutsHandler::~ShortcutsHandler()
     }
 
 
-KAction *ShortcutsHandler::addAction(
+QAction *ShortcutsHandler::addAction(
         const QString &id,
         const QString &text,
         const KShortcut &shortcut )
@@ -69,11 +72,11 @@ KAction *ShortcutsHandler::addAction(
     if (_actions->action(id))
         {
         qDebug() << id << " already present. Using new id!";
-        realId = QString::fromLatin1(createShortcutID());
+        realId = QUuid::createUuid().toString();
         }
 
     // Create the action
-    KAction *newAction = _actions->addAction(realId);
+    QAction *newAction = _actions->addAction(realId);
     if (!newAction)
         {
         return 0;
@@ -86,9 +89,9 @@ KAction *ShortcutsHandler::addAction(
         newAction->setProperty("isConfigurationAction", QVariant(true));
         }
     newAction->setText(text);
-    newAction->setGlobalShortcut( shortcut, KAction::ActiveShortcut );
+    KGlobalAccel::self()->setDefaultShortcut(newAction, QList<QKeySequence>() << shortcut.primary());
     // Enable global shortcut. If that fails there is no sense in proceeding
-    if (!newAction->isGlobalShortcutEnabled())
+    if (!KGlobalAccel::self()->hasShortcut(newAction))
         {
         kWarning() << "Failed to enable global shortcut for '" 
                    << text << "' " << id;
@@ -97,10 +100,6 @@ KAction *ShortcutsHandler::addAction(
         }
     Q_ASSERT(newAction->isEnabled());
 
-    connect(
-        newAction, SIGNAL(globalShortcutChanged(QKeySequence)),
-        this, SIGNAL(shortcutChanged()));
-
     return newAction;
     }
 
