@@ -35,7 +35,7 @@
 #include <QMap>
 #include <QTreeWidget>
 #include <QAction>
-#include <QtGui/qevent.h>
+#include <QContextMenuEvent>
 #include <QTreeWidgetItemIterator>
 
 //Local
@@ -47,7 +47,6 @@
 //#include "nicsignals.h"
 
 //Kde
-#include <klocale.h>
 
 class InfoPanel;
 class DevInfoPlugin;
@@ -89,7 +88,7 @@ class DeviceListing : public QTreeWidget
     Solid::Networking::Notifier *network;
     //NicSignals *nicSig;
     
-  public slots:
+  public Q_SLOTS:
     void itemActivatedSlot(QTreeWidgetItem *,const int);
     void deviceAddedSlot(const QString);
     void deviceRemovedSlot(const QString);
