@@ -30,12 +30,13 @@
 
 //Solid
 #include <solid/device.h>
+#include <solid/processor.h>
 #include <solid/deviceinterface.h>
+#include <solid/networkinterface.h>
+#include <solid/storagedrive.h>
 
 //KDE
-#include <kicon.h>
-#include <kdebug.h>
-#include <klocale.h>
+#include <KLocalizedString>
 
 //Local
 #include "qvlistlayout.h"
