@@ -23,7 +23,8 @@
 #include "soldevicetypes.h"
 
 #include <kcapacitybar.h>
-#include <kdiskfreespaceinfo.h>
+
+#include <KFormat>
 // ---- Processor
 
 SolProcessorDevice::SolProcessorDevice(QTreeWidgetItem *parent, const Solid::Device &device) :
@@ -38,7 +39,7 @@ SolProcessorDevice::SolProcessorDevice(const Solid::DeviceInterface::Type &type)
 { 
   deviceTypeHolder = Solid::DeviceInterface::Processor;
   
-  setDeviceIcon(KIcon("cpu"));
+  setDeviceIcon(QIcon::fromTheme(QStringLiteral("cpu")));
   setDeviceText(i18n("Processors"));
   setDefaultListing(type);
 }
@@ -106,7 +107,7 @@ SolStorageDevice::SolStorageDevice(const Solid::DeviceInterface::Type &type) :
 { 
   deviceTypeHolder = Solid::DeviceInterface::StorageDrive;
   
-  setDeviceIcon(KIcon("drive-harddisk"));
+  setDeviceIcon(QIcon::fromTheme(QStringLiteral("drive-harddisk")));
   setDeviceText(i18n("Storage Drives"));
   setDefaultListing(type);
 }
@@ -121,27 +122,28 @@ void SolStorageDevice::setDefaultDeviceText()
   const Solid::StorageDrive *stodev = interface<const Solid::StorageDrive>();
   if(!stodev) return;
   
-  QString deviceText;
+  QString storageType;
   switch (stodev->driveType())
   {
     case Solid::StorageDrive::HardDisk: 
-      deviceText = i18n("Hard Disk Drive");  break;
+      storageType = i18n("Hard Disk Drive");  break;
     case Solid::StorageDrive::CompactFlash:
-      deviceText = i18n("Compact Flash Reader");  break;
+      storageType = i18n("Compact Flash Reader");  break;
     case Solid::StorageDrive::SmartMedia:
-      deviceText = i18n("Smart Media Reader");  break;
+      storageType = i18n("Smart Media Reader");  break;
     case Solid::StorageDrive::SdMmc:
-      deviceText = i18n("SD/MMC Reader"); break;
+      storageType = i18n("SD/MMC Reader"); break;
     case Solid::StorageDrive::CdromDrive:
-      deviceText = i18n("Optical Drive"); break;
+      storageType = i18n("Optical Drive"); break;
     case Solid::StorageDrive::MemoryStick:
-      deviceText = i18n("Memory Stick Reader"); break;
+      storageType = i18n("Memory Stick Reader"); break;
     case Solid::StorageDrive::Xd:
-      deviceText = i18n("xD Reader"); break;
+      storageType = i18n("xD Reader"); break;
     default:
-      deviceText = i18n("Unknown Drive");
+      storageType = i18n("Unknown Drive"); 
   }
   
+  QString deviceText = storageType;
   setDeviceText(deviceText);
 }
 
@@ -199,7 +201,7 @@ SolNetworkDevice::SolNetworkDevice(const Solid::DeviceInterface::Type &type) :
 { 
   deviceTypeHolder = Solid::DeviceInterface::NetworkInterface;
   
-  setDeviceIcon(KIcon("network-wired"));
+  setDeviceIcon(QIcon::fromTheme(QStringLiteral("network-wired")));
   setDeviceText(i18n("Network Interfaces"));
   setDefaultListing(type);
 }
@@ -230,9 +232,9 @@ void SolNetworkDevice::setDefaultDeviceIcon()
   
   if(netdev->isWireless() == true)
   {
-    setDeviceIcon(KIcon("network-wireless"));
+    setDeviceIcon(QIcon::fromTheme(QStringLiteral("network-wireless")));
   } else {
-    setDeviceIcon(KIcon("network-wired"));
+    setDeviceIcon(QIcon::fromTheme(QStringLiteral("network-wired")));
   }
 }
 
@@ -247,10 +249,8 @@ QVListLayout *SolNetworkDevice::infoPanelLayout()
   labels << i18n("Hardware Address: ")
   << InfoPanel::friendlyString(netdev->hwAddress())
   << i18n("Wireless?")
-  << InfoPanel::convertTf(netdev->isWireless())
-  << i18n("Loopback?")
-  << InfoPanel::convertTf(netdev->isLoopback());
-
+  << InfoPanel::convertTf(netdev->isWireless());
+  
   deviceInfoLayout->applyQListToLayout(labels); 
   return deviceInfoLayout;
 }
@@ -337,8 +337,8 @@ QVListLayout *SolVolumeDevice::infoPanelLayout()
         usageBar->setText(
               i18nc("Available space out of total partition size (percent used)",
                     "%1 free of %2 (%3% used)",
-                    KGlobal::locale()->formatByteSize(mountSpaceInfo.available()),
-                    KGlobal::locale()->formatByteSize(mountSpaceInfo.size()),
+                    KFormat().formatByteSize(mountSpaceInfo.available()),
+                    KFormat().formatByteSize(mountSpaceInfo.size()),
                     usageBar->value()));
       }
       else
@@ -369,7 +369,7 @@ SolAudioDevice::SolAudioDevice(const Solid::DeviceInterface::Type &type) :
 { 
   deviceTypeHolder = Solid::DeviceInterface::AudioInterface;
   
-  setDeviceIcon(KIcon("audio-card"));
+  setDeviceIcon(QIcon::fromTheme(QStringLiteral("audio-card")));
   setDeviceText(i18n("Audio Interfaces"));
   setDefaultListing(type);
 }
@@ -412,11 +412,11 @@ void SolAudioDevice::createSubItems(const SubMenus &menus)
   if(menus == ALSA)
   {
     alsaSubItem = new SolDevice(this);
-    alsaSubItem->setDeviceIcon(KIcon("audio-card"));
+    alsaSubItem->setDeviceIcon(QIcon::fromTheme(QStringLiteral("audio-card")));
     alsaSubItem->setText(0,i18n("Alsa Interfaces"));
   } else {
     ossSubItem = new SolDevice(this);
-    ossSubItem->setDeviceIcon(KIcon("audio-card"));
+    ossSubItem->setDeviceIcon(QIcon::fromTheme(QStringLiteral("audio-card")));
     ossSubItem->setText(0,i18n("Open Sound System Interfaces"));
   }
 }
@@ -503,7 +503,7 @@ SolButtonDevice::SolButtonDevice(const Solid::DeviceInterface::Type &type) :
 { 
   deviceTypeHolder = Solid::DeviceInterface::Button;
   
-  setDeviceIcon(KIcon("insert-button"));
+  setDeviceIcon(QIcon::fromTheme(QStringLiteral("insert-button")));
   setDeviceText(i18n("Device Buttons"));
   setDefaultListing(type);
 }
@@ -515,7 +515,7 @@ void SolButtonDevice::setDefaultListing(const Solid::DeviceInterface::Type &type
 
 void SolButtonDevice::setDefaultDeviceIcon() 
 {
-  setDeviceIcon(KIcon("insert-button"));
+  setDeviceIcon(QIcon::fromTheme(QStringLiteral("insert-button")));
 }
 
 QVListLayout *SolButtonDevice::infoPanelLayout() 
@@ -563,7 +563,7 @@ SolMediaPlayerDevice::SolMediaPlayerDevice(const Solid::DeviceInterface::Type &t
 { 
   deviceTypeHolder = Solid::DeviceInterface::PortableMediaPlayer;
 
-  setDeviceIcon(KIcon("multimedia-player"));
+  setDeviceIcon(QIcon::fromTheme(QStringLiteral("multimedia-player")));
   setDeviceText(i18n("Multimedia Players"));
   setDefaultListing(type);
 }
@@ -603,7 +603,7 @@ SolCameraDevice::SolCameraDevice(const Solid::DeviceInterface::Type &type) :
 { 
   deviceTypeHolder = Solid::DeviceInterface::Camera;
 
-  setDeviceIcon(KIcon("camera-web"));
+  setDeviceIcon(QIcon::fromTheme(QStringLiteral("camera-web")));
   setDeviceText(i18n("Cameras"));
   setDefaultListing(type);
 }
@@ -643,7 +643,7 @@ SolBatteryDevice::SolBatteryDevice(const Solid::DeviceInterface::Type &type) :
 { 
   deviceTypeHolder = Solid::DeviceInterface::Battery;
 
-  setDeviceIcon(KIcon("battery"));
+  setDeviceIcon(QIcon::fromTheme(QStringLiteral("battery")));
   setDeviceText(i18n("Batteries"));
   setDefaultListing(type);
 }
@@ -664,27 +664,33 @@ QVListLayout *SolBatteryDevice::infoPanelLayout()
   QString type;
   switch(batdev->type()) 
   {
-      case Solid::Battery::PrimaryBattery:
-        type = i18n("Primary"); break;
+      case Solid::Battery::PdaBattery:
+	type = i18n("PDA"); break;
       case Solid::Battery::UpsBattery:
-        type = i18n("UPS"); break;
-      case Solid::Battery::UsbBattery:
-        type = i18n("USB"); break;
+	type = i18n("UPS"); break;
+      case Solid::Battery::MouseBattery:
+	type = i18n("Mouse"); break;
+      case Solid::Battery::PrimaryBattery:
+	type = i18n("Primary"); break;
+      case Solid::Battery::KeyboardBattery:
+	type = i18n("Keyboard"); break;
+      case Solid::Battery::KeyboardMouseBattery:
+	type = i18n("Keyboard + Mouse"); break;
+      case Solid::Battery::CameraBattery:
+	type = i18n("Camera"); break;
       default:
-        type = i18nc("unknown battery type", "Unknown");
+	type = i18nc("unknown battery type", "Unknown");
   }
   
   QString state;
-    switch(batdev->chargeState())
+    switch(batdev->type())
     {
       case Solid::Battery::Charging:
-        state = i18n("Charging"); break;
+	state = i18n("Charging"); break;
       case Solid::Battery::Discharging:
-        state = i18n("Discharging"); break;
-      case Solid::Battery::FullyCharged:
-        state = i18n("Fully Charged"); break;
+	state = i18n("Discharging"); break;
       default:
-        state = i18nc("unknown battery charge", "Unknown");
+	state = i18n("No Charge"); 
     }
   
   labels << i18n("Battery Type: ")
@@ -709,7 +715,7 @@ SolAcAdapterDevice::SolAcAdapterDevice(const Solid::DeviceInterface::Type &type)
 { 
   deviceTypeHolder = Solid::DeviceInterface::AcAdapter;
 
-  setDeviceIcon(KIcon("kde"));
+  setDeviceIcon(QIcon::fromTheme(QStringLiteral("kde")));
   setDeviceText(i18n("AC Adapters"));
   setDefaultListing(type);
 }
@@ -734,6 +740,173 @@ QVListLayout *SolAcAdapterDevice::infoPanelLayout()
   return deviceInfoLayout;
 }
 
+// DVB
+
+SolDvbDevice::SolDvbDevice(QTreeWidgetItem *parent, const Solid::Device &device) :
+  SolDevice(parent, device) 
+{
+  deviceTypeHolder = Solid::DeviceInterface::DvbInterface;
+}
+
+SolDvbDevice::SolDvbDevice(const Solid::DeviceInterface::Type &type) :
+  SolDevice(type)
+{ 
+  deviceTypeHolder = Solid::DeviceInterface::DvbInterface;
+
+  setDeviceIcon(QIcon::fromTheme(QStringLiteral("kde")));
+  setDeviceText(i18n("Digital Video Broadcasting Devices"));
+  setDefaultListing(type);
+}
+
+void SolDvbDevice::setDefaultListing(const Solid::DeviceInterface::Type &type) 
+{ 
+  createDeviceChildren<SolDvbDevice>(this,QString(),type);
+}
+
+QVListLayout *SolDvbDevice::infoPanelLayout() 
+{  
+  QStringList labels;
+  const Solid::DvbInterface *dvbdev = interface<const Solid::DvbInterface>(); 
+  
+  if(!dvbdev) return NULL;
+  deviceInfoLayout = new QVListLayout();
+  
+  QString type;
+  switch(dvbdev->deviceType()) 
+  {
+    case Solid::DvbInterface::DvbAudio:
+      type = i18n("Audio"); break;
+    case Solid::DvbInterface::DvbCa:
+      type = i18n("Conditional access system"); break;
+    case Solid::DvbInterface::DvbDemux:
+      type = i18n("Demux"); break;
+    case Solid::DvbInterface::DvbDvr:
+      type = i18n("Digital video recorder"); break;
+    case Solid::DvbInterface::DvbFrontend:
+      type = i18n("Front end"); break;
+    case Solid::DvbInterface::DvbNet:
+      type = i18n("Network"); break;
+    case Solid::DvbInterface::DvbOsd:
+      type = i18n("On-Screen display"); break;
+    case Solid::DvbInterface::DvbSec:
+      type = i18n("Security and content protection"); break;
+    case Solid::DvbInterface::DvbVideo:
+      type = i18n("Video"); break;
+    case Solid::DvbInterface::DvbUnknown:
+    default:
+      type = i18nc("unknown device type", "Unknown"); 
+  }
+  
+  labels << i18n("Device Type: ")
+  << type;
+  
+  deviceInfoLayout->applyQListToLayout(labels);
+  return deviceInfoLayout;
+}
+
+// Serial
+
+SolSerialDevice::SolSerialDevice(QTreeWidgetItem *parent, const Solid::Device &device) :
+  SolDevice(parent, device) 
+{
+  deviceTypeHolder = Solid::DeviceInterface::SerialInterface;
+}
+
+SolSerialDevice::SolSerialDevice(const Solid::DeviceInterface::Type &type) :
+  SolDevice(type)
+{ 
+  deviceTypeHolder = Solid::DeviceInterface::SerialInterface;
+  
+  setDeviceIcon(QIcon::fromTheme(QStringLiteral("printer")));
+  setDeviceText(i18n("Serial Devices"));
+  setDefaultListing(type);
+}
+
+void SolSerialDevice::setDefaultListing(const Solid::DeviceInterface::Type &type) 
+{ 
+  createDeviceChildren<SolSerialDevice>(this,QString(),type);
+}
+
+QVListLayout *SolSerialDevice::infoPanelLayout() 
+{  
+  QStringList labels;
+  const Solid::SerialInterface *serdev = interface<const Solid::SerialInterface>(); 
+  
+  if(!serdev) return NULL;
+  deviceInfoLayout = new QVListLayout();
+  
+  QString type;
+  switch(serdev->serialType())
+  {
+    case Solid::SerialInterface::Platform:
+      type = i18nc("platform serial interface type", "Platform"); break;
+    case Solid::SerialInterface::Usb:
+      type = i18n("USB"); break;
+    default:
+      type = i18nc("unknown serial interface type", "Unknown"); 
+  }
+  
+  QString port = i18nc("unknown port", "Unknown");
+  if(serdev->port() != -1) port = QString::number(serdev->port());
+
+  labels << i18n("Serial Type: ")
+  << type
+  << i18n("Port: ")
+  << port;
+  
+  deviceInfoLayout->applyQListToLayout(labels);
+  return deviceInfoLayout;
+}
+
+//  Smartcard
+
+SolSmartCardDevice::SolSmartCardDevice(QTreeWidgetItem *parent, const Solid::Device &device) :
+  SolDevice(parent, device) 
+{
+  deviceTypeHolder = Solid::DeviceInterface::SmartCardReader;
+}
+
+SolSmartCardDevice::SolSmartCardDevice(const Solid::DeviceInterface::Type &type) :
+  SolDevice(type)
+{ 
+  deviceTypeHolder = Solid::DeviceInterface::SmartCardReader;
+
+  setDeviceIcon(QIcon::fromTheme(QStringLiteral("kde")));
+  setDeviceText(i18n("Smart Card Devices"));
+  setDefaultListing(type);
+}
+
+void SolSmartCardDevice::setDefaultListing(const Solid::DeviceInterface::Type &type) 
+{ 
+  createDeviceChildren<SolSmartCardDevice>(this,QString(),type);
+}
+
+QVListLayout *SolSmartCardDevice::infoPanelLayout() 
+{  
+  QStringList labels;
+  const Solid::SmartCardReader *scdev = interface<const Solid::SmartCardReader>(); 
+  
+  if(!scdev) return NULL;
+  deviceInfoLayout = new QVListLayout();
+  
+  QString type;
+  switch(scdev->readerType())
+  {
+    case Solid::SmartCardReader::CardReader:
+      type = i18n("Card Reader"); break;
+    case Solid::SmartCardReader::CryptoToken:
+      type = i18n("Crypto Token"); break;
+    default:
+      type = i18nc("unknown smart card type", "Unknown"); 
+  }
+ 
+  labels << i18n("Smart Card Type: ")
+  << type;
+  
+  deviceInfoLayout->applyQListToLayout(labels);
+  return deviceInfoLayout;
+}
+
 // Video
 
 SolVideoDevice::SolVideoDevice(QTreeWidgetItem *parent, const Solid::Device &device) :
@@ -747,7 +920,7 @@ SolVideoDevice::SolVideoDevice(const Solid::DeviceInterface::Type &type) :
 { 
   deviceTypeHolder = Solid::DeviceInterface::Video;
   
-  setDeviceIcon(KIcon("video-display"));
+  setDeviceIcon(QIcon::fromTheme(QStringLiteral("video-display")));
   setDeviceText(i18n("Video Devices"));
   setDefaultListing(type);
 }
@@ -773,41 +946,3 @@ QVListLayout *SolVideoDevice::infoPanelLayout()
   deviceInfoLayout->applyQListToLayout(labels);
   return deviceInfoLayout;
 }
-
-// Graphic
-
-SolGraphicDevice::SolGraphicDevice(QTreeWidgetItem *parent, const Solid::Device &device) :
-  SolDevice(parent, device)
-{
-  deviceTypeHolder = Solid::DeviceInterface::Video;
-}
-
-SolGraphicDevice::SolGraphicDevice(const Solid::DeviceInterface::Type &type) :
-  SolDevice(type)
-{
-  deviceTypeHolder = Solid::DeviceInterface::Video;
-  
-  setDeviceIcon(KIcon("video-display"));
-  setDeviceText(i18n("Graphic Displays"));
-  setDefaultListing(type);
-}
-
-void SolGraphicDevice::setDefaultListing(const Solid::DeviceInterface::Type &type)
-{
-  createDeviceChildren<SolGraphicDevice>(this,QString(),type);
-}
-
-QVListLayout *SolGraphicDevice::infoPanelLayout()
-{
-  QStringList labels;
-  const Solid::Graphic *graphdev = interface<const Solid::Graphic>();
-
-  if(!graphdev) return NULL;
-  deviceInfoLayout = new QVListLayout();
-
-  labels << i18n("Driver: ")
-  << graphdev->driver();
-  
-  deviceInfoLayout->applyQListToLayout(labels);
-  return deviceInfoLayout;
-}
