@@ -25,11 +25,12 @@
 #include <unistd.h>
 
 #include <QDir>
+#include <QDrag>
 #include <QHeaderView>
 #include <QPainter>
 #include <QRegExp>
 #include <QPixmap>
-#include <QtGui/qevent.h>
+#include <QDropEvent>
 #include <QMenu>
 #include <QApplication>
 #include <QtDBus/QtDBus>
@@ -43,16 +44,17 @@
 #include <KGlobal>
 #include <KIconLoader>
 #include <KInputDialog>
-#include <KLocale>
+#include <KLocalizedString>
 #include <KMessageBox>
 #include <KService>
 #include <KServiceGroup>
 #include <KConfig>
 #include <KConfigGroup>
 #include <KStandardDirs>
-#include <KIO/NetAccess>
+#include <kio/netaccess.h>
+#include <KUrl>
 
-#include "moc_treeview.cpp"
+#include "treeview.moc"
 #include "menufile.h"
 #include "menuinfo.h"
 
@@ -227,6 +229,13 @@ bool TreeItem::isLayoutDirty() const
 static QPixmap appIcon(const QString &iconName)
 {
     QPixmap normal = KIconLoader::global()->loadIcon(iconName, KIconLoader::Small, 0, KIconLoader::DefaultState, QStringList(), 0L, true);
+    // make sure they are not larger than 20x20
+    if (normal.width() > 20 || normal.height() > 20)
+    {
+       QImage tmp = normal.toImage();
+       tmp = tmp.scaled(20, 20, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+       normal = QPixmap::fromImage(tmp);
+    }
     return normal;
 }
 
@@ -251,37 +260,37 @@ TreeView::TreeView( KActionCollection *ac, QWidget *parent, const char *name )
     header()->hide();
 
     // listen for creation
-    connect(m_ac->action(NEW_ITEM_ACTION_NAME), SIGNAL(triggered()), SLOT(newitem()));
-    connect(m_ac->action(NEW_SUBMENU_ACTION_NAME), SIGNAL(triggered()), SLOT(newsubmenu()));
-    connect(m_ac->action(NEW_SEPARATOR_ACTION_NAME), SIGNAL(triggered()), SLOT(newsep()));
+    connect(m_ac->action(NEW_ITEM_ACTION_NAME), SIGNAL(activated()), SLOT(newitem()));
+    connect(m_ac->action(NEW_SUBMENU_ACTION_NAME), SIGNAL(activated()), SLOT(newsubmenu()));
+    connect(m_ac->action(NEW_SEPARATOR_ACTION_NAME), SIGNAL(activated()), SLOT(newsep()));
 
     // listen for copy
-    connect(m_ac->action(CUT_ACTION_NAME), SIGNAL(triggered()), SLOT(cut()));
-    connect(m_ac->action(COPY_ACTION_NAME), SIGNAL(triggered()), SLOT(copy()));
-    connect(m_ac->action(PASTE_ACTION_NAME), SIGNAL(triggered()), SLOT(paste()));
+    connect(m_ac->action(CUT_ACTION_NAME), SIGNAL(activated()), SLOT(cut()));
+    connect(m_ac->action(COPY_ACTION_NAME), SIGNAL(activated()), SLOT(copy()));
+    connect(m_ac->action(PASTE_ACTION_NAME), SIGNAL(activated()), SLOT(paste()));
 
     // listen for deleting
-    connect(m_ac->action(DELETE_ACTION_NAME), SIGNAL(triggered()), SLOT(del()));
+    connect(m_ac->action(DELETE_ACTION_NAME), SIGNAL(activated()), SLOT(del()));
 
     // listen for sorting
     m_sortSignalMapper = new QSignalMapper(this);
     QAction *action = m_ac->action(SORT_BY_NAME_ACTION_NAME);
-    connect(action, SIGNAL(triggered()), m_sortSignalMapper, SLOT(map()));
+    connect(action, SIGNAL(activated()), m_sortSignalMapper, SLOT(map()));
     m_sortSignalMapper->setMapping(action, SortByName);
     action = m_ac->action(SORT_BY_DESCRIPTION_ACTION_NAME);
-    connect(action, SIGNAL(triggered()), m_sortSignalMapper, SLOT(map()));
+    connect(action, SIGNAL(activated()), m_sortSignalMapper, SLOT(map()));
     m_sortSignalMapper->setMapping(action, SortByDescription);
     action = m_ac->action(SORT_ALL_BY_NAME_ACTION_NAME);
-    connect(action, SIGNAL(triggered()), m_sortSignalMapper, SLOT(map()));
+    connect(action, SIGNAL(activated()), m_sortSignalMapper, SLOT(map()));
     m_sortSignalMapper->setMapping(action, SortAllByName);
     action = m_ac->action(SORT_ALL_BY_DESCRIPTION_ACTION_NAME);
-    connect(action, SIGNAL(triggered()), m_sortSignalMapper, SLOT(map()));
+    connect(action, SIGNAL(activated()), m_sortSignalMapper, SLOT(map()));
     m_sortSignalMapper->setMapping(action, SortAllByDescription);
     connect(m_sortSignalMapper, SIGNAL(mapped(const int)), this, SLOT(sort(const int)));
 
     // connect moving up/down actions
-    connect(m_ac->action(MOVE_UP_ACTION_NAME), SIGNAL(triggered()), SLOT(moveUpItem()));
-    connect(m_ac->action(MOVE_DOWN_ACTION_NAME), SIGNAL(triggered()), SLOT(moveDownItem()));
+    connect(m_ac->action(MOVE_UP_ACTION_NAME), SIGNAL(activated()), SLOT(moveUpItem()));
+    connect(m_ac->action(MOVE_DOWN_ACTION_NAME), SIGNAL(activated()), SLOT(moveDownItem()));
 
     // listen for selection
     connect(this, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)),
@@ -359,18 +368,17 @@ void TreeView::readMenuFolderInfo(MenuFolderInfo *folderInfo, KServiceGroup::Ptr
     folderInfo->id = id;
     folderInfo->fullId = prefix + id;
 
-    foreach(const KSycocaEntry::Ptr &e, folder->entries(true, !m_showHidden, true, m_detailedMenuEntries && !m_detailedEntriesNamesFirst))
+    for (const KSycocaEntry::Ptr &e : folder->entries(true, !m_showHidden, true, m_detailedMenuEntries && !m_detailedEntriesNamesFirst))
     {
         if (e->isType(KST_KServiceGroup))
         {
-            KServiceGroup::Ptr g(KServiceGroup::Ptr::staticCast(e));
             MenuFolderInfo *subFolderInfo = new MenuFolderInfo();
-            readMenuFolderInfo(subFolderInfo, g, folderInfo->fullId);
+            readMenuFolderInfo(subFolderInfo, KServiceGroup::Ptr(e), folderInfo->fullId);
             folderInfo->add(subFolderInfo, true);
         }
         else if (e->isType(KST_KService))
         {
-            folderInfo->add(new MenuEntryInfo(KService::Ptr::staticCast(e)), true);
+            folderInfo->add(new MenuEntryInfo(KService::Ptr(e)), true);
         }
         else if (e->isType(KST_KServiceSeparator))
         {
@@ -396,11 +404,8 @@ QString TreeView::findName(KDesktopFile *df, bool deleted)
           name.clear();
        if (name.isEmpty())
        {
-          QString file = df->fileName();
-          QString res = df->resource();
-
           bool isLocal = true;
-          const QStringList files = KGlobal::dirs()->findAllResources(res.toLatin1(), file);
+          const QStringList files = QStandardPaths::locateAll(df->resource(), df->fileName(), QStandardPaths::LocateFile);
           for(QStringList::ConstIterator it = files.constBegin();
               it != files.constEnd();
               ++it)
@@ -811,8 +816,7 @@ void TreeView::startDrag(Qt::DropActions supportedActions)
     }
 
     QDrag *drag = new QDrag(this);
-    int iconSize = KIconLoader::global()->currentSize(KIconLoader::Small);
-    drag->setPixmap(selectedItem()->icon(0).pixmap(iconSize, iconSize));
+    drag->setPixmap(selectedItem()->icon(0).pixmap(24, 24));
     drag->setMimeData(data);
     drag->exec(supportedActions, Qt::MoveAction);
 }
@@ -834,7 +838,7 @@ static QString createDesktopFile(const QString &file, QString *menuId, QStringLi
    QRegExp r("(.*)(?=-\\d+)");
    base = (r.indexIn(base) > -1) ? r.cap(1) : base;
 
-   QString result = KService::newServicePath(base, menuId, excludeList);
+   QString result = KService::newServicePath(true, base, menuId, excludeList);
    excludeList->append(*menuId);
    // Todo for Undo-support: Undo menuId allocation:
 
@@ -1610,8 +1614,6 @@ void TreeView::moveUpOrDownItem(bool isMovingUpAction)
 {
     // get the selected item and its parent
     TreeItem *sourceItem = static_cast<TreeItem*>(selectedItem());
-    if (!sourceItem)
-       return;
     TreeItem *parentItem = getParentItem(sourceItem);
 
     // get selected item index
@@ -1908,9 +1910,9 @@ void TreeView::restoreMenuSystem()
     }
 
     QString xdgdir = KGlobal::dirs()->KStandardDirs::localxdgdatadir();
-    if ( !KIO::NetAccess::del( QString(xdgdir + "/applications") , this) )
+    if ( !KIO::NetAccess::del( QUrl::fromLocalFile(xdgdir + QStringLiteral("/applications")) , this) )
         qWarning()<<"Could not delete dir :"<<( xdgdir+"/applications" );
-    if ( !KIO::NetAccess::del( QString(xdgdir +"/desktop-directories") , this) )
+    if ( !KIO::NetAccess::del( QUrl::fromLocalFile(xdgdir + QStringLiteral("/desktop-directories")) , this) )
         qWarning()<<"Could not delete dir :"<<( xdgdir + "/desktop-directories");
 
     KBuildSycocaProgressDialog::rebuildKSycoca(this);
