@@ -19,17 +19,17 @@
 
 */
 
-#include <QtGui/QDrag>
-#include <QtGui/QLabel>
-#include <QtGui/qevent.h>
-#include <QtGui/QPainter>
-#include <QtGui/QPixmap>
-#include <QtGui/QVBoxLayout>
+#include <QDrag>
+#include <QLabel>
+#include <QMouseEvent>
+#include <QPainter>
+#include <QPixmap>
+#include <QVBoxLayout>
 #include <QMimeData>
 
 #include <kdebug.h>
 #include <kicon.h>
-#include <klocale.h>
+#include <KLocalizedString>
 #include <kmessagebox.h>
 #include <ksgrd/SensorManager.h>
 #include <kfilterproxysearchline.h>
@@ -650,4 +650,4 @@ HostInfo *SensorInfo::hostInfo() const
 
 
 
-#include "moc_SensorBrowser.cpp"
+#include "SensorBrowser.moc"
