@@ -19,9 +19,9 @@
 
 */
 
-#include <klocale.h>
+#include <KLocalizedString>
 #include <ksgrd/SensorManager.h>
-#include <QtGui/qevent.h>
+#include <QMouseEvent>
 #include <QGridLayout>
 
 #include "DummyDisplay.h"
@@ -53,4 +53,4 @@ bool DummyDisplay::eventFilter( QObject* object, QEvent* event )
   return QWidget::eventFilter( object, event );
 }
 
-#include "moc_DummyDisplay.cpp"
+#include "DummyDisplay.moc"
