@@ -20,16 +20,18 @@
 */
 
 #include <QtXml/qdom.h>
-#include <QtGui/QImage>
-#include <QtGui/QToolTip>
-#include <QtGui/QVBoxLayout>
-#include <QtGui/QHBoxLayout>
-#include <QtGui/QLabel>
-#include <QtGui/qevent.h>
+#include <QImage>
+#include <QToolTip>
+#include <QVBoxLayout>
+#include <QHBoxLayout>
+#include <QLabel>
+#include <QFontInfo>
+#include <QResizeEvent>
 
 
 #include <kdebug.h>
-#include <klocale.h>
+#include <KGlobal>
+#include <KLocalizedString>
 #include <kmessagebox.h>
 #include <kapplication.h>
 #include <ksignalplotter.h>
@@ -289,18 +291,23 @@ void FancyPlotter::configureSettings()
 
     connect( mSettingsDialog, SIGNAL(applyClicked()), this, SLOT(applySettings()) );
     connect( mSettingsDialog, SIGNAL(okClicked()), this, SLOT(applySettings()) );
-    connect( mSettingsDialog, SIGNAL(finished()), this, SLOT(settingsFinished()) );
+    connect( mSettingsDialog, SIGNAL(finished(int)), this, SLOT(settingsFinished()) );
 
     mSettingsDialog->show();
 }
 
 void FancyPlotter::settingsFinished()
 {
-    mSettingsDialog->delayedDestruct();
+    applySettings();
+    mSettingsDialog->hide();
+    mSettingsDialog->deleteLater();
     mSettingsDialog = 0;
 }
 
 void FancyPlotter::applySettings() {
+    if (!mSettingsDialog) {
+        return;
+    }
     setTitle( mSettingsDialog->title() );
 
     mUseManualRange = mSettingsDialog->useManualRange();
@@ -443,7 +450,7 @@ bool FancyPlotter::removeBeam( uint beamId )
 {
     if ( beamId >= mBeams ) {
         kDebug(1215) << "FancyPlotter::removeBeam: beamId out of range ("
-            << beamId << ")";
+            << beamId << ")" << endl;
         return false;
     }
 
@@ -512,7 +519,7 @@ void FancyPlotter::setTooltip()
             if (sensor->unit() == "%")
                 lastValue = i18nc("units", "%1%", lastValue);
             else if( !sensor->unit().isEmpty() )
-                lastValue = i18nc("units", QString(QString("%1 ") + sensor->unit()).toUtf8(), lastValue);
+                lastValue = i18nc("units", QString(QString("%1 ") + sensor->unit()).toUtf8().constData(), lastValue);
         } else {
             lastValue = i18n("Error");
         }
@@ -533,14 +540,14 @@ void FancyPlotter::setTooltip()
         if(sensor->isLocalhost()) {
             tooltip += QString( "%1%2 %3 (%4)" ).arg( neednewline  ? "<br>" : "")
                 .arg("<font color=\"" + mPlotter->beamColor( beamId ).name() + "\">"+mIndicatorSymbol+"</font>")
-                .arg( i18n(description.toUtf8()) )
+                .arg( i18n(description.toUtf8().constData()) )
                 .arg( lastValue );
 
         } else {
             tooltip += QString( "%1%2 %3:%4 (%5)" ).arg( neednewline ? "<br>" : "" )
                 .arg("<font color=\"" + mPlotter->beamColor( beamId ).name() + "\">"+mIndicatorSymbol+"</font>")
                 .arg( sensor->hostName() )
-                .arg( i18n(description.toUtf8()) )
+                .arg( i18n(description.toUtf8().constData()) )
                 .arg( lastValue );
         }
         neednewline = true;
@@ -583,7 +590,7 @@ void FancyPlotter::sendDataToPlotter( )
                         if (sensor->unit() == "%")
                             lastValue = i18nc("units", "%1%", lastValue);
                         else if( !sensor->unit().isEmpty() )  {
-                            lastValue = i18nc("units", QString("%1 " + sensor->unit()).toUtf8(), lastValue);
+                            lastValue = i18nc("units", QString("%1 " + sensor->unit()).toUtf8().constData(), lastValue);
                         }
                     }
 
@@ -655,7 +662,7 @@ void FancyPlotter::plotterAxisScaleChanged()
 #endif
         mPlotter->setScaleDownBy(1);
         //translate any others
-        unit = ki18nc("units", QString("%1 " + mUnit).toUtf8());
+        unit = ki18nc("units", QString("%1 " + mUnit).toUtf8().constData());
     }
     mPlotter->setUnit(unit);
     //reconnect
@@ -855,7 +862,6 @@ bool FancyPlotter::saveSettings( QDomDocument &doc, QDomElement &element)
 
     element.setAttribute( "version", 1 );
     element.setAttribute( "labels", mPlotter->showAxis() );
-    element.setAttribute( "fontSize", mPlotter->font().pointSize() );
 
     QHash<QString,QDomElement> hash;
     int beamId = -1;
@@ -934,4 +940,4 @@ QColor FPSensorProperties::color() const
 }
 
 
-#include "moc_FancyPlotter.cpp"
+#include "FancyPlotter.moc"
