@@ -31,6 +31,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "focuschain.h"
 #include "netinfo.h"
 #include "workspace.h"
+#ifdef KWIN_BUILD_ACTIVITIES
+#include "activities.h"
+#endif
 
 #include <fixx11h.h>
 #include <kstartupinfo.h>
@@ -42,7 +45,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "rules.h"
 #include "screens.h"
 #include "useractions.h"
-#include <QtGui/qx11info_x11.h>
+#include <QDebug>
+#include <QX11Info>
 
 namespace KWin
 {
@@ -159,6 +163,26 @@ namespace KWin
          (SELI TODO - but what if they do?)
 
  Special cases I can think of:
+    - konqueror reusing, i.e. kfmclient tells running Konqueror instance
+        to open new window
+        - without focus stealing prevention - no problem
+        - with ASN (application startup notification) - ASN is forwarded,
+            and because it's newer than the instance's user timestamp,
+            it takes precedence
+        - without ASN - user timestamp needs to be reset, otherwise it would
+            be used, and it's old; moreover this new window mustn't be detected
+            as window belonging to already running application, or it wouldn't
+            be activated - see Client::sameAppWindowRoleMatch() for the (rather ugly)
+            hack
+    - konqueror preloading, i.e. window is created in advance, and kfmclient
+        tells this Konqueror instance to show it later
+        - without focus stealing prevention - no problem
+        - with ASN - ASN is forwarded, and because it's newer than the instance's
+            user timestamp, it takes precedence
+        - without ASN - user timestamp needs to be reset, otherwise it would
+            be used, and it's old; also, creation timestamp is changed to
+            the time the instance starts (re-)initializing the window,
+            this ensures creation timestamp will still work somewhat even in this case
     - KUniqueApplication - when the window is already visible, and the new instance
         wants it to activate
         - without focus stealing prevention - _NET_ACTIVE_WINDOW - no problem
@@ -216,7 +240,7 @@ void Workspace::setActiveClient(Client* c)
     }
     StackingUpdatesBlocker blocker(this);
     ++set_active_client_recursion;
-    updateFocusMousePosition(cursorPos());
+    updateFocusMousePosition(Cursor::pos());
     if (active_client != NULL) {
         // note that this may call setActiveClient( NULL ), therefore the recursion counter
         active_client->setActive(false);
@@ -238,7 +262,6 @@ void Workspace::setActiveClient(Client* c)
             }
         }
     }
-    pending_take_activity = NULL;
 
     updateToolWindows(false);
     if (c)
@@ -278,6 +301,14 @@ void Workspace::activateClient(Client* c, bool force)
         VirtualDesktopManager::self()->setCurrent(c->desktop());
         --block_focus;
     }
+#ifdef KWIN_BUILD_ACTIVITIES
+    if (!c->isOnCurrentActivity()) {
+        ++block_focus;
+        //DBUS!
+        Activities::self()->setCurrent(c->activities().first()); //first isn't necessarily best, but it's easiest
+        --block_focus;
+    }
+#endif
     if (c->isMinimized())
         c->unminimize();
 
@@ -304,10 +335,10 @@ void Workspace::activateClient(Client* c, bool force)
  */
 void Workspace::requestFocus(Client* c, bool force)
 {
-    takeActivity(c, ActivityFocus | (force ? ActivityFocusForce : 0), false);
+    takeActivity(c, force ? ActivityFocusForce : ActivityFocus);
 }
 
-void Workspace::takeActivity(Client* c, int flags, bool handled)
+void Workspace::takeActivity(Client* c, ActivityFlags flags)
 {
     // the 'if ( c == active_client ) return;' optimization mustn't be done here
     if (!focusChangeEnabled() && (c != active_client))
@@ -333,7 +364,6 @@ void Workspace::takeActivity(Client* c, int flags, bool handled)
             if (flags & ActivityRaise)
                 raiseClient(c);
             c = modal;
-            handled = false;
         }
         cancelDelayFocus();
     }
@@ -346,30 +376,23 @@ void Workspace::takeActivity(Client* c, int flags, bool handled)
             focusToNull();
         }
         flags &= ~ActivityFocus;
-        handled = false; // no point, can't get clicks
     }
     if (c->tabGroup() && c->tabGroup()->current() != c)
         c->tabGroup()->setCurrent(c);
     if (!c->isShown(true)) {  // shouldn't happen, call activateClient() if needed
-        kWarning(1212) << "takeActivity: not shown" ;
+        qWarning() << "takeActivity: not shown" ;
         return;
     }
-    c->takeActivity(flags, handled);
+
+    if (flags & ActivityFocus)
+        c->takeFocus();
+    if (flags & ActivityRaise)
+        workspace()->raiseClient(c);
+
     if (!c->isOnActiveScreen())
         screens()->setCurrent(c->screen());
 }
 
-void Workspace::handleTakeActivity(KWin::Client *c, xcb_timestamp_t /*timestamp*/, int flags)
-{
-    if (pending_take_activity != c)   // pending_take_activity is reset when doing restack or activation
-        return;
-    if ((flags & ActivityRaise) != 0)
-        raiseClient(c);
-    if ((flags & ActivityFocus) != 0 && c->isShown(false))
-        c->takeFocus();
-    pending_take_activity = NULL;
-}
-
 /*!
   Informs the workspace that the client \a c has been hidden. If it
   was the active client (or to-become the active client),
@@ -379,7 +402,7 @@ void Workspace::handleTakeActivity(KWin::Client *c, xcb_timestamp_t /*timestamp*
  */
 void Workspace::clientHidden(Client* c)
 {
-    assert(!c->isShown(true) || !c->isOnCurrentDesktop());
+    assert(!c->isShown(true) || !c->isOnCurrentDesktop() || !c->isOnCurrentActivity());
     activateNextClient(c);
 }
 
@@ -395,7 +418,7 @@ Client *Workspace::clientUnderMouse(int screen) const
         // rule out clients which are not really visible.
         // the screen test is rather superfluous for xrandr & twinview since the geometry would differ -> TODO: might be dropped
         if (!(client->isShown(false) && client->isOnCurrentDesktop() &&
-                client->isOnScreen(screen)))
+                client->isOnCurrentActivity() && client->isOnScreen(screen)))
             continue;
 
         if (client->geometry().contains(Cursor::pos())) {
@@ -542,18 +565,18 @@ bool Workspace::allowClientActivation(const KWin::Client *c, xcb_timestamp_t tim
     if (!ignore_desktop && !c->isOnCurrentDesktop())
         return false; // allow only with level == 0
     if (ac == NULL || ac->isDesktop()) {
-        kDebug(1212) << "Activation: No client active, allowing";
+        qDebug() << "Activation: No client active, allowing";
         return true; // no active client -> always allow
     }
     // TODO window urgency  -> return true?
     if (Client::belongToSameApplication(c, ac, true)) {
-        kDebug(1212) << "Activation: Belongs to active application";
+        qDebug() << "Activation: Belongs to active application";
         return true;
     }
     if (level == 3)   // high
         return false;
     if (time == -1U) {  // no time known
-        kDebug(1212) << "Activation: No timestamp at all";
+        qDebug() << "Activation: No timestamp at all";
         if (level == 1)   // low
             return true;
         // no timestamp at all, don't activate - because there's also creation timestamp
@@ -563,9 +586,9 @@ bool Workspace::allowClientActivation(const KWin::Client *c, xcb_timestamp_t tim
     }
     // level == 2 // normal
     Time user_time = ac->userTime();
-    kDebug(1212) << "Activation, compared:" << c << ":" << time << ":" << user_time
-                 << ":" << (timestampCompare(time, user_time) >= 0);
-    return timestampCompare(time, user_time) >= 0;   // time >= user_time
+    qDebug() << "Activation, compared:" << c << ":" << time << ":" << user_time
+                 << ":" << (NET::timestampCompare(time, user_time) >= 0) << endl;
+    return NET::timestampCompare(time, user_time) >= 0;   // time >= user_time
 }
 
 // basically the same like allowClientActivation(), this time allowing
@@ -584,20 +607,20 @@ bool Workspace::allowFullClientRaising(const KWin::Client *c, xcb_timestamp_t ti
     if (level == 4)   // extreme
         return false;
     if (ac == NULL || ac->isDesktop()) {
-        kDebug(1212) << "Raising: No client active, allowing";
+        qDebug() << "Raising: No client active, allowing";
         return true; // no active client -> always allow
     }
     // TODO window urgency  -> return true?
     if (Client::belongToSameApplication(c, ac, true)) {
-        kDebug(1212) << "Raising: Belongs to active application";
+        qDebug() << "Raising: Belongs to active application";
         return true;
     }
     if (level == 3)   // high
         return false;
     xcb_timestamp_t user_time = ac->userTime();
-    kDebug(1212) << "Raising, compared:" << time << ":" << user_time
-                 << ":" << (timestampCompare(time, user_time) >= 0);
-    return timestampCompare(time, user_time) >= 0;   // time >= user_time
+    qDebug() << "Raising, compared:" << time << ":" << user_time
+                 << ":" << (NET::timestampCompare(time, user_time) >= 0) << endl;
+    return NET::timestampCompare(time, user_time) >= 0;   // time >= user_time
 }
 
 // called from Client after FocusIn that wasn't initiated by KWin and the client
@@ -642,7 +665,7 @@ void Client::updateUserTime(xcb_timestamp_t time)
         time = xTime();
     if (time != -1U
             && (m_userTime == XCB_TIME_CURRENT_TIME
-                || timestampCompare(time, m_userTime) > 0)) {    // time > user_time
+                || NET::timestampCompare(time, m_userTime) > 0)) {    // time > user_time
         m_userTime = time;
         shade_below = NULL; // do not hover re-shade a window after it got interaction
     }
@@ -667,34 +690,31 @@ void Client::demandAttention(bool set)
     if (demands_attention == set)
         return;
     demands_attention = set;
-    info->setState(set ? NET::DemandsAttention : 0, NET::DemandsAttention);
+    info->setState(set ? NET::DemandsAttention : NET::States(0), NET::DemandsAttention);
     workspace()->clientAttentionChanged(this, set);
     emit demandsAttentionChanged();
 }
 
-// TODO I probably shouldn't be lazy here and do it without the macro, so that people can read it
-KWIN_COMPARE_PREDICATE(SameApplicationActiveHackPredicate, Client, const Client*,
-                       // ignore already existing splashes, toolbars, utilities and menus,
-                       // as the app may show those before the main window
-                       !cl->isSplash() && !cl->isToolbar() && !cl->isUtility() && !cl->isMenu()
-                       && Client::belongToSameApplication(cl, value, true) && cl != value);
-
 xcb_timestamp_t Client::readUserTimeMapTimestamp(const KStartupInfoId *asn_id, const KStartupInfoData *asn_data,
                                                  bool session) const
 {
     xcb_timestamp_t time = info->userTime();
-    //kDebug( 1212 ) << "User timestamp, initial:" << time;
+    //qDebug() << "User timestamp, initial:" << time;
     //^^ this deadlocks kwin --replace sometimes.
 
     // newer ASN timestamp always replaces user timestamp, unless user timestamp is 0
     // helps e.g. with konqy reusing
     if (asn_data != NULL && time != 0) {
+        // prefer timestamp from ASN id (timestamp from data is obsolete way)
         if (asn_id->timestamp() != 0
-                && (time == -1U || timestampCompare(asn_id->timestamp(), time) > 0)) {
+                && (time == -1U || NET::timestampCompare(asn_id->timestamp(), time) > 0)) {
             time = asn_id->timestamp();
+        } else if (asn_data->timestamp() != -1U
+                  && (time == -1U || NET::timestampCompare(asn_data->timestamp(), time) > 0)) {
+            time = asn_data->timestamp();
         }
     }
-    kDebug(1212) << "User timestamp, ASN:" << time;
+    qDebug() << "User timestamp, ASN:" << time;
     if (time == -1U) {
         // The window doesn't have any timestamp.
         // If it's the first window for its application
@@ -706,22 +726,28 @@ xcb_timestamp_t Client::readUserTimeMapTimestamp(const KStartupInfoId *asn_id, c
         Client* act = workspace()->mostRecentlyActivatedClient();
         if (act != NULL && !belongToSameApplication(act, this, true)) {
             bool first_window = true;
+            auto sameApplicationActiveHackPredicate = [this](const Client *cl) {
+                // ignore already existing splashes, toolbars, utilities and menus,
+                // as the app may show those before the main window
+                return !cl->isSplash() && !cl->isToolbar() && !cl->isUtility() && !cl->isMenu()
+                        && cl != this && Client::belongToSameApplication(cl, this, true);
+            };
             if (isTransient()) {
                 if (act->hasTransient(this, true))
                     ; // is transient for currently active window, even though it's not
                 // the same app (e.g. kcookiejar dialog) -> allow activation
                 else if (groupTransient() &&
-                        findClientInList(mainClients(), SameApplicationActiveHackPredicate(this)) == NULL)
+                        findInList<Client>(mainClients(), sameApplicationActiveHackPredicate) == NULL)
                     ; // standalone transient
                 else
                     first_window = false;
             } else {
-                if (workspace()->findClient(SameApplicationActiveHackPredicate(this)))
+                if (workspace()->findClient(sameApplicationActiveHackPredicate))
                     first_window = false;
             }
             // don't refuse if focus stealing prevention is turned off
             if (!first_window && rules()->checkFSP(options->focusStealingPreventionLevel()) > 0) {
-                kDebug(1212) << "User timestamp, already exists:" << 0;
+                qDebug() << "User timestamp, already exists:" << 0;
                 return 0; // refuse activation
             }
         }
@@ -738,7 +764,7 @@ xcb_timestamp_t Client::readUserTimeMapTimestamp(const KStartupInfoId *asn_id, c
             return -1U;
         time = readUserCreationTime();
     }
-    kDebug(1212) << "User timestamp, final:" << this << ":" << time;
+    qDebug() << "User timestamp, final:" << this << ":" << time;
     return time;
 }
 
@@ -750,7 +776,7 @@ xcb_timestamp_t Client::userTime() const
     assert(group() != NULL);
     if (time == -1U
             || (group()->userTime() != -1U
-                && timestampCompare(group()->userTime(), time) > 0))
+                && NET::timestampCompare(group()->userTime(), time) > 0))
         time = group()->userTime();
     return time;
 }
@@ -814,6 +840,8 @@ void Client::startupIdChanged()
     if (asn_data.xinerama() != -1)
         workspace()->sendClientToScreen(this, asn_data.xinerama());
     Time timestamp = asn_id.timestamp();
+    if (timestamp == 0 && asn_data.timestamp() != -1U)
+        timestamp = asn_data.timestamp();
     if (timestamp != 0) {
         bool activate = workspace()->allowClientActivation(this, timestamp);
         if (asn_data.desktop() != 0 && !isOnCurrentDesktop())
@@ -831,11 +859,6 @@ void Client::updateUrgency()
         demandAttention();
 }
 
-void Client::shortcutActivated()
-{
-    workspace()->activateClient(this, true);   // force
-}
-
 //****************************************
 // Group
 //****************************************
@@ -847,9 +870,12 @@ void Group::startupIdChanged()
     bool asn_valid = workspace()->checkStartupNotification(leader_wid, asn_id, asn_data);
     if (!asn_valid)
         return;
-    if (asn_id.timestamp() != -1U && user_time != -1U
-        && timestampCompare(asn_id.timestamp(), user_time) > 0) {
+    if (asn_id.timestamp() != 0 && user_time != -1U
+            && NET::timestampCompare(asn_id.timestamp(), user_time) > 0) {
         user_time = asn_id.timestamp();
+    } else if (asn_data.timestamp() != -1U && user_time != -1U
+              && NET::timestampCompare(asn_data.timestamp(), user_time) > 0) {
+        user_time = asn_data.timestamp();
     }
 }
 
@@ -860,7 +886,7 @@ void Group::updateUserTime(Time time)
         time = xTime();
     if (time != -1U
             && (user_time == CurrentTime
-                || timestampCompare(time, user_time) > 0))    // time > user_time
+                || NET::timestampCompare(time, user_time) > 0))    // time > user_time
         user_time = time;
 }
 
