@@ -28,24 +28,22 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "tabgroup.h"
 #include "toplevel.h"
 #include "xcbutils.h"
-// KDE
-#include <KShortcut>
-#include <NETWinInfo>
 // Qt
 #include <QPixmap>
+#include <QWindow>
 // X
-#ifdef HAVE_XSYNC
-#include <X11/extensions/sync.h>
-#endif
+#include <xcb/sync.h>
 #include <X11/Xutil.h>
 #include <fixx11h.h>
 
 // TODO: Cleanup the order of things in this .h file
 
-#include <QTimer>
+class QTimer;
 class KStartupInfoData;
 class KStartupInfoId;
 
+struct xcb_sync_alarm_notify_event_t;
+
 namespace KWin
 {
 namespace TabBox
@@ -57,6 +55,19 @@ class TabBoxClientImpl;
 class Bridge;
 class PaintRedirector;
 
+
+/**
+ * @brief Defines Predicates on how to search for a Client.
+ *
+ * Used by Workspace::findClient.
+ */
+enum class Predicate {
+    WindowMatch,
+    WrapperIdMatch,
+    FrameIdMatch,
+    InputIdMatch
+};
+
 class Client
     : public Toplevel
 {
@@ -206,8 +217,7 @@ class Client
      * Because of that no changed signal is provided.
      **/
     Q_PROPERTY(bool wantsInput READ wantsInput)
-    // TODO: a QIcon with all icon sizes?
-    Q_PROPERTY(QPixmap icon READ icon NOTIFY iconChanged)
+    Q_PROPERTY(QIcon icon READ icon NOTIFY iconChanged)
     /**
      * Whether the Client should be excluded from window switching effects.
      **/
@@ -273,6 +283,7 @@ public:
     xcb_window_t wrapperId() const;
     xcb_window_t decorationId() const;
     xcb_window_t inputId() const { return m_decoInputExtent; }
+    virtual xcb_window_t frameId() const override;
 
     const Client* transientFor() const;
     Client* transientFor();
@@ -312,11 +323,9 @@ public:
     virtual QSize clientSize() const;
     QPoint inputPos() const { return input_offset; } // Inside of geometry()
 
-    bool windowEvent(XEvent* e);
+    bool windowEvent(xcb_generic_event_t *e);
     virtual bool eventFilter(QObject* o, QEvent* e);
-#ifdef HAVE_XSYNC
-    void syncEvent(XSyncAlarmNotifyEvent* e);
-#endif
+    void syncEvent(xcb_sync_alarm_notify_event_t* e);
     NET::WindowType windowType(bool direct = false, int supported_types = 0) const;
 
     bool manage(xcb_window_t w, bool isMapped);
@@ -333,11 +342,7 @@ public:
     QSize adjustedSize(const QSize&, Sizemode mode = SizemodeAny) const;
     QSize adjustedSize() const;
 
-    QPixmap icon() const;
-    QPixmap icon(const QSize& size) const;
-    QPixmap miniIcon() const;
-    QPixmap bigIcon() const;
-    QPixmap hugeIcon() const;
+    const QIcon &icon() const;
 
     bool isActive() const;
     void setActive(bool);
@@ -348,6 +353,13 @@ public:
 
     void sendToScreen(int screen);
 
+    virtual QStringList activities() const;
+    void setOnActivity(const QString &activity, bool enable);
+    void setOnAllActivities(bool set);
+    void setOnActivities(QStringList newActivitiesList);
+    void updateActivities(bool includeTransients);
+    void blockActivityUpdates(bool b = true);
+
     /// Is not minimized and not hidden. I.e. normally visible on some virtual desktop.
     bool isShown(bool shaded_is_shown) const;
     bool isHiddenInternal() const; // For compositing
@@ -415,7 +427,6 @@ public:
     bool isMovableAcrossScreens() const;
     bool isCloseable() const; ///< May be closed by the user (May have a close button)
 
-    void takeActivity(int flags, bool handled);   // Takes ActivityFlags as arg (in utils.h)
     void takeFocus();
     bool isDemandingAttention() const {
         return demands_attention;
@@ -431,6 +442,7 @@ public:
 
     void updateShape();
 
+    enum ForceGeometry_t { NormalGeometrySet, ForceGeometrySet };
     void setGeometry(int x, int y, int w, int h, ForceGeometry_t force = NormalGeometrySet);
     void setGeometry(const QRect& r, ForceGeometry_t force = NormalGeometrySet);
     void move(int x, int y, ForceGeometry_t force = NormalGeometrySet);
@@ -461,11 +473,11 @@ public:
     void shrinkVertical();
 
     bool providesContextHelp() const;
-    KShortcut shortcut() const;
+    const QKeySequence &shortcut() const;
     void setShortcut(const QString& cut);
 
     WindowOperation mouseButtonToWindowOperation(Qt::MouseButtons button);
-    bool performMouseCommand(Options::MouseCommand, const QPoint& globalPos, bool handled = false);
+    bool performMouseCommand(Options::MouseCommand, const QPoint& globalPos);
 
     QRect adjustedClientArea(const QRect& desktop, const QRect& area) const;
 
@@ -508,7 +520,6 @@ public:
 
     static bool belongToSameApplication(const Client* c1, const Client* c2, bool active_hack = false);
     static bool sameAppWindowRoleMatch(const Client* c1, const Client* c2, bool active_hack);
-    static void readIcons(xcb_window_t win, QPixmap* icon, QPixmap* miniicon, QPixmap* bigicon, QPixmap* hugeicon);
 
     void setMinimized(bool set);
     void minimize(bool avoid_animation = false);
@@ -617,6 +628,7 @@ public:
         m_firstInTabBox = enable;
     }
     void updateFirstInTabBox();
+    void updateColorScheme();
 
     //sets whether the client should be treated as a SessionInteract window
     void setSessionInteract(bool needed);
@@ -624,18 +636,47 @@ public:
     // a helper for the workspace window packing. tests for screen validity and updates since in maximization case as with normal moving
     void packTo(int left, int top);
 
+#ifdef KWIN_BUILD_KAPPMENU
+    // Used by workspace
+    void emitShowRequest() {
+        emit showRequest();
+    }
+    void emitMenuHidden() {
+        emit menuHidden();
+    }
+    void setAppMenuAvailable();
+    void setAppMenuUnavailable();
+    void showApplicationMenu(const QPoint&);
+    bool menuAvailable() {
+        return m_menuAvailable;
+    }
+#endif
+
     template <typename T>
     void print(T &stream) const;
 
-public slots:
+    void cancelFocusOutTimer();
+
+    QPalette palette() const;
+
+    /**
+     * Restores the Client after it had been hidden due to show on screen edge functionality.
+     * In addition the property gets deleted so that the Client knows that it is visible again.
+     **/
+    void showOnScreenEdge();
+
+    void sendPointerButtonEvent(uint32_t button, InputRedirection::PointerButtonState state) override;
+    void sendPointerAxisEvent(InputRedirection::PointerAxis axis, qreal delta) override;
+    void sendKeybordKeyEvent(uint32_t key, InputRedirection::KeyboardKeyState state) override;
+
+public Q_SLOTS:
     void closeWindow();
     void updateCaption();
 
-private slots:
+private Q_SLOTS:
     void autoRaise();
     void shadeHover();
     void shadeUnhover();
-    void shortcutActivated();
     void delayedMoveResize();
 
 private:
@@ -650,16 +691,16 @@ private:
     void updateCursor();
 
     // Handlers for X11 events
-    bool mapRequestEvent(XMapRequestEvent* e);
-    void unmapNotifyEvent(XUnmapEvent* e);
-    void destroyNotifyEvent(XDestroyWindowEvent* e);
-    void configureRequestEvent(XConfigureRequestEvent* e);
-    virtual void propertyNotifyEvent(XPropertyEvent* e);
-    void clientMessageEvent(XClientMessageEvent* e);
-    void enterNotifyEvent(XCrossingEvent* e);
-    void leaveNotifyEvent(XCrossingEvent* e);
-    void focusInEvent(XFocusInEvent* e);
-    void focusOutEvent(XFocusOutEvent* e);
+    bool mapRequestEvent(xcb_map_request_event_t *e);
+    void unmapNotifyEvent(xcb_unmap_notify_event_t *e);
+    void destroyNotifyEvent(xcb_destroy_notify_event_t *e);
+    void configureRequestEvent(xcb_configure_request_event_t *e);
+    virtual void propertyNotifyEvent(xcb_property_notify_event_t *e) override;
+    void clientMessageEvent(xcb_client_message_event_t *e);
+    void enterNotifyEvent(xcb_enter_notify_event_t *e);
+    void leaveNotifyEvent(xcb_leave_notify_event_t *e);
+    void focusInEvent(xcb_focus_in_event_t *e);
+    void focusOutEvent(xcb_focus_out_event_t *e);
     virtual void damageNotifyEvent();
 
     bool buttonPressEvent(xcb_window_t w, int button, int state, int x, int y, int x_root, int y_root);
@@ -676,7 +717,7 @@ protected:
     virtual void debug(QDebug& stream) const;
     virtual bool shouldUnredirect() const;
 
-private slots:
+private Q_SLOTS:
     void delayedSetShortcut();
     void performMoveResize();
     void removeSyncSupport();
@@ -686,7 +727,7 @@ private slots:
     //Signals make an excellent way for communication
     //in between objects as compared to simple function
     //calls
-signals:
+Q_SIGNALS:
     void clientManaging(KWin::Client*);
     void clientFullScreenSet(KWin::Client*, bool, bool);
     void clientMaximizedStateChanged(KWin::Client*, KDecorationDefines::MaximizeMode);
@@ -718,6 +759,23 @@ signals:
      **/
     void tabGroupChanged();
 
+    /**
+     * Emitted whenever the Client want to show it menu
+     */
+    void showRequest();
+    /**
+     * Emitted whenever the Client's menu is closed
+     */
+    void menuHidden();
+    /**
+     * Emitted whenever the Client's menu is available
+     **/
+    void appMenuAvailable();
+    /**
+     * Emitted whenever the Client's menu is unavailable
+     */
+    void appMenuUnavailable();
+
     /**
      * Emitted whenever the demands attention state changes.
      **/
@@ -744,7 +802,7 @@ private:
     void setCaption(const QString& s, bool force = false);
     bool hasTransientInternal(const Client* c, bool indirect, ConstClientList& set) const;
     void finishWindowRules();
-    void setShortcutInternal(const KShortcut& cut);
+    void setShortcutInternal(const QKeySequence &cut = QKeySequence());
 
     void configureRequest(int value_mask, int rx, int ry, int rw, int rh, int gravity, bool from_tool);
     NETExtendedStrut strut() const;
@@ -796,11 +854,21 @@ private:
 
     bool tabTo(Client *other, bool behind, bool activate);
 
-    xcb_window_t m_client;
+    /**
+     * Reads the property and creates/destroys the screen edge if required
+     * and shows/hides the client.
+     **/
+    void updateShowOnScreenEdge();
+
+    Xcb::Window m_client;
     Xcb::Window m_wrapper;
+    Xcb::Window m_frame;
     KDecoration* decoration;
     Bridge* bridge;
     int desk;
+    QStringList activityList;
+    int m_activityUpdatesBlocked;
+    bool m_blockedActivityUpdatesRequireTransients;
     bool buttonDown;
     bool moveResizeMode;
     Xcb::Window m_moveResizeGrabWindow;
@@ -850,7 +918,6 @@ private:
     uint original_skip_taskbar : 1; ///< Unaffected by KWin
     uint Pdeletewindow : 1; ///< Does the window understand the DeleteWindow protocol?
     uint Ptakefocus : 1;///< Does the window understand the TakeFocus protocol?
-    uint Ptakeactivity : 1; ///< Does it support _NET_WM_TAKE_ACTIVITY
     uint Pcontexthelp : 1; ///< Does the window understand the ContextHelp protocol?
     uint Pping : 1; ///< Does it support _NET_WM_PING?
     uint input : 1; ///< Does the window want input in its wm_hints
@@ -872,12 +939,8 @@ private:
     bool blocks_compositing;
     WindowRules client_rules;
     void getWMHints();
-    void readIcons();
     void getWindowProtocols();
-    QPixmap icon_pix;
-    QPixmap miniicon_pix;
-    QPixmap bigicon_pix;
-    QPixmap hugeicon_pix;
+    QIcon m_icon;
     Qt::CursorShape m_cursor;
     // DON'T reorder - Saved to config files !!!
     enum FullScreenMode {
@@ -902,7 +965,7 @@ private:
     qint64 m_killHelperPID;
     xcb_timestamp_t m_pingTimestamp;
     xcb_timestamp_t m_userTime;
-    unsigned long allowed_actions;
+    NET::Actions allowed_actions;
     QSize client_size;
     int block_geometry_updates; // > 0 = New geometry is remembered, but not actually set
     enum PendingGeometry_t {
@@ -914,22 +977,19 @@ private:
     QRect geom_before_block;
     QRect deco_rect_before_block;
     bool shade_geometry_change;
-#ifdef HAVE_XSYNC
     struct {
-        XSyncCounter counter;
-        XSyncValue value;
-        XSyncAlarm alarm;
+        xcb_sync_counter_t counter;
+        xcb_sync_int64_t value;
+        xcb_sync_alarm_t alarm;
         QTimer *timeout, *failsafeTimeout;
         bool isPending;
     } syncRequest;
-#endif
     int border_left, border_right, border_top, border_bottom;
     int padding_left, padding_right, padding_top, padding_bottom;
     QRegion _mask;
     static bool check_active_modal; ///< \see Client::checkActiveModal()
-    KShortcut _shortcut;
+    QKeySequence _shortcut;
     int sm_stacking_order;
-    friend struct FetchNameInternalPredicate;
     friend struct ResetupRulesProcedure;
     friend class GeometryUpdatesBlocker;
     PaintRedirector* paintRedirector;
@@ -941,11 +1001,22 @@ private:
 
     friend bool performTransiencyCheck();
 
+    void checkActivities();
+    bool activitiesDefined; //whether the x property was actually set
+
     bool needsSessionInteract;
     bool needsXWindowMove;
 
+#ifdef KWIN_BUILD_KAPPMENU
+    bool m_menuAvailable;
+#endif
     Xcb::Window m_decoInputExtent;
     QPoint input_offset;
+
+    QTimer *m_focusOutTimer;
+
+    QPalette m_palette;
+    QList<QMetaObject::Connection> m_connections;
 };
 
 /**
@@ -974,7 +1045,7 @@ inline xcb_window_t Client::wrapperId() const
 inline xcb_window_t Client::decorationId() const
 {
     if (decoration) {
-        return decoration->widget()->winId();
+        return decoration->window()->winId();
     }
     return XCB_WINDOW_NONE;
 }
@@ -1057,24 +1128,9 @@ inline ShadeMode Client::shadeMode() const
     return shade_mode;
 }
 
-inline QPixmap Client::icon() const
+inline const QIcon &Client::icon() const
 {
-    return icon_pix;
-}
-
-inline QPixmap Client::miniIcon() const
-{
-    return miniicon_pix;
-}
-
-inline QPixmap Client::bigIcon() const
-{
-    return bigicon_pix;
-}
-
-inline QPixmap Client::hugeIcon() const
-{
-    return hugeicon_pix;
+    return m_icon;
 }
 
 inline QRect Client::geometryRestore() const
@@ -1197,7 +1253,7 @@ inline xcb_window_t Client::moveResizeGrabWindow() const
     return m_moveResizeGrabWindow;
 }
 
-inline KShortcut Client::shortcut() const
+inline const QKeySequence &Client::shortcut() const
 {
     return _shortcut;
 }
@@ -1212,6 +1268,11 @@ inline bool Client::hiddenPreview() const
     return mapping_state == Kept;
 }
 
+inline QPalette Client::palette() const
+{
+    return m_palette;
+}
+
 template <typename T>
 inline void Client::print(T &stream) const
 {
@@ -1219,9 +1280,6 @@ inline void Client::print(T &stream) const
            << resourceName() << ";Caption:" << caption() << "\'";
 }
 
-KWIN_COMPARE_PREDICATE(WrapperIdMatchPredicate, Client, Window, cl->wrapperId() == value);
-KWIN_COMPARE_PREDICATE(InputIdMatchPredicate, Client, Window, cl->inputId() == value);
-
 } // namespace
 Q_DECLARE_METATYPE(KWin::Client*)
 Q_DECLARE_METATYPE(QList<KWin::Client*>)
