@@ -20,46 +20,58 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "config-kwin.h"
 
 #include <QApplication>
-#include <QDeclarativeComponent>
-#include <QDeclarativeContext>
-#include <QDeclarativeEngine>
-#include <QDeclarativeItem>
-#include <QGraphicsView>
-#include <QPaintEngine>
+#include <QDebug>
+#include <QMutex>
+#include <QOpenGLFramebufferObject>
+#include <QPainter>
+#include <QQmlComponent>
+#include <QQmlContext>
+#include <QQmlEngine>
+#include <QQuickItem>
+#include <QQuickWindow>
+#include <QStandardPaths>
+#include <QWidget>
 
 #include <KConfig>
 #include <KConfigGroup>
-#include <KDebug>
+#include <KSharedConfig>
 #include <KPluginInfo>
 #include <KServiceTypeTrader>
-#include <KStandardDirs>
+
+K_PLUGIN_FACTORY_WITH_JSON(AuroraePluginFactory,
+                           "aurorae.json",
+                           registerPlugin<Aurorae::AuroraeFactory>(QString(), &Aurorae::AuroraeFactory::createInstance);)
+K_EXPORT_PLUGIN_VERSION(KWIN_DECORATION_API_VERSION)
 
 namespace Aurorae
 {
 
-AuroraeFactory::AuroraeFactory()
-        : QObject()
-        , KDecorationFactory()
+AuroraeFactory::AuroraeFactory(QObject *parent)
+        : KDecorationFactory(parent)
         , m_theme(new AuroraeTheme(this))
-        , m_engine(new QDeclarativeEngine(this))
-        , m_component(new QDeclarativeComponent(m_engine, this))
+        , m_engine(new QQmlEngine(this))
+        , m_component(new QQmlComponent(m_engine, this))
         , m_engineType(AuroraeEngine)
+        , m_mutex(new QMutex(QMutex::Recursive))
 {
     init();
+    connect(options(), &KDecorationOptions::buttonsChanged, this, &AuroraeFactory::buttonsChanged);
+    connect(options(), &KDecorationOptions::fontsChanged, this, &AuroraeFactory::titleFontChanged);
+    connect(options(), &KDecorationOptions::configChanged, this, &AuroraeFactory::updateConfiguration);
 }
 
 void AuroraeFactory::init()
 {
     qRegisterMetaType<uint>("Qt::MouseButtons");
 
-    KConfig conf("auroraerc");
+    KConfig conf(QStringLiteral("auroraerc"));
     KConfigGroup group(&conf, "Engine");
     if (!group.hasKey("EngineType") && !group.hasKey("ThemeName")) {
         // neither engine type and theme name are configured, use the only available theme
         initQML(group);
     } else if (group.hasKey("EngineType")) {
         const QString engineType = group.readEntry("EngineType", "aurorae").toLower();
-        if (engineType == "qml") {
+        if (engineType == QStringLiteral("qml")) {
             initQML(group);
         } else {
             // fallback to classic Aurorae Themes
@@ -80,7 +92,8 @@ void AuroraeFactory::initAurorae(KConfig &conf, KConfigGroup &group)
         initQML(group);
         return;
     }
-    KConfig config("aurorae/themes/" + themeName + '/' + themeName + "rc", KConfig::FullConfig, "data");
+    KConfig config(QStringLiteral("aurorae/themes/") + themeName + QStringLiteral("/") + themeName + QStringLiteral("rc"),
+                   KConfig::FullConfig, QStandardPaths::GenericDataLocation);
     KConfigGroup themeGroup(&conf, themeName);
     m_theme->loadTheme(themeName, config);
     m_theme->setBorderSize((KDecorationDefines::BorderSize)themeGroup.readEntry<int>("BorderSize", KDecorationDefines::BorderNormal));
@@ -90,13 +103,13 @@ void AuroraeFactory::initAurorae(KConfig &conf, KConfigGroup &group)
     /* use logic from KDeclarative::setupBindings():
     "addImportPath adds the path at the beginning, so to honour user's
      paths we need to traverse the list in reverse order" */
-    QStringListIterator paths(KGlobal::dirs()->findDirs("module", "imports"));
+    QStringListIterator paths(QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("module/imports"), QStandardPaths::LocateDirectory));
     paths.toBack();
     while (paths.hasPrevious()) {
         m_engine->addImportPath(paths.previous());
     }
-    m_component->loadUrl(QUrl(KStandardDirs::locate("data", "kwin/aurorae/aurorae.qml")));
-    m_engine->rootContext()->setContextProperty("auroraeTheme", m_theme);
+    m_component->loadUrl(QUrl(QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("kwin/aurorae/aurorae.qml"))));
+    m_engine->rootContext()->setContextProperty(QStringLiteral("auroraeTheme"), m_theme);
     m_themeName = themeName;
 }
 
@@ -104,23 +117,22 @@ void AuroraeFactory::initQML(const KConfigGroup &group)
 {
     // try finding the QML package
     const QString themeName = group.readEntry("ThemeName", "kwin4_decoration_qml_plastik");
-    kDebug(1212) << "Trying to load QML Decoration " << themeName;
+    qCDebug(AURORAE) << "Trying to load QML Decoration " << themeName;
     const QString internalname = themeName.toLower();
 
-    QString constraint = QString("[X-KDE-PluginInfo-Name] == '%1'").arg(internalname);
-    KService::List offers = KServiceTypeTrader::self()->query("KWin/Decoration", constraint);
+    QString constraint = QStringLiteral("[X-KDE-PluginInfo-Name] == '%1'").arg(internalname);
+    KService::List offers = KServiceTypeTrader::self()->query(QStringLiteral("KWin/Decoration"), constraint);
     if (offers.isEmpty()) {
-        kError(1212) << "Couldn't find QML Decoration " << themeName;
+        qCCritical(AURORAE) << "Couldn't find QML Decoration " << themeName << endl;
         // TODO: what to do in error case?
         return;
     }
     KService::Ptr service = offers.first();
-    KPluginInfo plugininfo(service);
-    const QString pluginName = service->property("X-KDE-PluginInfo-Name").toString();
-    const QString scriptName = service->property("X-Plasma-MainScript").toString();
-    const QString file = KStandardDirs::locate("data", "kwin/decorations/" + pluginName + "/contents/" + scriptName);
+    const QString pluginName = service->property(QStringLiteral("X-KDE-PluginInfo-Name")).toString();
+    const QString scriptName = service->property(QStringLiteral("X-Plasma-MainScript")).toString();
+    const QString file = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral(KWIN_NAME) + QStringLiteral("/decorations/") + pluginName + QStringLiteral("/contents/") + scriptName);
     if (file.isNull()) {
-        kDebug(1212) << "Could not find script file for " << pluginName;
+        qCDebug(AURORAE) << "Could not find script file for " << pluginName;
         // TODO: what to do in error case?
         return;
     }
@@ -129,7 +141,7 @@ void AuroraeFactory::initQML(const KConfigGroup &group)
     /* use logic from KDeclarative::setupBindings():
     "addImportPath adds the path at the beginning, so to honour user's
      paths we need to traverse the list in reverse order" */
-    QStringListIterator paths(KGlobal::dirs()->findDirs("module", "imports"));
+    QStringListIterator paths(QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("module/imports"), QStandardPaths::LocateDirectory));
     paths.toBack();
     while (paths.hasPrevious()) {
         m_engine->addImportPath(paths.previous());
@@ -140,7 +152,7 @@ void AuroraeFactory::initQML(const KConfigGroup &group)
 
 AuroraeFactory::~AuroraeFactory()
 {
-    s_instance = NULL;
+    s_instance = nullptr;
 }
 
 AuroraeFactory *AuroraeFactory::instance()
@@ -152,39 +164,35 @@ AuroraeFactory *AuroraeFactory::instance()
     return s_instance;
 }
 
-bool AuroraeFactory::reset(unsigned long changed)
+QObject *AuroraeFactory::createInstance(QWidget*, QObject*, const QList< QVariant >&)
 {
-    if (changed & SettingButtons) {
-        emit buttonsChanged();
-    }
-    if (changed & SettingFont){
-        emit titleFontChanged();
-    }
-    if (changed == SettingCompositing) {
-        return false;
-    }
-    const KConfig conf("auroraerc");
+    return instance();
+}
+
+void AuroraeFactory::updateConfiguration()
+{
+    const KConfig conf(QStringLiteral("auroraerc"));
     const KConfigGroup group(&conf, "Engine");
     const QString themeName = group.readEntry("ThemeName", "example-deco");
-    const KConfig config("aurorae/themes/" + themeName + '/' + themeName + "rc", KConfig::FullConfig, "data");
+    const KConfig config(QStringLiteral("aurorae/themes/") + themeName + QStringLiteral("/") + themeName + QStringLiteral("rc"),
+                         KConfig::FullConfig, QStandardPaths::GenericDataLocation);
     const KConfigGroup themeGroup(&conf, themeName);
     if (themeName != m_themeName) {
         m_engine->clearComponentCache();
         init();
-        // recreate all decorations
-        return true;
+        emit recreateDecorations();
     }
     if (m_engineType == AuroraeEngine) {
         m_theme->setBorderSize((KDecorationDefines::BorderSize)themeGroup.readEntry<int>("BorderSize", KDecorationDefines::BorderNormal));
         m_theme->setButtonSize((KDecorationDefines::BorderSize)themeGroup.readEntry<int>("ButtonSize", KDecorationDefines::BorderNormal));
     }
     emit configChanged();
-    return changed & (SettingDecoration | SettingButtons | SettingBorder); // need hard reset
 }
 
 bool AuroraeFactory::supports(Ability ability) const
 {
     switch (ability) {
+    case AbilityAnnounceButtons:
     case AbilityUsesAlphaChannel:
     case AbilityAnnounceAlphaChannel:
     case AbilityButtonMenu:
@@ -198,10 +206,13 @@ bool AuroraeFactory::supports(Ability ability) const
     case AbilityButtonShade:
     case AbilityButtonOnAllDesktops:
     case AbilityButtonHelp:
+    case AbilityButtonApplicationMenu:
     case AbilityProvidesShadow:
         return true; // TODO: correct value from theme
     case AbilityTabbing:
         return false;
+    case AbilityUsesBlurBehind:
+        return true;
     default:
         return false;
     }
@@ -220,72 +231,67 @@ QList< KDecorationDefines::BorderSize > AuroraeFactory::borderSizes() const
         BorderVeryHuge << BorderOversized;
 }
 
-QDeclarativeItem *AuroraeFactory::createQmlDecoration(Aurorae::AuroraeClient *client)
+QQuickItem *AuroraeFactory::createQmlDecoration(Aurorae::AuroraeClient *client)
 {
-    QDeclarativeContext *context = new QDeclarativeContext(m_engine->rootContext(), this);
-    context->setContextProperty("decoration", client);
-    return qobject_cast< QDeclarativeItem* >(m_component->create(context));
+    QQmlContext *context = new QQmlContext(m_engine->rootContext(), this);
+    context->setContextProperty(QStringLiteral("decoration"), client);
+    return qobject_cast< QQuickItem* >(m_component->create(context));
 }
 
-AuroraeFactory *AuroraeFactory::s_instance = NULL;
+AuroraeFactory *AuroraeFactory::s_instance = nullptr;
 
 /*******************************************************
 * Client
 *******************************************************/
 AuroraeClient::AuroraeClient(KDecorationBridge *bridge, KDecorationFactory *factory)
     : KDecoration(bridge, factory)
-    , m_view(NULL)
-    , m_scene(new QGraphicsScene(this))
+    , m_view(nullptr)
     , m_item(AuroraeFactory::instance()->createQmlDecoration(this))
 {
-    connect(this, SIGNAL(keepAboveChanged(bool)), SIGNAL(keepAboveChangedWrapper()));
-    connect(this, SIGNAL(keepBelowChanged(bool)), SIGNAL(keepBelowChangedWrapper()));
     connect(AuroraeFactory::instance(), SIGNAL(buttonsChanged()), SIGNAL(buttonsChanged()));
     connect(AuroraeFactory::instance(), SIGNAL(configChanged()), SIGNAL(configChanged()));
     connect(AuroraeFactory::instance(), SIGNAL(titleFontChanged()), SIGNAL(fontChanged()));
-    connect(m_item, SIGNAL(alphaChanged()), SLOT(slotAlphaChanged()));
+    connect(m_item.data(), SIGNAL(alphaChanged()), SLOT(slotAlphaChanged()));
+    connect(this, SIGNAL(appMenuAvailable()), SIGNAL(appMenuAvailableChanged()));
+    connect(this, SIGNAL(appMenuUnavailable()), SIGNAL(appMenuAvailableChanged()));
 }
 
 AuroraeClient::~AuroraeClient()
 {
-    if (m_item) {
-        m_item->setParent(NULL);
-        m_item->deleteLater();
-    }
-    m_scene->setParent(NULL);
-    m_scene->deleteLater();
-    m_view->setParent(NULL);
-    m_view->deleteLater();
 }
 
 void AuroraeClient::init()
 {
-    m_scene->setItemIndexMethod(QGraphicsScene::NoIndex);
-    // HACK: we need to add the GraphicsView as a child widget to a normal widget
-    // the GraphicsView eats the mouse release event and by that kwin core starts to move
-    // the decoration each time the decoration is clicked
-    // therefore we use two widgets and inject an own mouse release event to the parent widget
-    // when the graphics view eats a mouse event
-    createMainWidget();
-    widget()->setAttribute(Qt::WA_TranslucentBackground);
-    widget()->setAttribute(Qt::WA_NoSystemBackground);
-    widget()->installEventFilter(this);
-    m_view = new QGraphicsView(m_scene, widget());
-    m_view->setAttribute(Qt::WA_TranslucentBackground);
-    m_view->setWindowFlags(Qt::X11BypassWindowManagerHint);
-    m_view->setFrameShape(QFrame::NoFrame);
-    m_view->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-    m_view->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-    m_view->setOptimizationFlags(QGraphicsView::DontSavePainterState);
-    m_view->setViewportUpdateMode(QGraphicsView::BoundingRectViewportUpdate);
-    QPalette pal = m_view->palette();
-    pal.setColor(m_view->backgroundRole(), Qt::transparent);
-    m_view->setPalette(pal);
-    QPalette pal2 = widget()->palette();
-    pal2.setColor(widget()->backgroundRole(), Qt::transparent);
-    widget()->setPalette(pal2);
-    if (m_item)
-        m_scene->addItem(m_item);
+    m_view = new QQuickWindow();
+    m_view->setFlags(initialWFlags());
+    m_view->setColor(Qt::transparent);
+    setMainWindow(m_view);
+    if (compositingActive()) {
+        connect(m_view, &QQuickWindow::beforeRendering, [this]() {
+            int left, right, top, bottom;
+            left = right = top = bottom = 0;
+            padding(left, right, top, bottom);
+            if (m_fbo.isNull() || m_fbo->size() != QSize(width() + left + right, height() + top + bottom)) {
+                m_fbo.reset(new QOpenGLFramebufferObject(QSize(width() + left + right, height() + top + bottom),
+                                                         QOpenGLFramebufferObject::CombinedDepthStencil));
+                if (!m_fbo->isValid()) {
+                    qCWarning(AURORAE) << "Creating FBO as render target failed";
+                    m_fbo.reset();
+                    return;
+                }
+            }
+            m_view->setRenderTarget(m_fbo.data());
+        });
+        connect(m_view, &QQuickWindow::afterRendering, [this]{
+            QMutexLocker locker(AuroraeFactory::instance()->mutex());
+            m_buffer = m_fbo->toImage();
+        });
+        connect(m_view, &QQuickWindow::afterRendering, this,
+                static_cast<void (KDecoration::*)(void)>(&KDecoration::update), Qt::QueuedConnection);
+    }
+    if (m_item) {
+        m_item->setParentItem(m_view->contentItem());
+    }
     slotAlphaChanged();
 
     AuroraeFactory::instance()->theme()->setCompositingActive(compositingActive());
@@ -308,45 +314,13 @@ bool AuroraeClient::eventFilter(QObject *object, QEvent *event)
     return false;
 }
 
-void AuroraeClient::activeChange()
-{
-    emit activeChanged();
-}
-
-void AuroraeClient::captionChange()
-{
-    emit captionChanged();
-}
-
-void AuroraeClient::iconChange()
-{
-    emit iconChanged();
-}
-
-void AuroraeClient::desktopChange()
-{
-    emit desktopChanged();
-}
-
-void AuroraeClient::maximizeChange()
-{
-    emit maximizeChanged();
-}
-
 void AuroraeClient::resize(const QSize &s)
 {
     if (m_item) {
         m_item->setWidth(s.width());
         m_item->setHeight(s.height());
     }
-    m_scene->setSceneRect(QRectF(QPoint(0, 0), s));
     m_view->resize(s);
-    widget()->resize(s);
-}
-
-void AuroraeClient::shadeChange()
-{
-    emit shadeChanged();
 }
 
 void AuroraeClient::borders(int &left, int &right, int &top, int &bottom) const
@@ -355,23 +329,26 @@ void AuroraeClient::borders(int &left, int &right, int &top, int &bottom) const
         left = right = top = bottom = 0;
         return;
     }
-    const bool maximized = maximizeMode() == MaximizeFull;
-    QObject *borders = NULL;
-    if (maximized) {
-        borders = m_item->findChild<QObject*>("maximizedBorders");
+    QObject *borders = nullptr;
+    if (maximizeMode() == MaximizeFull) {
+        borders = m_item->findChild<QObject*>(QStringLiteral("maximizedBorders"));
     } else {
-        borders = m_item->findChild<QObject*>("borders");
+        borders = m_item->findChild<QObject*>(QStringLiteral("borders"));
     }
     sizesFromBorders(borders, left, right, top, bottom);
 }
 
 void AuroraeClient::padding(int &left, int &right, int &top, int &bottom) const
 {
-    if (!m_item || maximizeMode() == MaximizeFull) {
+    if (!m_item) {
         left = right = top = bottom = 0;
         return;
     }
-    sizesFromBorders(m_item->findChild<QObject*>("padding"), left, right, top, bottom);
+    if (maximizeMode() == MaximizeFull) {
+        left = right = top = bottom = 0;
+        return;
+    }
+    sizesFromBorders(m_item->findChild<QObject*>(QStringLiteral("padding")), left, right, top, bottom);
 }
 
 void AuroraeClient::sizesFromBorders(const QObject *borders, int &left, int &right, int &top, int &bottom) const
@@ -387,7 +364,7 @@ void AuroraeClient::sizesFromBorders(const QObject *borders, int &left, int &rig
 
 QSize AuroraeClient::minimumSize() const
 {
-    return widget()->minimumSize();
+    return m_view->minimumSize();
 }
 
 KDecorationDefines::Position AuroraeClient::mousePosition(const QPoint &point) const
@@ -435,16 +412,16 @@ KDecorationDefines::Position AuroraeClient::mousePosition(const QPoint &point) c
     return Position(pos);
 }
 
-void AuroraeClient::reset(long unsigned int changed)
-{
-    KDecoration::reset(changed);
-}
-
 void AuroraeClient::menuClicked()
 {
     showWindowMenu(QCursor::pos());
 }
 
+void AuroraeClient::appMenuClicked()
+{
+    showApplicationMenu(QCursor::pos());
+}
+
 void AuroraeClient::toggleShade()
 {
     setShade(!isShade());
@@ -460,68 +437,30 @@ void AuroraeClient::toggleKeepBelow()
     setKeepBelow(!keepBelow());
 }
 
-bool AuroraeClient::isMaximized() const
-{
-    return maximizeMode()==KDecorationDefines::MaximizeFull;
-}
-
 void AuroraeClient::titlePressed(int button, int buttons)
 {
     titlePressed(static_cast<Qt::MouseButton>(button), static_cast<Qt::MouseButtons>(buttons));
 }
 
-void AuroraeClient::titleReleased(int button, int buttons)
-{
-    titleReleased(static_cast<Qt::MouseButton>(button), static_cast<Qt::MouseButtons>(buttons));
-}
-
-void AuroraeClient::titleMouseMoved(int button, int buttons)
-{
-    titleMouseMoved(static_cast<Qt::MouseButton>(button), static_cast<Qt::MouseButtons>(buttons));
-}
-
 void AuroraeClient::titlePressed(Qt::MouseButton button, Qt::MouseButtons buttons)
 {
     const QPoint cursor = QCursor::pos();
-    QMouseEvent *event = new QMouseEvent(QEvent::MouseButtonPress, widget()->mapFromGlobal(cursor),
+    QMouseEvent *event = new QMouseEvent(QEvent::MouseButtonPress, m_view->mapFromGlobal(cursor),
                                          cursor, button, buttons, Qt::NoModifier);
     processMousePressEvent(event);
     delete event;
-    event = 0;
-}
-
-void AuroraeClient::titleReleased(Qt::MouseButton button, Qt::MouseButtons buttons)
-{
-    const QPoint cursor = QCursor::pos();
-    QMouseEvent *event = new QMouseEvent(QEvent::MouseButtonRelease, widget()->mapFromGlobal(cursor),
-                                         cursor, button, buttons, Qt::NoModifier);
-    QApplication::sendEvent(widget(), event);
-    delete event;
-    event = 0;
-}
-
-void AuroraeClient::titleMouseMoved(Qt::MouseButton button, Qt::MouseButtons buttons)
-{
-    const QPoint cursor = QCursor::pos();
-    QMouseEvent *event = new QMouseEvent(QEvent::MouseMove, widget()->mapFromGlobal(cursor),
-                                         cursor, button, buttons, Qt::NoModifier);
-    QApplication::sendEvent(widget(), event);
-    delete event;
-    event = 0;
+    event = nullptr;
 }
 
 void AuroraeClient::themeChanged()
 {
-    m_scene->clear();
-    m_item = AuroraeFactory::instance()->createQmlDecoration(this);
+    m_item.reset(AuroraeFactory::instance()->createQmlDecoration(this));
     if (!m_item) {
         return;
     }
 
-    m_item->setWidth(m_scene->sceneRect().width());
-    m_item->setHeight(m_scene->sceneRect().height());
-    m_scene->addItem(m_item);
-    connect(m_item, SIGNAL(alphaChanged()), SLOT(slotAlphaChanged()));
+    m_item->setParentItem(m_view->contentItem());
+    connect(m_item.data(), SIGNAL(alphaChanged()), SLOT(slotAlphaChanged()));
     slotAlphaChanged();
 }
 
@@ -570,7 +509,7 @@ void AuroraeClient::doTitlebarDblClickOperation()
 
 QVariant AuroraeClient::readConfig(const QString &key, const QVariant &defaultValue)
 {
-    KSharedConfigPtr config = KSharedConfig::openConfig("auroraerc");
+    KSharedConfigPtr config = KSharedConfig::openConfig(QStringLiteral("auroraerc"));
     return config->group(AuroraeFactory::instance()->currentThemeName()).readEntry(key, defaultValue);
 }
 
@@ -603,7 +542,7 @@ QRegion AuroraeClient::region(KDecorationDefines::Region r)
     }
     int left, right, top, bottom;
     left = right = top = bottom = 0;
-    sizesFromBorders(m_item->findChild<QObject*>("extendedBorders"), left, right, top, bottom);
+    sizesFromBorders(m_item->findChild<QObject*>(QStringLiteral("extendedBorders")), left, right, top, bottom);
     if (top == 0 && right == 0 && bottom == 0 && left == 0) {
         // no extended borders
         return QRegion();
@@ -612,7 +551,7 @@ QRegion AuroraeClient::region(KDecorationDefines::Region r)
     int paddingLeft, paddingRight, paddingTop, paddingBottom;
     paddingLeft = paddingRight = paddingTop = paddingBottom = 0;
     padding(paddingLeft, paddingRight, paddingTop, paddingBottom);
-    QRect rect = widget()->rect().adjusted(paddingLeft, paddingTop, -paddingRight, -paddingBottom);
+    QRect rect = this->rect().adjusted(paddingLeft, paddingTop, -paddingRight, -paddingBottom);
     rect.translate(-paddingLeft, -paddingTop);
 
     return QRegion(rect.adjusted(-left, -top, right, bottom)).subtract(rect);
@@ -620,27 +559,17 @@ QRegion AuroraeClient::region(KDecorationDefines::Region r)
 
 bool AuroraeClient::animationsSupported() const
 {
-    if (!compositingActive()) {
-        return false;
-    }
-    QPixmap pix(1,1);
-    QPainter p(&pix);
-    const bool raster = p.paintEngine()->type() == QPaintEngine::Raster;
-    p.end();
-    return raster;
+    return compositingActive();
+}
+
+void AuroraeClient::render(QPaintDevice *device, const QRegion &sourceRegion)
+{
+    QMutexLocker locker(AuroraeFactory::instance()->mutex());
+    QPainter painter(device);
+    painter.setClipRegion(sourceRegion);
+    painter.drawImage(QPoint(0, 0), m_buffer);
 }
 
 } // namespace Aurorae
 
-extern "C"
-{
-    KDE_EXPORT KDecorationFactory *create_factory() {
-        return Aurorae::AuroraeFactory::instance();
-    }
-    KWIN_EXPORT int decoration_version() {
-        return KWIN_DECORATION_API_VERSION;
-    }
-}
-
-
-#include "moc_aurorae.cpp"
+#include "aurorae.moc"
