@@ -17,14 +17,11 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
-
-#include <config-X11.h>
-#include <config-kwin.h>
-
 #include "composite.h"
 #include "compositingadaptor.h"
 
 #include "utils.h"
+#include <QTextStream>
 #include "workspace.h"
 #include "client.h"
 #include "unmanaged.h"
@@ -33,24 +30,28 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "overlaywindow.h"
 #include "scene.h"
 #include "scene_xrender.h"
+#include "scene_opengl.h"
+#include "scene_qpainter.h"
 #include "shadow.h"
 #include "useractions.h"
 #include "compositingprefs.h"
 #include "xcbutils.h"
+#if HAVE_WAYLAND
+#include "wayland_backend.h"
+#endif
 
 #include <stdio.h>
 
-#include <QTextStream>
-#include <QFile>
+#include <QtConcurrentRun>
+#include <QFutureWatcher>
 #include <QMenu>
-#include <QtCore/qcoreevent.h>
+#include <QTimerEvent>
 #include <QDateTime>
 #include <QDBusConnection>
-#include <kaction.h>
-#include <kactioncollection.h>
-#include <KGlobal>
+#include <KGlobalAccel>
 #include <KLocalizedString>
 #include <KNotification>
+#include <KSelectionWatcher>
 
 #include <xcb/composite.h>
 #include <xcb/damage.h>
@@ -62,6 +63,16 @@ namespace KWin
 
 extern int currentRefreshRate();
 
+CompositorSelectionOwner::CompositorSelectionOwner(const char *selection) : KSelectionOwner(selection), owning(false)
+{
+    connect (this, SIGNAL(lostOwnership()), SLOT(looseOwnership()));
+}
+
+void CompositorSelectionOwner::looseOwnership()
+{
+    owning = false;
+}
+
 KWIN_SINGLETON_FACTORY_VARIABLE(Compositor, s_compositor)
 
 static inline qint64 milliToNano(int milli) { return milli * 1000 * 1000; }
@@ -71,18 +82,22 @@ Compositor::Compositor(QObject* workspace)
     : QObject(workspace)
     , m_suspended(options->isUseCompositing() ? NoReasonSuspend : UserSuspend)
     , cm_selection(NULL)
+    , vBlankInterval(0)
     , fpsInterval(0)
     , m_xrrRefreshRate(0)
     , forceUnredirectCheck(false)
     , m_finishing(false)
     , m_timeSinceLastVBlank(0)
     , m_scene(NULL)
+    , m_waitingForFrameRendered(false)
 {
     qRegisterMetaType<Compositor::SuspendReason>("Compositor::SuspendReason");
     new CompositingAdaptor(this);
     QDBusConnection dbus = QDBusConnection::sessionBus();
-    dbus.registerObject("/Compositor", this);
-    dbus.registerService("org.kde.kwin.Compositing");
+    dbus.registerObject(QStringLiteral("/Compositor"), this);
+    dbus.registerService(QStringLiteral("org.kde.kwin.Compositing"));
+    dbus.connect(QString(), QStringLiteral("/Compositor"), QStringLiteral("org.kde.kwin.Compositing"),
+                 QStringLiteral("reinit"), this, SLOT(slotReinitialize()));
     connect(&unredirectTimer, SIGNAL(timeout()), SLOT(delayedCheckUnredirect()));
     connect(&compositeResetTimer, SIGNAL(timeout()), SLOT(restart()));
     connect(workspace, SIGNAL(configChanged()), SLOT(slotConfigChanged()));
@@ -94,9 +109,19 @@ Compositor::Compositor(QObject* workspace)
     // 2 sec which should be enough to restart the compositor
     static const int compositorLostMessageDelay = 2000;
 
+    m_releaseSelectionTimer.setSingleShot(true);
+    m_releaseSelectionTimer.setInterval(compositorLostMessageDelay);
+    connect(&m_releaseSelectionTimer, SIGNAL(timeout()), SLOT(releaseCompositorSelection()));
+
     m_unusedSupportPropertyTimer.setInterval(compositorLostMessageDelay);
     m_unusedSupportPropertyTimer.setSingleShot(true);
     connect(&m_unusedSupportPropertyTimer, SIGNAL(timeout()), SLOT(deleteUnusedSupportProperties()));
+#if HAVE_WAYLAND
+    if (kwinApp()->operationMode() != Application::OperationModeX11) {
+        connect(Wayland::WaylandBackend::self(), &Wayland::WaylandBackend::systemCompositorDied, this, &Compositor::finish);
+        connect(Wayland::WaylandBackend::self(), &Wayland::WaylandBackend::backendReady, this, &Compositor::setup);
+    }
+#endif
 
     // delay the call to setup by one event cycle
     // The ctor of this class is invoked from the Workspace ctor, that means before
@@ -109,10 +134,7 @@ Compositor::~Compositor()
 {
     finish();
     deleteUnusedSupportProperties();
-    if (cm_selection) {
-        disconnect(cm_selection, 0, this, 0);
-        delete cm_selection;
-    }
+    delete cm_selection;
     s_compositor = NULL;
 }
 
@@ -122,60 +144,118 @@ void Compositor::setup()
     if (hasScene())
         return;
     if (m_suspended) {
-        kDebug(1212) << "Compositing is suspended, reason:" << m_suspended;
+        qDebug() << "Compositing is suspended, reason:" << m_suspended;
         return;
     } else if (!CompositingPrefs::compositingPossible()) {
-        kError(1212) << "Compositing is not possible";
+        qCritical() << "Compositing is not possible";
         return;
     }
     m_starting = true;
 
     if (!options->isCompositingInitialized()) {
-        // options->reloadCompositingSettings(true) initializes the CompositingPrefs
         options->reloadCompositingSettings(true);
+        slotCompositingOptionsInitialized();
+    } else {
+        slotCompositingOptionsInitialized();
     }
+}
 
+extern int screen_number; // main.cpp
+extern bool is_multihead;
+
+void Compositor::slotCompositingOptionsInitialized()
+{
+    char selection_name[ 100 ];
+    sprintf(selection_name, "_NET_WM_CM_S%d", DefaultScreen(display()));
     if (!cm_selection) {
-        const int selection_sreen = DefaultScreen(display());
-        char selection_name[ 100 ];
-        ::memset(selection_name, '\0', sizeof(selection_name) * sizeof(char));
-        ::sprintf(selection_name, "_NET_WM_CM_S%d", selection_sreen);
-        cm_selection = new KSelectionOwner(selection_name, selection_sreen, this);
-    }
-    if (cm_selection->ownerWindow() == XNone) {
-        cm_selection->claim(true);   // force claiming
+        cm_selection = new CompositorSelectionOwner(selection_name);
         connect(cm_selection, SIGNAL(lostOwnership()), SLOT(finish()));
     }
-
-    // There might still be a deleted around, needs to be cleared before creating the scene (BUG 333275)
-    while (!Workspace::self()->deletedList().isEmpty()) {
-        Workspace::self()->deletedList().first()->discard();
+    if (!cm_selection->owning) {
+        cm_selection->claim(true);   // force claiming
+        cm_selection->owning = true;
     }
-
     switch(options->compositingMode()) {
-#ifdef KWIN_BUILD_COMPOSITE
+    case OpenGLCompositing: {
+        qDebug() << "Initializing OpenGL compositing";
+
+        // Some broken drivers crash on glXQuery() so to prevent constant KWin crashes:
+        KSharedConfigPtr unsafeConfigPtr = KSharedConfig::openConfig();
+        KConfigGroup unsafeConfig(unsafeConfigPtr, "Compositing");
+        const QString openGLIsUnsafe = QStringLiteral("OpenGLIsUnsafe") + (is_multihead ? QString::number(screen_number) : QString());
+        if (unsafeConfig.readEntry(openGLIsUnsafe, false))
+            qWarning() << "KWin has detected that your OpenGL library is unsafe to use";
+        else {
+            unsafeConfig.writeEntry(openGLIsUnsafe, true);
+            unsafeConfig.sync();
+#ifndef KWIN_HAVE_OPENGLES
+            if (!CompositingPrefs::hasGlx()) {
+                unsafeConfig.writeEntry(openGLIsUnsafe, false);
+                unsafeConfig.sync();
+                qDebug() << "No glx extensions available";
+                break;
+            }
+#endif
+
+            m_scene = SceneOpenGL::createScene();
+            connect(m_scene, SIGNAL(resetCompositing()), SLOT(restart()));
+
+            // TODO: Add 30 second delay to protect against screen freezes as well
+            unsafeConfig.writeEntry(openGLIsUnsafe, false);
+            unsafeConfig.sync();
+
+            if (m_scene && !m_scene->initFailed())
+                break; // -->
+            delete m_scene;
+            m_scene = NULL;
+        }
+
+        // Do not Fall back to XRender - it causes problems when selfcheck fails during startup, but works later on
+        break;
+    }
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     case XRenderCompositing:
-        kDebug(1212) << "Initializing XRender compositing";
-        m_scene = new SceneXrender(Workspace::self());
+        qDebug() << "Initializing XRender compositing";
+        m_scene = SceneXrender::createScene();
         break;
 #endif
+    case QPainterCompositing:
+        qDebug() << "Initializing QPainter compositing";
+        m_scene = SceneQPainter::createScene();
+        break;
     default:
-        kDebug(1212) << "No compositing enabled";
+        qDebug() << "No compositing enabled";
         m_starting = false;
+        cm_selection->owning = false;
         cm_selection->release();
+        if (kwinApp()->requiresCompositing()) {
+            qCritical() << "The used windowing system requires compositing";
+            qCritical() << "We are going to quit KWin now as it is broken";
+            qApp->quit();
+        }
         return;
     }
     if (m_scene == NULL || m_scene->initFailed()) {
-        kError(1212) << "Failed to initialize compositing, compositing disabled";
-        kError(1212) << "Consult http://techbase.kde.org/Projects/KWin/4.0-release-notes#Setting_up";
+        qCritical() << "Failed to initialize compositing, compositing disabled";
         delete m_scene;
         m_scene = NULL;
         m_starting = false;
+        cm_selection->owning = false;
         cm_selection->release();
+        if (kwinApp()->requiresCompositing()) {
+            qCritical() << "The used windowing system requires compositing";
+            qCritical() << "We are going to quit KWin now as it is broken";
+            qApp->quit();
+        }
         return;
     }
     m_xrrRefreshRate = KWin::currentRefreshRate();
     fpsInterval = options->maxFpsInterval();
+    if (m_scene->syncsToVBlank()) {  // if we do vsync, set the fps to the next multiple of the vblank rate
+        vBlankInterval = milliToNano(1000) / m_xrrRefreshRate;
+        fpsInterval = qMax((fpsInterval / vBlankInterval) * vBlankInterval, vBlankInterval);
+    } else
+        vBlankInterval = milliToNano(1); // no sync - DO NOT set "0", would cause div-by-zero segfaults.
     m_timeSinceLastVBlank = fpsInterval - (options->vBlankTime() + 1); // means "start now" - we don't have even a slight idea when the first vsync will occur
     scheduleRepaint();
     xcb_composite_redirect_subwindows(connection(), rootWindow(), XCB_COMPOSITE_REDIRECT_MANUAL);
@@ -196,6 +276,9 @@ void Compositor::setup()
     emit compositingToggled(true);
 
     m_starting = false;
+    if (m_releaseSelectionTimer.isActive()) {
+        m_releaseSelectionTimer.stop();
+    }
 
     // render at least once
     performCompositing();
@@ -212,6 +295,7 @@ void Compositor::finish()
     if (!hasScene())
         return;
     m_finishing = true;
+    m_releaseSelectionTimer.start();
     foreach (Client * c, Workspace::self()->clientList())
         m_scene->windowClosed(c, NULL);
     foreach (Client * c, Workspace::self()->desktopList())
@@ -221,13 +305,13 @@ void Compositor::finish()
     foreach (Deleted * c, Workspace::self()->deletedList())
         m_scene->windowDeleted(c);
     foreach (Client * c, Workspace::self()->clientList())
-        c->finishCompositing();
+    c->finishCompositing();
     foreach (Client * c, Workspace::self()->desktopList())
-        c->finishCompositing();
+    c->finishCompositing();
     foreach (Unmanaged * c, Workspace::self()->unmanagedList())
-        c->finishCompositing();
+    c->finishCompositing();
     foreach (Deleted * c, Workspace::self()->deletedList())
-        c->finishCompositing();
+    c->finishCompositing();
     xcb_composite_unredirect_subwindows(connection(), rootWindow(), XCB_COMPOSITE_REDIRECT_MANUAL);
     delete effects;
     effects = NULL;
@@ -240,7 +324,7 @@ void Compositor::finish()
             ++it) {
         // forward all opacity values to the frame in case there'll be other CM running
         if ((*it)->opacity() != 1.0) {
-            NETWinInfo i(display(), (*it)->frameId(), rootWindow(), 0);
+            NETWinInfo i(connection(), (*it)->frameId(), rootWindow(), 0, 0);
             i.setOpacity(static_cast< unsigned long >((*it)->opacity() * 0xffffffff));
         }
     }
@@ -249,13 +333,28 @@ void Compositor::finish()
         Workspace::self()->deletedList().first()->discard();
     m_finishing = false;
     emit compositingToggled(false);
-    if (cm_selection) {
-        kDebug(1212) << "Releasing compositor selection";
-        disconnect(cm_selection, 0, this, 0);
-        cm_selection->release();
-        delete cm_selection;
-        cm_selection = 0;
+}
+
+void Compositor::releaseCompositorSelection()
+{
+    if (hasScene() && !m_finishing) {
+        // compositor is up and running again, no need to release the selection
+        return;
     }
+    if (m_starting) {
+        // currently still starting the compositor, it might fail, so restart the timer to test again
+        m_releaseSelectionTimer.start();
+        return;
+    }
+
+    if (m_finishing) {
+        // still shutting down, a restart might follow, so restart the timer to test again
+        m_releaseSelectionTimer.start();
+        return;
+    }
+    qDebug() << "Releasing compositor selection";
+    cm_selection->owning = false;
+    cm_selection->release();
 }
 
 void Compositor::keepSupportProperty(xcb_atom_t atom)
@@ -283,10 +382,21 @@ void Compositor::deleteUnusedSupportProperties()
     }
     foreach (const xcb_atom_t &atom, m_unusedSupportProperties) {
         // remove property from root window
-        XDeleteProperty(QX11Info::display(), rootWindow(), atom);
+        xcb_delete_property(connection(), rootWindow(), atom);
     }
 }
 
+// OpenGL self-check failed, fallback to XRender
+void Compositor::fallbackToXRenderCompositing()
+{
+    finish();
+    KConfigGroup config(KSharedConfig::openConfig(), "Compositing");
+    config.writeEntry("Backend", "XRender");
+    config.sync();
+    options->setCompositingMode(XRenderCompositing);
+    setup();
+}
+
 void Compositor::slotConfigChanged()
 {
     if (!m_suspended) {
@@ -294,15 +404,14 @@ void Compositor::slotConfigChanged()
         if (effects)   // setupCompositing() may fail
             effects->reconfigure();
         addRepaintFull();
-    } else {
+    } else
         finish();
-    }
 }
 
 void Compositor::slotReinitialize()
 {
     // Reparse config. Config options will be reloaded by setup()
-    KGlobal::config()->reparseConfiguration();
+    KSharedConfig::openConfig()->reparseConfiguration();
 
     // Restart compositing
     finish();
@@ -319,6 +428,10 @@ void Compositor::slotReinitialize()
 // for the shortcut
 void Compositor::slotToggleCompositing()
 {
+    if (kwinApp()->requiresCompositing()) {
+        // we are not allowed to turn on/off compositing
+        return;
+    }
     if (m_suspended) { // direct user call; clear all bits
         resume(AllReasonSuspend);
     } else { // but only set the user one (sufficient to suspend)
@@ -332,14 +445,12 @@ void Compositor::toggleCompositing()
     slotToggleCompositing(); // TODO only operate on script level here?
     if (m_suspended) {
         // when disabled show a shortcut how the user can get back compositing
-        QString shortcut, message;
-        if (KAction* action = qobject_cast<KAction*>(Workspace::self()->actionCollection()->action("Suspend Compositing")))
-            shortcut = action->globalShortcut().primary().toString(QKeySequence::NativeText);
-        if (!shortcut.isEmpty()) {
+        const auto shortcuts = KGlobalAccel::self()->shortcut(workspace()->findChild<QAction*>(QStringLiteral("Suspend Compositing")));
+        if (!shortcuts.isEmpty()) {
             // display notification only if there is the shortcut
-            message = i18n("Desktop effects have been suspended by another application.<br/>"
-                           "You can resume using the '%1' shortcut.", shortcut);
-            KNotification::event("compositingsuspendeddbus", message);
+            const QString message = i18n("Desktop effects have been suspended by another application.<br/>"
+                                         "You can resume using the '%1' shortcut.", shortcuts.first().toString(QKeySequence::NativeText));
+            KNotification::event(QStringLiteral("compositingsuspendeddbus"), message);
         }
     }
 }
@@ -351,6 +462,9 @@ void Compositor::updateCompositeBlocking()
 
 void Compositor::updateCompositeBlocking(Client *c)
 {
+    if (kwinApp()->requiresCompositing()) {
+        return;
+    }
     if (c) { // if c == 0 we just check if we can resume
         if (c->isBlockingCompositing()) {
             if (!(m_suspended & BlockRuleSuspend)) // do NOT attempt to call suspend(true); from within the eventchain!
@@ -373,6 +487,9 @@ void Compositor::updateCompositeBlocking(Client *c)
 
 void Compositor::suspend(Compositor::SuspendReason reason)
 {
+    if (kwinApp()->requiresCompositing()) {
+        return;
+    }
     Q_ASSERT(reason != NoReasonSuspend);
     m_suspended |= reason;
     finish();
@@ -387,6 +504,9 @@ void Compositor::resume(Compositor::SuspendReason reason)
 
 void Compositor::setCompositing(bool active)
 {
+    if (kwinApp()->requiresCompositing()) {
+        return;
+    }
     if (active) {
         resume(ScriptSuspend);
     } else {
@@ -442,10 +562,23 @@ void Compositor::timerEvent(QTimerEvent *te)
         QObject::timerEvent(te);
 }
 
+void Compositor::lastFrameRendered()
+{
+    if (!m_waitingForFrameRendered) {
+        return;
+    }
+    m_waitingForFrameRendered = false;
+    performCompositing();
+}
+
 void Compositor::performCompositing()
 {
-    if (!isOverlayWindowVisible())
+    if (m_scene->usesOverlayWindow() && !isOverlayWindowVisible())
         return; // nothing is visible anyway
+    if (!m_scene->isLastFrameRendered()) {
+        m_waitingForFrameRendered = true;
+        return; // frame wouldn't make it on the screen
+    }
 
     // Create a list of all windows in the stacking order
     ToplevelList windows = Workspace::self()->xStackingOrder();
@@ -470,6 +603,15 @@ void Compositor::performCompositing()
 
     // Get the replies
     foreach (Toplevel *win, damaged) {
+        // Discard the cached lanczos texture
+        if (win->effectWindow()) {
+            const QVariant texture = win->effectWindow()->data(LanczosCacheRole);
+            if (texture.isValid()) {
+                delete static_cast<GLTexture *>(texture.value<void*>());
+                win->effectWindow()->setData(LanczosCacheRole, QVariant());
+            }
+        }
+
         win->getDamageRegionReply();
     }
 
@@ -488,12 +630,9 @@ void Compositor::performCompositing()
     // TODO ?
     // this cannot be used so carelessly - needs protections against broken clients, the window
     // should not get focus before it's displayed, handle unredirected windows properly and so on.
-    QMutableListIterator<Toplevel*> windows_it(windows);
-    while (windows_it.hasNext()) {
-        const Toplevel *t = windows_it.next();
+    foreach (Toplevel *t, windows)
         if (!t->readyForPainting())
-            windows_it.remove();
-    }
+            windows.removeAll(t);
 
     QRegion repaints = repaints_region;
     // clear all repaints, so that post-pass can add repaints for the next repaint
@@ -512,22 +651,18 @@ void Compositor::performCompositing()
 
 bool Compositor::windowRepaintsPending() const
 {
-    foreach (const Toplevel * c, Workspace::self()->clientList()) {
-        if (!c->repaints().isEmpty())
-            return true;
-    }
-    foreach (const Toplevel * c, Workspace::self()->desktopList()) {
-        if (!c->repaints().isEmpty())
-            return true;
-    }
-    foreach (const Toplevel * c, Workspace::self()->unmanagedList()) {
-        if (!c->repaints().isEmpty())
-            return true;
-    }
-    foreach (const Toplevel * c, Workspace::self()->deletedList()) {
-        if (!c->repaints().isEmpty())
-            return true;
-    }
+    foreach (Toplevel * c, Workspace::self()->clientList())
+    if (!c->repaints().isEmpty())
+        return true;
+    foreach (Toplevel * c, Workspace::self()->desktopList())
+    if (!c->repaints().isEmpty())
+        return true;
+    foreach (Toplevel * c, Workspace::self()->unmanagedList())
+    if (!c->repaints().isEmpty())
+        return true;
+    foreach (Toplevel * c, Workspace::self()->deletedList())
+    if (!c->repaints().isEmpty())
+        return true;
     return false;
 }
 
@@ -542,14 +677,52 @@ void Compositor::setCompositeTimer()
         return;
 
     uint waitTime = 1;
-    // w/o blocking vsync we just jump to the next demanded tick
-    if (fpsInterval > m_timeSinceLastVBlank) {
-        waitTime = nanoToMilli(fpsInterval - m_timeSinceLastVBlank);
-        if (!waitTime) {
-            waitTime = 1; // will ensure we don't block out the eventloop - the system's just not faster ...
+
+    if (m_scene->blocksForRetrace()) {
+
+        // TODO: make vBlankTime dynamic?!
+        // It's required because glXWaitVideoSync will *likely* block a full frame if one enters
+        // a retrace pass which can last a variable amount of time, depending on the actual screen
+        // Now, my ooold 19" CRT can do such retrace so that 2ms are entirely sufficient,
+        // while another ooold 15" TFT requires about 6ms
+
+        qint64 padding = m_timeSinceLastVBlank;
+        if (padding > fpsInterval) {
+            // we're at low repaints or spent more time in painting than the user wanted to wait for that frame
+            padding = vBlankInterval - (padding%vBlankInterval); // -> align to next vblank
+        } else {  // -> align to the next maxFps tick
+            padding = ((vBlankInterval - padding%vBlankInterval) + (fpsInterval/vBlankInterval-1)*vBlankInterval);
+            //               "remaining time of the first vsync" + "time for the other vsyncs of the frame"
+        }
+
+        if (padding < options->vBlankTime()) { // we'll likely miss this frame
+            waitTime = nanoToMilli(padding + vBlankInterval - options->vBlankTime()); // so we add one
+        } else {
+            waitTime = nanoToMilli(padding - options->vBlankTime());
+        }
+    }
+    else { // w/o blocking vsync we just jump to the next demanded tick
+        if (fpsInterval > m_timeSinceLastVBlank) {
+            waitTime = nanoToMilli(fpsInterval - m_timeSinceLastVBlank);
+            if (!waitTime) {
+                waitTime = 1; // will ensure we don't block out the eventloop - the system's just not faster ...
+            }
+        }/* else if (m_scene->syncsToVBlank() && m_timeSinceLastVBlank - fpsInterval < (vBlankInterval<<1)) {
+            // NOTICE - "for later" ------------------------------------------------------------------
+            // It can happen that we push two frames within one refresh cycle.
+            // Swapping will then block even with triple buffering when the GPU does not discard but
+            // queues frames
+            // now here's the mean part: if we take that as "OMG, we're late - next frame ASAP",
+            // there'll immediately be 2 frames in the pipe, swapping will block, we think we're
+            // late ... ewww
+            // so instead we pad to the clock again and add 2ms safety to ensure the pipe is really
+            // free
+            // NOTICE: obviously m_timeSinceLastVBlank can be too big because we're too slow as well
+            // So if this code was enabled, we'd needlessly half the framerate once more (15 instead of 30)
+            waitTime = nanoToMilli(vBlankInterval - (m_timeSinceLastVBlank - fpsInterval)%vBlankInterval) + 2;
+        }*/ else {
+            waitTime = 1; // ... "0" would be sufficient, but the compositor isn't the WMs only task
         }
-    } else {
-        waitTime = 1; // ... "0" would be sufficient, but the compositor isn't the WMs only task
     }
     compositeTimer.start(qMin(waitTime, 250u), this); // force 4fps minimum
 }
@@ -567,7 +740,7 @@ void Compositor::checkUnredirect()
 // force is needed when the list of windows changes (e.g. a window goes away)
 void Compositor::checkUnredirect(bool force)
 {
-    if (!hasScene() || m_scene->overlayWindow()->window() == None || !options->isUnredirectFullscreen())
+    if (!hasScene() || !m_scene->overlayWindow() || m_scene->overlayWindow()->window() == None || !options->isUnredirectFullscreen())
         return;
     if (force)
         forceUnredirectCheck = true;
@@ -577,14 +750,14 @@ void Compositor::checkUnredirect(bool force)
 
 void Compositor::delayedCheckUnredirect()
 {
-    if (!hasScene() || m_scene->overlayWindow()->window() == None || !(options->isUnredirectFullscreen() || sender() == options))
+    if (!hasScene() || !m_scene->overlayWindow() || m_scene->overlayWindow()->window() == None || !(options->isUnredirectFullscreen() || sender() == options))
         return;
     ToplevelList list;
     bool changed = forceUnredirectCheck;
     foreach (Client * c, Workspace::self()->clientList())
-        list.append(c);
+    list.append(c);
     foreach (Unmanaged * c, Workspace::self()->unmanagedList())
-        list.append(c);
+    list.append(c);
     foreach (Toplevel * c, list) {
         if (c->updateUnredirectedState())
             changed = true;
@@ -656,17 +829,30 @@ QString Compositor::compositingNotPossibleReason() const
     return CompositingPrefs::compositingNotPossibleReason();
 }
 
+bool Compositor::isOpenGLBroken() const
+{
+    return CompositingPrefs::openGlIsBroken();
+}
+
 QString Compositor::compositingType() const
 {
     if (!hasScene()) {
-        return "none";
+        return QStringLiteral("none");
     }
     switch (m_scene->compositingType()) {
     case XRenderCompositing:
-        return "xrender";
+        return QStringLiteral("xrender");
+    case OpenGL2Compositing:
+#ifdef KWIN_HAVE_OPENGLES
+        return QStringLiteral("gles");
+#else
+        return QStringLiteral("gl2");
+#endif
+    case QPainterCompositing:
+        return "qpainter";
     case NoCompositing:
     default:
-        return "none";
+        return QStringLiteral("none");
     }
 }
 
@@ -705,7 +891,7 @@ bool Toplevel::setupCompositing()
     // and us setting up damage tracking.  If the client wins we won't get a damage event even
     // though the window has been painted.  To avoid this we mark the whole window as damaged
     // and schedule a repaint immediately after creating the damage object.
-    if (qobject_cast<Unmanaged*>(this))
+    if (dynamic_cast<Unmanaged*>(this))
         addDamageFull();
 
     return true;
@@ -753,7 +939,6 @@ bool Toplevel::compositing() const
 
 void Client::damageNotifyEvent()
 {
-#ifdef HAVE_XSYNC
     if (syncRequest.isPending && isResize()) {
         emit damaged(this, QRect());
         m_isDamaged = true;
@@ -761,13 +946,9 @@ void Client::damageNotifyEvent()
     }
 
     if (!ready_for_painting) { // avoid "setReadyForPainting()" function calling overhead
-        if (syncRequest.counter == None)   // cannot detect complete redraw, consider done now
+        if (syncRequest.counter == XCB_NONE)   // cannot detect complete redraw, consider done now
             setReadyForPainting();
     }
-#else
-    if (!ready_for_painting)
-        setReadyForPainting();
-#endif
 
     Toplevel::damageNotifyEvent();
 }
@@ -937,10 +1118,10 @@ bool Toplevel::updateUnredirectedState()
     lastUnredirect.start();
     unredirect = should;
     if (unredirect) {
-        kDebug(1212) << "Unredirecting:" << this;
+        qDebug() << "Unredirecting:" << this;
         xcb_composite_unredirect_window(connection(), frameId(), XCB_COMPOSITE_REDIRECT_MANUAL);
     } else {
-        kDebug(1212) << "Redirecting:" << this;
+        qDebug() << "Redirecting:" << this;
         xcb_composite_redirect_window(connection(), frameId(), XCB_COMPOSITE_REDIRECT_MANUAL);
         discardWindowPixmap();
     }
@@ -1009,7 +1190,14 @@ bool Client::shouldUnredirect() const
 
 bool Unmanaged::shouldUnredirect() const
 {
-    // it must cover whole display or one xinerama screen, and be the topmost there
+    // the pixmap is needed for the login effect, a nicer solution would be the login effect increasing
+    // refcount for the window pixmap (which would prevent unredirect), avoiding this hack
+    if (resourceClass() == "ksplashx"
+            || resourceClass() == "ksplashsimple"
+            || resourceClass() == "ksplashqml"
+            )
+        return false;
+// it must cover whole display or one xinerama screen, and be the topmost there
     const int desktop = VirtualDesktopManager::self()->current();
     if (geometry() == workspace()->clientArea(FullArea, geometry().center(), desktop)
             || geometry() == workspace()->clientArea(ScreenArea, geometry().center(), desktop)) {
