@@ -21,7 +21,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "cursor.h"
 // kwin
 #include <kwinglobals.h>
+#include "input.h"
+#include "main.h"
 #include "utils.h"
+// KDE
+#include <KConfig>
+#include <KConfigGroup>
+#include <KSharedConfig>
 // Qt
 #include <QTimer>
 // Xlib
@@ -32,14 +38,34 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 namespace KWin
 {
+Cursor *Cursor::s_self = nullptr;
 
-KWIN_SINGLETON_FACTORY_FACTORED(Cursor, X11Cursor)
+Cursor *Cursor::create(QObject *parent)
+{
+    Q_ASSERT(!s_self);
+#ifndef KCMRULES
+    if (kwinApp()->operationMode() == Application::OperationModeX11) {
+        s_self = new X11Cursor(parent);
+    } else {
+        s_self = new InputRedirectionCursor(parent);
+    }
+#else
+    Q_UNUSED(parent)
+#endif
+    return s_self;
+}
 
 Cursor::Cursor(QObject *parent)
     : QObject(parent)
     , m_mousePollingCounter(0)
     , m_cursorTrackingCounter(0)
+    , m_themeName("default")
+    , m_themeSize(24)
 {
+    loadThemeSettings();
+    // TODO: we need to connect for cursor theme changes
+    // in KDE4 times this was done through KGlobaSettings::cursorChanged
+    // which got emitted from the cursors KCM, this needs porting
 }
 
 Cursor::~Cursor()
@@ -47,6 +73,18 @@ Cursor::~Cursor()
     s_self = NULL;
 }
 
+void Cursor::loadThemeSettings()
+{
+    KConfigGroup mousecfg(KSharedConfig::openConfig("kcminputrc", KConfig::NoGlobals), "Mouse");
+    m_themeName = mousecfg.readEntry("cursorTheme", "default");
+    bool ok = false;
+    m_themeSize = mousecfg.readEntry("cursorSize", QString("24")).toUInt(&ok);
+    if (!ok) {
+        m_themeSize = 24;
+    }
+    emit themeChanged();
+}
+
 QPoint Cursor::pos()
 {
     s_self->doGetPos();
@@ -227,13 +265,15 @@ void X11Cursor::doStopCursorTracking()
 
 void X11Cursor::mousePolled()
 {
-    const QPoint last = currentPos();
-    const uint16_t lastMask = m_buttonMask;
+    static QPoint lastPos = currentPos();
+    static uint16_t lastMask = m_buttonMask;
     doGetPos(); // Update if needed
-    if (last != currentPos() || lastMask != m_buttonMask) {
-        emit mouseChanged(currentPos(), last,
+    if (lastPos != currentPos() || lastMask != m_buttonMask) {
+        emit mouseChanged(currentPos(), lastPos,
             x11ToQtMouseButtons(m_buttonMask), x11ToQtMouseButtons(lastMask),
             x11ToQtKeyboardModifiers(m_buttonMask), x11ToQtKeyboardModifiers(lastMask));
+        lastPos = currentPos();
+        lastMask = m_buttonMask;
     }
 }
 
@@ -253,9 +293,7 @@ xcb_cursor_t X11Cursor::createCursor(Qt::CursorShape shape)
         return XCB_CURSOR_NONE;
     }
     // XCursor is an XLib only lib
-    const char *theme = XcursorGetTheme(display());
-    const int size = XcursorGetDefaultSize(display());
-    XcursorImage *ximg = XcursorLibraryLoadImage(name.constData(), theme, size);
+    XcursorImage *ximg = XcursorLibraryLoadImage(name.constData(), themeName().toUtf8().constData(), themeSize());
     if (!ximg) {
         return XCB_CURSOR_NONE;
     }
@@ -265,7 +303,7 @@ xcb_cursor_t X11Cursor::createCursor(Qt::CursorShape shape)
     return cursor;
 }
 
-QByteArray X11Cursor::cursorName(Qt::CursorShape shape) const
+QByteArray Cursor::cursorName(Qt::CursorShape shape) const
 {
     switch (shape) {
     case Qt::ArrowCursor:
@@ -315,4 +353,59 @@ QByteArray X11Cursor::cursorName(Qt::CursorShape shape) const
     }
 }
 
+InputRedirectionCursor::InputRedirectionCursor(QObject *parent)
+    : Cursor(parent)
+    , m_oldButtons(Qt::NoButton)
+    , m_currentButtons(Qt::NoButton)
+{
+    connect(input(), SIGNAL(globalPointerChanged(QPointF)), SLOT(slotPosChanged(QPointF)));
+    connect(input(), SIGNAL(pointerButtonStateChanged(uint32_t,InputRedirection::PointerButtonState)),
+            SLOT(slotPointerButtonChanged()));
+#ifndef KCMRULES
+    connect(input(), &InputRedirection::keyboardModifiersChanged,
+            this, &InputRedirectionCursor::slotModifiersChanged);
+#endif
+}
+
+InputRedirectionCursor::~InputRedirectionCursor()
+{
+}
+
+void InputRedirectionCursor::doSetPos()
+{
+    // no support for pointer warping - reset to true position
+    slotPosChanged(input()->globalPointer());
+}
+
+void InputRedirectionCursor::slotPosChanged(const QPointF &pos)
+{
+    const QPoint oldPos = currentPos();
+    updatePos(pos.toPoint());
+    emit mouseChanged(pos.toPoint(), oldPos, m_currentButtons, m_oldButtons,
+                      input()->keyboardModifiers(), input()->keyboardModifiers());
+}
+
+void InputRedirectionCursor::slotModifiersChanged(Qt::KeyboardModifiers mods, Qt::KeyboardModifiers oldMods)
+{
+    emit mouseChanged(currentPos(), currentPos(), m_currentButtons, m_currentButtons, mods, oldMods);
+}
+
+void InputRedirectionCursor::slotPointerButtonChanged()
+{
+    m_oldButtons = m_currentButtons;
+    m_currentButtons = input()->qtButtonStates();
+}
+
+void InputRedirectionCursor::doStartCursorTracking()
+{
+    xcb_xfixes_select_cursor_input(connection(), rootWindow(), XCB_XFIXES_CURSOR_NOTIFY_MASK_DISPLAY_CURSOR);
+    // TODO: also track the Wayland cursor
+}
+
+void InputRedirectionCursor::doStopCursorTracking()
+{
+    xcb_xfixes_select_cursor_input(connection(), rootWindow(), 0);
+    // TODO: also track the Wayland cursor
+}
+
 } // namespace
