@@ -27,11 +27,16 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "scene.h"
 #include "xcbutils.h"
 
-#include <QStack>
 #include <QHash>
+#include <Plasma/FrameSvg>
+#include <KService>
 
-#include <QDBusPendingCallWatcher>
-#include <QDBusServiceWatcher>
+namespace Plasma {
+class Theme;
+}
+
+class QDBusPendingCallWatcher;
+class QDBusServiceWatcher;
 class KService;
 class OrgFreedesktopScreenSaverInterface;
 
@@ -46,6 +51,7 @@ class WindowThumbnailItem;
 class Client;
 class Compositor;
 class Deleted;
+class EffectLoader;
 class Unmanaged;
 class ScreenLockerWatcher;
 
@@ -59,112 +65,126 @@ class EffectsHandlerImpl : public EffectsHandler
 public:
     EffectsHandlerImpl(Compositor *compositor, Scene *scene);
     virtual ~EffectsHandlerImpl();
-    virtual void prePaintScreen(ScreenPrePaintData& data, int time);
-    virtual void paintScreen(int mask, QRegion region, ScreenPaintData& data);
+    void prePaintScreen(ScreenPrePaintData& data, int time) override;
+    void paintScreen(int mask, QRegion region, ScreenPaintData& data) override;
     /**
      * Special hook to perform a paintScreen but just with the windows on @p desktop.
      **/
     void paintDesktop(int desktop, int mask, QRegion region, ScreenPaintData& data);
-    virtual void postPaintScreen();
-    virtual void prePaintWindow(EffectWindow* w, WindowPrePaintData& data, int time);
-    virtual void paintWindow(EffectWindow* w, int mask, QRegion region, WindowPaintData& data);
-    virtual void postPaintWindow(EffectWindow* w);
-    virtual void paintEffectFrame(EffectFrame* frame, QRegion region, double opacity, double frameOpacity);
+    void postPaintScreen() override;
+    void prePaintWindow(EffectWindow* w, WindowPrePaintData& data, int time) override;
+    void paintWindow(EffectWindow* w, int mask, QRegion region, WindowPaintData& data) override;
+    void postPaintWindow(EffectWindow* w) override;
+    void paintEffectFrame(EffectFrame* frame, QRegion region, double opacity, double frameOpacity) override;
 
     Effect *provides(Effect::Feature ef);
 
-    virtual void drawWindow(EffectWindow* w, int mask, QRegion region, WindowPaintData& data);
+    void drawWindow(EffectWindow* w, int mask, QRegion region, WindowPaintData& data) override;
 
-    virtual void buildQuads(EffectWindow* w, WindowQuadList& quadList);
+    void buildQuads(EffectWindow* w, WindowQuadList& quadList) override;
 
-    virtual void activateWindow(EffectWindow* c);
-    virtual EffectWindow* activeWindow() const;
-    virtual void moveWindow(EffectWindow* w, const QPoint& pos, bool snap = false, double snapAdjust = 1.0);
-    virtual void windowToDesktop(EffectWindow* w, int desktop);
-    virtual void windowToScreen(EffectWindow* w, int screen);
-    virtual void setShowingDesktop(bool showing);
+    void activateWindow(EffectWindow* c) override;
+    EffectWindow* activeWindow() const override;
+    void moveWindow(EffectWindow* w, const QPoint& pos, bool snap = false, double snapAdjust = 1.0) override;
+    void windowToDesktop(EffectWindow* w, int desktop) override;
+    void windowToScreen(EffectWindow* w, int screen) override;
+    void setShowingDesktop(bool showing) override;
 
-    virtual int currentDesktop() const;
-    virtual int numberOfDesktops() const;
-    virtual void setCurrentDesktop(int desktop);
-    virtual void setNumberOfDesktops(int desktops);
-    virtual QSize desktopGridSize() const;
-    virtual int desktopGridWidth() const;
-    virtual int desktopGridHeight() const;
-    virtual int workspaceWidth() const;
-    virtual int workspaceHeight() const;
-    virtual int desktopAtCoords(QPoint coords) const;
-    virtual QPoint desktopGridCoords(int id) const;
-    virtual QPoint desktopCoords(int id) const;
-    virtual int desktopAbove(int desktop = 0, bool wrap = true) const;
-    virtual int desktopToRight(int desktop = 0, bool wrap = true) const;
-    virtual int desktopBelow(int desktop = 0, bool wrap = true) const;
-    virtual int desktopToLeft(int desktop = 0, bool wrap = true) const;
-    virtual QString desktopName(int desktop) const;
-    virtual bool optionRollOverDesktops() const;
+    QString currentActivity() const override;
+    int currentDesktop() const override;
+    int numberOfDesktops() const override;
+    void setCurrentDesktop(int desktop) override;
+    void setNumberOfDesktops(int desktops) override;
+    QSize desktopGridSize() const override;
+    int desktopGridWidth() const override;
+    int desktopGridHeight() const override;
+    int workspaceWidth() const override;
+    int workspaceHeight() const override;
+    int desktopAtCoords(QPoint coords) const override;
+    QPoint desktopGridCoords(int id) const override;
+    QPoint desktopCoords(int id) const override;
+    int desktopAbove(int desktop = 0, bool wrap = true) const override;
+    int desktopToRight(int desktop = 0, bool wrap = true) const override;
+    int desktopBelow(int desktop = 0, bool wrap = true) const override;
+    int desktopToLeft(int desktop = 0, bool wrap = true) const override;
+    QString desktopName(int desktop) const override;
+    bool optionRollOverDesktops() const override;
 
-    virtual int displayWidth() const;
-    virtual int displayHeight() const;
-    virtual QPoint cursorPos() const;
-    virtual bool grabKeyboard(Effect* effect);
-    virtual void ungrabKeyboard();
+    QPoint cursorPos() const override;
+    bool grabKeyboard(Effect* effect) override;
+    void ungrabKeyboard() override;
     // not performing XGrabPointer
-    virtual void startMouseInterception(Effect *effect, Qt::CursorShape shape);
-    virtual void stopMouseInterception(Effect *effect);
-    virtual void* getProxy(QString name);
-    virtual void startMousePolling();
-    virtual void stopMousePolling();
-    virtual EffectWindow* findWindow(WId id) const;
-    virtual EffectWindowList stackingOrder() const;
-    virtual void setElevatedWindow(EffectWindow* w, bool set);
+    void startMouseInterception(Effect *effect, Qt::CursorShape shape) override;
+    void stopMouseInterception(Effect *effect) override;
+    void registerGlobalShortcut(const QKeySequence &shortcut, QAction *action) override;
+    void registerPointerShortcut(Qt::KeyboardModifiers modifiers, Qt::MouseButton pointerButtons, QAction *action) override;
+    void registerAxisShortcut(Qt::KeyboardModifiers modifiers, PointerAxisDirection axis, QAction *action) override;
+    void* getProxy(QString name) override;
+    void startMousePolling() override;
+    void stopMousePolling() override;
+    EffectWindow* findWindow(WId id) const override;
+    EffectWindowList stackingOrder() const override;
+    void setElevatedWindow(EffectWindow* w, bool set) override;
 
-    virtual void setTabBoxWindow(EffectWindow*);
-    virtual void setTabBoxDesktop(int);
-    virtual EffectWindowList currentTabBoxWindowList() const;
-    virtual void refTabBox();
-    virtual void unrefTabBox();
-    virtual void closeTabBox();
-    virtual QList< int > currentTabBoxDesktopList() const;
-    virtual int currentTabBoxDesktop() const;
-    virtual EffectWindow* currentTabBoxWindow() const;
+    void setTabBoxWindow(EffectWindow*) override;
+    void setTabBoxDesktop(int) override;
+    EffectWindowList currentTabBoxWindowList() const override;
+    void refTabBox() override;
+    void unrefTabBox() override;
+    void closeTabBox() override;
+    QList< int > currentTabBoxDesktopList() const override;
+    int currentTabBoxDesktop() const override;
+    EffectWindow* currentTabBoxWindow() const override;
 
-    virtual void setActiveFullScreenEffect(Effect* e);
-    virtual Effect* activeFullScreenEffect() const;
+    void setActiveFullScreenEffect(Effect* e) override;
+    Effect* activeFullScreenEffect() const override;
 
-    virtual void addRepaintFull();
-    virtual void addRepaint(const QRect& r);
-    virtual void addRepaint(const QRegion& r);
-    virtual void addRepaint(int x, int y, int w, int h);
-    virtual int activeScreen() const;
-    virtual int numScreens() const;
-    virtual int screenNumber(const QPoint& pos) const;
-    virtual QRect clientArea(clientAreaOption, int screen, int desktop) const;
-    virtual QRect clientArea(clientAreaOption, const EffectWindow* c) const;
-    virtual QRect clientArea(clientAreaOption, const QPoint& p, int desktop) const;
-    virtual double animationTimeFactor() const;
-    virtual WindowQuadType newWindowQuadType();
+    void addRepaintFull() override;
+    void addRepaint(const QRect& r) override;
+    void addRepaint(const QRegion& r) override;
+    void addRepaint(int x, int y, int w, int h) override;
+    int activeScreen() const override;
+    int numScreens() const override;
+    int screenNumber(const QPoint& pos) const override;
+    QRect clientArea(clientAreaOption, int screen, int desktop) const override;
+    QRect clientArea(clientAreaOption, const EffectWindow* c) const override;
+    QRect clientArea(clientAreaOption, const QPoint& p, int desktop) const override;
+    QSize virtualScreenSize() const override;
+    QRect virtualScreenGeometry() const override;
+    double animationTimeFactor() const override;
+    WindowQuadType newWindowQuadType() override;
 
-    virtual void defineCursor(Qt::CursorShape shape);
-    virtual bool checkInputWindowEvent(XEvent* e);
-    virtual void checkInputWindowStacking();
+    void defineCursor(Qt::CursorShape shape) override;
+    bool checkInputWindowEvent(xcb_button_press_event_t *e);
+    bool checkInputWindowEvent(xcb_motion_notify_event_t *e);
+    bool checkInputWindowEvent(QMouseEvent *e);
+    void checkInputWindowStacking();
 
-    virtual void reserveElectricBorder(ElectricBorder border, Effect *effect);
-    virtual void unreserveElectricBorder(ElectricBorder border, Effect *effect);
+    void reserveElectricBorder(ElectricBorder border, Effect *effect) override;
+    void unreserveElectricBorder(ElectricBorder border, Effect *effect) override;
 
-    virtual unsigned long xrenderBufferPicture();
-    virtual void reconfigure();
-    virtual void registerPropertyType(long atom, bool reg);
-    virtual QByteArray readRootProperty(long atom, long type, int format) const;
-    virtual void deleteRootProperty(long atom) const;
-    virtual xcb_atom_t announceSupportProperty(const QByteArray& propertyName, Effect* effect);
-    virtual void removeSupportProperty(const QByteArray& propertyName, Effect* effect);
+    unsigned long xrenderBufferPicture() override;
+    QPainter* scenePainter() override;
+    void reconfigure() override;
+    void registerPropertyType(long atom, bool reg) override;
+    QByteArray readRootProperty(long atom, long type, int format) const override;
+    void deleteRootProperty(long atom) const override;
+    xcb_atom_t announceSupportProperty(const QByteArray& propertyName, Effect* effect) override;
+    void removeSupportProperty(const QByteArray& propertyName, Effect* effect) override;
 
-    virtual bool hasDecorationShadows() const;
+    bool hasDecorationShadows() const override;
 
-    virtual EffectFrame* effectFrame(bool staticSize, const QPoint& position, Qt::Alignment alignment) const;
+    bool decorationsHaveAlpha() const override;
 
-    virtual QVariant kwinOption(KWinOption kwopt);
-    virtual bool isScreenLocked() const;
+    bool decorationSupportsBlurBehind() const override;
+
+    EffectFrame* effectFrame(EffectFrameStyle style, bool staticSize, const QPoint& position, Qt::Alignment alignment) const override;
+
+    QVariant kwinOption(KWinOption kwopt) override;
+    bool isScreenLocked() const override;
+
+    bool makeOpenGLContextCurrent() override;
+    void doneOpenGLContextCurrent() override;
 
     // internal (used by kwin core or compositing code)
     void startPaint();
@@ -172,7 +192,7 @@ public:
     bool hasKeyboardGrab() const;
     void desktopResized(const QSize &size);
 
-    virtual void reloadEffect(Effect *effect);
+    void reloadEffect(Effect *effect) override;
     QStringList loadedEffects() const;
     QStringList listOfEffects() const;
 
@@ -199,42 +219,31 @@ public Q_SLOTS:
 
     // slots for D-Bus interface
     Q_SCRIPTABLE void reconfigureEffect(const QString& name);
-    Q_SCRIPTABLE bool loadEffect(const QString& name, bool checkDefault = false);
+    Q_SCRIPTABLE bool loadEffect(const QString& name);
     Q_SCRIPTABLE void toggleEffect(const QString& name);
     Q_SCRIPTABLE void unloadEffect(const QString& name);
     Q_SCRIPTABLE bool isEffectLoaded(const QString& name) const;
+    Q_SCRIPTABLE bool isEffectSupported(const QString& name);
+    Q_SCRIPTABLE QList<bool> areEffectsSupported(const QStringList &names);
     Q_SCRIPTABLE QString supportInformation(const QString& name) const;
     Q_SCRIPTABLE QString debug(const QString& name, const QString& parameter = QString()) const;
 
 protected Q_SLOTS:
-    void slotDesktopChanged(int old, KWin::Client *withClient);
-    void slotDesktopPresenceChanged(KWin::Client *c, int old);
-    void slotClientAdded(KWin::Client *c);
     void slotClientShown(KWin::Toplevel*);
-    void slotUnmanagedAdded(KWin::Unmanaged *u);
     void slotUnmanagedShown(KWin::Toplevel*);
     void slotWindowClosed(KWin::Toplevel *c);
-    void slotClientActivated(KWin::Client *c);
-    void slotDeletedRemoved(KWin::Deleted *d);
     void slotClientMaximized(KWin::Client *c, KDecorationDefines::MaximizeMode maxMode);
-    void slotClientStartUserMovedResized(KWin::Client *c);
-    void slotClientStepUserMovedResized(KWin::Client *c, const QRect &geometry);
-    void slotClientFinishUserMovedResized(KWin::Client *c);
     void slotOpacityChanged(KWin::Toplevel *t, qreal oldOpacity);
-    void slotClientMinimized(KWin::Client *c, bool animate);
-    void slotClientUnminimized(KWin::Client *c, bool animate);
     void slotClientModalityChanged();
     void slotGeometryShapeChanged(KWin::Toplevel *t, const QRect &old);
     void slotPaddingChanged(KWin::Toplevel *t, const QRect &old);
     void slotWindowDamaged(KWin::Toplevel *t, const QRect& r);
     void slotPropertyNotify(KWin::Toplevel *t, long atom);
-    void slotPropertyNotify(long atom);
 
 protected:
-    QLibrary* findEffectLibrary(KService* service);
     void effectsChanged();
-    void setupClientConnections(const KWin::Client *c);
-    void setupUnmanagedConnections(const KWin::Unmanaged *u);
+    void setupClientConnections(KWin::Client *c);
+    void setupUnmanagedConnections(KWin::Unmanaged *u);
 
     Effect* keyboard_grab_effect;
     Effect* fullscreen_effect;
@@ -262,6 +271,7 @@ private:
     int m_currentRenderedDesktop;
     Xcb::Window m_mouseInterceptionWindow;
     QList<Effect*> m_grabbedMouseEffects;
+    EffectLoader *m_effectLoader;
 };
 
 class EffectWindowImpl : public EffectWindow
@@ -271,27 +281,27 @@ public:
     explicit EffectWindowImpl(Toplevel *toplevel);
     virtual ~EffectWindowImpl();
 
-    virtual void enablePainting(int reason);
-    virtual void disablePainting(int reason);
-    virtual bool isPaintingEnabled();
+    void enablePainting(int reason) override;
+    void disablePainting(int reason) override;
+    bool isPaintingEnabled() override;
 
-    virtual void refWindow();
-    virtual void unrefWindow();
+    void refWindow() override;
+    void unrefWindow() override;
 
-    virtual const EffectWindowGroup* group() const;
+    const EffectWindowGroup* group() const override;
 
-    virtual QRegion shape() const;
-    virtual QRect decorationInnerRect() const;
-    virtual QByteArray readProperty(long atom, long type, int format) const;
-    virtual void deleteProperty(long atom) const;
+    QRegion shape() const override;
+    QRect decorationInnerRect() const override;
+    QByteArray readProperty(long atom, long type, int format) const override;
+    void deleteProperty(long atom) const override;
 
-    virtual EffectWindow* findModal();
-    virtual EffectWindowList mainWindows() const;
+    EffectWindow* findModal() override;
+    EffectWindowList mainWindows() const override;
 
-    virtual WindowQuadList buildQuads(bool force = false) const;
+    WindowQuadList buildQuads(bool force = false) const override;
 
-    virtual void referencePreviousWindowPixmap();
-    virtual void unreferencePreviousWindowPixmap();
+    void referencePreviousWindowPixmap() override;
+    void unreferencePreviousWindowPixmap() override;
 
     const Toplevel* window() const;
     Toplevel* window();
@@ -331,7 +341,7 @@ class EffectWindowGroupImpl
 {
 public:
     explicit EffectWindowGroupImpl(Group* g);
-    virtual EffectWindowList members() const;
+    EffectWindowList members() const override;
 private:
     Group* group;
 };
@@ -341,34 +351,65 @@ class EffectFrameImpl
 {
     Q_OBJECT
 public:
-    explicit EffectFrameImpl(bool staticSize = true, QPoint position = QPoint(-1, -1),
+    explicit EffectFrameImpl(EffectFrameStyle style, bool staticSize = true, QPoint position = QPoint(-1, -1),
                              Qt::Alignment alignment = Qt::AlignCenter);
     virtual ~EffectFrameImpl();
 
-    virtual void free();
-    virtual void render(QRegion region = infiniteRegion(), double opacity = 1.0, double frameOpacity = 1.0);
-    virtual Qt::Alignment alignment() const;
-    virtual void setAlignment(Qt::Alignment alignment);
-    virtual const QFont& font() const;
-    virtual void setFont(const QFont& font);
-    virtual const QRect& geometry() const;
-    virtual void setGeometry(const QRect& geometry, bool force = false);
-    virtual const QPixmap& icon() const;
-    virtual void setIcon(const QPixmap& icon);
-    virtual const QSize& iconSize() const;
-    virtual void setIconSize(const QSize& size);
-    virtual void setPosition(const QPoint& point);
-    virtual const QString& text() const;
-    virtual void setText(const QString& text);
+    void free() override;
+    void render(QRegion region = infiniteRegion(), double opacity = 1.0, double frameOpacity = 1.0) override;
+    Qt::Alignment alignment() const override;
+    void setAlignment(Qt::Alignment alignment) override;
+    const QFont& font() const override;
+    void setFont(const QFont& font) override;
+    const QRect& geometry() const override;
+    void setGeometry(const QRect& geometry, bool force = false) override;
+    const QIcon& icon() const override;
+    void setIcon(const QIcon& icon) override;
+    const QSize& iconSize() const override;
+    void setIconSize(const QSize& size) override;
+    void setPosition(const QPoint& point) override;
+    const QString& text() const override;
+    void setText(const QString& text) override;
+    EffectFrameStyle style() const override {
+        return m_style;
+    };
+    Plasma::FrameSvg& frame() {
+        return m_frame;
+    }
     bool isStatic() const {
         return m_static;
     };
     void finalRender(QRegion region, double opacity, double frameOpacity) const;
+    void setShader(GLShader* shader) override {
+        m_shader = shader;
+    }
+    GLShader* shader() const override {
+        return m_shader;
+    }
+    void setSelection(const QRect& selection) override;
+    const QRect& selection() const {
+        return m_selectionGeometry;
+    }
+    Plasma::FrameSvg& selectionFrame() {
+        return m_selection;
+    }
+    /**
+     * The foreground text color as specified by the default Plasma theme.
+     */
+    QColor styledTextColor();
+
+private Q_SLOTS:
+    void plasmaThemeChanged();
+
 private:
     Q_DISABLE_COPY(EffectFrameImpl)   // As we need to use Qt slots we cannot copy this class
     void align(QRect &geometry);   // positions geometry around m_point respecting m_alignment
     void autoResize(); // Auto-resize if not a static size
 
+    EffectFrameStyle m_style;
+    Plasma::FrameSvg m_frame; // TODO: share between all EffectFrames
+    Plasma::FrameSvg m_selection;
+
     // Position
     bool m_static;
     QPoint m_point;
@@ -378,10 +419,14 @@ private:
     // Contents
     QString m_text;
     QFont m_font;
-    QPixmap m_icon;
+    QIcon m_icon;
     QSize m_iconSize;
+    QRect m_selectionGeometry;
 
     Scene::EffectFrame* m_sceneFrame;
+    GLShader* m_shader;
+
+    Plasma::Theme *m_theme;
 };
 
 class ScreenLockerWatcher : public QObject
@@ -399,6 +444,8 @@ private Q_SLOTS:
     void setLocked(bool activated);
     void activeQueried(QDBusPendingCallWatcher *watcher);
     void serviceOwnerChanged(const QString &serviceName, const QString &oldOwner, const QString &newOwner);
+    void serviceRegisteredQueried();
+    void serviceOwnerQueried();
 private:
     OrgFreedesktopScreenSaverInterface *m_interface;
     QDBusServiceWatcher *m_serviceWatcher;
