@@ -23,18 +23,13 @@
 // KConfigSkeleton
 #include "blurconfig.h"
 
-#include <X11/Xatom.h>
-
 #include <QMatrix4x4>
 #include <QLinkedList>
-#include <KDebug>
 
 namespace KWin
 {
 
-KWIN_EFFECT(blur, BlurEffect)
-KWIN_EFFECT_SUPPORTED(blur, BlurEffect::supported())
-KWIN_EFFECT_ENABLEDBYDEFAULT(blur, BlurEffect::enabledByDefault())
+static const QByteArray s_blurAtomName = QByteArrayLiteral("_KDE_NET_WM_BLUR_BEHIND_REGION");
 
 BlurEffect::BlurEffect()
 {
@@ -42,7 +37,7 @@ BlurEffect::BlurEffect()
 
     // Offscreen texture that's used as the target for the horizontal blur pass
     // and the source for the vertical pass.
-    tex = GLTexture(displayWidth(), displayHeight());
+    tex = GLTexture(effects->virtualScreenSize());
     tex.setFilter(GL_LINEAR);
     tex.setWrapMode(GL_CLAMP_TO_EDGE);
 
@@ -53,7 +48,7 @@ BlurEffect::BlurEffect()
     // ### Hackish way to announce support.
     //     Should be included in _NET_SUPPORTED instead.
     if (shader && shader->isValid() && target->valid()) {
-        net_wm_blur_region = effects->announceSupportProperty("_KDE_NET_WM_BLUR_BEHIND_REGION", this);
+        net_wm_blur_region = effects->announceSupportProperty(s_blurAtomName, this);
     } else {
         net_wm_blur_region = 0;
     }
@@ -85,7 +80,7 @@ void BlurEffect::reconfigure(ReconfigureFlags flags)
 {
     Q_UNUSED(flags)
 
-    BlurConfig::self()->readConfig();
+    BlurConfig::self()->read();
     int radius = qBound(2, BlurConfig::blurRadius(), 14);
     if (shader)
         shader->setRadius(radius);
@@ -95,14 +90,14 @@ void BlurEffect::reconfigure(ReconfigureFlags flags)
     windows.clear();
 
     if (!shader || !shader->isValid())
-        XDeleteProperty(display(), rootWindow(), net_wm_blur_region);
+        effects->removeSupportProperty(s_blurAtomName, this);
 }
 
 void BlurEffect::updateBlurRegion(EffectWindow *w) const
 {
     QRegion region;
 
-    const QByteArray value = w->readProperty(net_wm_blur_region, XA_CARDINAL, 32);
+    const QByteArray value = w->readProperty(net_wm_blur_region, XCB_ATOM_CARDINAL, 32);
     if (value.size() > 0 && !(value.size() % (4 * sizeof(unsigned long)))) {
         const unsigned long *cardinals = reinterpret_cast<const unsigned long*>(value.constData());
         for (unsigned int i = 0; i < value.size() / sizeof(unsigned long);) {
@@ -141,7 +136,7 @@ void BlurEffect::slotPropertyNotify(EffectWindow *w, long atom)
         updateBlurRegion(w);
         CacheEntry it = windows.find(w);
         if (it != windows.end()) {
-            const QRect screen(0, 0, displayWidth(), displayHeight());
+            const QRect screen = effects->virtualScreenGeometry();
             it->damagedRegion = expand(blurRegion(w).translated(w->pos())) & screen;
         }
     }
@@ -153,10 +148,6 @@ bool BlurEffect::enabledByDefault()
 
     if (gl->isIntel() && gl->chipClass() < SandyBridge)
         return false;
-    if (gl->driver() == Driver_Catalyst && effects->compositingType() == OpenGL1Compositing) {
-        // fglrx supports only ARB shaders and those tend to crash KWin (see Bug #270818 and #286795)
-        return false;
-    }
 
     return true;
 }
@@ -164,17 +155,13 @@ bool BlurEffect::enabledByDefault()
 bool BlurEffect::supported()
 {
     bool supported = GLRenderTarget::supported() && GLTexture::NPOTTextureSupported() && GLSLBlurShader::supported();
-#ifdef KWIN_HAVE_OPENGL_1
-    if (effects->compositingType() == OpenGL1Compositing) {
-        supported = GLRenderTarget::supported() && GLTexture::NPOTTextureSupported() && ARBBlurShader::supported();
-    }
-#endif
 
     if (supported) {
         int maxTexSize;
         glGetIntegerv(GL_MAX_TEXTURE_SIZE, &maxTexSize);
 
-        if (displayWidth() > maxTexSize || displayHeight() > maxTexSize)
+        const QSize screenSize = effects->virtualScreenSize();
+        if (screenSize.width() > maxTexSize || screenSize.height() > maxTexSize)
             supported = false;
     }
     return supported;
@@ -305,7 +292,7 @@ void BlurEffect::prePaintWindow(EffectWindow* w, WindowPrePaintData& data, int t
     }
 
     // in case this window has regions to be blurred
-    const QRect screen(0, 0, displayWidth(), displayHeight());
+    const QRect screen = effects->virtualScreenGeometry();
     const QRegion blurArea = blurRegion(w).translated(w->pos()) & screen;
     const QRegion expandedBlur = expand(blurArea) & screen;
 
@@ -404,7 +391,7 @@ bool BlurEffect::shouldBlur(const EffectWindow *w, int mask, const WindowPaintDa
 
 void BlurEffect::drawWindow(EffectWindow *w, int mask, QRegion region, WindowPaintData &data)
 {
-    const QRect screen(0, 0, displayWidth(), displayHeight());
+    const QRect screen = effects->virtualScreenGeometry();
     if (shouldBlur(w, mask, data)) {
         QRegion shape = region & blurRegion(w).translated(w->pos()) & screen;
 
@@ -430,7 +417,7 @@ void BlurEffect::drawWindow(EffectWindow *w, int mask, QRegion region, WindowPai
 
 void BlurEffect::paintEffectFrame(EffectFrame *frame, QRegion region, double opacity, double frameOpacity)
 {
-    const QRect screen(0, 0, displayWidth(), displayHeight());
+    const QRect screen = effects->virtualScreenGeometry();
     bool valid = target->valid() && shader && shader->isValid();
     QRegion shape = frame->geometry().adjusted(-5, -5, 5, 5) & screen;
     if (valid && !shape.isEmpty() && region.intersects(shape.boundingRect()) && frame->style() != EffectFrameNone) {
@@ -469,16 +456,9 @@ void BlurEffect::doBlur(const QRegion& shape, const QRect& screen, const float o
 
     // Set up the texture matrix to transform from screen coordinates
     // to texture coordinates.
-#ifdef KWIN_HAVE_OPENGL_1
-    if (effects->compositingType() == OpenGL1Compositing) {
-        glMatrixMode(GL_TEXTURE);
-        pushMatrix();
-    }
-#endif
     QMatrix4x4 textureMatrix;
     textureMatrix.scale(1.0 / scratch.width(), -1.0 / scratch.height(), 1);
     textureMatrix.translate(-r.x(), -scratch.height() - r.y(), 0);
-    loadMatrix(textureMatrix);
     shader->setTextureMatrix(textureMatrix);
 
     vbo->draw(GL_TRIANGLES, 0, expanded.rectCount() * 6);
@@ -506,19 +486,11 @@ void BlurEffect::doBlur(const QRegion& shape, const QRect& screen, const float o
     textureMatrix.setToIdentity();
     textureMatrix.scale(1.0 / tex.width(), -1.0 / tex.height(), 1);
     textureMatrix.translate(0, -tex.height(), 0);
-    loadMatrix(textureMatrix);
     shader->setTextureMatrix(textureMatrix);
 
     vbo->draw(GL_TRIANGLES, expanded.rectCount() * 6, shape.rectCount() * 6);
     vbo->unbindArrays();
 
-#ifdef KWIN_HAVE_OPENGL_1
-    if (effects->compositingType() == OpenGL1Compositing) {
-        popMatrix();
-        glMatrixMode(GL_MODELVIEW);
-    }
-#endif
-
     if (opacity < 1.0) {
         glDisable(GL_BLEND);
     }
@@ -529,7 +501,7 @@ void BlurEffect::doBlur(const QRegion& shape, const QRect& screen, const float o
 
 void BlurEffect::doCachedBlur(EffectWindow *w, const QRegion& region, const float opacity)
 {
-    const QRect screen(0, 0, displayWidth(), displayHeight());
+    const QRect screen = effects->virtualScreenGeometry();
     const QRegion blurredRegion = blurRegion(w).translated(w->pos()) & screen;
     const QRegion expanded = expand(blurredRegion) & screen;
     const QRect r = expanded.boundingRect();
@@ -559,17 +531,6 @@ void BlurEffect::doCachedBlur(EffectWindow *w, const QRegion& region, const floa
     shader->bind();
     QMatrix4x4 textureMatrix;
     QMatrix4x4 modelViewProjectionMatrix;
-#ifdef KWIN_HAVE_OPENGL_1
-    if (effects->compositingType() == OpenGL1Compositing) {
-        glMatrixMode(GL_MODELVIEW);
-        pushMatrix();
-        glLoadIdentity();
-        glMatrixMode(GL_TEXTURE);
-        pushMatrix();
-        glMatrixMode(GL_PROJECTION);
-        pushMatrix();
-    }
-#endif
 
     /**
      * Which part of the background texture can be updated ?
@@ -638,20 +599,12 @@ void BlurEffect::doCachedBlur(EffectWindow *w, const QRegion& region, const floa
 
         modelViewProjectionMatrix.ortho(0, r.width(), r.height(), 0 , 0, 65535);
         modelViewProjectionMatrix.translate(-r.x(), -r.y(), 0);
-        loadMatrix(modelViewProjectionMatrix);
         shader->setModelViewProjectionMatrix(modelViewProjectionMatrix);
 
         // Set up the texture matrix to transform from screen coordinates
         // to texture coordinates.
         textureMatrix.scale(1.0 / tex.width(), -1.0 / tex.height(), 1);
         textureMatrix.translate(-updateRect.x(), -updateRect.height() - updateRect.y(), 0);
-#ifdef KWIN_HAVE_OPENGL_1
-        if (effects->compositingType() == OpenGL1Compositing) {
-            glMatrixMode(GL_TEXTURE);
-            loadMatrix(textureMatrix);
-            glMatrixMode(GL_PROJECTION);
-        }
-#endif
         shader->setTextureMatrix(textureMatrix);
 
         vbo->draw(GL_TRIANGLES, horizontalOffset, horizontalCount);
@@ -677,8 +630,8 @@ void BlurEffect::doCachedBlur(EffectWindow *w, const QRegion& region, const floa
     }
 
     modelViewProjectionMatrix.setToIdentity();
-    modelViewProjectionMatrix.ortho(0, displayWidth(), displayHeight(), 0, 0, 65535);
-    loadMatrix(modelViewProjectionMatrix);
+    const QSize screenSize = effects->virtualScreenSize();
+    modelViewProjectionMatrix.ortho(0, screenSize.width(), screenSize.height(), 0, 0, 65535);
     shader->setModelViewProjectionMatrix(modelViewProjectionMatrix);
 
     // Set the up the texture matrix to transform from screen coordinates
@@ -686,28 +639,11 @@ void BlurEffect::doCachedBlur(EffectWindow *w, const QRegion& region, const floa
     textureMatrix.setToIdentity();
     textureMatrix.scale(1.0 / targetTexture.width(), -1.0 / targetTexture.height(), 1);
     textureMatrix.translate(-r.x(), -targetTexture.height() - r.y(), 0);
-#ifdef KWIN_HAVE_OPENGL_1
-    if (effects->compositingType() == OpenGL1Compositing) {
-        glMatrixMode(GL_TEXTURE);
-        loadMatrix(textureMatrix);
-        glMatrixMode(GL_PROJECTION);
-    }
-#endif
     shader->setTextureMatrix(textureMatrix);
 
     vbo->draw(GL_TRIANGLES, verticalOffset, verticalCount);
     vbo->unbindArrays();
 
-#ifdef KWIN_HAVE_OPENGL_1
-    if (effects->compositingType() == OpenGL1Compositing) {
-        popMatrix();
-        glMatrixMode(GL_TEXTURE);
-        popMatrix();
-        glMatrixMode(GL_MODELVIEW);
-        popMatrix();
-    }
-#endif
-
     if (opacity < 1.0) {
         glDisable(GL_BLEND);
     }
