@@ -21,15 +21,19 @@
 // KConfigSkeleton
 #include "blurconfig.h"
 
-#include <kwineffects.h>
+#include <kwineffects_interface.h>
+#include <KAboutData>
+#include <KPluginFactory>
+
+K_PLUGIN_FACTORY_WITH_JSON(BlurEffectConfigFactory,
+                           "blur_config.json",
+                           registerPlugin<KWin::BlurEffectConfig>();)
 
 namespace KWin
 {
 
-KWIN_EFFECT_CONFIG_FACTORY
-
 BlurEffectConfig::BlurEffectConfig(QWidget *parent, const QVariantList &args)
-    : KCModule(EffectFactory::componentData(), parent, args)
+    : KCModule(KAboutData::pluginData(QStringLiteral("blur")), parent, args)
 {
     ui.setupUi(this);
     addConfig(BlurConfig::self(), this);
@@ -45,7 +49,10 @@ void BlurEffectConfig::save()
 {
     KCModule::save();
 
-    EffectsHandler::sendReloadMessage("blur");
+    OrgKdeKwinEffectsInterface interface(QStringLiteral("org.kde.kwin.Effects"),
+                                         QStringLiteral("/Effects"),
+                                         QDBusConnection::sessionBus());
+    interface.reconfigureEffect(QStringLiteral("kwin4_effect_blur"));
 }
 
 } // namespace KWin
