@@ -23,28 +23,20 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <kwinconfig.h>
 #include <QFont>
+#include <QIcon>
 #include <QMatrix4x4>
 #include <QMouseEvent>
-#include <KDE/KLocalizedString>
-#include <kapplication.h>
+#include <KLocalizedString>
 #include <kcolorscheme.h>
-#include <kglobal.h>
-#include <kstandarddirs.h>
-#include <KDE/KIcon>
 
 #include <kwinglutils.h>
 #include <kwinglplatform.h>
 
 #include <math.h>
 
-#include <kdebug.h>
-
 namespace KWin
 {
 
-KWIN_EFFECT(coverswitch, CoverSwitchEffect)
-KWIN_EFFECT_SUPPORTED(coverswitch, CoverSwitchEffect::supported())
-
 CoverSwitchEffect::CoverSwitchEffect()
     : mActivated(0)
     , angle(60.0)
@@ -68,15 +60,15 @@ CoverSwitchEffect::CoverSwitchEffect()
     captionFont.setPointSize(captionFont.pointSize() * 2);
 
     if (effects->compositingType() == OpenGL2Compositing) {
-        QString shadersDir = "kwin/shaders/1.10/";
+        QString shadersDir = QStringLiteral("kwin/shaders/1.10/");
 #ifdef KWIN_HAVE_OPENGLES
         const qint64 coreVersionNumber = kVersionNumber(3, 0);
 #else
         const qint64 coreVersionNumber = kVersionNumber(1, 40);
 #endif
         if (GLPlatform::instance()->glslVersion() >= coreVersionNumber)
-            shadersDir = "kwin/shaders/1.40/";
-        const QString fragmentshader = KGlobal::dirs()->findResource("data", shadersDir + "coverswitch-reflection.glsl");
+            shadersDir = QStringLiteral("kwin/shaders/1.40/");
+        const QString fragmentshader = QStandardPaths::locate(QStandardPaths::GenericDataLocation, shadersDir + QStringLiteral("coverswitch-reflection.glsl"));
         m_reflectionShader = ShaderManager::instance()->loadFragmentShader(ShaderManager::GenericShader, fragmentshader);
     } else {
         m_reflectionShader = NULL;
@@ -101,7 +93,7 @@ bool CoverSwitchEffect::supported()
 
 void CoverSwitchEffect::reconfigure(ReconfigureFlags)
 {
-    CoverSwitchConfig::self()->readConfig();
+    CoverSwitchConfig::self()->read();
     animationDuration = animationTime<CoverSwitchConfig>(200);
     animateSwitch     = CoverSwitchConfig::animateSwitch();
     animateStart      = CoverSwitchConfig::animateStart();
@@ -195,22 +187,12 @@ void CoverSwitchEffect::paintScreen(int mask, QRegion region, ScreenPaintData& d
             projection.frustum(xmin * xminFactor, xmax * xmaxFactor, ymin * yminFactor, ymax * ymaxFactor, zNear, zFar);
             QMatrix4x4 modelview;
             modelview.translate(xTranslate, yTranslate, 0.0);
-            if (shaderManager->isShaderBound()) {
-                GLShader *shader = shaderManager->pushShader(ShaderManager::GenericShader);
-                origProjection = shader->getUniformMatrix4x4("projection");
-                origModelview = shader->getUniformMatrix4x4("modelview");
-                shader->setUniform("projection", projection);
-                shader->setUniform("modelview", origModelview * modelview);
-                shaderManager->popShader();
-            } else {
-#ifdef KWIN_HAVE_OPENGL_1
-                glMatrixMode(GL_PROJECTION);
-                pushMatrix();
-                loadMatrix(projection);
-                glMatrixMode(GL_MODELVIEW);
-                pushMatrix(modelview);
-#endif
-            }
+            GLShader *shader = shaderManager->pushShader(ShaderManager::GenericShader);
+            origProjection = shader->getUniformMatrix4x4("projection");
+            origModelview = shader->getUniformMatrix4x4("modelview");
+            shader->setUniform("projection", projection);
+            shader->setUniform("modelview", origModelview * modelview);
+            shaderManager->popShader();
         }
 
         QList< EffectWindow* > tempList = currentWindowList;
@@ -313,7 +295,7 @@ void CoverSwitchEffect::paintScreen(int mask, QRegion region, ScreenPaintData& d
             glScissor(area.x(), y, area.width(), area.height());
             glEnable(GL_SCISSOR_TEST);
 
-            if (shaderManager->isValid() && m_reflectionShader && m_reflectionShader->isValid()) {
+            if (m_reflectionShader && m_reflectionShader->isValid()) {
                 shaderManager->pushShader(m_reflectionShader);
                 QMatrix4x4 windowTransformation;
                 windowTransformation.translate(area.x() + area.width() * 0.5f, 0.0, 0.0);
@@ -343,27 +325,6 @@ void CoverSwitchEffect::paintScreen(int mask, QRegion region, ScreenPaintData& d
                 vbo->render(GL_TRIANGLES);
 
                 shaderManager->popShader();
-            } else {
-#ifdef KWIN_HAVE_OPENGL_1
-                glPushMatrix();
-                QRect fullRect = effects->clientArea(FullArea, activeScreen, effects->currentDesktop());
-                if (effects->numScreens() > 1 && area.x() != fullRect.x()) {
-                    // have to change the reflection area in horizontal layout and right screen
-                    glTranslatef(-area.x(), 0.0, 0.0);
-                }
-                glTranslatef(area.x() + area.width() * 0.5f, 0.0, 0.0);
-                glColor4fv(mirrorColor[0]);
-                glBegin(GL_POLYGON);
-                glVertex3f(vertices[0], vertices[1], vertices[2]);
-                glVertex3f(vertices[3], vertices[4], vertices[5]);
-                // rearground
-                glColor4fv(mirrorColor[1]);
-                glVertex3f(vertices[6], vertices[7], vertices[8]);
-                glVertex3f(vertices[9], vertices[10], vertices[11]);
-                glEnd();
-
-                glPopMatrix();
-#endif
             }
             glDisable(GL_SCISSOR_TEST);
             glDisable(GL_BLEND);
@@ -371,20 +332,10 @@ void CoverSwitchEffect::paintScreen(int mask, QRegion region, ScreenPaintData& d
         paintScene(frontWindow, leftWindows, rightWindows);
 
         if (effects->numScreens() > 1) {
-            if (shaderManager->isShaderBound())  {
-                GLShader *shader = shaderManager->pushShader(ShaderManager::GenericShader);
-                shader->setUniform("projection", origProjection);
-                shader->setUniform("modelview", origModelview);
-                shaderManager->popShader();
-            } else {
-#ifdef KWIN_HAVE_OPENGL_1
-                popMatrix();
-                // revert change of projection matrix
-                glMatrixMode(GL_PROJECTION);
-                popMatrix();
-                glMatrixMode(GL_MODELVIEW);
-#endif
-            }
+            GLShader *shader = shaderManager->pushShader(ShaderManager::GenericShader);
+            shader->setUniform("projection", origProjection);
+            shader->setUniform("modelview", origModelview);
+            shaderManager->popShader();
         }
 
         // Render the caption frame
@@ -553,16 +504,17 @@ void CoverSwitchEffect::slotTabBoxAdded(int mode)
 
                 // Calculation of correct area
                 area = effects->clientArea(FullScreenArea, activeScreen, effects->currentDesktop());
-                scaleFactor = (zPosition + 1100) * 2.0 * tan(60.0 * M_PI / 360.0f) / displayWidth();
-                if (displayWidth() - area.width() != 0) {
+                const QSize screenSize = effects->virtualScreenSize();
+                scaleFactor = (zPosition + 1100) * 2.0 * tan(60.0 * M_PI / 360.0f) / screenSize.width();
+                if (screenSize.width() - area.width() != 0) {
                     // one of the screens is smaller than the other (horizontal)
-                    if (area.width() < displayWidth() - area.width())
-                        scaleFactor *= (float)area.width() / (float)(displayWidth() - area.width());
-                    else if (area.width() != displayWidth() - area.width()) {
+                    if (area.width() < screenSize.width() - area.width())
+                        scaleFactor *= (float)area.width() / (float)(screenSize.width() - area.width());
+                    else if (area.width() != screenSize.width() - area.width()) {
                         // vertical layout with different width
                         // but we don't want to catch screens with same width and different height
-                        if (displayHeight() != area.height())
-                            scaleFactor *= (float)area.width() / (float)(displayWidth());
+                        if (screenSize.height() != area.height())
+                            scaleFactor *= (float)area.width() / (float)(screenSize.width());
                     }
                 }
 
@@ -734,34 +686,22 @@ void CoverSwitchEffect::paintWindowCover(EffectWindow* w, bool reflectedWindow,
     }
 
     if (reflectedWindow) {
-        if (effects->compositingType() == OpenGL2Compositing) {
-            GLShader *shader = ShaderManager::instance()->pushShader(ShaderManager::GenericShader);
-            QMatrix4x4 origMatrix = shader->getUniformMatrix4x4("screenTransformation");
-            QMatrix4x4 reflectionMatrix;
-            reflectionMatrix.scale(1.0, -1.0, 1.0);
-            shader->setUniform("screenTransformation", origMatrix * reflectionMatrix);
-            data.setYTranslation(- area.height() - windowRect.y() - windowRect.height());
-            if (start) {
-                data.multiplyOpacity(timeLine.currentValue());
-            } else if (stop) {
-                data.multiplyOpacity(1.0 - timeLine.currentValue());
-            }
-            effects->drawWindow(w,
-                                 PAINT_WINDOW_TRANSFORMED,
-                                 infiniteRegion(), data);
-            shader->setUniform("screenTransformation", origMatrix);
-            ShaderManager::instance()->popShader();
-        } else {
-#ifdef KWIN_HAVE_OPENGL_1
-            glPushMatrix();
-            glScalef(1.0, -1.0, 1.0);
-            data.setYTranslation(- area.height() - windowRect.y() - windowRect.height());
-            effects->paintWindow(w,
-                                 PAINT_WINDOW_TRANSFORMED,
-                                 infiniteRegion(), data);
-            glPopMatrix();
-#endif
+        GLShader *shader = ShaderManager::instance()->pushShader(ShaderManager::GenericShader);
+        QMatrix4x4 origMatrix = shader->getUniformMatrix4x4("screenTransformation");
+        QMatrix4x4 reflectionMatrix;
+        reflectionMatrix.scale(1.0, -1.0, 1.0);
+        shader->setUniform("screenTransformation", origMatrix * reflectionMatrix);
+        data.setYTranslation(- area.height() - windowRect.y() - windowRect.height());
+        if (start) {
+            data.multiplyOpacity(timeLine.currentValue());
+        } else if (stop) {
+            data.multiplyOpacity(1.0 - timeLine.currentValue());
         }
+        effects->drawWindow(w,
+                                PAINT_WINDOW_TRANSFORMED,
+                                infiniteRegion(), data);
+        shader->setUniform("screenTransformation", origMatrix);
+        ShaderManager::instance()->popShader();
     } else {
         effects->paintWindow(w,
                              PAINT_WINDOW_TRANSFORMED,
@@ -786,10 +726,11 @@ void CoverSwitchEffect::paintFrontWindow(EffectWindow* frontWindow, int width, i
     }
     if (animation) {
       float distance = 0.0;
+      const QSize screenSize = effects->virtualScreenSize();
       if (direction == Right) {
             // move to right
             distance = -frontWindow->geometry().width() * 0.5f + area.width() * 0.5f +
-                       (((float)displayWidth() * 0.5 * scaleFactor) - (float)area.width() * 0.5f) / rightWindows;
+                       (((float)screenSize.width() * 0.5 * scaleFactor) - (float)area.width() * 0.5f) / rightWindows;
             data.translate(distance * timeLine.currentValue());
             data.setRotationAxis(Qt::YAxis);
             data.setRotationAngle(-angle * timeLine.currentValue());
@@ -797,7 +738,7 @@ void CoverSwitchEffect::paintFrontWindow(EffectWindow* frontWindow, int width, i
         } else {
             // move to left
             distance = frontWindow->geometry().width() * 0.5f - area.width() * 0.5f +
-                       ((float)width * 0.5f - ((float)displayWidth() * 0.5 * scaleFactor)) / leftWindows;
+                       ((float)width * 0.5f - ((float)screenSize.width() * 0.5 * scaleFactor)) / leftWindows;
             float factor = 1.0;
             if (specialHandlingForward)
                 factor = 2.0;
@@ -824,9 +765,10 @@ void CoverSwitchEffect::paintWindows(const EffectWindowList& windows, bool left,
         rotateFactor = -1;
     }
 
-    float xTranslate = -((float)(width) * 0.5f - ((float)displayWidth() * 0.5 * scaleFactor));
+    const QSize screenSize = effects->virtualScreenSize();
+    float xTranslate = -((float)(width) * 0.5f - ((float)screenSize.width() * 0.5 * scaleFactor));
     if (!left)
-        xTranslate = ((float)displayWidth() * 0.5 * scaleFactor) - (float)width * 0.5f;
+        xTranslate = ((float)screenSize.width() * 0.5 * scaleFactor) - (float)width * 0.5f;
     // handling for additional window from other side
     // has to appear on this side after half of the time
     if (animation && timeLine.currentValue() >= 0.5 && additionalWindow != NULL) {
@@ -1010,7 +952,7 @@ void CoverSwitchEffect::updateCaption()
     if (selected_window->isDesktop()) {
         captionFrame->setText(i18nc("Special entry in alt+tab list for minimizing all windows",
                      "Show Desktop"));
-        static QPixmap pix = KIcon("user-desktop").pixmap(captionFrame->iconSize());
+        static QPixmap pix = QIcon::fromTheme(QStringLiteral("user-desktop")).pixmap(captionFrame->iconSize());
         captionFrame->setIcon(pix);
     } else {
         captionFrame->setText(selected_window->caption());
