@@ -2,7 +2,7 @@
  KWin - the KDE window manager
  This file is part of the KDE project.
 
- Copyright (C) 2008 Martin Gräßlin <ubuntu@martin-graesslin.com
+ Copyright (C) 2008 Martin Gräßlin <mgraesslin@kde.org>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,22 +21,26 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // KConfigSkeleton
 #include "coverswitchconfig.h"
 
-#include <kwineffects.h>
+#include <kwineffects_interface.h>
+#include <KAboutData>
+#include <KPluginFactory>
 
 #include <QVBoxLayout>
 
+K_PLUGIN_FACTORY_WITH_JSON(CoverSwitchEffectConfigFactory,
+                           "coverswitch_config.json",
+                           registerPlugin<KWin::CoverSwitchEffectConfig>();)
+
 namespace KWin
 {
 
-KWIN_EFFECT_CONFIG_FACTORY
-
 CoverSwitchEffectConfigForm::CoverSwitchEffectConfigForm(QWidget* parent) : QWidget(parent)
 {
     setupUi(this);
 }
 
 CoverSwitchEffectConfig::CoverSwitchEffectConfig(QWidget* parent, const QVariantList& args) :
-    KCModule(EffectFactory::componentData(), parent, args)
+    KCModule(KAboutData::pluginData(QStringLiteral("coverswitch")), parent, args)
 {
     m_ui = new CoverSwitchEffectConfigForm(this);
 
@@ -44,27 +48,18 @@ CoverSwitchEffectConfig::CoverSwitchEffectConfig(QWidget* parent, const QVariant
 
     layout->addWidget(m_ui);
 
-    connect(m_ui->kcfg_Thumbnails, SIGNAL(stateChanged(int)), this, SLOT(thumbnailsChanged()));
-    connect(m_ui->kcfg_DynamicThumbnails, SIGNAL(stateChanged(int)), this, SLOT(thumbnailsChanged()));
-
     addConfig(CoverSwitchConfig::self(), m_ui);
 }
 
 void CoverSwitchEffectConfig::save()
 {
     KCModule::save();
-    EffectsHandler::sendReloadMessage("coverswitch");
+    OrgKdeKwinEffectsInterface interface(QStringLiteral("org.kde.kwin.Effects"),
+                                         QStringLiteral("/Effects"),
+                                         QDBusConnection::sessionBus());
+    interface.reconfigureEffect(QStringLiteral("kwin4_effect_coverswitch"));
 }
 
-void CoverSwitchEffectConfig::thumbnailsChanged()
-{
-    bool enabled = m_ui->kcfg_Thumbnails->isChecked() && m_ui->kcfg_DynamicThumbnails->isChecked();
-    m_ui->kcfg_DynamicThumbnails->setEnabled(m_ui->kcfg_Thumbnails->isChecked());
-    m_ui->kcfg_ThumbnailWindows->setEnabled(enabled);
-    m_ui->labelThumbnailWindows->setEnabled(enabled);
-}
-
-
 } // namespace
 
 #include "coverswitch_config.moc"
