@@ -21,15 +21,19 @@
 // KConfigSkeleton
 #include "dashboardconfig.h"
 
-#include <kwineffects.h>
+#include <kwineffects_interface.h>
+#include <KAboutData>
+#include <KPluginFactory>
+
+K_PLUGIN_FACTORY_WITH_JSON(DashboardEffectConfigFactory,
+                           "dashboard_config.json",
+                           registerPlugin<KWin::DashboardEffectConfig>();)
 
 namespace KWin
 {
 
-KWIN_EFFECT_CONFIG_FACTORY
-
 DashboardEffectConfig::DashboardEffectConfig(QWidget *parent, const QVariantList &args)
-    : KCModule(EffectFactory::componentData(), parent, args)
+    : KCModule(KAboutData::pluginData(QStringLiteral("dashboard")), parent, args)
 {
     ui.setupUi(this);
 
@@ -46,8 +50,12 @@ void DashboardEffectConfig::save()
 {
     KCModule::save();
 
-    EffectsHandler::sendReloadMessage("dashboard");
+    OrgKdeKwinEffectsInterface interface(QStringLiteral("org.kde.kwin.Effects"),
+                                         QStringLiteral("/Effects"),
+                                         QDBusConnection::sessionBus());
+    interface.reconfigureEffect(QStringLiteral("kwin4_effect_dashboard"));
 }
 
 } // namespace KWin
 
+#include "dashboard_config.moc"
