@@ -28,29 +28,23 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <math.h>
 
-#include <kaction.h>
-#include <kactioncollection.h>
-#include <kdebug.h>
-#include <KDE/KGlobal>
-#include <KDE/KLocalizedString>
-#include <KDE/KStandardDirs>
-#include <kdeclarative.h>
+#include <QAction>
+#include <QApplication>
+#include <KGlobalAccel>
+#include <KLocalizedString>
 #include <netwm_def.h>
 #include <QEvent>
 #include <QMouseEvent>
-#include <kglobalsettings.h>
 #include <QtGui/QVector2D>
-#include <QDeclarativeEngine>
-#include <QDeclarativeContext>
-#include <QGraphicsObject>
+#include <QQmlContext>
+#include <QQmlEngine>
+#include <QQuickItem>
 
 namespace KWin
 {
 
 // WARNING, TODO: This effect relies on the desktop layout being EWMH-compliant.
 
-KWIN_EFFECT(desktopgrid, DesktopGridEffect)
-
 DesktopGridEffect::DesktopGridEffect()
     : activated(false)
     , timeline()
@@ -70,13 +64,15 @@ DesktopGridEffect::DesktopGridEffect()
     , m_proxy(0)
 {
     // Load shortcuts
-    KActionCollection* actionCollection = new KActionCollection(this);
-    KAction* a = (KAction*) actionCollection->addAction("ShowDesktopGrid");
+    QAction* a = new QAction(this);
+    a->setObjectName(QStringLiteral("ShowDesktopGrid"));
     a->setText(i18n("Show Desktop Grid"));
-    a->setGlobalShortcut(KShortcut(Qt::CTRL + Qt::Key_F8));
-    shortcut = a->globalShortcut();
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::CTRL + Qt::Key_F8);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::CTRL + Qt::Key_F8);
+    shortcut = KGlobalAccel::self()->shortcut(a);
+    effects->registerGlobalShortcut(Qt::CTRL + Qt::Key_F8, a);
     connect(a, SIGNAL(triggered(bool)), this, SLOT(toggle()));
-    connect(a, SIGNAL(globalShortcutChanged(QKeySequence)), this, SLOT(globalShortcutChanged(QKeySequence)));
+    connect(KGlobalAccel::self(), &KGlobalAccel::globalShortcutChanged, this, &DesktopGridEffect::globalShortcutChanged);
     connect(effects, SIGNAL(windowAdded(KWin::EffectWindow*)), this, SLOT(slotWindowAdded(KWin::EffectWindow*)));
     connect(effects, SIGNAL(windowClosed(KWin::EffectWindow*)), this, SLOT(slotWindowClosed(KWin::EffectWindow*)));
     connect(effects, SIGNAL(windowDeleted(KWin::EffectWindow*)), this, SLOT(slotWindowDeleted(KWin::EffectWindow*)));
@@ -99,7 +95,7 @@ DesktopGridEffect::~DesktopGridEffect()
 
 void DesktopGridEffect::reconfigure(ReconfigureFlags)
 {
-    DesktopGridConfig::self()->readConfig();
+    DesktopGridConfig::self()->read();
 
     foreach (ElectricBorder border, borderActivate) {
         effects->unreserveElectricBorder(border, this);
@@ -484,7 +480,7 @@ void DesktopGridEffect::windowInputMouseEvent(QEvent* e)
     if (e->type() == QEvent::MouseMove) {
         int d = posToDesktop(me->pos());
         if (windowMove != NULL &&
-                (me->pos() - dragStartPos).manhattanLength() > KGlobalSettings::dndEventDelay()) {
+                (me->pos() - dragStartPos).manhattanLength() > QApplication::startDragDistance()) {
             // Handle window moving
             if (!wasWindowMove) { // Activate on move
                 if (isUsingPresentWindows() && windowMove->isOnAllDesktops()) {
@@ -537,7 +533,7 @@ void DesktopGridEffect::windowInputMouseEvent(QEvent* e)
             }
             effects->addRepaintFull();
         } else if ((me->buttons() & Qt::LeftButton) && !wasDesktopMove &&
-                  (me->pos() - dragStartPos).manhattanLength() > KGlobalSettings::dndEventDelay()) {
+                  (me->pos() - dragStartPos).manhattanLength() > QApplication::startDragDistance()) {
             wasDesktopMove = true;
             effects->defineCursor(Qt::ClosedHandCursor);
         }
@@ -1096,7 +1092,7 @@ void DesktopGridEffect::setup()
 
     // setup the motion managers
     if (m_usePresentWindows)
-        m_proxy = static_cast<PresentWindowsEffectProxy*>(effects->getProxy("presentwindows"));
+        m_proxy = static_cast<PresentWindowsEffectProxy*>(effects->getProxy(QStringLiteral("presentwindows")));
     if (isUsingPresentWindows()) {
         for (int i = 1; i <= effects->numberOfDesktops(); i++) {
             for (int j = 0; j < effects->numScreens(); j++) {
@@ -1120,9 +1116,9 @@ void DesktopGridEffect::setup()
         connect(view, SIGNAL(addDesktop()), SLOT(slotAddDesktop()));
         connect(view, SIGNAL(removeDesktop()), SLOT(slotRemoveDesktop()));
         const QRect screenRect = effects->clientArea(FullScreenArea, i, 1);
-        view->setGeometry(screenRect.right() + 1 - view->sceneRect().width(),
-                          screenRect.bottom() + 1 - view->sceneRect().height(),
-                          view->sceneRect().width(), view->sceneRect().height());
+        view->setGeometry(screenRect.right() + 1 - view->width(),
+                          screenRect.bottom() + 1 - view->height(),
+                          view->width(), view->height());
         view->show();
         m_desktopButtonsViews.insert(view, NULL);
     }
@@ -1215,9 +1211,13 @@ void DesktopGridEffect::finish()
     }
 }
 
-void DesktopGridEffect::globalShortcutChanged(const QKeySequence& seq)
+void DesktopGridEffect::globalShortcutChanged(QAction *action, const QKeySequence& seq)
 {
-    shortcut = KShortcut(seq);
+    if (action->objectName() != QStringLiteral("ShowDesktopGrid")) {
+        return;
+    }
+    shortcut.clear();
+    shortcut.append(seq);
 }
 
 bool DesktopGridEffect::isMotionManagerMovingWindows() const
@@ -1385,29 +1385,19 @@ bool DesktopGridEffect::isRelevantWithPresentWindows(EffectWindow *w) const
 /************************************************
 * DesktopButtonView
 ************************************************/
-DesktopButtonsView::DesktopButtonsView(QWidget *parent)
-    : QDeclarativeView(parent)
+DesktopButtonsView::DesktopButtonsView(QWindow *parent)
+    : QQuickView(parent)
 {
-    setWindowFlags(Qt::X11BypassWindowManagerHint);
-    setAttribute(Qt::WA_TranslucentBackground);
-    QPalette pal = palette();
-    pal.setColor(backgroundRole(), Qt::transparent);
-    setPalette(pal);
-    foreach (const QString &importPath, KGlobal::dirs()->findDirs("module", "imports")) {
-        engine()->addImportPath(importPath);
-    }
-    KDeclarative kdeclarative;
-    kdeclarative.setDeclarativeEngine(engine());
-    kdeclarative.initialize();
-    kdeclarative.setupBindings();
+    setFlags(Qt::X11BypassWindowManagerHint);
+    setColor(Qt::transparent);
 
-    rootContext()->setContextProperty("add", QVariant(true));
-    rootContext()->setContextProperty("remove", QVariant(true));
-    setSource(QUrl(KStandardDirs::locate("data", QLatin1String("kwin/effects/desktopgrid/main.qml"))));
-    if (QObject *item = rootObject()->findChild<QObject*>("addButton")) {
+    rootContext()->setContextProperty(QStringLiteral("add"), QVariant(true));
+    rootContext()->setContextProperty(QStringLiteral("remove"), QVariant(true));
+    setSource(QUrl(QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("kwin/effects/desktopgrid/main.qml"))));
+    if (QObject *item = rootObject()->findChild<QObject*>(QStringLiteral("addButton"))) {
         connect(item, SIGNAL(clicked()), SIGNAL(addDesktop()));
     }
-    if (QObject *item = rootObject()->findChild<QObject*>("removeButton")) {
+    if (QObject *item = rootObject()->findChild<QObject*>(QStringLiteral("removeButton"))) {
         connect(item, SIGNAL(clicked()), SIGNAL(removeDesktop()));
     }
 }
@@ -1427,12 +1417,12 @@ void DesktopButtonsView::windowInputMouseEvent(QMouseEvent *e)
 
 void DesktopButtonsView::setAddDesktopEnabled(bool enable)
 {
-    rootContext()->setContextProperty("add", QVariant(enable));
+    rootContext()->setContextProperty(QStringLiteral("add"), QVariant(enable));
 }
 
 void DesktopButtonsView::setRemoveDesktopEnabled(bool enable)
 {
-    rootContext()->setContextProperty("remove", QVariant(enable));
+    rootContext()->setContextProperty(QStringLiteral("remove"), QVariant(enable));
 }
 
 } // namespace
