@@ -22,27 +22,33 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "desktopgrid_config.h"
 // KConfigSkeleton
 #include "desktopgridconfig.h"
+#include <kwineffects_interface.h>
 
-#include <kwineffects.h>
+#include <QAction>
 
 #include <kconfiggroup.h>
 #include <KActionCollection>
-#include <kaction.h>
+#include <KAboutData>
+#include <KGlobalAccel>
+#include <KLocalizedString>
+#include <KPluginFactory>
 
 #include <QVBoxLayout>
 
+K_PLUGIN_FACTORY_WITH_JSON(DesktopGridEffectConfigFactory,
+                           "desktopgrid_config.json",
+                           registerPlugin<KWin::DesktopGridEffectConfig>();)
+
 namespace KWin
 {
 
-KWIN_EFFECT_CONFIG_FACTORY
-
 DesktopGridEffectConfigForm::DesktopGridEffectConfigForm(QWidget* parent) : QWidget(parent)
 {
     setupUi(this);
 }
 
 DesktopGridEffectConfig::DesktopGridEffectConfig(QWidget* parent, const QVariantList& args)
-    :   KCModule(EffectFactory::componentData(), parent, args)
+    :   KCModule(KAboutData::pluginData(QStringLiteral("desktopgrid")), parent, args)
 {
     m_ui = new DesktopGridEffectConfigForm(this);
 
@@ -51,18 +57,20 @@ DesktopGridEffectConfig::DesktopGridEffectConfig(QWidget* parent, const QVariant
     layout->addWidget(m_ui);
 
     // Shortcut config. The shortcut belongs to the component "kwin"!
-    m_actionCollection = new KActionCollection(this, KComponentData("kwin"));
+    m_actionCollection = new KActionCollection(this, QStringLiteral("kwin"));
 
-    m_actionCollection->setConfigGroup("DesktopGrid");
+    m_actionCollection->setConfigGroup(QStringLiteral("DesktopGrid"));
     m_actionCollection->setConfigGlobal(true);
 
-    KAction* a = (KAction*) m_actionCollection->addAction("ShowDesktopGrid");
+    QAction* a = m_actionCollection->addAction(QStringLiteral("ShowDesktopGrid"));
     a->setText(i18n("Show Desktop Grid"));
     a->setProperty("isConfigurationAction", true);
-    a->setGlobalShortcut(KShortcut(Qt::CTRL + Qt::Key_F8));
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::CTRL + Qt::Key_F8);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::CTRL + Qt::Key_F8);
 
     m_ui->shortcutEditor->addCollection(m_actionCollection);
 
+
     m_ui->desktopNameAlignmentCombo->addItem(i18nc("Desktop name alignment:", "Disabled"), QVariant(Qt::Alignment(0)));
     m_ui->desktopNameAlignmentCombo->addItem(i18n("Top"), QVariant(Qt::AlignHCenter | Qt::AlignTop));
     m_ui->desktopNameAlignmentCombo->addItem(i18n("Top-Right"), QVariant(Qt::AlignRight | Qt::AlignTop));
@@ -95,11 +103,14 @@ void DesktopGridEffectConfig::save()
     DesktopGridConfig::setDesktopNameAlignment(m_ui->desktopNameAlignmentCombo->itemData(m_ui->desktopNameAlignmentCombo->currentIndex()).toInt());
     KCModule::save();
 
-    KConfigGroup conf = EffectsHandler::effectConfig("DesktopGrid");
+    KConfigGroup conf = EffectsHandler::effectConfig(QStringLiteral("DesktopGrid"));
     conf.writeEntry("DesktopNameAlignment", DesktopGridConfig::desktopNameAlignment());
     conf.sync();
 
-    EffectsHandler::sendReloadMessage("desktopgrid");
+    OrgKdeKwinEffectsInterface interface(QStringLiteral("org.kde.kwin.Effects"),
+                                         QStringLiteral("/Effects"),
+                                         QDBusConnection::sessionBus());
+    interface.reconfigureEffect(QStringLiteral("kwin4_effect_desktopgrid"));
 }
 
 void DesktopGridEffectConfig::load()
