@@ -22,29 +22,30 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // KConfigSkeleton
 #include "diminactiveconfig.h"
 
-#include <kwineffects.h>
+#include <kwineffects_interface.h>
 
 #include <KLocalizedString>
-#include <kdebug.h>
 #include <kconfiggroup.h>
-#include <kaction.h>
-#include <KShortcutsEditor>
+#include <KAboutData>
+#include <KPluginFactory>
 
 #include <QWidget>
 #include <QVBoxLayout>
 
+K_PLUGIN_FACTORY_WITH_JSON(DimInactiveEffectConfigFactory,
+                           "diminactive_config.json",
+                           registerPlugin<KWin::DimInactiveEffectConfig>();)
+
 namespace KWin
 {
 
-KWIN_EFFECT_CONFIG_FACTORY
-
 DimInactiveEffectConfigForm::DimInactiveEffectConfigForm(QWidget* parent) : QWidget(parent)
 {
     setupUi(this);
 }
 
 DimInactiveEffectConfig::DimInactiveEffectConfig(QWidget* parent, const QVariantList& args) :
-    KCModule(EffectFactory::componentData(), parent, args)
+    KCModule(KAboutData::pluginData(QStringLiteral("diminactive")), parent, args)
 {
     m_ui = new DimInactiveEffectConfigForm(this);
 
@@ -60,9 +61,12 @@ DimInactiveEffectConfig::DimInactiveEffectConfig(QWidget* parent, const QVariant
 void DimInactiveEffectConfig::save()
 {
     KCModule::save();
-    EffectsHandler::sendReloadMessage("diminactive");
+    OrgKdeKwinEffectsInterface interface(QStringLiteral("org.kde.kwin.Effects"),
+                                         QStringLiteral("/Effects"),
+                                         QDBusConnection::sessionBus());
+    interface.reconfigureEffect(QStringLiteral("kwin4_effect_diminactive"));
 }
 
 } // namespace
 
-#include "moc_diminactive_config.cpp"
+#include "diminactive_config.moc"
