@@ -20,27 +20,33 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "flipswitch_config.h"
 // KConfigSkeleton
 #include "flipswitchconfig.h"
+#include <kwineffects_interface.h>
 
-#include <kwineffects.h>
+#include <QAction>
 
 #include <kconfiggroup.h>
-#include <KAction>
 #include <KActionCollection>
+#include <KAboutData>
+#include <KGlobalAccel>
+#include <KLocalizedString>
+#include <KPluginFactory>
 
 #include <QVBoxLayout>
 
+K_PLUGIN_FACTORY_WITH_JSON(FlipSwitchEffectConfigFactory,
+                           "flipswitch_config.json",
+                           registerPlugin<KWin::FlipSwitchEffectConfig>();)
+
 namespace KWin
 {
 
-KWIN_EFFECT_CONFIG_FACTORY
-
 FlipSwitchEffectConfigForm::FlipSwitchEffectConfigForm(QWidget* parent) : QWidget(parent)
 {
     setupUi(this);
 }
 
 FlipSwitchEffectConfig::FlipSwitchEffectConfig(QWidget* parent, const QVariantList& args) :
-    KCModule(EffectFactory::componentData(), parent, args)
+    KCModule(KAboutData::pluginData(QStringLiteral("flipswitch")), parent, args)
 {
     m_ui = new FlipSwitchEffectConfigForm(this);
 
@@ -49,15 +55,15 @@ FlipSwitchEffectConfig::FlipSwitchEffectConfig(QWidget* parent, const QVariantLi
     layout->addWidget(m_ui);
 
     // Shortcut config. The shortcut belongs to the component "kwin"!
-    m_actionCollection = new KActionCollection(this, KComponentData("kwin"));
-    KAction* a = (KAction*)m_actionCollection->addAction("FlipSwitchCurrent");
+    m_actionCollection = new KActionCollection(this, QStringLiteral("kwin"));
+    QAction* a = m_actionCollection->addAction(QStringLiteral("FlipSwitchCurrent"));
     a->setText(i18n("Toggle Flip Switch (Current desktop)"));
-    a->setGlobalShortcut(KShortcut(), KAction::ActiveShortcut);
-    KAction* b = (KAction*)m_actionCollection->addAction("FlipSwitchAll");
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>());
+    QAction* b = m_actionCollection->addAction(QStringLiteral("FlipSwitchAll"));
     b->setText(i18n("Toggle Flip Switch (All desktops)"));
-    b->setGlobalShortcut(KShortcut(), KAction::ActiveShortcut);
+    KGlobalAccel::self()->setShortcut(b, QList<QKeySequence>());
 
-    m_actionCollection->setConfigGroup("FlipSwitch");
+    m_actionCollection->setConfigGroup(QStringLiteral("FlipSwitch"));
     m_actionCollection->setConfigGlobal(true);
 
     m_ui->shortcutEditor->addCollection(m_actionCollection);
@@ -76,7 +82,10 @@ void FlipSwitchEffectConfig::save()
     KCModule::save();
     m_ui->shortcutEditor->save();
 
-    EffectsHandler::sendReloadMessage("flipswitch");
+    OrgKdeKwinEffectsInterface interface(QStringLiteral("org.kde.kwin.Effects"),
+                                         QStringLiteral("/Effects"),
+                                         QDBusConnection::sessionBus());
+    interface.reconfigureEffect(QStringLiteral("kwin4_effect_flipswitch"));
 }
 
 
