@@ -31,16 +31,20 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 namespace KWin
 {
 
-KWIN_EFFECT(glide, GlideEffect)
-KWIN_EFFECT_SUPPORTED(glide, GlideEffect::supported())
-
 static const int IsGlideWindow = 0x22A982D4;
-static Atom slideAtom;
+static xcb_atom_t slideAtom;
+static const QByteArray s_slideAtomName = QByteArrayLiteral("_KDE_SLIDE");
 
 GlideEffect::GlideEffect()
 {
-    slideAtom = XInternAtom( display(), "_KDE_SLIDE", False );
-    effects->registerPropertyType( slideAtom, true );
+    slideAtom = XCB_ATOM_NONE;
+    xcb_connection_t *c = connection();
+    const auto cookie = xcb_intern_atom(c, false, s_slideAtomName.length(), s_slideAtomName.constData());
+    QScopedPointer<xcb_intern_atom_reply_t, QScopedPointerPodDeleter> atom(xcb_intern_atom_reply(c, cookie, nullptr));
+    if (atom) {
+        slideAtom = atom->atom;
+        effects->registerPropertyType( slideAtom, true );
+    }
     reconfigure(ReconfigureAll);
     connect(effects, SIGNAL(windowAdded(KWin::EffectWindow*)), this, SLOT(slotWindowAdded(KWin::EffectWindow*)));
     connect(effects, SIGNAL(windowClosed(KWin::EffectWindow*)), this, SLOT(slotWindowClosed(KWin::EffectWindow*)));
@@ -49,7 +53,9 @@ GlideEffect::GlideEffect()
 
 GlideEffect::~GlideEffect()
 {
-    effects->registerPropertyType( slideAtom, false );
+    if (slideAtom) {
+        effects->registerPropertyType( slideAtom, false );
+    }
 }
 
 bool GlideEffect::supported()
@@ -60,7 +66,7 @@ bool GlideEffect::supported()
 void GlideEffect::reconfigure(ReconfigureFlags)
 {
     // Fetch config with KConfigXT
-    GlideConfig::self()->readConfig();
+    GlideConfig::self()->read();
     duration = animationTime<GlideConfig>(350);
     effect = (EffectStyle) GlideConfig::glideEffect();
     angle = GlideConfig::glideAngle();
@@ -107,38 +113,32 @@ void GlideEffect::paintWindow(EffectWindow* w, int mask, QRegion region, WindowP
         default:
         case GlideInOut:
             if (info->added)
-                glideIn(w, data);
+                glideIn(w, data, info);
             else if (info->closed)
-                glideOut(w, data);
+                glideOut(w, data, info);
             break;
         case GlideOutIn:
             if (info->added)
-                glideOut(w, data);
+                glideOut(w, data, info);
             if (info->closed)
-                glideIn(w, data);
+                glideIn(w, data, info);
             break;
-        case GlideIn: glideIn(w, data); break;
-        case GlideOut: glideOut(w, data); break;
+        case GlideIn: glideIn(w, data, info); break;
+        case GlideOut: glideOut(w, data, info); break;
         }
     }
     effects->paintWindow(w, mask, region, data);
 }
 
-void GlideEffect::glideIn(EffectWindow* w, WindowPaintData& data)
+void GlideEffect::glideIn(EffectWindow* w, WindowPaintData& data, const InfoHash::const_iterator &info)
 {
-    InfoHash::const_iterator info = windows.constFind(w);
-    if (info == windows.constEnd())
-        return;
     const double progress = info->timeLine->currentValue();
     data *= progress;
     data.translate(int(w->width() / 2 * (1 - progress)), int(w->height() / 2 * (1 - progress)));
 }
 
-void GlideEffect::glideOut(EffectWindow* w, WindowPaintData& data)
+void GlideEffect::glideOut(EffectWindow* w, WindowPaintData& data, const InfoHash::const_iterator &info)
 {
-    InfoHash::const_iterator info = windows.constFind(w);
-    if (info == windows.constEnd())
-        return;
     const double progress = info->timeLine->currentValue();
     data *= (2 - progress);
     data.translate(- int(w->width() / 2 * (1 - progress)), - int(w->height() / 2 * (1 - progress)));
@@ -219,13 +219,13 @@ bool GlideEffect::isGlideWindow(EffectWindow* w)
         return false;
     if (w->data(IsGlideWindow).toBool())
         return true;
-    if (!w->readProperty( slideAtom, slideAtom, 32 ).isNull())
+    if (slideAtom && !w->readProperty( slideAtom, slideAtom, 32 ).isNull())
         return false;
     if (w->hasDecoration())
         return true;
     if (!w->isManaged() || w->isMenu() ||  w->isNotification() || w->isDesktop() ||
             w->isDock() ||  w->isSplash() || w->isToolbar() ||
-            w->windowClass() == "dashboard dashboard")
+            w->windowClass() == QStringLiteral("dashboard dashboard"))
         return false;
     return true;
 }
