@@ -47,6 +47,10 @@ public:
     virtual void postPaintWindow(EffectWindow* w);
     virtual bool isActive() const;
 
+    int requestedEffectChainPosition() const override {
+        return 50;
+    }
+
     static bool supported();
 
     // for properties
@@ -67,8 +71,8 @@ public Q_SLOTS:
 private:
     class WindowInfo;
     typedef QMap< const EffectWindow*, WindowInfo > InfoHash;
-    void glideIn(EffectWindow* w, WindowPaintData& data);
-    void glideOut(EffectWindow* w, WindowPaintData& data);
+    void glideIn(EffectWindow* w, WindowPaintData& data, const InfoHash::const_iterator &info);
+    void glideOut(EffectWindow* w, WindowPaintData& data, const InfoHash::const_iterator &info);
     bool isGlideWindow(EffectWindow* w);
     InfoHash windows;
     float duration;
