@@ -22,15 +22,19 @@
 // KConfigSkeleton
 #include "glideconfig.h"
 
-#include <kwineffects.h>
+#include <kwineffects_interface.h>
+#include <KAboutData>
+#include <KPluginFactory>
+
+K_PLUGIN_FACTORY_WITH_JSON(GlideEffectConfigFactory,
+                           "glide_config.json",
+                           registerPlugin<KWin::GlideEffectConfig>();)
 
 namespace KWin
 {
 
-KWIN_EFFECT_CONFIG_FACTORY
-
 GlideEffectConfig::GlideEffectConfig(QWidget *parent, const QVariantList &args)
-    : KCModule(EffectFactory::componentData(), parent, args)
+    : KCModule(KAboutData::pluginData(QStringLiteral("glide")), parent, args)
 {
     ui.setupUi(this);
     addConfig(GlideConfig::self(), this);
@@ -44,7 +48,10 @@ GlideEffectConfig::~GlideEffectConfig()
 void GlideEffectConfig::save()
 {
     KCModule::save();
-    EffectsHandler::sendReloadMessage("glide");
+    OrgKdeKwinEffectsInterface interface(QStringLiteral("org.kde.kwin.Effects"),
+                                         QStringLiteral("/Effects"),
+                                         QDBusConnection::sessionBus());
+    interface.reconfigureEffect(QStringLiteral("kwin4_effect_glide"));
 }
 } // namespace KWin
 #include "glide_config.moc"
