@@ -19,39 +19,45 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 
 #include "invert_config.h"
+#include <kwineffects_interface.h>
 
-#include <kwineffects.h>
+#include <QAction>
 
-#include <KDE/KLocalizedString>
-#include <kdebug.h>
+#include <KGlobalAccel>
+#include <KLocalizedString>
 #include <KActionCollection>
-#include <kaction.h>
 #include <KShortcutsEditor>
+#include <KAboutData>
+#include <KPluginFactory>
 
 #include <QVBoxLayout>
 
+K_PLUGIN_FACTORY_WITH_JSON(InvertEffectConfigFactory,
+                           "invert_config.json",
+                           registerPlugin<KWin::InvertEffectConfig>();)
+
 namespace KWin
 {
 
-KWIN_EFFECT_CONFIG_FACTORY
-
 InvertEffectConfig::InvertEffectConfig(QWidget* parent, const QVariantList& args) :
-    KCModule(EffectFactory::componentData(), parent, args)
+    KCModule(KAboutData::pluginData(QStringLiteral("invert")), parent, args)
 {
     QVBoxLayout* layout = new QVBoxLayout(this);
 
     // Shortcut config. The shortcut belongs to the component "kwin"!
-    KActionCollection *actionCollection = new KActionCollection(this, KComponentData("kwin"));
+    KActionCollection *actionCollection = new KActionCollection(this, QStringLiteral("kwin"));
 
-    KAction* a = static_cast<KAction*>(actionCollection->addAction("Invert"));
+    QAction* a = actionCollection->addAction(QStringLiteral("Invert"));
     a->setText(i18n("Toggle Invert Effect"));
     a->setProperty("isConfigurationAction", true);
-    a->setGlobalShortcut(KShortcut(Qt::CTRL + Qt::META + Qt::Key_I));
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::CTRL + Qt::META + Qt::Key_I);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::CTRL + Qt::META + Qt::Key_I);
 
-    KAction* b = static_cast<KAction*>(actionCollection->addAction("InvertWindow"));
+    QAction* b = actionCollection->addAction(QStringLiteral("InvertWindow"));
     b->setText(i18n("Toggle Invert Effect on Window"));
     b->setProperty("isConfigurationAction", true);
-    b->setGlobalShortcut(KShortcut(Qt::CTRL + Qt::META + Qt::Key_U));
+    KGlobalAccel::self()->setDefaultShortcut(b, QList<QKeySequence>() << Qt::CTRL + Qt::META + Qt::Key_U);
+    KGlobalAccel::self()->setShortcut(b, QList<QKeySequence>() << Qt::CTRL + Qt::META + Qt::Key_U);
 
     mShortcutEditor = new KShortcutsEditor(actionCollection, this,
                                            KShortcutsEditor::GlobalAction, KShortcutsEditor::LetterShortcutsDisallowed);
@@ -81,12 +87,16 @@ void InvertEffectConfig::save()
     mShortcutEditor->save();    // undo() will restore to this state from now on
 
     emit changed(false);
-    EffectsHandler::sendReloadMessage("invert");
+    OrgKdeKwinEffectsInterface interface(QStringLiteral("org.kde.kwin.Effects"),
+                                         QStringLiteral("/Effects"),
+                                         QDBusConnection::sessionBus());
+    interface.reconfigureEffect(QStringLiteral("kwin4_effect_invert"));
 }
 
 void InvertEffectConfig::defaults()
 {
     mShortcutEditor->allDefault();
+
     emit changed(true);
 }
 
