@@ -29,10 +29,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 namespace KWin
 {
 
+class GLRenderTarget;
+class GLTexture;
+
 class LogoutEffect
     : public Effect
 {
     Q_OBJECT
+    Q_PROPERTY(bool useBlur READ isUseBlur)
 public:
     LogoutEffect();
     ~LogoutEffect();
@@ -43,6 +47,14 @@ public:
     virtual void paintWindow(EffectWindow* w, int mask, QRegion region, WindowPaintData& data);
     virtual bool isActive() const;
 
+    int requestedEffectChainPosition() const override {
+        return 85;
+    }
+
+    // for properties
+    bool isUseBlur() const {
+        return useBlur;
+    }
 public Q_SLOTS:
     void slotWindowAdded(KWin::EffectWindow* w);
     void slotWindowClosed(KWin::EffectWindow *w);
@@ -57,14 +69,22 @@ private:
     bool logoutWindowPassed;
 
     // Persistent effect
-    long logoutAtom;
+    xcb_atom_t logoutAtom;
     bool canDoPersistent;
     EffectWindowList ignoredWindows;
 
+    void renderVignetting();
+    void renderBlurTexture();
     int frameDelay;
+    bool blurSupported, useBlur;
+    GLTexture* blurTexture;
+    GLRenderTarget* blurTarget;
     double windowOpacity;
     EffectWindowList windows;
     QHash< EffectWindow*, double > windowsOpacities;
+    GLShader *m_vignettingShader;
+    GLShader *m_blurShader;
+    QString m_shadersDir;
 };
 
 } // namespace
