@@ -20,21 +20,20 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 
-#include "config-kwin.h"
-
 #include "magnifier.h"
 // KConfigSkeleton
 #include "magnifierconfig.h"
 
-#include <kaction.h>
-#include <kactioncollection.h>
+#include <QAction>
+#include <kwinconfig.h>
 #include <kstandardaction.h>
 
-#ifdef KWIN_BUILD_COMPOSITE
+#include <kwinglutils.h>
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
 #include <kwinxrenderutils.h>
-#include <xcbutils.h>
 #include <xcb/render.h>
 #endif
+#include <KGlobalAccel>
 
 namespace KWin
 {
@@ -45,18 +44,28 @@ MagnifierEffect::MagnifierEffect()
     : zoom(1)
     , target_zoom(1)
     , polling(false)
-#ifdef KWIN_BUILD_COMPOSITE
+    , m_texture(0)
+    , m_fbo(0)
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     , m_pixmap(XCB_PIXMAP_NONE)
 #endif
 {
-    KActionCollection* actionCollection = new KActionCollection(this);
-    KAction* a;
-    a = static_cast< KAction* >(actionCollection->addAction(KStandardAction::ZoomIn, this, SLOT(zoomIn())));
-    a->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_Equal));
-    a = static_cast< KAction* >(actionCollection->addAction(KStandardAction::ZoomOut, this, SLOT(zoomOut())));
-    a->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_Minus));
-    a = static_cast< KAction* >(actionCollection->addAction(KStandardAction::ActualSize, this, SLOT(toggle())));
-    a->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_0));
+    QAction* a;
+    a = KStandardAction::zoomIn(this, SLOT(zoomIn()), this);
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Equal);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Equal);
+    effects->registerGlobalShortcut(Qt::META + Qt::Key_Equal, a);
+
+    a = KStandardAction::zoomOut(this, SLOT(zoomOut()), this);
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Minus);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Minus);
+    effects->registerGlobalShortcut(Qt::META + Qt::Key_Minus, a);
+
+    a = KStandardAction::actualSize(this, SLOT(toggle()), this);
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_0);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_0);
+    effects->registerGlobalShortcut(Qt::META + Qt::Key_0, a);
+
     connect(effects, SIGNAL(mouseChanged(QPoint,QPoint,Qt::MouseButtons,Qt::MouseButtons,Qt::KeyboardModifiers,Qt::KeyboardModifiers)),
             this, SLOT(slotMouseChanged(QPoint,QPoint,Qt::MouseButtons,Qt::MouseButtons,Qt::KeyboardModifiers,Qt::KeyboardModifiers)));
     reconfigure(ReconfigureAll);
@@ -64,16 +73,18 @@ MagnifierEffect::MagnifierEffect()
 
 MagnifierEffect::~MagnifierEffect()
 {
+    delete m_fbo;
+    delete m_texture;
     destroyPixmap();
     // Save the zoom value.
-    KConfigGroup conf = EffectsHandler::effectConfig("Magnifier");
+    KConfigGroup conf = EffectsHandler::effectConfig(QStringLiteral("Magnifier"));
     conf.writeEntry("InitialZoom", target_zoom);
     conf.sync();
 }
 
 void MagnifierEffect::destroyPixmap()
 {
-#ifdef KWIN_BUILD_COMPOSITE
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     if (effects->compositingType() != XRenderCompositing) {
         return;
     }
@@ -85,9 +96,15 @@ void MagnifierEffect::destroyPixmap()
 #endif
 }
 
+bool MagnifierEffect::supported()
+{
+    return  effects->compositingType() == XRenderCompositing ||
+            (effects->isOpenGLCompositing() && GLRenderTarget::blitSupported());
+}
+
 void MagnifierEffect::reconfigure(ReconfigureFlags)
 {
-    MagnifierConfig::self()->readConfig();
+    MagnifierConfig::self()->read();
     int width, height;
     width = MagnifierConfig::width();
     height = MagnifierConfig::height();
@@ -107,6 +124,11 @@ void MagnifierEffect::prePaintScreen(ScreenPrePaintData& data, int time)
         else {
             zoom = qMax(zoom * qMin(1 - diff, 0.8), target_zoom);
             if (zoom == 1.0) {
+                // zoom ended - delete FBO and texture
+                delete m_fbo;
+                delete m_texture;
+                m_fbo = NULL;
+                m_texture = NULL;
                 destroyPixmap();
             }
         }
@@ -127,8 +149,51 @@ void MagnifierEffect::paintScreen(int mask, QRegion region, ScreenPaintData& dat
         QRect srcArea(cursor.x() - (double)area.width() / (zoom*2),
                       cursor.y() - (double)area.height() / (zoom*2),
                       (double)area.width() / zoom, (double)area.height() / zoom);
+        if (effects->isOpenGLCompositing()) {
+            m_fbo->blitFromFramebuffer(srcArea);
+            // paint magnifier
+            m_texture->bind();
+            m_texture->render(infiniteRegion(), area);
+            m_texture->unbind();
+            QVector<float> verts;
+            GLVertexBuffer *vbo = GLVertexBuffer::streamingBuffer();
+            vbo->reset();
+            vbo->setColor(QColor(0, 0, 0));
+            // top frame
+            verts << area.right() + FRAME_WIDTH << area.top() - FRAME_WIDTH;
+            verts << area.left() - FRAME_WIDTH << area.top() - FRAME_WIDTH;
+            verts << area.left() - FRAME_WIDTH << area.top() - 1;
+            verts << area.left() - FRAME_WIDTH << area.top() - 1;
+            verts << area.right() + FRAME_WIDTH << area.top() - 1;
+            verts << area.right() + FRAME_WIDTH << area.top() - FRAME_WIDTH;
+            // left frame
+            verts << area.left() - 1 << area.top() - FRAME_WIDTH;
+            verts << area.left() - FRAME_WIDTH << area.top() - FRAME_WIDTH;
+            verts << area.left() - FRAME_WIDTH << area.bottom() + FRAME_WIDTH;
+            verts << area.left() - FRAME_WIDTH << area.bottom() + FRAME_WIDTH;
+            verts << area.left() - 1 << area.bottom() + FRAME_WIDTH;
+            verts << area.left() - 1 << area.top() - FRAME_WIDTH;
+            // right frame
+            verts << area.right() + FRAME_WIDTH << area.top() - FRAME_WIDTH;
+            verts << area.right() + 1 << area.top() - FRAME_WIDTH;
+            verts << area.right() + 1 << area.bottom() + FRAME_WIDTH;
+            verts << area.right() + 1 << area.bottom() + FRAME_WIDTH;
+            verts << area.right() + FRAME_WIDTH << area.bottom() + FRAME_WIDTH;
+            verts << area.right() + FRAME_WIDTH << area.top() - FRAME_WIDTH;
+            // bottom frame
+            verts << area.right() + FRAME_WIDTH << area.bottom() + 1;
+            verts << area.left() - FRAME_WIDTH << area.bottom() + 1;
+            verts << area.left() - FRAME_WIDTH << area.bottom() + FRAME_WIDTH;
+            verts << area.left() - FRAME_WIDTH << area.bottom() + FRAME_WIDTH;
+            verts << area.right() + FRAME_WIDTH << area.bottom() + FRAME_WIDTH;
+            verts << area.right() + FRAME_WIDTH << area.bottom() + 1;
+            vbo->setData(verts.size() / 2, 2, verts.constData(), NULL);
+
+            ShaderBinder binder(ShaderManager::ColorShader);
+            vbo->render(GL_TRIANGLES);
+        }
         if (effects->compositingType() == XRenderCompositing) {
-#ifdef KWIN_BUILD_COMPOSITE
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
             if (m_pixmap == XCB_PIXMAP_NONE || m_pixmapSize != srcArea.size()) {
                 destroyPixmap();
                 m_pixmap = xcb_generate_id(connection());
@@ -136,21 +201,23 @@ void MagnifierEffect::paintScreen(int mask, QRegion region, ScreenPaintData& dat
                 xcb_create_pixmap(connection(), 32, m_pixmap, rootWindow(), m_pixmapSize.width(), m_pixmapSize.height());
                 m_picture.reset(new XRenderPicture(m_pixmap, 32));
             }
+#define DOUBLE_TO_FIXED(d) ((xcb_render_fixed_t) ((d) * 65536))
             static xcb_render_transform_t identity = {
-                KWIN_DOUBLE_TO_FIXED(1), KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(0),
-                KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(1), KWIN_DOUBLE_TO_FIXED(0),
-                KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(1)
+                DOUBLE_TO_FIXED(1), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0),
+                DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(1), DOUBLE_TO_FIXED(0),
+                DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(1)
             };
             static xcb_render_transform_t xform = {
-                KWIN_DOUBLE_TO_FIXED(1), KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(0),
-                KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(1), KWIN_DOUBLE_TO_FIXED(0),
-                KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(1)
+                DOUBLE_TO_FIXED(1), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0),
+                DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(1), DOUBLE_TO_FIXED(0),
+                DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(1)
             };
             xcb_render_composite(connection(), XCB_RENDER_PICT_OP_SRC, effects->xrenderBufferPicture(), 0, *m_picture,
                                 srcArea.x(), srcArea.y(), 0, 0, 0, 0, srcArea.width(), srcArea.height());
             xcb_flush(connection());
-            xform.matrix11 = KWIN_DOUBLE_TO_FIXED(1.0/zoom);
-            xform.matrix22 = KWIN_DOUBLE_TO_FIXED(1.0/zoom);
+            xform.matrix11 = DOUBLE_TO_FIXED(1.0/zoom);
+            xform.matrix22 = DOUBLE_TO_FIXED(1.0/zoom);
+#undef DOUBLE_TO_FIXED
             xcb_render_set_picture_transform(connection(), *m_picture, xform);
             xcb_render_set_picture_filter(connection(), *m_picture, 4, const_cast<char*>("good"), 0, NULL);
             xcb_render_composite(connection(), XCB_RENDER_PICT_OP_SRC, *m_picture, 0, effects->xrenderBufferPicture(),
@@ -192,6 +259,12 @@ void MagnifierEffect::zoomIn()
         polling = true;
         effects->startMousePolling();
     }
+    if (effects->isOpenGLCompositing() && !m_texture) {
+        effects->makeOpenGLContextCurrent();
+        m_texture = new GLTexture(magnifier_size.width(), magnifier_size.height());
+        m_texture->setYInverted(false);
+        m_fbo = new GLRenderTarget(*m_texture);
+    }
     effects->addRepaint(magnifierArea().adjusted(-FRAME_WIDTH, -FRAME_WIDTH, FRAME_WIDTH, FRAME_WIDTH));
 }
 
@@ -205,6 +278,11 @@ void MagnifierEffect::zoomOut()
             effects->stopMousePolling();
         }
         if (zoom == target_zoom) {
+            effects->makeOpenGLContextCurrent();
+            delete m_fbo;
+            delete m_texture;
+            m_fbo = NULL;
+            m_texture = NULL;
             destroyPixmap();
         }
     }
@@ -221,6 +299,12 @@ void MagnifierEffect::toggle()
             polling = true;
             effects->startMousePolling();
         }
+        if (effects->isOpenGLCompositing() && !m_texture) {
+            effects->makeOpenGLContextCurrent();
+            m_texture = new GLTexture(magnifier_size.width(), magnifier_size.height());
+            m_texture->setYInverted(false);
+            m_fbo = new GLRenderTarget(*m_texture);
+        }
     } else {
         target_zoom = 1;
         if (polling) {
@@ -247,4 +331,4 @@ bool MagnifierEffect::isActive() const
 
 } // namespace
 
-#include "moc_magnifier.cpp"
+#include "magnifier.moc"
