@@ -21,31 +21,35 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "magnifier_config.h"
 // KConfigSkeleton
 #include "magnifierconfig.h"
+#include <kwineffects_interface.h>
 
-#include <kwineffects.h>
+#include <QAction>
 
+#include <KGlobalAccel>
 #include <KLocalizedString>
-#include <kdebug.h>
 #include <kconfiggroup.h>
 #include <KActionCollection>
-#include <kaction.h>
-#include <KShortcutsEditor>
+#include <KAboutData>
+#include <KPluginFactory>
 
+#include <QDebug>
 #include <QWidget>
 #include <QVBoxLayout>
 
+K_PLUGIN_FACTORY_WITH_JSON(MagnifierEffectConfigFactory,
+                           "magnifier_config.json",
+                           registerPlugin<KWin::MagnifierEffectConfig>();)
+
 namespace KWin
 {
 
-KWIN_EFFECT_CONFIG_FACTORY
-
 MagnifierEffectConfigForm::MagnifierEffectConfigForm(QWidget* parent) : QWidget(parent)
 {
     setupUi(this);
 }
 
 MagnifierEffectConfig::MagnifierEffectConfig(QWidget* parent, const QVariantList& args) :
-    KCModule(EffectFactory::componentData(), parent, args)
+    KCModule(KAboutData::pluginData(QStringLiteral("magnifier")), parent, args)
 {
     m_ui = new MagnifierEffectConfigForm(this);
 
@@ -58,25 +62,29 @@ MagnifierEffectConfig::MagnifierEffectConfig(QWidget* parent, const QVariantList
     connect(m_ui->editor, SIGNAL(keyChange()), this, SLOT(changed()));
 
     // Shortcut config. The shortcut belongs to the component "kwin"!
-    m_actionCollection = new KActionCollection(this, KComponentData("kwin"));
+    m_actionCollection = new KActionCollection(this, QStringLiteral("kwin"));
 
-    m_actionCollection->setConfigGroup("Magnifier");
+    m_actionCollection->setConfigGroup(QStringLiteral("Magnifier"));
     m_actionCollection->setConfigGlobal(true);
 
-    KAction* a;
-    a = static_cast< KAction* >(m_actionCollection->addAction(KStandardAction::ZoomIn));
+    QAction* a;
+    a = m_actionCollection->addAction(KStandardAction::ZoomIn);
     a->setProperty("isConfigurationAction", true);
-    a->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_Plus));
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Equal);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Equal);
 
-    a = static_cast< KAction* >(m_actionCollection->addAction(KStandardAction::ZoomOut));
+    a = m_actionCollection->addAction(KStandardAction::ZoomOut);
     a->setProperty("isConfigurationAction", true);
-    a->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_Minus));
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Minus);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Minus);
 
-    a = static_cast< KAction* >(m_actionCollection->addAction(KStandardAction::ActualSize));
+    a = m_actionCollection->addAction(KStandardAction::ActualSize);
     a->setProperty("isConfigurationAction", true);
-    a->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_0));
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_0);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_0);
 
     m_ui->editor->addCollection(m_actionCollection);
+
     load();
 }
 
@@ -88,11 +96,14 @@ MagnifierEffectConfig::~MagnifierEffectConfig()
 
 void MagnifierEffectConfig::save()
 {
-    kDebug(1212) << "Saving config of Magnifier" ;
+    qDebug() << "Saving config of Magnifier" ;
 
     m_ui->editor->save();   // undo() will restore to this state from now on
     KCModule::save();
-    EffectsHandler::sendReloadMessage("magnifier");
+    OrgKdeKwinEffectsInterface interface(QStringLiteral("org.kde.kwin.Effects"),
+                                         QStringLiteral("/Effects"),
+                                         QDBusConnection::sessionBus());
+    interface.reconfigureEffect(QStringLiteral("kwin4_effect_magnifier"));
 }
 
 void MagnifierEffectConfig::defaults()
@@ -103,4 +114,4 @@ void MagnifierEffectConfig::defaults()
 
 } // namespace
 
-#include "moc_magnifier_config.cpp"
+#include "magnifier_config.moc"
