@@ -22,6 +22,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // KConfigSkeleton
 #include "mouseclickconfig.h"
 
+#include <QAction>
 #include <kwinglutils.h>
 
 #ifdef KWIN_HAVE_XRENDER_COMPOSITING
@@ -30,25 +31,28 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <xcb/render.h>
 #endif
 
-#include <KDE/KAction>
-#include <KDE/KActionCollection>
-#include <KDE/KConfigGroup>
+#include <KConfigGroup>
+#include <KGlobalAccel>
+
+#include <QPainter>
 
 #include <math.h>
 
 namespace KWin
 {
 
-KWIN_EFFECT(mouseclick, MouseClickEffect)
-
 MouseClickEffect::MouseClickEffect()
 {
     m_enabled = false;
-    KActionCollection* actionCollection = new KActionCollection(this);
-    KAction* a = static_cast<KAction*>(actionCollection->addAction("ToggleMouseClick"));
+
+    QAction* a = new QAction(this);
+    a->setObjectName(QStringLiteral("ToggleMouseClick"));
     a->setText(i18n("Toggle Effect"));
-    a->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_Asterisk));
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Asterisk);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Asterisk);
+    effects->registerGlobalShortcut(Qt::META + Qt::Key_Asterisk, a);
     connect(a, SIGNAL(triggered(bool)), this, SLOT(toggleEnabled()));
+
     reconfigure(ReconfigureAll);
 
     m_buttons[0] = new MouseButton(i18n("Left"), Qt::LeftButton);
@@ -72,7 +76,7 @@ MouseClickEffect::~MouseClickEffect()
 
 void MouseClickEffect::reconfigure(ReconfigureFlags)
 {
-    MouseClickConfig::self()->readConfig();
+    MouseClickConfig::self()->read();
     m_colors[0] = MouseClickConfig::color1();
     m_colors[1] = MouseClickConfig::color2();
     m_colors[2] = MouseClickConfig::color3();
@@ -254,6 +258,8 @@ void MouseClickEffect::drawCircle(const QColor& color, float cx, float cy, float
         drawCircleGl(color, cx, cy, r);
     if (effects->compositingType() == XRenderCompositing)
         drawCircleXr(color, cx, cy, r);
+    if (effects->compositingType() == QPainterCompositing)
+        drawCircleQPainter(color, cx, cy, r);
 }
 
 void MouseClickEffect::paintScreenSetup(int mask, QRegion region, ScreenPaintData& data)
@@ -357,11 +363,18 @@ void MouseClickEffect::drawCircleXr(const QColor& color, float cx, float cy, flo
 #endif
 }
 
+void MouseClickEffect::drawCircleQPainter(const QColor &color, float cx, float cy, float r)
+{
+    QPainter *painter = effects->scenePainter();
+    painter->save();
+    painter->setPen(color);
+    painter->drawArc(cx - r, cy - r, r * 2, r * 2, 0, 5760);
+    painter->restore();
+}
+
 void MouseClickEffect::paintScreenSetupGl(int, QRegion, ScreenPaintData&)
 {
-    if (ShaderManager::instance()->isValid()) {
-        ShaderManager::instance()->pushShader(ShaderManager::ColorShader);
-    }
+    ShaderManager::instance()->pushShader(ShaderManager::ColorShader);
 
     glLineWidth(m_lineWidth);
     glEnable(GL_BLEND);
@@ -372,9 +385,7 @@ void MouseClickEffect::paintScreenFinishGl(int, QRegion, ScreenPaintData&)
 {
     glDisable(GL_BLEND);
 
-    if (ShaderManager::instance()->isValid()) {
-        ShaderManager::instance()->popShader();
-    }
+    ShaderManager::instance()->popShader();
 }
 
 } // namespace
