@@ -19,22 +19,22 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 
-#include "config-kwin.h"
-
 #include "mousemark.h"
 
 // KConfigSkeleton
 #include "mousemarkconfig.h"
 
-#include <kaction.h>
-#include <kactioncollection.h>
+#include <QAction>
+#include <kwinconfig.h>
+#include <kwinglutils.h>
+#include <KGlobalAccel>
 #include <KLocalizedString>
 
+#include <QPainter>
+
 #include <math.h>
 
-#include <kdebug.h>
-
-#ifdef KWIN_BUILD_COMPOSITE
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
 #include <xcb/render.h>
 #endif
 
@@ -45,15 +45,21 @@ namespace KWin
 
 MouseMarkEffect::MouseMarkEffect()
 {
-    KActionCollection* actionCollection = new KActionCollection(this);
-    KAction* a = static_cast< KAction* >(actionCollection->addAction("ClearMouseMarks"));
+    QAction* a = new QAction(this);
+    a->setObjectName(QStringLiteral("ClearMouseMarks"));
     a->setText(i18n("Clear All Mouse Marks"));
-    a->setGlobalShortcut(KShortcut(Qt::SHIFT + Qt::META + Qt::Key_F11));
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::SHIFT + Qt::META + Qt::Key_F11);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::SHIFT + Qt::META + Qt::Key_F11);
+    effects->registerGlobalShortcut(Qt::SHIFT + Qt::META + Qt::Key_F11, a);
     connect(a, SIGNAL(triggered(bool)), this, SLOT(clear()));
-    a = static_cast< KAction* >(actionCollection->addAction("ClearLastMouseMark"));
+    a = new QAction(this);
+    a->setObjectName(QStringLiteral("ClearLastMouseMark"));
     a->setText(i18n("Clear Last Mouse Mark"));
-    a->setGlobalShortcut(KShortcut(Qt::SHIFT + Qt::META + Qt::Key_F12));
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::SHIFT + Qt::META + Qt::Key_F12);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::SHIFT + Qt::META + Qt::Key_F12);
+    effects->registerGlobalShortcut(Qt::SHIFT + Qt::META + Qt::Key_F12, a);
     connect(a, SIGNAL(triggered(bool)), this, SLOT(clearLast()));
+
     connect(effects, SIGNAL(mouseChanged(QPoint,QPoint,Qt::MouseButtons,Qt::MouseButtons,Qt::KeyboardModifiers,Qt::KeyboardModifiers)),
             this, SLOT(slotMouseChanged(QPoint,QPoint,Qt::MouseButtons,Qt::MouseButtons,Qt::KeyboardModifiers,Qt::KeyboardModifiers)));
     connect(effects, SIGNAL(screenLockingChanged(bool)), SLOT(screenLockingChanged(bool)));
@@ -70,14 +76,14 @@ MouseMarkEffect::~MouseMarkEffect()
 static int width_2 = 1;
 void MouseMarkEffect::reconfigure(ReconfigureFlags)
 {
-    MouseMarkConfig::self()->readConfig();
+    MouseMarkConfig::self()->read();
     width = MouseMarkConfig::lineWidth();
     width_2 = width / 2;
     color = MouseMarkConfig::color();
     color.setAlphaF(1.0);
 }
 
-#ifdef KWIN_BUILD_COMPOSITE
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
 void MouseMarkEffect::addRect(const QPoint &p1, const QPoint &p2, xcb_rectangle_t *r, xcb_render_color_t *c)
 {
     r->x = qMin(p1.x(), p2.x()) - width_2;
@@ -109,7 +115,41 @@ void MouseMarkEffect::paintScreen(int mask, QRegion region, ScreenPaintData& dat
     effects->paintScreen(mask, region, data);   // paint normal screen
     if (marks.isEmpty() && drawing.isEmpty())
         return;
-#ifdef KWIN_BUILD_COMPOSITE
+    if ( effects->isOpenGLCompositing()) {
+#ifndef KWIN_HAVE_OPENGLES
+        glEnable(GL_LINE_SMOOTH);
+#endif
+        glLineWidth(width);
+        GLVertexBuffer *vbo = GLVertexBuffer::streamingBuffer();
+        vbo->reset();
+        vbo->setUseColor(true);
+        vbo->setColor(color);
+        ShaderBinder binder(ShaderManager::ColorShader);
+        QVector<float> verts;
+        foreach (const Mark & mark, marks) {
+            verts.clear();
+            verts.reserve(mark.size() * 2);
+            foreach (const QPoint & p, mark) {
+                verts << p.x() << p.y();
+            }
+            vbo->setData(verts.size() / 2, 2, verts.data(), NULL);
+            vbo->render(GL_LINE_STRIP);
+        }
+        if (!drawing.isEmpty()) {
+            verts.clear();
+            verts.reserve(drawing.size() * 2);
+            foreach (const QPoint & p, drawing) {
+                verts << p.x() << p.y();
+            }
+            vbo->setData(verts.size() / 2, 2, verts.data(), NULL);
+            vbo->render(GL_LINE_STRIP);
+        }
+        glLineWidth(1.0);
+    #ifndef KWIN_HAVE_OPENGLES
+        glDisable(GL_LINE_SMOOTH);
+    #endif
+    }
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     if ( effects->compositingType() == XRenderCompositing) {
         xcb_render_color_t c = preMultiply(color);
         for (int i = 0; i < marks.count(); ++i) {
@@ -133,6 +173,28 @@ void MouseMarkEffect::paintScreen(int mask, QRegion region, ScreenPaintData& dat
         }
     }
 #endif
+    if (effects->compositingType() == QPainterCompositing) {
+        QPainter *painter = effects->scenePainter();
+        painter->save();
+        QPen pen(color);
+        pen.setWidth(width);
+        painter->setPen(pen);
+        foreach (const Mark &mark, marks) {
+            drawMark(painter, mark);
+        }
+        drawMark(painter, drawing);
+        painter->restore();
+    }
+}
+
+void MouseMarkEffect::drawMark(QPainter *painter, const Mark &mark)
+{
+    if (mark.count() <= 1) {
+        return;
+    }
+    for (int i = 0; i < mark.count() - 1; ++i) {
+        painter->drawLine(mark[i], mark[i+1]);
+    }
 }
 
 void MouseMarkEffect::slotMouseChanged(const QPoint& pos, const QPoint&,
@@ -223,4 +285,4 @@ bool MouseMarkEffect::isActive() const
 
 } // namespace
 
-#include "moc_mousemark.cpp"
+#include "mousemark.moc"
