@@ -22,33 +22,37 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 // KConfigSkeleton
 #include "mousemarkconfig.h"
+#include <kwineffects_interface.h>
 
-#include <kwineffects.h>
+#include <QAction>
 
 #include <KLocalizedString>
-#include <kdebug.h>
 #include <KActionCollection>
-#include <kaction.h>
-#include <KShortcutsEditor>
+#include <KAboutData>
+#include <KGlobalAccel>
+#include <KPluginFactory>
 
+#include <QDebug>
 #include <QWidget>
 
+K_PLUGIN_FACTORY_WITH_JSON(MouseMarkEffectConfigFactory,
+                           "mousemark_config.json",
+                           registerPlugin<KWin::MouseMarkEffectConfig>();)
+
 namespace KWin
 {
 
-KWIN_EFFECT_CONFIG_FACTORY
-
 MouseMarkEffectConfigForm::MouseMarkEffectConfigForm(QWidget* parent) : QWidget(parent)
 {
     setupUi(this);
 }
 
 MouseMarkEffectConfig::MouseMarkEffectConfig(QWidget* parent, const QVariantList& args) :
-    KCModule(EffectFactory::componentData(), parent, args)
+    KCModule(KAboutData::pluginData(QStringLiteral("mousemark")), parent, args)
 {
     m_ui = new MouseMarkEffectConfigForm(this);
 
-    m_ui->kcfg_LineWidth->setSuffix(ki18np(" pixel", " pixels"));
+    m_ui->kcfg_LineWidth->setSuffix(ki18np(" pixel", " pixels").toString());
 
     QVBoxLayout* layout = new QVBoxLayout(this);
 
@@ -57,17 +61,19 @@ MouseMarkEffectConfig::MouseMarkEffectConfig(QWidget* parent, const QVariantList
     addConfig(MouseMarkConfig::self(), m_ui);
 
     // Shortcut config. The shortcut belongs to the component "kwin"!
-    m_actionCollection = new KActionCollection(this, KComponentData("kwin"));
+    m_actionCollection = new KActionCollection(this, QStringLiteral("kwin"));
 
-    KAction* a = static_cast< KAction* >(m_actionCollection->addAction("ClearMouseMarks"));
+    QAction* a = m_actionCollection->addAction(QStringLiteral("ClearMouseMarks"));
     a->setText(i18n("Clear Mouse Marks"));
     a->setProperty("isConfigurationAction", true);
-    a->setGlobalShortcut(KShortcut(Qt::SHIFT + Qt::META + Qt::Key_F11));
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::SHIFT + Qt::META + Qt::Key_F11);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::SHIFT + Qt::META + Qt::Key_F11);
 
-    a = static_cast< KAction* >(m_actionCollection->addAction("ClearLastMouseMark"));
+    a = m_actionCollection->addAction(QStringLiteral("ClearLastMouseMark"));
     a->setText(i18n("Clear Last Mouse Mark"));
     a->setProperty("isConfigurationAction", true);
-    a->setGlobalShortcut(KShortcut(Qt::SHIFT + Qt::META + Qt::Key_F12));
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::SHIFT + Qt::META + Qt::Key_F12);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::SHIFT + Qt::META + Qt::Key_F12);
 
     m_ui->editor->addCollection(m_actionCollection);
 
@@ -82,15 +88,18 @@ MouseMarkEffectConfig::~MouseMarkEffectConfig()
 
 void MouseMarkEffectConfig::save()
 {
-    kDebug(1212) << "Saving config of MouseMark" ;
+    qDebug() << "Saving config of MouseMark" ;
     KCModule::save();
 
     m_actionCollection->writeSettings();
     m_ui->editor->save();   // undo() will restore to this state from now on
 
-    EffectsHandler::sendReloadMessage("mousemark");
+    OrgKdeKwinEffectsInterface interface(QStringLiteral("org.kde.kwin.Effects"),
+                                         QStringLiteral("/Effects"),
+                                         QDBusConnection::sessionBus());
+    interface.reconfigureEffect(QStringLiteral("kwin4_effect_mousemark"));
 }
 
 } // namespace
 
-#include "moc_mousemark_config.cpp"
+#include "mousemark_config.moc"
