@@ -18,19 +18,19 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 
-#include "config-kwin.h"
-
 #include "resize.h"
 // KConfigSkeleton
 #include "resizeconfig.h"
 
-#ifdef KWIN_BUILD_COMPOSITE
+#include <kwinglutils.h>
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
 #include "kwinxrenderutils.h"
 #endif
 
 #include <KColorScheme>
 
 #include <QtGui/QVector2D>
+#include <QPainter>
 
 namespace KWin
 {
@@ -81,7 +81,31 @@ void ResizeEffect::paintWindow(EffectWindow* w, int mask, QRegion region, Window
             float alpha = 0.8f;
             QColor color = KColorScheme(QPalette::Normal, KColorScheme::Selection).background().color();
 
-#ifdef KWIN_BUILD_COMPOSITE
+            if (effects->isOpenGLCompositing()) {
+                GLVertexBuffer *vbo = GLVertexBuffer::streamingBuffer();
+                vbo->reset();
+                vbo->setUseColor(true);
+                ShaderBinder binder(ShaderManager::ColorShader);
+                glEnable(GL_BLEND);
+                glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+                color.setAlphaF(alpha);
+                vbo->setColor(color);
+                QVector<float> verts;
+                verts.reserve(paintRegion.rects().count() * 12);
+                foreach (const QRect & r, paintRegion.rects()) {
+                    verts << r.x() + r.width() << r.y();
+                    verts << r.x() << r.y();
+                    verts << r.x() << r.y() + r.height();
+                    verts << r.x() << r.y() + r.height();
+                    verts << r.x() + r.width() << r.y() + r.height();
+                    verts << r.x() + r.width() << r.y();
+                }
+                vbo->setData(verts.count() / 2, 2, verts.data(), NULL);
+                vbo->render(GL_TRIANGLES);
+                glDisable(GL_BLEND);
+            }
+
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
             if (effects->compositingType() == XRenderCompositing) {
                 QVector<xcb_rectangle_t> rects;
                 foreach (const QRect & r, paintRegion.rects()) {
@@ -93,6 +117,15 @@ void ResizeEffect::paintWindow(EffectWindow* w, int mask, QRegion region, Window
                                            rects.count(), rects.constData());
             }
 #endif
+            if (effects->compositingType() == QPainterCompositing) {
+                QPainter *painter = effects->scenePainter();
+                painter->save();
+                color.setAlphaF(alpha);
+                foreach (const QRect &r, paintRegion.rects()) {
+                    painter->fillRect(r, color);
+                }
+                painter->restore();
+            }
         }
     } else {
         AnimationEffect::paintWindow(w, mask, region, data);
@@ -102,7 +135,7 @@ void ResizeEffect::paintWindow(EffectWindow* w, int mask, QRegion region, Window
 void ResizeEffect::reconfigure(ReconfigureFlags)
 {
     m_features = 0;
-    ResizeConfig::self()->readConfig();
+    ResizeConfig::self()->read();
     if (ResizeConfig::textureScale())
         m_features |= TextureScale;
     if (ResizeConfig::outline())
