@@ -21,8 +21,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "screenshot.h"
 #include <kwinglutils.h>
 #include <kwinxrenderutils.h>
-#include <KDE/KDebug>
-#include <KDE/KTemporaryFile>
+#include <QtCore/QTemporaryFile>
+#include <QtCore/QDir>
 #include <QtDBus/QDBusConnection>
 #include <QVarLengthArray>
 #include <QtGui/QPainter>
@@ -32,9 +32,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 namespace KWin
 {
 
-KWIN_EFFECT(screenshot, ScreenShotEffect)
-KWIN_EFFECT_SUPPORTED(screenshot, ScreenShotEffect::supported())
-
 bool ScreenShotEffect::supported()
 {
     return  effects->compositingType() == XRenderCompositing ||
@@ -45,14 +42,14 @@ ScreenShotEffect::ScreenShotEffect()
     : m_scheduledScreenshot(0)
 {
     connect ( effects, SIGNAL(windowClosed(KWin::EffectWindow*)), SLOT(windowClosed(KWin::EffectWindow*)) );
-    QDBusConnection::sessionBus().registerObject("/Screenshot", this, QDBusConnection::ExportScriptableContents);
-    QDBusConnection::sessionBus().registerService("org.kde.kwin.Screenshot");
+    QDBusConnection::sessionBus().registerObject(QStringLiteral("/Screenshot"), this, QDBusConnection::ExportScriptableContents);
+    QDBusConnection::sessionBus().registerService(QStringLiteral("org.kde.kwin.Screenshot"));
 }
 
 ScreenShotEffect::~ScreenShotEffect()
 {
-    QDBusConnection::sessionBus().unregisterObject("/Screenshot");
-    QDBusConnection::sessionBus().unregisterService("org.kde.kwin.Screenshot");
+    QDBusConnection::sessionBus().unregisterObject(QStringLiteral("/Screenshot"));
+    QDBusConnection::sessionBus().unregisterService(QStringLiteral("org.kde.kwin.Screenshot"));
 }
 
 #ifdef KWIN_HAVE_XRENDER_COMPOSITING
@@ -190,40 +187,20 @@ void ScreenShotEffect::setMatrix(int width, int height)
     QMatrix4x4 projection;
     QMatrix4x4 identity;
     projection.ortho(QRect(0, 0, width, height));
-    if (effects->compositingType() == OpenGL2Compositing) {
-        ShaderBinder binder(ShaderManager::GenericShader);
-        GLShader *shader = binder.shader();
-        s_origProjection = shader->getUniformMatrix4x4("projection");
-        s_origModelview = shader->getUniformMatrix4x4("modelview");
-        shader->setUniform(GLShader::ProjectionMatrix, projection);
-        shader->setUniform(GLShader::ModelViewMatrix, identity);
-    } else if (effects->compositingType() == OpenGL1Compositing) {
-#ifdef KWIN_HAVE_OPENGL_1
-        glMatrixMode(GL_PROJECTION);
-        glPushMatrix();
-        loadMatrix(projection);
-        glMatrixMode(GL_MODELVIEW);
-        glPushMatrix();
-        glLoadIdentity();
-#endif
-    }
+    ShaderBinder binder(ShaderManager::GenericShader);
+    GLShader *shader = binder.shader();
+    s_origProjection = shader->getUniformMatrix4x4("projection");
+    s_origModelview = shader->getUniformMatrix4x4("modelview");
+    shader->setUniform(GLShader::ProjectionMatrix, projection);
+    shader->setUniform(GLShader::ModelViewMatrix, identity);
 }
 
 void ScreenShotEffect::restoreMatrix()
 {
-    if (effects->compositingType() == OpenGL2Compositing) {
-        ShaderBinder binder(ShaderManager::GenericShader);
-        GLShader *shader = binder.shader();
-        shader->setUniform(GLShader::ProjectionMatrix, s_origProjection);
-        shader->setUniform(GLShader::ModelViewMatrix, s_origModelview);
-    } else if (effects->compositingType() == OpenGL1Compositing) {
-#ifdef KWIN_HAVE_OPENGL_1
-        glMatrixMode(GL_PROJECTION);
-        glPopMatrix();
-        glMatrixMode(GL_MODELVIEW);
-        glPopMatrix();
-#endif
-    }
+    ShaderBinder binder(ShaderManager::GenericShader);
+    GLShader *shader = binder.shader();
+    shader->setUniform(GLShader::ProjectionMatrix, s_origProjection);
+    shader->setUniform(GLShader::ModelViewMatrix, s_origModelview);
 }
 
 void ScreenShotEffect::screenshotWindowUnderCursor(int mask)
@@ -257,7 +234,7 @@ void ScreenShotEffect::screenshotForWindow(qulonglong winid, int mask)
 
 QString ScreenShotEffect::screenshotFullscreen()
 {
-    return blitScreenshot(QRect(0, 0, displayWidth(), displayHeight()));
+    return blitScreenshot(effects->virtualScreenGeometry());
 }
 
 QString ScreenShotEffect::screenshotScreen(int screen)
@@ -274,14 +251,14 @@ QString ScreenShotEffect::blitScreenshot(const QRect &geometry)
 {
 #ifdef KWIN_HAVE_OPENGLES
     Q_UNUSED(geometry)
-    kDebug(1212) << "Framebuffer Blit not supported";
+    qCDebug(KWINEFFECTS) << "Framebuffer Blit not supported";
     return QString();
 #else
     QImage img;
     if (effects->isOpenGLCompositing())
     {
         if (!GLRenderTarget::blitSupported()) {
-            kDebug(1212) << "Framebuffer Blit not supported";
+            qCDebug(KWINEFFECTS) << "Framebuffer Blit not supported";
             return QString();
         }
         GLTexture tex(geometry.width(), geometry.height());
@@ -304,8 +281,7 @@ QString ScreenShotEffect::blitScreenshot(const QRect &geometry)
 #endif
     }
 
-    KTemporaryFile temp;
-    temp.setSuffix(".png");
+    QTemporaryFile temp(QDir::tempPath() + QDir::separator() + QLatin1String("kwin_screenshot_XXXXXX.png"));
     temp.setAutoRemove(false);
     if (!temp.open()) {
         return QString();
