@@ -18,14 +18,15 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 
-#include "config-kwin.h"
-
 #include "showfps.h"
 
 // KConfigSkeleton
 #include "showfpsconfig.h"
 
-#ifdef KWIN_BUILD_COMPOSITE
+#include <kwinconfig.h>
+
+#include <kwinglutils.h>
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
 #include <kwinxrenderutils.h>
 #include <xcb/render.h>
 #endif
@@ -44,7 +45,7 @@ const int MAX_TIME = 100;
 ShowFpsEffect::ShowFpsEffect()
     : paints_pos(0)
     , frames_pos(0)
-    , m_noBenchmark(effects->effectFrame(false))
+    , m_noBenchmark(effects->effectFrame(EffectFrameUnstyled, false))
 {
     for (int i = 0;
             i < NUM_PAINTS;
@@ -63,18 +64,19 @@ ShowFpsEffect::ShowFpsEffect()
 
 void ShowFpsEffect::reconfigure(ReconfigureFlags)
 {
-    ShowFpsConfig::self()->readConfig();
+    ShowFpsConfig::self()->read();
     alpha = ShowFpsConfig::alpha();
     x = ShowFpsConfig::x();
     y = ShowFpsConfig::y();
+    const QSize screenSize = effects->virtualScreenSize();
     if (x == -10000)   // there's no -0 :(
-        x = displayWidth() - 2 * NUM_PAINTS - FPS_WIDTH;
+        x = screenSize.width() - 2 * NUM_PAINTS - FPS_WIDTH;
     else if (x < 0)
-        x = displayWidth() - 2 * NUM_PAINTS - FPS_WIDTH - x;
+        x = screenSize.width() - 2 * NUM_PAINTS - FPS_WIDTH - x;
     if (y == -10000)
-        y = displayHeight() - MAX_TIME;
+        y = screenSize.height() - MAX_TIME;
     else if (y < 0)
-        y = displayHeight() - MAX_TIME - y;
+        y = screenSize.height() - MAX_TIME - y;
     fps_rect = QRect(x, y, FPS_WIDTH + 2 * NUM_PAINTS, MAX_TIME);
     m_noBenchmark->setPosition(fps_rect.bottomRight() + QPoint(-6, 6));
 
@@ -93,15 +95,15 @@ void ShowFpsEffect::reconfigure(ReconfigureFlags)
         textAlign = Qt::AlignTop | Qt::AlignLeft;
         break;
     case TOP_RIGHT:
-        fpsTextRect = QRect(displayWidth() - 100, 0, 100, 100);
+        fpsTextRect = QRect(screenSize.width() - 100, 0, 100, 100);
         textAlign = Qt::AlignTop | Qt::AlignRight;
         break;
     case BOTTOM_LEFT:
-        fpsTextRect = QRect(0, displayHeight() - 100, 100, 100);
+        fpsTextRect = QRect(0, screenSize.height() - 100, 100, 100);
         textAlign = Qt::AlignBottom | Qt::AlignLeft;
         break;
     case BOTTOM_RIGHT:
-        fpsTextRect = QRect(displayWidth() - 100, displayHeight() - 100, 100, 100);
+        fpsTextRect = QRect(screenSize.width() - 100, screenSize.height() - 100, 100, 100);
         textAlign = Qt::AlignBottom | Qt::AlignRight;
         break;
     case NOWHERE:
@@ -155,16 +157,99 @@ void ShowFpsEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
             ++fps; // count all frames in the last second
     if (fps > MAX_TIME)
         fps = MAX_TIME; // keep it the same height
-#ifdef KWIN_BUILD_COMPOSITE
+    if (effects->isOpenGLCompositing()) {
+        paintGL(fps);
+        glFinish(); // make sure all rendering is done
+    }
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     if (effects->compositingType() == XRenderCompositing) {
         paintXrender(fps);
-        XSync(display(), False);   // make sure all rendering is done
+        xcb_flush(connection());   // make sure all rendering is done
     }
 #endif
+    if (effects->compositingType() == QPainterCompositing) {
+        paintQPainter(fps);
+    }
     m_noBenchmark->render(infiniteRegion(), 1.0, alpha);
 }
 
-#ifdef KWIN_BUILD_COMPOSITE
+void ShowFpsEffect::paintGL(int fps)
+{
+    int x = this->x;
+    int y = this->y;
+    glEnable(GL_BLEND);
+    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    // TODO painting first the background white and then the contents
+    // means that the contents also blend with the background, I guess
+    ShaderBinder binder(ShaderManager::ColorShader);
+    GLVertexBuffer *vbo = GLVertexBuffer::streamingBuffer();
+    vbo->reset();
+    QColor color(255, 255, 255);
+    color.setAlphaF(alpha);
+    vbo->setColor(color);
+    QVector<float> verts;
+    verts.reserve(12);
+    verts << x + 2 * NUM_PAINTS + FPS_WIDTH << y;
+    verts << x << y;
+    verts << x << y + MAX_TIME;
+    verts << x << y + MAX_TIME;
+    verts << x + 2 * NUM_PAINTS + FPS_WIDTH << y + MAX_TIME;
+    verts << x + 2 * NUM_PAINTS + FPS_WIDTH << y;
+    vbo->setData(6, 2, verts.constData(), NULL);
+    vbo->render(GL_TRIANGLES);
+    y += MAX_TIME; // paint up from the bottom
+    color.setRed(0);
+    color.setGreen(0);
+    vbo->setColor(color);
+    verts.clear();
+    verts << x + FPS_WIDTH << y - fps;
+    verts << x << y - fps;
+    verts << x << y;
+    verts << x << y;
+    verts << x + FPS_WIDTH << y;
+    verts << x + FPS_WIDTH << y - fps;
+    vbo->setData(6, 2, verts.constData(), NULL);
+    vbo->render(GL_TRIANGLES);
+
+
+    color.setBlue(0);
+    vbo->setColor(color);
+    QVector<float> vertices;
+    for (int i = 10;
+            i < MAX_TIME;
+            i += 10) {
+        vertices << x << y - i;
+        vertices << x + FPS_WIDTH << y - i;
+    }
+    vbo->setData(vertices.size() / 2, 2, vertices.constData(), NULL);
+    vbo->render(GL_LINES);
+    x += FPS_WIDTH;
+
+    // Paint FPS graph
+    paintFPSGraph(x, y);
+    x += NUM_PAINTS;
+
+    // Paint amount of rendered pixels graph
+    paintDrawSizeGraph(x, y);
+
+    // Paint FPS numerical value
+    if (fpsTextRect.isValid()) {
+        fpsText.reset(new GLTexture(fpsTextImage(fps)));
+        fpsText->bind();
+        ShaderBinder binder(ShaderManager::SimpleShader);
+        if (effects->compositingType() == OpenGL2Compositing) {
+            binder.shader()->setUniform("offset", QVector2D(0, 0));
+        }
+        fpsText->render(QRegion(fpsTextRect), fpsTextRect);
+        fpsText->unbind();
+        effects->addRepaint(fpsTextRect);
+    }
+
+    // Paint paint sizes
+    glDisable(GL_BLEND);
+}
+
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
 /*
  Differences between OpenGL and XRender:
  - differently specified rectangles (X: width/height, O: x2,y2)
@@ -222,6 +307,39 @@ void ShowFpsEffect::paintXrender(int fps)
 }
 #endif
 
+void ShowFpsEffect::paintQPainter(int fps)
+{
+    QPainter *painter = effects->scenePainter();
+    painter->save();
+
+    QColor color(255, 255, 255);
+    color.setAlphaF(alpha);
+
+    painter->setCompositionMode(QPainter::CompositionMode_SourceOver);
+    painter->fillRect(x, y, 2 * NUM_PAINTS + FPS_WIDTH, MAX_TIME, color);
+    color.setRed(0);
+    color.setGreen(0);
+    painter->fillRect(x, y + MAX_TIME - fps, FPS_WIDTH, fps, color);
+
+    color.setBlue(0);
+    for (int i = 10; i < MAX_TIME; i += 10) {
+        painter->setPen(color);
+        painter->drawLine(x, y + MAX_TIME - i, x + FPS_WIDTH, y + MAX_TIME - i);
+    }
+
+    // Paint FPS graph
+    paintFPSGraph(x + FPS_WIDTH, y + MAX_TIME - 1);
+
+    // Paint amount of rendered pixels graph
+    paintDrawSizeGraph(x + FPS_WIDTH + NUM_PAINTS, y + MAX_TIME - 1);
+
+    // Paint FPS numerical value
+    painter->setPen(Qt::black);
+    painter->drawText(fpsTextRect, textAlign, QString::number(fps));
+
+    painter->restore();
+}
+
 void ShowFpsEffect::paintFPSGraph(int x, int y)
 {
     // Paint FPS graph
@@ -270,7 +388,52 @@ void ShowFpsEffect::paintDrawSizeGraph(int x, int y)
 
 void ShowFpsEffect::paintGraph(int x, int y, QList<int> values, QList<int> lines, bool colorize)
 {
-#ifdef KWIN_BUILD_COMPOSITE
+    if (effects->isOpenGLCompositing()) {
+        QColor color(0, 0, 0);
+        color.setAlphaF(alpha);
+        GLVertexBuffer *vbo = GLVertexBuffer::streamingBuffer();
+        vbo->reset();
+        vbo->setColor(color);
+        QVector<float> verts;
+        // First draw the lines
+        foreach (int h, lines) {
+            verts << x << y - h;
+            verts << x + values.count() << y - h;
+        }
+        vbo->setData(verts.size() / 2, 2, verts.constData(), NULL);
+        vbo->render(GL_LINES);
+        // Then the graph values
+        int lastValue = 0;
+        verts.clear();
+        for (int i = 0; i < values.count(); i++) {
+            int value = values[ i ];
+            if (colorize && value != lastValue) {
+                if (!verts.isEmpty()) {
+                    vbo->setData(verts.size() / 2, 2, verts.constData(), NULL);
+                    vbo->render(GL_LINES);
+                }
+                verts.clear();
+                if (value <= 10) {
+                    color = QColor(0, 255, 0);
+                } else if (value <= 20) {
+                    color = QColor(255, 255, 0);
+                } else if (value <= 50) {
+                    color = QColor(255, 0, 0);
+                } else {
+                    color = QColor(0, 0, 0);
+                }
+                vbo->setColor(color);
+            }
+            verts << x + values.count() - i << y;
+            verts << x + values.count() - i << y - value;
+            lastValue = value;
+        }
+        if (!verts.isEmpty()) {
+            vbo->setData(verts.size() / 2, 2, verts.constData(), NULL);
+            vbo->render(GL_LINES);
+        }
+    }
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     if (effects->compositingType() == XRenderCompositing) {
         xcb_pixmap_t pixmap = xcb_generate_id(connection());
         xcb_create_pixmap(connection(), 32, pixmap, rootWindow(), values.count(), MAX_TIME);
@@ -329,6 +492,32 @@ void ShowFpsEffect::paintGraph(int x, int y, QList<int> values, QList<int> lines
                              XCB_RENDER_PICTURE_NONE, effects->xrenderBufferPicture(), 0, 0, 0, 0, x, y, values.count(), MAX_TIME);
     }
 #endif
+    if (effects->compositingType() == QPainterCompositing) {
+        QPainter *painter = effects->scenePainter();
+        painter->setPen(Qt::black);
+        // First draw the lines
+        foreach (int h, lines) {
+            painter->drawLine(x, y - h, x + values.count(), y - h);
+        }
+        QColor color(0, 0, 0);
+        color.setAlphaF(alpha);
+        for (int i = 0; i < values.count(); i++) {
+            int value = values[ i ];
+            if (colorize) {
+                if (value <= 10) {
+                    color = QColor(0, 255, 0);
+                } else if (value <= 20) {
+                    color = QColor(255, 255, 0);
+                } else if (value <= 50) {
+                    color = QColor(255, 0, 0);
+                } else {
+                    color = QColor(0, 0, 0);
+                }
+            }
+            painter->setPen(color);
+            painter->drawLine(x + values.count() - i, y, x + values.count() - i, y - value);
+        }
+    }
 }
 
 void ShowFpsEffect::postPaintScreen()
