@@ -20,16 +20,16 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "snaphelper.h"
 
-#include "kwinglutils.h"
+#include <kwinglutils.h>
 #ifdef KWIN_HAVE_XRENDER_COMPOSITING
-#include "kwinxrenderutils.h"
+#include <kwinxrenderutils.h>
+#include <xcb/render.h>
 #endif
+#include <QPainter>
 
 namespace KWin
 {
 
-KWIN_EFFECT(snaphelper, SnapHelperEffect)
-
 SnapHelperEffect::SnapHelperEffect()
     : m_active(false)
     , m_window(NULL)
@@ -39,21 +39,11 @@ SnapHelperEffect::SnapHelperEffect()
     connect(effects, SIGNAL(windowClosed(KWin::EffectWindow*)), this, SLOT(slotWindowClosed(KWin::EffectWindow*)));
     connect(effects, SIGNAL(windowStartUserMovedResized(KWin::EffectWindow*)), this, SLOT(slotWindowStartUserMovedResized(KWin::EffectWindow*)));
     connect(effects, SIGNAL(windowFinishUserMovedResized(KWin::EffectWindow*)), this, SLOT(slotWindowFinishUserMovedResized(KWin::EffectWindow*)));
-    connect(effects, SIGNAL(windowGeometryShapeChanged(KWin::EffectWindow*, const QRect&)), this, SLOT(slotWindowResized(KWin::EffectWindow*, const QRect&)));
-
-    /*if ( effects->compositingType() == XRenderCompositing )
-        {
-        XGCValues gcattr;
-        // TODO: Foreground color
-        gcattr.line_width = 4;
-        m_gc = XCreateGC( display(), rootWindow(), GCLineWidth, &gcattr );
-        }*/
+    connect(effects, SIGNAL(windowGeometryShapeChanged(KWin::EffectWindow*,QRect)), this, SLOT(slotWindowResized(KWin::EffectWindow*,QRect)));
 }
 
 SnapHelperEffect::~SnapHelperEffect()
 {
-    //if ( effects->compositingType() == XRenderCompositing )
-    //    XFreeGC( display(), m_gc );
 }
 
 void SnapHelperEffect::reconfigure(ReconfigureFlags)
@@ -134,7 +124,7 @@ void SnapHelperEffect::postPaintScreen()
                 int halfWidth = m_window->width() / 2;
                 int halfHeight = m_window->height() / 2;
 
-                XRectangle rects[6];
+                xcb_rectangle_t rects[6];
                 // Center lines
                 rects[0].x = rect.x() + rect.width() / 2 - 2;
                 rects[0].y = rect.y();
@@ -164,11 +154,36 @@ void SnapHelperEffect::postPaintScreen()
                 rects[5].width = 4;
                 rects[5].height = 2*halfHeight - 4;
 
-                XRenderColor c = preMultiply(QColor(128, 128, 128, m_timeline.currentValue()*128));
-                XRenderFillRectangles(display(), PictOpOver, effects->xrenderBufferPicture(), &c, rects, 6);
+                xcb_render_fill_rectangles(connection(), XCB_RENDER_PICT_OP_OVER, effects->xrenderBufferPicture(),
+                                           preMultiply(QColor(128, 128, 128, m_timeline.currentValue()*128)), 6, rects);
             }
 #endif
         }
+        if (effects->compositingType() == QPainterCompositing) {
+            QPainter *painter = effects->scenePainter();
+            painter->save();
+            QColor color;
+            color.setRedF(0.5);
+            color.setGreenF(0.5);
+            color.setBlueF(0.5);
+            color.setAlphaF(m_timeline.currentValue() * 0.5);
+            QPen pen(color);
+            pen.setWidth(4);
+            painter->setPen(pen);
+            painter->setBrush(Qt::NoBrush);
+
+            for (int i = 0; i < effects->numScreens(); ++i) {
+                const QRect &rect = effects->clientArea(ScreenArea, i, 0);
+                // Center lines
+                painter->drawLine(rect.center().x(), rect.y(), rect.center().x(), rect.y() + rect.height());
+                painter->drawLine(rect.x(), rect.center().y(), rect.x() + rect.width(), rect.center().y());
+
+                // window outline
+                QRect windowRect(rect.center(), m_window->geometry().size());
+                painter->drawRect(windowRect.translated(-windowRect.width()/2, -windowRect.height()/2));
+            }
+            painter->restore();
+        }
     } else if (m_window && !m_active) {
         if (m_window->isDeleted())
             m_window->unrefWindow();
