@@ -21,31 +21,34 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "thumbnailaside_config.h"
 // KConfigSkeleton
 #include "thumbnailasideconfig.h"
+#include <kwineffects_interface.h>
 
-#include <kwineffects.h>
+#include <QAction>
 
 #include <KLocalizedString>
-#include <kdebug.h>
 #include <kconfiggroup.h>
 #include <KActionCollection>
-#include <kaction.h>
-#include <KShortcutsEditor>
+#include <KAboutData>
+#include <KGlobalAccel>
+#include <KPluginFactory>
 
 #include <QWidget>
 #include <QVBoxLayout>
 
+K_PLUGIN_FACTORY_WITH_JSON(ThumbnailAsideEffectConfigFactory,
+                           "thumbnailaside_config.json",
+                           registerPlugin<KWin::ThumbnailAsideEffectConfig>();)
+
 namespace KWin
 {
 
-KWIN_EFFECT_CONFIG_FACTORY
-
 ThumbnailAsideEffectConfigForm::ThumbnailAsideEffectConfigForm(QWidget* parent) : QWidget(parent)
 {
     setupUi(this);
 }
 
 ThumbnailAsideEffectConfig::ThumbnailAsideEffectConfig(QWidget* parent, const QVariantList& args) :
-    KCModule(EffectFactory::componentData(), parent, args)
+    KCModule(KAboutData::pluginData(QStringLiteral("thumbnailaside")), parent, args)
 {
     m_ui = new ThumbnailAsideEffectConfigForm(this);
 
@@ -58,15 +61,16 @@ ThumbnailAsideEffectConfig::ThumbnailAsideEffectConfig(QWidget* parent, const QV
     addConfig(ThumbnailAsideConfig::self(), this);
 
     // Shortcut config. The shortcut belongs to the component "kwin"!
-    m_actionCollection = new KActionCollection(this, KComponentData("kwin"));
+    m_actionCollection = new KActionCollection(this, QStringLiteral("kwin"));
 
-    m_actionCollection->setConfigGroup("ThumbnailAside");
+    m_actionCollection->setConfigGroup(QStringLiteral("ThumbnailAside"));
     m_actionCollection->setConfigGlobal(true);
 
-    KAction* a = (KAction*)m_actionCollection->addAction("ToggleCurrentThumbnail");
+    QAction* a = m_actionCollection->addAction(QStringLiteral("ToggleCurrentThumbnail"));
     a->setText(i18n("Toggle Thumbnail for Current Window"));
     a->setProperty("isConfigurationAction", true);
-    a->setGlobalShortcut(KShortcut(Qt::META + Qt::CTRL + Qt::Key_T));
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::META + Qt::CTRL + Qt::Key_T);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::META + Qt::CTRL + Qt::Key_T);
 
     m_ui->editor->addCollection(m_actionCollection);
 
@@ -82,9 +86,12 @@ ThumbnailAsideEffectConfig::~ThumbnailAsideEffectConfig()
 void ThumbnailAsideEffectConfig::save()
 {
     KCModule::save();
-    EffectsHandler::sendReloadMessage("thumbnailaside");
+    OrgKdeKwinEffectsInterface interface(QStringLiteral("org.kde.kwin.Effects"),
+                                         QStringLiteral("/Effects"),
+                                         QDBusConnection::sessionBus());
+    interface.reconfigureEffect(QStringLiteral("kwin4_effect_thumbnailaside"));
 }
 
 } // namespace
 
-#include "moc_thumbnailaside_config.cpp"
+#include "thumbnailaside_config.moc"
