@@ -19,24 +19,22 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 
-#include <config-kwin.h>
-
 #include "trackmouse.h"
 
 // KConfigSkeleton
 #include "trackmouseconfig.h"
 
-#include <kwinxrenderutils.h>
-#include <kglobal.h>
-#include <kstandarddirs.h>
-#include <kaction.h>
-#include <kactioncollection.h>
-#include <KLocalizedString>
-#include <kdebug.h>
+#include <QAction>
+#include <QPainter>
+#include <QTime>
+#include <QMatrix4x4>
 
-#ifdef KWIN_BUILD_COMPOSITE
-#include <xcbutils.h>
-#endif
+#include <kwinconfig.h>
+#include <kwinglutils.h>
+#include <kwinxrenderutils.h>
+
+#include <KGlobalAccel>
+#include <KLocalizedString>
 
 #include <math.h>
 
@@ -47,18 +45,25 @@ TrackMouseEffect::TrackMouseEffect()
     : m_active(false)
     , m_angle(0)
 {
-#ifdef KWIN_BUILD_COMPOSITE
+    m_texture[0] = m_texture[1] = 0;
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     m_picture[0] = m_picture[1] = 0;
     if ( effects->compositingType() == XRenderCompositing)
-        m_angleBase = M_PI_2;
+        m_angleBase = 1.57079632679489661923; // Pi/2
 #endif
+    if ( effects->isOpenGLCompositing() || effects->compositingType() == QPainterCompositing)
+        m_angleBase = 90.0;
     m_mousePolling = false;
-    KActionCollection *actionCollection = new KActionCollection(this);
-    m_action = static_cast< KAction* >(actionCollection->addAction("TrackMouse"));
+
+    m_action = new QAction(this);
+    m_action->setObjectName(QStringLiteral("TrackMouse"));
     m_action->setText(i18n("Track mouse"));
-    m_action->setGlobalShortcut(KShortcut());
+    KGlobalAccel::self()->setDefaultShortcut(m_action, QList<QKeySequence>());
+    KGlobalAccel::self()->setShortcut(m_action, QList<QKeySequence>());
+    effects->registerGlobalShortcut(QKeySequence(), m_action);
 
     connect(m_action, SIGNAL(triggered(bool)), this, SLOT(toggle()));
+
     connect(effects, SIGNAL(mouseChanged(QPoint,QPoint,Qt::MouseButtons,Qt::MouseButtons,Qt::KeyboardModifiers,Qt::KeyboardModifiers)),
                      SLOT(slotMouseChanged(QPoint,QPoint,Qt::MouseButtons,Qt::MouseButtons,Qt::KeyboardModifiers,Qt::KeyboardModifiers)));
     reconfigure(ReconfigureAll);
@@ -69,7 +74,8 @@ TrackMouseEffect::~TrackMouseEffect()
     if (m_mousePolling)
         effects->stopMousePolling();
     for (int i = 0; i < 2; ++i) {
-#ifdef KWIN_BUILD_COMPOSITE
+        delete m_texture[i]; m_texture[i] = 0;
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
         delete m_picture[i]; m_picture[i] = 0;
 #endif
     }
@@ -78,7 +84,7 @@ TrackMouseEffect::~TrackMouseEffect()
 void TrackMouseEffect::reconfigure(ReconfigureFlags)
 {
     m_modifiers = 0;
-    TrackMouseConfig::self()->readConfig();
+    TrackMouseConfig::self()->read();
     if (TrackMouseConfig::shift())
         m_modifiers |= Qt::ShiftModifier;
     if (TrackMouseConfig::alt())
@@ -116,7 +122,33 @@ void TrackMouseEffect::paintScreen(int mask, QRegion region, ScreenPaintData& da
     if (!m_active)
         return;
 
-#ifdef KWIN_BUILD_COMPOSITE
+    if ( effects->isOpenGLCompositing() && m_texture[0] && m_texture[1]) {
+        ShaderBinder binder(ShaderManager::GenericShader);
+        GLShader *shader(binder.shader());
+        if (!shader) {
+            return;
+        }
+        QMatrix4x4 modelview;
+        modelview = shader->getUniformMatrix4x4("modelview");
+        glEnable(GL_BLEND);
+        glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
+        QMatrix4x4 matrix(modelview);
+        const QPointF p = m_lastRect[0].topLeft() + QPoint(m_lastRect[0].width()/2.0, m_lastRect[0].height()/2.0);
+        for (int i = 0; i < 2; ++i) {
+            matrix.translate(p.x(), p.y(), 0.0);
+            matrix.rotate(i ? -2*m_angle : m_angle, 0, 0, 1.0);
+            matrix.translate(-p.x(), -p.y(), 0.0);
+            shader->setUniform(GLShader::ModelViewMatrix, matrix);
+            shader->setUniform(GLShader::Saturation, 1.0);
+            shader->setUniform(GLShader::ModulationConstant, QVector4D(1.0, 1.0, 1.0, 1.0));
+            m_texture[i]->bind();
+            m_texture[i]->render(region, m_lastRect[i]);
+            m_texture[i]->unbind();
+        }
+        glDisable(GL_BLEND);
+        shader->setUniform(GLShader::ModelViewMatrix, modelview);
+    }
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     if ( effects->compositingType() == XRenderCompositing && m_picture[0] && m_picture[1]) {
         float sine = sin(m_angle);
         const float cosine = cos(m_angle);
@@ -125,22 +157,34 @@ void TrackMouseEffect::paintScreen(int mask, QRegion region, ScreenPaintData& da
             const float dx = m_size[i].width()/2.0;
             const float dy = m_size[i].height()/2.0;
             const xcb_render_picture_t picture = *m_picture[i];
+#define DOUBLE_TO_FIXED(d) ((xcb_render_fixed_t) ((d) * 65536))
             xcb_render_transform_t xform = {
-                KWIN_DOUBLE_TO_FIXED( cosine ), KWIN_DOUBLE_TO_FIXED( -sine ), KWIN_DOUBLE_TO_FIXED( dx - cosine*dx + sine*dy ),
-                KWIN_DOUBLE_TO_FIXED( sine ), KWIN_DOUBLE_TO_FIXED( cosine ), KWIN_DOUBLE_TO_FIXED( dy - sine*dx - cosine*dy ),
-                KWIN_DOUBLE_TO_FIXED( 0.0 ), KWIN_DOUBLE_TO_FIXED( 0.0 ), KWIN_DOUBLE_TO_FIXED( 1.0 )
+                DOUBLE_TO_FIXED( cosine ), DOUBLE_TO_FIXED( -sine ), DOUBLE_TO_FIXED( dx - cosine*dx + sine*dy ),
+                DOUBLE_TO_FIXED( sine ), DOUBLE_TO_FIXED( cosine ), DOUBLE_TO_FIXED( dy - sine*dx - cosine*dy ),
+                DOUBLE_TO_FIXED( 0.0 ), DOUBLE_TO_FIXED( 0.0 ), DOUBLE_TO_FIXED( 1.0 )
             };
+#undef DOUBLE_TO_FIXED
             xcb_render_set_picture_transform(connection(), picture, xform);
             xcb_render_set_picture_filter(connection(), picture, 8, "bilinear", 0, NULL);
             const QRect &rect = m_lastRect[i];
             xcb_render_composite(connection(), XCB_RENDER_PICT_OP_OVER, picture, XCB_RENDER_PICTURE_NONE,
                                  effects->xrenderBufferPicture(), 0, 0, 0, 0,
-                                 qRound((rect.x()+rect.width()/2.0)*data.xScale() - rect.width()/2.0 + data.xTranslation()),
-                                 qRound((rect.y()+rect.height()/2.0)*data.yScale() - rect.height()/2.0 + data.yTranslation()),
-                                 rect.width(), rect.height());
+                                 rect.x(), rect.y(), rect.width(), rect.height());
         }
     }
 #endif
+    if (effects->compositingType() == QPainterCompositing && !m_image[0].isNull() && !m_image[1].isNull()) {
+        QPainter *painter = effects->scenePainter();
+        const QPointF p = m_lastRect[0].topLeft() + QPoint(m_lastRect[0].width()/2.0, m_lastRect[0].height()/2.0);
+        for (int i = 0; i < 2; ++i) {
+            painter->save();
+            painter->translate(p.x(), p.y());
+            painter->rotate(i ? -2*m_angle : m_angle);
+            painter->translate(-p.x(), -p.y());
+            painter->drawImage(m_lastRect[i], m_image[i]);
+            painter->restore();
+        }
+    }
 }
 
 void TrackMouseEffect::postPaintScreen()
@@ -153,15 +197,19 @@ void TrackMouseEffect::postPaintScreen()
 
 bool TrackMouseEffect::init()
 {
-#ifdef KWIN_BUILD_COMPOSITE
-    if (!m_picture[0]) {
+    effects->makeOpenGLContextCurrent();
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
+    if (!(m_texture[0] || m_picture[0] || m_image[0].isNull())) {
         loadTexture();
-        if (!m_picture[0])
+        if (!(m_texture[0] || m_picture[0] || m_image[0].isNull()))
             return false;
     }
 #else
-    loadTexture();
-    return false;
+    if (!m_texture[0] || m_image[0].isNull()) {
+        loadTexture();
+        if (!m_texture[0] || m_image[0].isNull())
+            return false;
+    }
 #endif
     m_lastRect[0].moveCenter(cursorPos());
     m_lastRect[1].moveCenter(cursorPos());
@@ -202,20 +250,29 @@ void TrackMouseEffect::slotMouseChanged(const QPoint&, const QPoint&,
 
 void TrackMouseEffect::loadTexture()
 {
-    QString f[2] = {KGlobal::dirs()->findResource("appdata", "tm_outer.png"),
-                    KGlobal::dirs()->findResource("appdata", "tm_inner.png")};
+    QString f[2] = {QStandardPaths::locate(QStandardPaths::DataLocation, QStringLiteral("tm_outer.png")),
+                    QStandardPaths::locate(QStandardPaths::DataLocation, QStringLiteral("tm_inner.png"))};
     if (f[0].isEmpty() || f[1].isEmpty())
         return;
 
     for (int i = 0; i < 2; ++i) {
-#ifdef KWIN_BUILD_COMPOSITE
+        if ( effects->isOpenGLCompositing()) {
+            QImage img(f[i]);
+            m_texture[i] = new GLTexture(img);
+            m_lastRect[i].setSize(img.size());
+        }
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
         if ( effects->compositingType() == XRenderCompositing) {
-            QImage image(f[i]);
-            m_picture[i] = new XRenderPicture(image);
-            m_size[i] = image.size();
-            m_lastRect[i].setSize(image.size());
+            QImage pixmap(f[i]);
+            m_picture[i] = new XRenderPicture(pixmap);
+            m_size[i] = pixmap.size();
+            m_lastRect[i].setSize(pixmap.size());
         }
 #endif
+        if (effects->compositingType() == QPainterCompositing) {
+            m_image[i] = QImage(f[i]);
+            m_lastRect[i].setSize(m_image[i].size());
+        }
     }
 }
 
