@@ -19,13 +19,14 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 
-#include <kwineffects.h>
+#include <QAction>
+#include <kwineffects_interface.h>
 
 #include <KLocalizedString>
-#include <kdebug.h>
-#include <kaction.h>
 #include <KActionCollection>
-#include <KShortcutsEditor>
+#include <KAboutData>
+#include <KGlobalAccel>
+#include <KPluginFactory>
 
 #include <QVBoxLayout>
 #include <QLabel>
@@ -35,18 +36,20 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // KConfigSkeleton
 #include "trackmouseconfig.h"
 
+K_PLUGIN_FACTORY_WITH_JSON(TrackMouseEffectConfigFactory,
+                           "trackmouse_config.json",
+                           registerPlugin<KWin::TrackMouseEffectConfig>();)
+
 namespace KWin
 {
 
-KWIN_EFFECT_CONFIG_FACTORY
-
 TrackMouseEffectConfigForm::TrackMouseEffectConfigForm(QWidget* parent) : QWidget(parent)
 {
     setupUi(this);
 }
 
 TrackMouseEffectConfig::TrackMouseEffectConfig(QWidget* parent, const QVariantList& args) :
-    KCModule(EffectFactory::componentData(), parent, args)
+    KCModule(KAboutData::pluginData(QStringLiteral("trackmouse")), parent, args)
 {
 
     m_ui = new TrackMouseEffectConfigForm(this);
@@ -55,14 +58,17 @@ TrackMouseEffectConfig::TrackMouseEffectConfig(QWidget* parent, const QVariantLi
 
     addConfig(TrackMouseConfig::self(), m_ui);
 
-    m_actionCollection = new KActionCollection(this, KComponentData("kwin"));
-    m_actionCollection->setConfigGroup("TrackMouse");
+    m_actionCollection = new KActionCollection(this, QStringLiteral("kwin"));
+    m_actionCollection->setConfigGroup(QStringLiteral("TrackMouse"));
     m_actionCollection->setConfigGlobal(true);
 
-    KAction *a = static_cast< KAction* >(m_actionCollection->addAction("TrackMouse"));
+    QAction *a = m_actionCollection->addAction(QStringLiteral("TrackMouse"));
     a->setText(i18n("Track mouse"));
     a->setProperty("isConfigurationAction", true);
-    a->setGlobalShortcut(KShortcut());
+
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>());
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>());
+
     connect(m_ui->shortcut, SIGNAL(keySequenceChanged(QKeySequence)),
                             SLOT(shortcutChanged(QKeySequence)));
 
@@ -84,8 +90,6 @@ void TrackMouseEffectConfig::checkModifiers()
 void TrackMouseEffectConfig::load()
 {
     KCModule::load();
-    if (KAction *a = qobject_cast<KAction*>(m_actionCollection->action("TrackMouse")))
-        m_ui->shortcut->setKeySequence(a->globalShortcut().primary());
 
     checkModifiers();
     emit changed(false);
@@ -95,7 +99,10 @@ void TrackMouseEffectConfig::save()
 {
     KCModule::save();
     m_actionCollection->writeSettings();
-    EffectsHandler::sendReloadMessage("trackmouse");
+    OrgKdeKwinEffectsInterface interface(QStringLiteral("org.kde.kwin.Effects"),
+                                         QStringLiteral("/Effects"),
+                                         QDBusConnection::sessionBus());
+    interface.reconfigureEffect(QStringLiteral("kwin4_effect_trackmouse"));
 }
 
 void TrackMouseEffectConfig::defaults()
@@ -107,12 +114,13 @@ void TrackMouseEffectConfig::defaults()
 
 void TrackMouseEffectConfig::shortcutChanged(const QKeySequence &seq)
 {
-    if (KAction *a = qobject_cast<KAction*>(m_actionCollection->action("TrackMouse")))
-        a->setGlobalShortcut(KShortcut(seq), KAction::ActiveShortcut, KAction::NoAutoloading);
+    if (QAction *a = m_actionCollection->action(QStringLiteral("TrackMouse"))) {
+        KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << seq, KGlobalAccel::NoAutoloading);
+    }
 //     m_actionCollection->writeSettings();
     emit changed(true);
 }
 
 } // namespace
 
-#include "moc_trackmouse_config.cpp"
+#include "trackmouse_config.moc"
