@@ -21,16 +21,22 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "windowgeometry_config.h"
 // KConfigSkeleton
 #include "windowgeometryconfig.h"
+#include <kwineffects_interface.h>
 
-#include <kwineffects.h>
+#include <QAction>
 #include <KActionCollection>
-#include <kaction.h>
+#include <KGlobalAccel>
 #include <KLocalizedString>
 #include <kconfiggroup.h>
+#include <KAboutData>
+#include <KPluginFactory>
+
+K_PLUGIN_FACTORY_WITH_JSON(WindowGeometryEffectConfigFactory,
+                           "windowgeometry_config.json",
+                           registerPlugin<KWin::WindowGeometryConfig>();)
 
 namespace KWin
 {
-KWIN_EFFECT_CONFIG_FACTORY
 
 WindowGeometryConfigForm::WindowGeometryConfigForm(QWidget* parent) : QWidget(parent)
 {
@@ -38,19 +44,21 @@ WindowGeometryConfigForm::WindowGeometryConfigForm(QWidget* parent) : QWidget(pa
 }
 
 WindowGeometryConfig::WindowGeometryConfig(QWidget* parent, const QVariantList& args)
-    : KCModule(KWin::EffectFactory::componentData(), parent, args)
+    : KCModule(KAboutData::pluginData(QStringLiteral("windowgeometry")), parent, args)
 {
     QVBoxLayout* layout = new QVBoxLayout(this);
     layout->addWidget(myUi = new WindowGeometryConfigForm(this));
 
     // Shortcut config. The shortcut belongs to the component "kwin"!
-    myActionCollection = new KActionCollection(this, KComponentData("kwin"));
-    KAction* a = (KAction*)myActionCollection->addAction("WindowGeometry");
+    myActionCollection = new KActionCollection(this, QStringLiteral("kwin"));
+    QAction* a = myActionCollection->addAction(QStringLiteral("WindowGeometry"));
     a->setText(i18n("Toggle KWin composited geometry display"));
     a->setProperty("isConfigurationAction", true);
-    a->setGlobalShortcut(KShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_F11));
 
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::CTRL + Qt::SHIFT + Qt::Key_F11);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::CTRL + Qt::SHIFT + Qt::Key_F11);
     myUi->shortcuts->addCollection(myActionCollection);
+
     connect(myUi->shortcuts,    SIGNAL(keyChange()), this, SLOT(changed()));
 
     addConfig(WindowGeometryConfiguration::self(), myUi);
@@ -68,7 +76,10 @@ void WindowGeometryConfig::save()
 {
     KCModule::save();
     myUi->shortcuts->save();   // undo() will restore to this state from now on
-    EffectsHandler::sendReloadMessage("windowgeometry");
+    OrgKdeKwinEffectsInterface interface(QStringLiteral("org.kde.kwin.Effects"),
+                                         QStringLiteral("/Effects"),
+                                         QDBusConnection::sessionBus());
+    interface.reconfigureEffect(QStringLiteral("kwin4_effect_windowgeometry"));
 }
 
 void WindowGeometryConfig::defaults()
@@ -78,4 +89,4 @@ void WindowGeometryConfig::defaults()
 }
 
 } //namespace
-#include "moc_windowgeometry_config.cpp"
+#include "windowgeometry_config.moc"
