@@ -22,19 +22,20 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "wobblywindows_config.h"
 // KConfigSkeleton
 #include "wobblywindowsconfig.h"
+#include <kwineffects_interface.h>
 
-#include <kwineffects.h>
-
-#include <klocale.h>
-#include <kdebug.h>
-#include <kaction.h>
+#include <KLocalizedString>
+#include <KAboutData>
 #include <kconfiggroup.h>
+#include <KPluginFactory>
+
+K_PLUGIN_FACTORY_WITH_JSON(WobblyWindowsEffectConfigFactory,
+                           "wobblywindows_config.json",
+                           registerPlugin<KWin::WobblyWindowsEffectConfig>();)
 
 namespace KWin
 {
 
-KWIN_EFFECT_CONFIG_FACTORY
-
 //-----------------------------------------------------------------------------
 // WARNING: This is (kinda) copied from wobblywindows.cpp
 
@@ -79,7 +80,7 @@ ParameterSet pset[5] = { set_0, set_1, set_2, set_3, set_4 };
 //-----------------------------------------------------------------------------
 
 WobblyWindowsEffectConfig::WobblyWindowsEffectConfig(QWidget* parent, const QVariantList& args) :
-    KCModule(EffectFactory::componentData(), parent, args)
+    KCModule(KAboutData::pluginData(QStringLiteral("wobblywindows")), parent, args)
 {
     m_ui.setupUi(this);
 
@@ -97,7 +98,10 @@ void WobblyWindowsEffectConfig::save()
 {
     KCModule::save();
 
-    EffectsHandler::sendReloadMessage("wobblywindows");
+    OrgKdeKwinEffectsInterface interface(QStringLiteral("org.kde.kwin.Effects"),
+                                         QStringLiteral("/Effects"),
+                                         QDBusConnection::sessionBus());
+    interface.reconfigureEffect(QStringLiteral("kwin4_effect_wobblywindows"));
 }
 
 void WobblyWindowsEffectConfig::wobblinessChanged()
