@@ -19,25 +19,23 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 
-#include "config-kwin.h"
-
 #include "zoom.h"
 // KConfigSkeleton
 #include "zoomconfig.h"
 
+#include <QAction>
 #include <QApplication>
 #include <QStyle>
-#include <QVector2D>
-#include <kaction.h>
-#include <kactioncollection.h>
+#include <QtGui/QVector2D>
+#include <QtDBus/QDBusConnection>
 #include <kstandardaction.h>
 #include <KConfigGroup>
+#include <KGlobalAccel>
 #include <KLocalizedString>
-#include <KDebug>
 
-#ifdef KWIN_BUILD_COMPOSITE
+#include <kwinglutils.h>
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
 #include <kwinxrenderutils.h>
-#include <xcbutils.h>
 #include <xcb/render.h>
 #endif
 #include <xcb/xfixes.h>
@@ -54,7 +52,10 @@ ZoomEffect::ZoomEffect()
     , polling(false)
     , zoomFactor(1.25)
     , mouseTracking(MouseTrackingProportional)
+    , enableFocusTracking(false)
+    , followFocus(true)
     , mousePointer(MousePointerScale)
+    , focusDelay(350)   // in milliseconds
     , imageWidth(0)
     , imageHeight(0)
     , isMouseHidden(false)
@@ -62,38 +63,71 @@ ZoomEffect::ZoomEffect()
     , yMove(0)
     , moveFactor(20.0)
 {
-    KActionCollection* actionCollection = new KActionCollection(this);
-    KAction* a = 0;
-    a = static_cast< KAction* >(actionCollection->addAction(KStandardAction::ZoomIn, this, SLOT(zoomIn())));
-    a->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_Equal));
-    a = static_cast< KAction* >(actionCollection->addAction(KStandardAction::ZoomOut, this, SLOT(zoomOut())));
-    a->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_Minus));
-    a = static_cast< KAction* >(actionCollection->addAction(KStandardAction::ActualSize, this, SLOT(actualSize())));
-    a->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_0));
+    QAction* a = 0;
+    a = KStandardAction::zoomIn(this, SLOT(zoomIn()), this);
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Equal);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Equal);
+    effects->registerGlobalShortcut(Qt::META + Qt::Key_Equal, a);
+    effects->registerAxisShortcut(Qt::ControlModifier | Qt::MetaModifier, PointerAxisDown, a);
 
-    a = static_cast< KAction* >(actionCollection->addAction("MoveZoomLeft"));
+    a = KStandardAction::zoomOut(this, SLOT(zoomOut()), this);
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Minus);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Minus);
+    effects->registerGlobalShortcut(Qt::META + Qt::Key_Minus, a);
+    effects->registerAxisShortcut(Qt::ControlModifier | Qt::MetaModifier, PointerAxisUp, a);
+
+    a = KStandardAction::actualSize(this, SLOT(actualSize()), this);
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_0);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_0);
+    effects->registerGlobalShortcut(Qt::META + Qt::Key_0, a);
+
+    a = new QAction(this);
+    a->setObjectName(QStringLiteral("MoveZoomLeft"));
     a->setText(i18n("Move Zoomed Area to Left"));
-    a->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_Left));
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Left);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Left);
+    effects->registerGlobalShortcut(Qt::META + Qt::Key_Left, a);
     connect(a, SIGNAL(triggered(bool)), this, SLOT(moveZoomLeft()));
 
-    a = static_cast< KAction* >(actionCollection->addAction("MoveZoomRight"));
+    a = new QAction(this);
+    a->setObjectName(QStringLiteral("MoveZoomRight"));
     a->setText(i18n("Move Zoomed Area to Right"));
-    a->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_Right));
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Right);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Right);
+    effects->registerGlobalShortcut(Qt::META + Qt::Key_Right, a);
     connect(a, SIGNAL(triggered(bool)), this, SLOT(moveZoomRight()));
 
-    a = static_cast< KAction* >(actionCollection->addAction("MoveZoomUp"));
+    a = new QAction(this);
+    a->setObjectName(QStringLiteral("MoveZoomUp"));
     a->setText(i18n("Move Zoomed Area Upwards"));
-    a->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_Up));
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Up);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Up);
+    effects->registerGlobalShortcut(Qt::META + Qt::Key_Up, a);
     connect(a, SIGNAL(triggered(bool)), this, SLOT(moveZoomUp()));
 
-    a = static_cast< KAction* >(actionCollection->addAction("MoveZoomDown"));
+    a = new QAction(this);
+    a->setObjectName(QStringLiteral("MoveZoomDown"));
     a->setText(i18n("Move Zoomed Area Downwards"));
-    a->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_Down));
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Down);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_Down);
+    effects->registerGlobalShortcut(Qt::META + Qt::Key_Down, a);
     connect(a, SIGNAL(triggered(bool)), this, SLOT(moveZoomDown()));
 
-    a = static_cast< KAction* >(actionCollection->addAction("MoveMouseToCenter"));
+    // TODO: these two actions don't belong into the effect. They need to be moved into KWin core
+    a = new QAction(this);
+    a->setObjectName(QStringLiteral("MoveMouseToFocus"));
+    a->setText(i18n("Move Mouse to Focus"));
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_F5);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_F5);
+    effects->registerGlobalShortcut(Qt::META + Qt::Key_F5, a);
+    connect(a, SIGNAL(triggered(bool)), this, SLOT(moveMouseToFocus()));
+
+    a = new QAction(this);
+    a->setObjectName(QStringLiteral("MoveMouseToCenter"));
     a->setText(i18n("Move Mouse to Center"));
-    a->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_F6));
+    KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_F6);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << Qt::META + Qt::Key_F6);
+    effects->registerGlobalShortcut(Qt::META + Qt::Key_F6, a);
     connect(a, SIGNAL(triggered(bool)), this, SLOT(moveMouseToCenter()));
 
     timeline.setDuration(350);
@@ -111,7 +145,7 @@ ZoomEffect::~ZoomEffect()
     // switch off and free resources
     showCursor();
     // Save the zoom value.
-    KConfigGroup conf = EffectsHandler::effectConfig("Zoom");
+    KConfigGroup conf = EffectsHandler::effectConfig(QStringLiteral("Zoom"));
     conf.writeEntry("InitialZoom", target_zoom);
     conf.sync();
 }
@@ -119,9 +153,10 @@ ZoomEffect::~ZoomEffect()
 void ZoomEffect::showCursor()
 {
     if (isMouseHidden) {
-        // show the previously hidden mouse-pointer again and free the loaded picture.
+        // show the previously hidden mouse-pointer again and free the loaded texture/picture.
         xcb_xfixes_show_cursor(connection(), rootWindow());
-#ifdef KWIN_BUILD_COMPOSITE
+        texture.reset();
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
         xrenderPicture.reset();
 #endif
         isMouseHidden = false;
@@ -133,11 +168,13 @@ void ZoomEffect::hideCursor()
     if (mouseTracking == MouseTrackingProportional && mousePointer == MousePointerKeep)
         return; // don't replace the actual cursor by a static image for no reason.
     if (!isMouseHidden) {
-        // try to load the cursor-theme and if successful then hide the mouse-pointer
+        // try to load the cursor-theme into a OpenGL texture and if successful then hide the mouse-pointer
         recreateTexture();
         bool shouldHide = false;
-        if (effects->compositingType() == XRenderCompositing) {
-#ifdef KWIN_BUILD_COMPOSITE
+        if (effects->isOpenGLCompositing()) {
+            shouldHide = !texture.isNull();
+        } else if (effects->compositingType() == XRenderCompositing) {
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
             shouldHide = !xrenderPicture.isNull();
 #endif
         }
@@ -150,8 +187,9 @@ void ZoomEffect::hideCursor()
 
 void ZoomEffect::recreateTexture()
 {
+    effects->makeOpenGLContextCurrent();
     // read details about the mouse-cursor theme define per default
-    KConfigGroup mousecfg(KSharedConfig::openConfig("kcminputrc"), "Mouse");
+    KConfigGroup mousecfg(KSharedConfig::openConfig(QStringLiteral("kcminputrc")), "Mouse");
     QString theme = mousecfg.readEntry("cursorTheme", QString());
     QString size  = mousecfg.readEntry("cursorSize", QString());
 
@@ -162,36 +200,61 @@ void ZoomEffect::recreateTexture()
         iconSize = QApplication::style()->pixelMetric(QStyle::PM_LargeIconSize);
 
     // load the cursor-theme image from the Xcursor-library
-    XcursorImage *ximg = XcursorLibraryLoadImage("left_ptr", theme.toLocal8Bit(), iconSize);
+    XcursorImage *ximg = XcursorLibraryLoadImage("left_ptr", theme.toLocal8Bit().constData(), iconSize);
     if (!ximg) // default is better then nothing, so keep it as backup
         ximg = XcursorLibraryLoadImage("left_ptr", "default", iconSize);
     if (ximg) {
-        // turn the XcursorImage into a QImage that will be used to create the XRenderPicture.
+        // turn the XcursorImage into a QImage that will be used to create the GLTexture/XRenderPicture.
         imageWidth = ximg->width;
         imageHeight = ximg->height;
         cursorHotSpot = QPoint(ximg->xhot, ximg->yhot);
         QImage img((uchar*)ximg->pixels, imageWidth, imageHeight, QImage::Format_ARGB32_Premultiplied);
-#ifdef KWIN_BUILD_COMPOSITE
+        if (effects->isOpenGLCompositing())
+            texture.reset(new GLTexture(img));
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
         if (effects->compositingType() == XRenderCompositing)
             xrenderPicture.reset(new XRenderPicture(img));
 #endif
         XcursorImageDestroy(ximg);
     }
     else {
-        kDebug(1212) << "Loading cursor image (" << theme << ") FAILED -> falling back to proportional mouse tracking!";
+        qCDebug(KWINEFFECTS) << "Loading cursor image (" << theme << ") FAILED -> falling back to proportional mouse tracking!";
         mouseTracking = MouseTrackingProportional;
     }
 }
 
 void ZoomEffect::reconfigure(ReconfigureFlags)
 {
-    ZoomConfig::self()->readConfig();
+    ZoomConfig::self()->read();
     // On zoom-in and zoom-out change the zoom by the defined zoom-factor.
     zoomFactor = qMax(0.1, ZoomConfig::zoomFactor());
     // Visibility of the mouse-pointer.
     mousePointer = MousePointerType(ZoomConfig::mousePointer());
     // Track moving of the mouse.
     mouseTracking = MouseTrackingType(ZoomConfig::mouseTracking());
+    // Enable tracking of the focused location.
+    bool _enableFocusTracking = ZoomConfig::enableFocusTracking();
+    if (enableFocusTracking != _enableFocusTracking) {
+        enableFocusTracking = _enableFocusTracking;
+        if (QDBusConnection::sessionBus().isConnected()) {
+            if (enableFocusTracking)
+                QDBusConnection::sessionBus().connect(QStringLiteral("org.kde.kaccessibleapp"),
+                                                      QStringLiteral("/Adaptor"),
+                                                      QStringLiteral("org.kde.kaccessibleapp.Adaptor"),
+                                                      QStringLiteral("focusChanged"),
+                                                      this, SLOT(focusChanged(int,int,int,int,int,int)));
+            else
+                QDBusConnection::sessionBus().disconnect(QStringLiteral("org.kde.kaccessibleapp"),
+                                                         QStringLiteral("/Adaptor"),
+                                                         QStringLiteral("org.kde.kaccessibleapp.Adaptor"),
+                                                         QStringLiteral("focusChanged"),
+                                                         this, SLOT(focusChanged(int,int,int,int,int,int)));
+        }
+    }
+    // When the focus changes, move the zoom area to the focused location.
+    followFocus = ZoomConfig::enableFollowFocus();
+    // The time in milliseconds to wait before a focus-event takes away a mouse-move.
+    focusDelay = qMax(uint(0), ZoomConfig::focusDelay());
     // The factor the zoom-area will be moved on touching an edge on push-mode or using the navigation KAction's.
     moveFactor = qMax(0.1, ZoomConfig::moveFactor());
     if (source_zoom < 0) {
@@ -207,7 +270,9 @@ void ZoomEffect::reconfigure(ReconfigureFlags)
 
 void ZoomEffect::prePaintScreen(ScreenPrePaintData& data, int time)
 {
+    bool altered = false;
     if (zoom != target_zoom) {
+        altered = true;
         const float zoomDist = qAbs(target_zoom - source_zoom);
         if (target_zoom > zoom)
             zoom = qMin(zoom + ((zoomDist * time) / animationTime(150*zoomFactor)), target_zoom);
@@ -217,6 +282,9 @@ void ZoomEffect::prePaintScreen(ScreenPrePaintData& data, int time)
 
     if (zoom == 1.0) {
         showCursor();
+        // reset the generic shader to avoid artifacts in plenty other effects
+        if (altered && effects->isOpenGLCompositing())
+            ShaderBinder binder(ShaderManager::GenericShader, true);
     } else {
         hideCursor();
         data.mask |= PAINT_SCREEN_TRANSFORMED;
@@ -229,6 +297,7 @@ void ZoomEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
 {
     if (zoom != 1.0) {
         data *= QVector2D(zoom, zoom);
+        const QSize screenSize = effects->virtualScreenSize();
 
         // mouse-tracking allows navigation of the zoom-area using the mouse.
         switch(mouseTracking) {
@@ -241,8 +310,8 @@ void ZoomEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
             prevPoint = cursorPoint;
             // fall through
         case MouseTrackingDisabled:
-            data.setXTranslation(qMin(0, qMax(int(displayWidth() - displayWidth() * zoom), int(displayWidth() / 2 - prevPoint.x() * zoom))));
-            data.setYTranslation(qMin(0, qMax(int(displayHeight() - displayHeight() * zoom), int(displayHeight() / 2 - prevPoint.y() * zoom))));
+            data.setXTranslation(qMin(0, qMax(int(screenSize.width() - screenSize.width() * zoom), int(screenSize.width() / 2 - prevPoint.x() * zoom))));
+            data.setYTranslation(qMin(0, qMax(int(screenSize.height() - screenSize.height() * zoom), int(screenSize.height() / 2 - prevPoint.y() * zoom))));
             break;
         case MouseTrackingPush: {
                 // touching an edge of the screen moves the zoom-area in that direction.
@@ -252,21 +321,37 @@ void ZoomEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
                 xMove = yMove = 0;
                 if (x < threshold)
                     xMove = (x - threshold) / zoom;
-                else if (x + threshold > displayWidth())
-                    xMove = (x + threshold - displayWidth()) / zoom;
+                else if (x + threshold > screenSize.width())
+                    xMove = (x + threshold - screenSize.width()) / zoom;
                 if (y < threshold)
                     yMove = (y - threshold) / zoom;
-                else if (y + threshold > displayHeight())
-                    yMove = (y + threshold - displayHeight()) / zoom;
+                else if (y + threshold > screenSize.height())
+                    yMove = (y + threshold - screenSize.height()) / zoom;
                 if (xMove)
-                    prevPoint.setX(qMax(0, qMin(displayWidth(), prevPoint.x() + xMove)));
+                    prevPoint.setX(qMax(0, qMin(screenSize.width(), prevPoint.x() + xMove)));
                 if (yMove)
-                    prevPoint.setY(qMax(0, qMin(displayHeight(), prevPoint.y() + yMove)));
+                    prevPoint.setY(qMax(0, qMin(screenSize.height(), prevPoint.y() + yMove)));
                 data.setXTranslation(- int(prevPoint.x() * (zoom - 1.0)));
                 data.setYTranslation(- int(prevPoint.y() * (zoom - 1.0)));
                 break;
             }
         }
+
+        // use the focusPoint if focus tracking is enabled
+        if (enableFocusTracking && followFocus) {
+            bool acceptFocus = true;
+            if (mouseTracking != MouseTrackingDisabled && focusDelay > 0) {
+                // Wait some time for the mouse before doing the switch. This serves as threshold
+                // to prevent the focus from jumping around to much while working with the mouse.
+                const int msecs = lastMouseEvent.msecsTo(lastFocusEvent);
+                acceptFocus = msecs > focusDelay;
+            }
+            if (acceptFocus) {
+                data.setXTranslation(- int(focusPoint.x() * (zoom - 1.0)));
+                data.setYTranslation(- int(focusPoint.y() * (zoom - 1.0)));
+                prevPoint = focusPoint;
+            }
+        }
     }
 
     effects->paintScreen(mask, region, data);
@@ -284,19 +369,28 @@ void ZoomEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
         const QPoint p = effects->cursorPos() - cursorHotSpot;
         QRect rect(p.x() * zoom + data.xTranslation(), p.y() * zoom + data.yTranslation(), w, h);
 
-#ifdef KWIN_BUILD_COMPOSITE
+        if (texture) {
+            texture->bind();
+            glEnable(GL_BLEND);
+            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+            texture->render(region, rect);
+            texture->unbind();
+            glDisable(GL_BLEND);
+        }
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
         if (xrenderPicture) {
+#define DOUBLE_TO_FIXED(d) ((xcb_render_fixed_t) ((d) * 65536))
             static xcb_render_transform_t xrenderIdentity = {
-                KWIN_DOUBLE_TO_FIXED(1), KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(0),
-                KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(1), KWIN_DOUBLE_TO_FIXED(0),
-                KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(1)
+                DOUBLE_TO_FIXED(1), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0),
+                DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(1), DOUBLE_TO_FIXED(0),
+                DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(1)
             };
             if (mousePointer == MousePointerScale) {
                 xcb_render_set_picture_filter(connection(), *xrenderPicture, 4, const_cast<char*>("good"), 0, NULL);
                 const xcb_render_transform_t xform = {
-                    KWIN_DOUBLE_TO_FIXED(1.0 / zoom), KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(0),
-                    KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(1.0 / zoom), KWIN_DOUBLE_TO_FIXED(0),
-                    KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(1)
+                    DOUBLE_TO_FIXED(1.0 / zoom), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0),
+                    DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(1.0 / zoom), DOUBLE_TO_FIXED(0),
+                    DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(1)
                 };
                 xcb_render_set_picture_transform(connection(), *xrenderPicture, xform);
             }
@@ -304,6 +398,7 @@ void ZoomEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
                                  effects->xrenderBufferPicture(), 0, 0, 0, 0, rect.x(), rect.y(), rect.width(), rect.height());
             if (mousePointer == MousePointerScale)
                 xcb_render_set_picture_transform(connection(), *xrenderPicture, xrenderIdentity);
+#undef DOUBLE_TO_FIXED
         }
 #endif
     }
@@ -362,8 +457,9 @@ void ZoomEffect::actualSize()
 
 void ZoomEffect::timelineFrameChanged(int /* frame */)
 {
-    prevPoint.setX(qMax(0, qMin(displayWidth(), prevPoint.x() + xMove)));
-    prevPoint.setY(qMax(0, qMin(displayHeight(), prevPoint.y() + yMove)));
+    const QSize screenSize = effects->virtualScreenSize();
+    prevPoint.setX(qMax(0, qMin(screenSize.width(), prevPoint.x() + xMove)));
+    prevPoint.setY(qMax(0, qMin(screenSize.height(), prevPoint.y() + yMove)));
     cursorPoint = prevPoint;
     effects->addRepaintFull();
 }
@@ -373,17 +469,18 @@ void ZoomEffect::moveZoom(int x, int y)
     if (timeline.state() == QTimeLine::Running)
         timeline.stop();
 
+    const QSize screenSize = effects->virtualScreenSize();
     if (x < 0)
-        xMove = - qMax(1.0, displayWidth() / zoom / moveFactor);
+        xMove = - qMax(1.0, screenSize.width() / zoom / moveFactor);
     else if (x > 0)
-        xMove = qMax(1.0, displayWidth() / zoom  / moveFactor);
+        xMove = qMax(1.0, screenSize.width() / zoom  / moveFactor);
     else
         xMove = 0;
 
     if (y < 0)
-        yMove = - qMax(1.0, displayHeight() / zoom / moveFactor);
+        yMove = - qMax(1.0, screenSize.height() / zoom / moveFactor);
     else if (y > 0)
-        yMove = qMax(1.0, displayHeight() / zoom / moveFactor);
+        yMove = qMax(1.0, screenSize.height() / zoom / moveFactor);
     else
         yMove = 0;
 
@@ -410,9 +507,14 @@ void ZoomEffect::moveZoomDown()
     moveZoom(0, 1);
 }
 
+void ZoomEffect::moveMouseToFocus()
+{
+    QCursor::setPos(focusPoint.x(), focusPoint.y());
+}
+
 void ZoomEffect::moveMouseToCenter()
 {
-    QRect r(0, 0, displayWidth(), displayHeight());
+    const QRect r = effects->virtualScreenGeometry();
     QCursor::setPos(r.x() + r.width() / 2, r.y() + r.height() / 2);
 }
 
@@ -428,6 +530,18 @@ void ZoomEffect::slotMouseChanged(const QPoint& pos, const QPoint& old, Qt::Mous
     }
 }
 
+void ZoomEffect::focusChanged(int px, int py, int rx, int ry, int rwidth, int rheight)
+{
+    if (zoom == 1.0)
+        return;
+    const QSize screenSize = effects->virtualScreenSize();
+    focusPoint = (px >= 0 && py >= 0) ? QPoint(px, py) : QPoint(rx + qMax(0, (qMin(screenSize.width(), rwidth) / 2) - 60), ry + qMax(0, (qMin(screenSize.height(), rheight) / 2) - 60));
+    if (enableFocusTracking) {
+        lastFocusEvent = QTime::currentTime();
+        effects->addRepaintFull();
+    }
+}
+
 bool ZoomEffect::isActive() const
 {
     return zoom != 1.0 || zoom != target_zoom;
@@ -435,4 +549,4 @@ bool ZoomEffect::isActive() const
 
 } // namespace
 
-#include "moc_zoom.cpp"
+#include "zoom.moc"
