@@ -25,16 +25,20 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
+//#define QT_CLEAN_NAMESPACE
+
 #include "group.h"
+#include <QTextStream>
 #include "workspace.h"
 #include "client.h"
 #include "effects.h"
 
-#include <kstartupinfo.h>
-#include <QtGui/qx11info_x11.h>
-#include <QTextStream>
-
 #include <assert.h>
+#include <kstartupinfo.h>
+#include <KWindowSystem>
+#include <QDebug>
+#include <QX11Info>
+
 
 /*
  TODO
@@ -69,15 +73,15 @@ bool performTransiencyCheck()
         if ((*it1)->deleting)
             continue;
         if ((*it1)->in_group == NULL) {
-            kDebug(1212) << "TC: " << *it1 << " in not in a group";
+            qDebug() << "TC: " << *it1 << " in not in a group" << endl;
             ret = false;
         } else if (!(*it1)->in_group->members().contains(*it1)) {
-            kDebug(1212) << "TC: " << *it1 << " has a group " << (*it1)->in_group << " but group does not contain it";
+            qDebug() << "TC: " << *it1 << " has a group " << (*it1)->in_group << " but group does not contain it" << endl;
             ret = false;
         }
         if (!(*it1)->isTransient()) {
             if (!(*it1)->mainClients().isEmpty()) {
-                kDebug(1212) << "TC: " << *it1 << " is not transient, has main clients:" << (*it1)->mainClients();
+                qDebug() << "TC: " << *it1 << " is not transient, has main clients:" << (*it1)->mainClients() << endl;
                 ret = false;
             }
         } else {
@@ -88,13 +92,13 @@ bool performTransiencyCheck()
                 if (transiencyCheckNonExistent
                         && !Workspace::self()->clients.contains(*it2)
                         && !Workspace::self()->desktops.contains(*it2)) {
-                    kDebug(1212) << "TC:" << *it1 << " has non-existent main client ";
-                    kDebug(1212) << "TC2:" << *it2; // this may crash
+                    qDebug() << "TC:" << *it1 << " has non-existent main client ";
+                    qDebug() << "TC2:" << *it2; // this may crash
                     ret = false;
                     continue;
                 }
                 if (!(*it2)->transients_list.contains(*it1)) {
-                    kDebug(1212) << "TC:" << *it1 << " has main client " << *it2 << " but main client does not have it as a transient";
+                    kdDebug(1212) << "TC:" << *it1 << " has main client " << *it2 << " but main client does not have it as a transient" << endl;
                     ret = false;
                 }
             }
@@ -106,13 +110,13 @@ bool performTransiencyCheck()
             if (transiencyCheckNonExistent
                     && !Workspace::self()->clients.contains(*it2)
                     && !Workspace::self()->desktops.contains(*it2)) {
-                kDebug(1212) << "TC:" << *it1 << " has non-existent transient ";
-                kDebug(1212) << "TC2:" << *it2; // this may crash
+                qDebug() << "TC:" << *it1 << " has non-existent transient ";
+                qDebug() << "TC2:" << *it2; // this may crash
                 ret = false;
                 continue;
             }
             if (!(*it2)->mainClients().contains(*it1)) {
-                kDebug(1212) << "TC:" << *it1 << " has transient " << *it2 << " but transient does not have it as a main client";
+                kdDebug(1212) << "TC:" << *it1 << " has transient " << *it2 << " but transient does not have it as a main client" << endl;
                 ret = false;
             }
         }
@@ -126,7 +130,7 @@ bool performTransiencyCheck()
                 it2 != members.constEnd();
                 ++it2) {
             if ((*it2)->in_group != *it1) {
-                kDebug(1212) << "TC: Group " << *it1 << " contains client " << *it2 << " but client is not in that group";
+                qDebug() << "TC: Group " << *it1 << " contains client " << *it2 << " but client is not in that group" << endl;
                 ret = false;
             }
         }
@@ -151,8 +155,8 @@ static void checkTransiency()
 {
     if (--transiencyCheck == 0) {
         if (!performTransiencyCheck()) {
-            kDebug(1212) << "BT:" << transiencyCheckStartBt;
-            kDebug(1212) << "CLIENT:" << transiencyCheckClient;
+            qDebug() << "BT:" << transiencyCheckStartBt << endl;
+            qDebug() << "CLIENT:" << transiencyCheckClient << endl;
             abort();
         }
         transiencyCheckNonExistent = false;
@@ -199,10 +203,9 @@ Group::Group(Window leader_P)
         refcount(0)
 {
     if (leader_P != None) {
-        leader_client = workspace()->findClient(WindowMatchPredicate(leader_P));
-        unsigned long properties[ 2 ] = { 0, NET::WM2StartupId };
-        leader_info = new NETWinInfo(display(), leader_P, rootWindow(),
-                                      properties, 2);
+        leader_client = workspace()->findClient(Predicate::WindowMatch, leader_P);
+        leader_info = new NETWinInfo(connection(), leader_P, rootWindow(),
+                                      0, NET::WM2StartupId);
     }
     effect_group = new EffectWindowGroupImpl(this);
     workspace()->addGroup(this);
@@ -214,67 +217,41 @@ Group::~Group()
     delete effect_group;
 }
 
-QPixmap Group::icon() const
+QIcon Group::icon() const
 {
     if (leader_client != NULL)
         return leader_client->icon();
     else if (leader_wid != None) {
-        QPixmap ic;
-        Client::readIcons(leader_wid, &ic, NULL, NULL, NULL);
+        QIcon ic;
+        auto readIcon = [&ic, this](int size, bool scale = true) {
+            const QPixmap pix = KWindowSystem::icon(leader_wid, size, size, scale, KWindowSystem::NETWM | KWindowSystem::WMHints);
+            if (!pix.isNull()) {
+                ic.addPixmap(pix);
+            }
+        };
+        readIcon(16);
+        readIcon(32);
+        readIcon(48, false);
+        readIcon(64, false);
+        readIcon(128, false);
         return ic;
     }
-    return QPixmap();
-}
-
-QPixmap Group::miniIcon() const
-{
-    if (leader_client != NULL)
-        return leader_client->miniIcon();
-    else if (leader_wid != None) {
-        QPixmap ic;
-        Client::readIcons(leader_wid, NULL, &ic, NULL, NULL);
-        return ic;
-    }
-    return QPixmap();
-}
-
-QPixmap Group::bigIcon() const
-{
-    if (leader_client != NULL)
-        return leader_client->bigIcon();
-    else if (leader_wid != None) {
-        QPixmap ic;
-        Client::readIcons(leader_wid, NULL, NULL, &ic, NULL);
-        return ic;
-    }
-    return QPixmap();
-}
-
-QPixmap Group::hugeIcon() const
-{
-    if (leader_client != NULL)
-        return leader_client->hugeIcon();
-    else if (leader_wid != None) {
-        QPixmap ic;
-        Client::readIcons(leader_wid, NULL, NULL, NULL, &ic);
-        return ic;
-    }
-    return QPixmap();
+    return QIcon();
 }
 
 void Group::addMember(Client* member_P)
 {
     TRANSIENCY_CHECK(member_P);
     _members.append(member_P);
-//    kDebug(1212) << "GROUPADD:" << this << ":" << member_P;
-//    kDebug(1212) << kBacktrace();
+//    qDebug() << "GROUPADD:" << this << ":" << member_P;
+//    qDebug() << kBacktrace();
 }
 
 void Group::removeMember(Client* member_P)
 {
     TRANSIENCY_CHECK(member_P);
-//    kDebug(1212) << "GROUPREMOVE:" << this << ":" << member_P;
-//    kDebug(1212) << kBacktrace();
+//    qDebug() << "GROUPREMOVE:" << this << ":" << member_P;
+//    qDebug() << kBacktrace();
     Q_ASSERT(_members.contains(member_P));
     _members.removeAll(member_P);
 // there are cases when automatic deleting of groups must be delayed,
@@ -382,7 +359,7 @@ void Workspace::updateMinimizedOfTransients(Client* c)
         }
         if (c->isModal()) { // if a modal dialog is minimized, minimize its mainwindow too
             foreach (Client * c2, c->mainClients())
-                c2->minimize();
+            c2->minimize();
         }
     } else {
         // else unmiminize the transients
@@ -396,7 +373,7 @@ void Workspace::updateMinimizedOfTransients(Client* c)
         }
         if (c->isModal()) {
             foreach (Client * c2, c->mainClients())
-                c2->unminimize();
+            c2->unminimize();
         }
     }
 }
@@ -435,8 +412,8 @@ void Workspace::checkTransients(xcb_window_t w)
 bool Toplevel::resourceMatch(const Toplevel* c1, const Toplevel* c2)
 {
     // xv has "xv" as resource name, and different strings starting with "XV" as resource class
-    if (qstrncmp(c1->resourceClass(), "xv", 2) == 0 && c1->resourceName() == "xv")
-        return qstrncmp(c2->resourceClass(), "xv", 2) == 0 && c2->resourceName() == "xv";
+    if (qstrncmp(c1->resourceClass().constData(), "xv", 2) == 0 && c1->resourceName() == "xv")
+        return qstrncmp(c2->resourceClass().constData(), "xv", 2) == 0 && c2->resourceName() == "xv";
     // Mozilla has "Mozilla" as resource name, and different strings as resource class
     if (c1->resourceName() == "mozilla")
         return c2->resourceName() == "mozilla";
@@ -606,7 +583,7 @@ void Client::setTransient(xcb_window_t new_transient_for_id)
         transient_for = NULL;
         m_transientForId = new_transient_for_id;
         if (m_transientForId != XCB_WINDOW_NONE && !groupTransient()) {
-            transient_for = workspace()->findClient(WindowMatchPredicate(m_transientForId));
+            transient_for = workspace()->findClient(Predicate::WindowMatch, m_transientForId);
             assert(transient_for != NULL);   // verifyTransient() had to check this
             transient_for->addTransient(this);
         } // checkGroup() will check 'check_active_modal'
@@ -637,28 +614,28 @@ void Client::removeFromMainClients()
 void Client::cleanGrouping()
 {
     TRANSIENCY_CHECK(this);
-//    kDebug(1212) << "CLEANGROUPING:" << this;
+//    qDebug() << "CLEANGROUPING:" << this;
 //    for ( ClientList::ConstIterator it = group()->members().begin();
 //         it != group()->members().end();
 //         ++it )
-//        kDebug(1212) << "CL:" << *it;
+//        qDebug() << "CL:" << *it;
 //    ClientList mains;
 //    mains = mainClients();
 //    for ( ClientList::ConstIterator it = mains.begin();
 //         it != mains.end();
 //         ++it )
-//        kDebug(1212) << "MN:" << *it;
+//        qDebug() << "MN:" << *it;
     removeFromMainClients();
-//    kDebug(1212) << "CLEANGROUPING2:" << this;
+//    qDebug() << "CLEANGROUPING2:" << this;
 //    for ( ClientList::ConstIterator it = group()->members().begin();
 //         it != group()->members().end();
 //         ++it )
-//        kDebug(1212) << "CL2:" << *it;
+//        qDebug() << "CL2:" << *it;
 //    mains = mainClients();
 //    for ( ClientList::ConstIterator it = mains.begin();
 //         it != mains.end();
 //         ++it )
-//        kDebug(1212) << "MN2:" << *it;
+//        qDebug() << "MN2:" << *it;
     for (ClientList::ConstIterator it = transients_list.constBegin();
             it != transients_list.constEnd();
        ) {
@@ -668,16 +645,16 @@ void Client::cleanGrouping()
         } else
             ++it;
     }
-//    kDebug(1212) << "CLEANGROUPING3:" << this;
+//    qDebug() << "CLEANGROUPING3:" << this;
 //    for ( ClientList::ConstIterator it = group()->members().begin();
 //         it != group()->members().end();
 //         ++it )
-//        kDebug(1212) << "CL3:" << *it;
+//        qDebug() << "CL3:" << *it;
 //    mains = mainClients();
 //    for ( ClientList::ConstIterator it = mains.begin();
 //         it != mains.end();
 //         ++it )
-//        kDebug(1212) << "MN3:" << *it;
+//        qDebug() << "MN3:" << *it;
     // HACK
     // removeFromMainClients() did remove 'this' from transient
     // lists of all group members, but then made windows that
@@ -690,11 +667,11 @@ void Client::cleanGrouping()
             it != group_members.constEnd();
             ++it)
         (*it)->removeTransient(this);
-//    kDebug(1212) << "CLEANGROUPING4:" << this;
+//    qDebug() << "CLEANGROUPING4:" << this;
 //    for ( ClientList::ConstIterator it = group_members.begin();
 //         it != group_members.end();
 //         ++it )
-//        kDebug(1212) << "CL4:" << *it;
+//        qDebug() << "CL4:" << *it;
 }
 
 // Make sure that no group transient is considered transient
@@ -769,7 +746,7 @@ xcb_window_t Client::verifyTransientFor(xcb_window_t new_transient_for, bool set
     }
     if (new_transient_for == window()) { // pointing to self
         // also fix the property itself
-        kWarning(1216) << "Client " << this << " has WM_TRANSIENT_FOR poiting to itself." ;
+        qWarning() << "Client " << this << " has WM_TRANSIENT_FOR poiting to itself." ;
         new_property_value = new_transient_for = rootWindow();
     }
 //  The transient_for window may be embedded in another application,
@@ -778,17 +755,17 @@ xcb_window_t Client::verifyTransientFor(xcb_window_t new_transient_for, bool set
     xcb_window_t before_search = new_transient_for;
     while (new_transient_for != XCB_WINDOW_NONE
             && new_transient_for != rootWindow()
-            && !workspace()->findClient(WindowMatchPredicate(new_transient_for))) {
+            && !workspace()->findClient(Predicate::WindowMatch, new_transient_for)) {
         Xcb::Tree tree(new_transient_for);
         if (tree.isNull()) {
             break;
         }
         new_transient_for = tree->parent;
     }
-    if (Client* new_transient_for_client = workspace()->findClient(WindowMatchPredicate(new_transient_for))) {
+    if (Client* new_transient_for_client = workspace()->findClient(Predicate::WindowMatch, new_transient_for)) {
         if (new_transient_for != before_search) {
-            kDebug(1212) << "Client " << this << " has WM_TRANSIENT_FOR poiting to non-toplevel window "
-                         << before_search << ", child of " << new_transient_for_client << ", adjusting.";
+            qDebug() << "Client " << this << " has WM_TRANSIENT_FOR poiting to non-toplevel window "
+                         << before_search << ", child of " << new_transient_for_client << ", adjusting." << endl;
             new_property_value = new_transient_for; // also fix the property
         }
     } else
@@ -799,17 +776,17 @@ xcb_window_t Client::verifyTransientFor(xcb_window_t new_transient_for, bool set
     int count = 20;
     xcb_window_t loop_pos = new_transient_for;
     while (loop_pos != XCB_WINDOW_NONE && loop_pos != rootWindow()) {
-        Client* pos = workspace()->findClient(WindowMatchPredicate(loop_pos));
+        Client* pos = workspace()->findClient(Predicate::WindowMatch, loop_pos);
         if (pos == NULL)
             break;
         loop_pos = pos->m_transientForId;
         if (--count == 0 || pos == this) {
-            kWarning(1216) << "Client " << this << " caused WM_TRANSIENT_FOR loop." ;
+            qWarning() << "Client " << this << " caused WM_TRANSIENT_FOR loop." ;
             new_transient_for = rootWindow();
         }
     }
     if (new_transient_for != rootWindow()
-            && workspace()->findClient(WindowMatchPredicate(new_transient_for)) == NULL) {
+            && workspace()->findClient(Predicate::WindowMatch, new_transient_for) == NULL) {
         // it's transient for a specific window, but that window is not mapped
         new_transient_for = rootWindow();
     }
@@ -827,19 +804,19 @@ void Client::addTransient(Client* cl)
     transients_list.append(cl);
     if (workspace()->mostRecentlyActivatedClient() == this && cl->isModal())
         check_active_modal = true;
-//    kDebug(1212) << "ADDTRANS:" << this << ":" << cl;
-//    kDebug(1212) << kBacktrace();
+//    qDebug() << "ADDTRANS:" << this << ":" << cl;
+//    qDebug() << kBacktrace();
 //    for ( ClientList::ConstIterator it = transients_list.begin();
 //         it != transients_list.end();
 //         ++it )
-//        kDebug(1212) << "AT:" << (*it);
+//        qDebug() << "AT:" << (*it);
 }
 
 void Client::removeTransient(Client* cl)
 {
     TRANSIENCY_CHECK(this);
-//    kDebug(1212) << "REMOVETRANS:" << this << ":" << cl;
-//    kDebug(1212) << kBacktrace();
+//    qDebug() << "REMOVETRANS:" << this << ":" << cl;
+//    qDebug() << kBacktrace();
     transients_list.removeAll(cl);
     // cl is transient for this, but this is going away
     // make cl group transient
@@ -921,7 +898,7 @@ ClientList Client::allMainClients() const
 {
     ClientList result = mainClients();
     foreach (const Client * cl, result)
-        result += cl->allMainClients();
+    result += cl->allMainClients();
     return result;
 }
 
