@@ -22,9 +22,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QDBusConnection>
 #include <QDBusMessage>
 
-#include <QLibrary>
-#include <KVBox>
-#include <KLocale>
+#include <KSharedConfig>
+#include <KLocalizedString>
+#include <KConfigGroup>
+#include <KPluginTrader>
+#include <QDialogButtonBox>
+#include <QPushButton>
 
 namespace KWin
 {
@@ -68,41 +71,43 @@ KWinDecorationConfigForm::KWinDecorationConfigForm(QWidget* parent)
 KWinDecorationConfigDialog::KWinDecorationConfigDialog(QString deco, const QList<QVariant>& borderSizes,
         KDecorationDefines::BorderSize size,
         QWidget* parent, Qt::WFlags flags)
-    : KDialog(parent, flags)
+    : QDialog(parent, flags)
     , m_borderSizes(borderSizes)
     , m_kwinConfig(KSharedConfig::openConfig("kwinrc"))
-    , m_pluginObject(0)
-    , m_pluginConfigWidget(0)
+    , m_pluginObject(nullptr)
+    , m_pluginConfigWidget(nullptr)
 {
     m_ui = new KWinDecorationConfigForm(this);
     setWindowTitle(i18n("Decoration Options"));
-    setButtons(KDialog::Ok | KDialog::Cancel | KDialog::Default | KDialog::Reset);
-    enableButton(KDialog::Reset, false);
+
+    m_buttons = new QDialogButtonBox(this);
+    m_buttons->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel | QDialogButtonBox::RestoreDefaults | QDialogButtonBox::Reset);
+    m_buttons->button(QDialogButtonBox::Reset)->setEnabled(false);
     QVBoxLayout* layout = new QVBoxLayout;
     layout->addWidget(m_ui);
+    connect(m_buttons, SIGNAL(accepted()), SLOT(accept()));
+    connect(m_buttons, SIGNAL(rejected()), SLOT(reject()));
 
-    QLibrary library(styleToConfigLib(deco));
-    if (library.load()) {
-        void *alloc_ptr = library.resolve("allocate_config");
-        if (alloc_ptr != NULL) {
-            allocatePlugin = (QObject * (*)(KConfigGroup & conf, QWidget * parent))alloc_ptr;
-            KConfigGroup config(m_kwinConfig, "Style");
-            m_pluginConfigWidget = new KVBox(this);
-            m_pluginObject = (QObject*)(allocatePlugin(config, m_pluginConfigWidget));
-
-            // connect required signals and slots together...
-            connect(this, SIGNAL(accepted()), this, SLOT(slotAccepted()));
-            connect(m_pluginObject, SIGNAL(changed()), this, SLOT(slotSelectionChanged()));
-            connect(this, SIGNAL(pluginSave(KConfigGroup&)), m_pluginObject, SLOT(save(KConfigGroup&)));
-            connect(this, SIGNAL(defaultClicked()), m_pluginObject, SLOT(defaults()));
-            connect(this, SIGNAL(defaultClicked()), SLOT(slotDefault()));
-        }
-    }
-
-    if (m_pluginConfigWidget) {
+    m_pluginConfigWidget = new QWidget(this);
+    m_pluginConfigWidget->setLayout(new QVBoxLayout);
+    m_pluginObject = KPluginTrader::self()->createInstanceFromQuery<QObject>(QStringLiteral("kf5/kwin/kdecorations/config"),
+                                                                              QString(),
+                                                                              QStringLiteral("[X-KDE-PluginInfo-Name] == '%1'").arg(deco),
+                                                                              m_pluginConfigWidget,
+                                                                              m_pluginConfigWidget,
+                                                                              QVariantList() << QStringLiteral("kwinrc") << QStringLiteral("Style"));
+    if (m_pluginObject) {
+        // connect required signals and slots together...
+        connect(this, SIGNAL(accepted()), this, SLOT(slotAccepted()));
+        connect(m_pluginObject, SIGNAL(changed()), this, SLOT(slotSelectionChanged()));
+        connect(this, SIGNAL(pluginSave(KConfigGroup&)), m_pluginObject, SLOT(save(KConfigGroup&)));
+        connect(m_buttons->button(QDialogButtonBox::RestoreDefaults), SIGNAL(clicked(bool)), m_pluginObject, SLOT(defaults()));
+        connect(m_buttons->button(QDialogButtonBox::RestoreDefaults), SIGNAL(clicked(bool)), SLOT(slotDefault()));
         layout->addWidget(m_pluginConfigWidget);
     }
 
+    layout->addWidget(m_buttons);
+
     if (borderSizes.count() >= 2) {
         foreach (const QVariant & borderSize, borderSizes) {
             KDecorationDefines::BorderSize currentSize =
@@ -115,9 +120,7 @@ KWinDecorationConfigDialog::KWinDecorationConfigDialog(QString deco, const QList
         m_ui->borderLabel->hide();
     }
 
-    QWidget* main = new QWidget(this);
-    main->setLayout(layout);
-    setMainWidget(main);
+    setLayout(layout);
 }
 
 KWinDecorationConfigDialog::~KWinDecorationConfigDialog()
@@ -152,15 +155,7 @@ void KWinDecorationConfigDialog::slotAccepted()
 
 void KWinDecorationConfigDialog::slotSelectionChanged()
 {
-    enableButton(KDialog::Reset, true);
-}
-
-QString KWinDecorationConfigDialog::styleToConfigLib(const QString& styleLib) const
-{
-    if (styleLib.startsWith(QLatin1String("kwin3_")))
-        return "kwin_" + styleLib.mid(6) + "_config";
-    else
-        return styleLib + "_config";
+    m_buttons->button(QDialogButtonBox::Reset)->setEnabled(true);
 }
 
 void KWinDecorationConfigDialog::slotDefault()
@@ -171,4 +166,4 @@ void KWinDecorationConfigDialog::slotDefault()
 
 } // namespace KWin
 
-#include "moc_configdialog.cpp"
+#include "configdialog.moc"
