@@ -25,18 +25,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QDir>
 #include <QFileInfo>
 #include <QApplication>
-#include <QtGui/QPainter>
-#include <QStyle>
-#include <QtGui/QTextDocument>
+#include <QStandardPaths>
 // KDE
 #include <KConfigGroup>
 #include <KDesktopFile>
-#include <KGlobalSettings>
-#include <KIcon>
 #include <KLocalizedString>
 #include <KServiceTypeTrader>
-#include <KStandardDirs>
 #include <KPluginInfo>
+#include <KPluginTrader>
 #include "kwindecoration.h"
 
 /* WARNING -------------------------------------------------------------------------
@@ -51,6 +47,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * (which is invoked on deco deconstruction)
 * WARNING ------------------------------------------------------------------------ */
 
+static const QString s_auroraePluginName = QStringLiteral("aurorae");
+
 namespace KWin
 {
 
@@ -59,17 +57,16 @@ DecorationModel::DecorationModel(KSharedConfigPtr config, QObject* parent)
     , m_plugins(new KDecorationPreviewPlugins(config))
     , m_preview(new KDecorationPreview())
     , m_customButtons(false)
-    , m_leftButtons(QString())
-    , m_rightButtons(QString())
+    , m_options(new KDecorationPreviewOptions)
 {
     QHash<int, QByteArray> roleNames;
     roleNames[Qt::DisplayRole] = "display";
-    roleNames[DecorationModel::PixmapRole] = "preview";
     roleNames[TypeRole] = "type";
     roleNames[AuroraeNameRole] = "auroraeThemeName";
     roleNames[QmlMainScriptRole] = "mainScript";
     roleNames[BorderSizeRole] = "borderSize";
     roleNames[ButtonSizeRole] = "buttonSize";
+    roleNames[LibraryNameRole] = "library";
     setRoleNames(roleNames);
     m_config = KSharedConfig::openConfig("auroraerc");
     findDecorations();
@@ -92,47 +89,37 @@ void DecorationModel::reload()
 void DecorationModel::findDecorations()
 {
     beginResetModel();
-    const QStringList dirList = KGlobal::dirs()->findDirs("data", "kwin");
-
-    foreach (const QString & dir, dirList) {
-        QDir d(dir);
-        if (d.exists()) {
-            foreach (const QFileInfo & fi, d.entryInfoList()) {
-                const QString filename(fi.absoluteFilePath());
-                if (KDesktopFile::isDesktopFile(filename)) {
-                    const KDesktopFile desktopFile(filename);
-                    const QString libName = desktopFile.desktopGroup().readEntry("X-KDE-Library");
-
-                    if (!libName.isEmpty() && libName.startsWith(QLatin1String("kwin3_"))) {
-                        if (libName == "kwin3_aurorae") {
-                            // read the Aurorae themes
-                            findAuroraeThemes();
-                            continue;
-                        }
-                        if (!m_plugins->canLoad(libName))
-                            continue;
-                        DecorationModelData data;
-                        data.name = desktopFile.readName();
-                        data.libraryName = libName;
-                        data.type = DecorationModelData::NativeDecoration;
-                        data.borderSize = KDecorationDefines::BorderNormal;
-                        data.closeDblClick = false;
-                        metaData(data, desktopFile);
-                        m_decorations.append(data);
-                    }
-                }
-            }
+    const auto decorations = KPluginTrader::self()->query(QStringLiteral("kf5/kwin/kdecorations"));
+    for (const KPluginInfo &plugin : decorations) {
+        if (plugin.pluginName() == s_auroraePluginName) {
+            // read the Aurorae themes
+            findAuroraeThemes();
+            continue;
         }
+        DecorationModelData data;
+        data.name = plugin.name();
+        data.pluginName = plugin.pluginName();
+        data.type = DecorationModelData::NativeDecoration;
+        data.borderSize = KDecorationDefines::BorderNormal;
+        data.closeDblClick = false;
+        data.comment = plugin.comment();
+        data.author = plugin.author();
+        data.email = plugin.email();
+        data.version = plugin.version();
+        data.license = plugin.license();
+        data.website = plugin.website();
+        m_decorations.append(data);
     }
-    const KService::List offers = KServiceTypeTrader::self()->query("KWin/Decoration");
-    foreach (const KService::Ptr service, offers) {
+
+    KService::List offers = KServiceTypeTrader::self()->query("KWin/Decoration");
+    foreach (KService::Ptr service, offers) {
         DecorationModelData data;
         data.name = service->name();
-        data.libraryName = "kwin3_aurorae";
+        data.pluginName = s_auroraePluginName;
         data.type = DecorationModelData::QmlDecoration;
         data.auroraeName = service->property("X-KDE-PluginInfo-Name").toString();
         QString scriptName = service->property("X-Plasma-MainScript").toString();
-        data.qmlPath = KStandardDirs::locate("data", "kwin/decorations/" + data.auroraeName + "/contents/" + scriptName);
+        data.qmlPath = QStandardPaths::locate(QStandardPaths::GenericDataLocation, "kwin/decorations/" + data.auroraeName + "/contents/" + scriptName);
         if (data.qmlPath.isEmpty()) {
             // not a valid QML theme
             continue;
@@ -140,7 +127,7 @@ void DecorationModel::findDecorations()
         KConfigGroup config(m_config, data.auroraeName);
         data.borderSize = (KDecorationDefines::BorderSize)config.readEntry< int >("BorderSize", KDecorationDefines::BorderNormal);
         data.buttonSize = (KDecorationDefines::BorderSize)config.readEntry< int >("ButtonSize", KDecorationDefines::BorderNormal);
-        data.closeDblClick = config.readEntry< bool >("CloseOnDoubleClickMenuButton", true);
+        data.closeDblClick = config.readEntry< bool >("CloseOnDoubleClickMenuButton", false);
         data.comment = service->comment();
         KPluginInfo info(service);
         data.author = info.author();
@@ -156,10 +143,21 @@ void DecorationModel::findDecorations()
 
 void DecorationModel::findAuroraeThemes()
 {
-    // get all desktop themes
-    QStringList themes = KGlobal::dirs()->findAllResources("data",
-                         "aurorae/themes/*/metadata.desktop",
-                         KStandardDirs::NoDuplicates);
+    // get all destop themes
+    QStringList themes;
+    const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, "aurorae/themes/", QStandardPaths::LocateDirectory);
+    QStringList themeDirectories;
+    for (const QString & dir : dirs) {
+        QDir directory = QDir(dir);
+        for (const QString &themeDir : directory.entryList(QDir::AllDirs | QDir::NoDotAndDotDot)) {
+            themeDirectories << dir + themeDir;
+        }
+    }
+    for (const QString &dir : themeDirectories) {
+        for (const QString & file : QDir(dir).entryList(QStringList() << QStringLiteral("metadata.desktop"))) {
+            themes.append(dir + '/' + file);
+        }
+    }
     foreach (const QString & theme, themes) {
         int themeSepIndex = theme.lastIndexOf('/', -1);
         QString themeRoot = theme.left(themeSepIndex);
@@ -174,13 +172,13 @@ void DecorationModel::findAuroraeThemes()
 
         DecorationModelData data;
         data.name = name;
-        data.libraryName = "kwin3_aurorae";
+        data.pluginName = s_auroraePluginName;
         data.type = DecorationModelData::AuroraeDecoration;
         data.auroraeName = packageName;
         KConfigGroup config(m_config, data.auroraeName);
         data.borderSize = (KDecorationDefines::BorderSize)config.readEntry< int >("BorderSize", KDecorationDefines::BorderNormal);
         data.buttonSize = (KDecorationDefines::BorderSize)config.readEntry< int >("ButtonSize", KDecorationDefines::BorderNormal);
-        data.closeDblClick = config.readEntry< bool >("CloseOnDoubleClickMenuButton", true);
+        data.closeDblClick = config.readEntry< bool >("CloseOnDoubleClickMenuButton", false);
         metaData(data, df);
         m_decorations.append(data);
     }
@@ -212,9 +210,7 @@ QVariant DecorationModel::data(const QModelIndex& index, int role) const
     case NameRole:
         return m_decorations[ index.row()].name;
     case LibraryNameRole:
-        return m_decorations[ index.row()].libraryName;
-    case PixmapRole:
-        return m_decorations[ index.row()].preview;
+        return m_decorations[ index.row()].pluginName;
     case TypeRole:
         return m_decorations[ index.row()].type;
     case AuroraeNameRole:
@@ -236,7 +232,7 @@ QVariant DecorationModel::data(const QModelIndex& index, int role) const
     case BorderSizesRole: {
         QList< QVariant > sizes;
         const bool mustDisablePreview = m_plugins->factory() && m_plugins->factory() == m_preview->factory();
-        if (m_plugins->loadPlugin(m_decorations[index.row()].libraryName) && m_plugins->factory()) {
+        if (m_plugins->loadPlugin(m_decorations[index.row()].pluginName) && m_plugins->factory()) {
             foreach (KDecorationDefines::BorderSize size, m_plugins->factory()->borderSizes())   // krazy:exclude=foreach
                 sizes << int(size);
             if (mustDisablePreview) // it's nuked with destroyPreviousPlugin()
@@ -276,7 +272,6 @@ bool DecorationModel::setData(const QModelIndex& index, const QVariant& value, i
         }
         emit dataChanged(index, index);
         emit configChanged(m_decorations[ index.row()].auroraeName);
-        regeneratePreview(index);
         return true;
     }
     if (role == ButtonSizeRole && (type == DecorationModelData::AuroraeDecoration || type == DecorationModelData::QmlDecoration)) {
@@ -286,7 +281,6 @@ bool DecorationModel::setData(const QModelIndex& index, const QVariant& value, i
         config.sync();
         emit dataChanged(index, index);
         emit configChanged(m_decorations[ index.row()].auroraeName);
-        regeneratePreview(index);
         return true;
     }
     if (role == CloseOnDblClickRole && (type == DecorationModelData::AuroraeDecoration || type == DecorationModelData::QmlDecoration)) {
@@ -307,95 +301,21 @@ bool DecorationModel::setData(const QModelIndex& index, const QVariant& value, i
 
 void DecorationModel::changeButtons(const KWin::DecorationButtons *buttons)
 {
-    bool regenerate = (buttons->customPositions() != m_customButtons);
-    if (!regenerate && buttons->customPositions())
-        regenerate = (buttons->leftButtons() != m_leftButtons) || (buttons->rightButtons() != m_rightButtons);
     m_customButtons = buttons->customPositions();
-    m_leftButtons = buttons->leftButtons();
-    m_rightButtons = buttons->rightButtons();
-    if (regenerate)
-        regeneratePreviews();
+    m_options->setCustomTitleButtonsEnabled(m_customButtons);
+    m_options->setCustomTitleButtons(buttons->leftButtons(), buttons->rightButtons());
 }
 
-void DecorationModel::setButtons(bool custom, const QString& left, const QString& right)
+void DecorationModel::setButtons(bool custom, const QList<KDecorationDefines::DecorationButton> &left, const QList<KDecorationDefines::DecorationButton> &right)
 {
     m_customButtons = custom;
-    m_leftButtons = left;
-    m_rightButtons = right;
-}
-
-void DecorationModel::regenerateNextPreview()
-{
-    if (m_nextPreviewIndex < m_lastUpdateIndex && m_nextPreviewIndex < m_decorations.count())
-        regeneratePreview(index(m_nextPreviewIndex),
-                          QSize(qobject_cast<KWinDecorationModule*>(QObject::parent())->itemWidth(), 150));
-    ++m_nextPreviewIndex;
-    if (m_nextPreviewIndex >= m_lastUpdateIndex && m_firstUpdateIndex > 0) {
-        // do the above ones
-        m_lastUpdateIndex = qMin(m_firstUpdateIndex, m_decorations.count());
-        m_firstUpdateIndex = m_nextPreviewIndex = 0;
-    }
-    if (m_nextPreviewIndex < m_lastUpdateIndex)
-        QMetaObject::invokeMethod(this, "regenerateNextPreview", Qt::QueuedConnection);
-}
-
-void DecorationModel::regeneratePreviews(int firstIndex)
-{
-    m_firstUpdateIndex = firstIndex;
-    m_lastUpdateIndex = m_decorations.count();
-    m_nextPreviewIndex = firstIndex;
-    regenerateNextPreview();
-}
-
-void DecorationModel::stopPreviewGeneration()
-{
-    m_firstUpdateIndex = m_lastUpdateIndex = m_nextPreviewIndex = 0;
-}
-
-void DecorationModel::regeneratePreview(const QModelIndex& index, const QSize& size)
-{
-    DecorationModelData& data = m_decorations[ index.row()];
-
-    switch(data.type) {
-    case DecorationModelData::NativeDecoration: {
-        bool enabled = false;
-        bool loaded;
-        // m_preview->deco management is not required
-        // either the deco loads and the following recreateDecoration will sanitize decos (on new factory)
-        // or the deco does not load and destroyPreviousPlugin() is not called
-        if ((loaded = m_plugins->loadPlugin(data.libraryName)) && m_preview->recreateDecoration(m_plugins)) {
-            enabled = true;
-        } else {
-            m_preview->disablePreview();
-        }
-        if (loaded)
-            m_plugins->destroyPreviousPlugin();
-        if (enabled) {
-            m_preview->resize(size);
-            m_preview->setTempButtons(m_plugins, m_customButtons, m_leftButtons, m_rightButtons);
-            m_preview->setTempBorderSize(m_plugins, data.borderSize);
-            data.preview = m_preview->preview();
-        } else {
-            m_decorations.removeAt(index.row());
-        }
-        break;
-    }
-    default:
-        // nothing
-        break;
-    }
-    emit dataChanged(index, index);
-}
-
-void DecorationModel::regeneratePreview(const QModelIndex& index)
-{
-    regeneratePreview(index, m_decorations.at(index.row()).preview.size());
+    m_options->setCustomTitleButtons(left, right);
 }
 
 QModelIndex DecorationModel::indexOfLibrary(const QString& libraryName) const
 {
     for (int i = 0; i < m_decorations.count(); i++) {
-        if (m_decorations.at(i).libraryName.compare(libraryName) == 0)
+        if (m_decorations.at(i).pluginName.compare(libraryName) == 0)
             return index(i);
     }
     return QModelIndex();
