@@ -20,15 +20,17 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #ifndef KWIN_DECORATIONMODEL_H
 #define KWIN_DECORATIONMODEL_H
-#include <QtCore/qabstractitemmodel.h>
+#include <QAbstractListModel>
 #include <QPixmap>
 #include <KConfig>
+#include <KSharedConfig>
 #include <kdecoration.h>
 
-#include <QWidget>
+class QWidget;
 class KDesktopFile;
 class KDecorationPlugins;
 class KDecorationPreview;
+class KDecorationPreviewOptions;
 
 namespace KWin
 {
@@ -44,8 +46,7 @@ public:
         QmlDecoration = 2
     };
     QString name;
-    QString libraryName;
-    QPixmap preview;
+    QString pluginName;
     DecorationType type;
     QString comment;
     QString author;
@@ -75,7 +76,6 @@ public:
     enum {
         NameRole = Qt::UserRole,
         LibraryNameRole = Qt::UserRole + 1,
-        PixmapRole = Qt::UserRole + 2,
         TypeRole = Qt::UserRole + 3,
         AuroraeNameRole = Qt::UserRole + 4,
         PackageDescriptionRole = Qt::UserRole + 5,
@@ -90,7 +90,7 @@ public:
         QmlMainScriptRole = Qt::UserRole + 14,
         CloseOnDblClickRole = Qt::UserRole + 15
     };
-    explicit DecorationModel(KSharedConfigPtr config, QObject* parent = 0);
+    explicit DecorationModel(KSharedConfigPtr config, QObject* parent = nullptr);
     ~DecorationModel();
 
     virtual QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const;
@@ -99,8 +99,6 @@ public:
 
     void reload();
 
-    void regeneratePreview(const QModelIndex& index);
-
     /**
     * Changes the button state and regenerates the preview.
     */
@@ -108,7 +106,7 @@ public:
     /**
     * Changes the button state without regenerating the preview.
     */
-    void setButtons(bool custom, const QString& left, const QString& right);
+    void setButtons(bool custom, const QList<KDecorationDefines::DecorationButton>& left, const QList<KDecorationDefines::DecorationButton>& right);
 
     void setBorderSize(const QModelIndex& index, KDecorationDefines::BorderSize size);
 
@@ -116,19 +114,12 @@ public:
     QModelIndex indexOfName(const QString& decoName) const;
     QModelIndex indexOfAuroraeName(const QString& auroraeName, const QString& type) const;
 
-    void regeneratePreviews(int firstIndex = 0);
-    void stopPreviewGeneration();
-
     Q_INVOKABLE QVariant readConfig(const QString &themeName, const QString &key, const QVariant &defaultValue = QVariant());
 
     void notifyConfigChanged(const QModelIndex &index);
 
 Q_SIGNALS:
     void configChanged(QString themeName);
-public slots:
-    void regeneratePreview(const QModelIndex& index, const QSize& size);
-private slots:
-    void regenerateNextPreview();
 private:
     void findDecorations();
     void findAuroraeThemes();
@@ -137,12 +128,8 @@ private:
     KDecorationPlugins* m_plugins;
     KDecorationPreview* m_preview;
     bool m_customButtons;
-    QString m_leftButtons;
-    QString m_rightButtons;
     KSharedConfigPtr m_config;
-    int m_nextPreviewIndex;
-    int m_firstUpdateIndex;
-    int m_lastUpdateIndex;
+    QScopedPointer<KDecorationPreviewOptions> m_options;
 };
 
 } // namespace KWin
