@@ -33,39 +33,36 @@
 #include "configdialog.h"
 #include "decorationmodel.h"
 #include "auroraetheme.h"
+#include "preview.h"
 // Qt
-#include <QFile>
-#include <QTimer>
 #include <QtDBus/QtDBus>
-#include <QtDeclarative/QDeclarativeContext>
-#include <QtDeclarative/QDeclarativeEngine>
-#include <QtDeclarative/QDeclarativeItem>
-#include <QtDeclarative/QDeclarativeView>
 #include <QSortFilterProxyModel>
-#include <QtGui/qgraphicsitem.h>
 #include <QScrollBar>
 #include <QUiLoader>
+#include <QtCore/QStandardPaths>
+#include <QQmlContext>
+#include <QQuickItem>
+#include <QQuickView>
 // KDE
 #include <KAboutData>
-#include <KDialog>
+#include <kconfigloader.h>
 #include <KLocalizedString>
 #include <KMessageBox>
-#include <KStandardDirs>
+#include <KNewStuff3/KNS3/DownloadDialog>
 #include <KConfigDialogManager>
 #include <KPluginFactory>
-#include <Plasma/ConfigLoader>
-#include <qdeclarative.h>
 
 // KCModule plugin interface
 // =========================
 K_PLUGIN_FACTORY(KWinDecoFactory,
                  registerPlugin<KWin::KWinDecorationModule>();
                 )
-K_EXPORT_PLUGIN(KWinDecoFactory("kcmkwindecoration"))
 
 namespace KWin
 {
 
+static const QString s_defaultPluginName = QStringLiteral("Oxygen");
+
 KWinDecorationForm::KWinDecorationForm(QWidget* parent)
     : QWidget(parent)
 {
@@ -74,29 +71,33 @@ KWinDecorationForm::KWinDecorationForm(QWidget* parent)
 
 
 KWinDecorationModule::KWinDecorationModule(QWidget* parent, const QVariantList &)
-    : KCModule(KWinDecoFactory::componentData(), parent)
+    : KCModule(parent)
     , kwinConfig(KSharedConfig::openConfig("kwinrc"))
     , m_showTooltips(false)
-    , m_model(NULL)
-    , m_proxyModel(NULL)
+    , m_model(nullptr)
+    , m_proxyModel(nullptr)
     , m_configLoaded(false)
     , m_decorationButtons(new DecorationButtons(this))
-    , m_lastPreviewWidth(-1)
-    , m_previewUpdateTimer(NULL)
+    , m_listView(new QQuickView())
 {
     qmlRegisterType<Aurorae::AuroraeTheme>("org.kde.kwin.aurorae", 0, 1, "AuroraeTheme");
+    qmlRegisterType<PreviewItem>("org.kde.kwin.kcmdecoration", 0, 1, "PreviewItem");
     m_ui = new KWinDecorationForm(this);
-    m_ui->configureDecorationButton->setIcon(KIcon("configure"));
-    m_ui->configureButtonsButton->setIcon(KIcon("configure"));
+    m_ui->configureDecorationButton->setIcon(QIcon::fromTheme("configure"));
+    m_ui->configureButtonsButton->setIcon(QIcon::fromTheme("configure"));
+    m_ui->ghnsButton->setIcon(QIcon::fromTheme("get-hot-new-stuff"));
+    QWidget *container = QWidget::createWindowContainer(m_listView.data(), m_ui->decorationList->viewport());
+    QVBoxLayout *containerLayout = new QVBoxLayout(m_ui->decorationList->viewport());
+    containerLayout->addWidget(container);
     QVBoxLayout* layout = new QVBoxLayout(this);
     layout->addWidget(m_ui);
 
     KAboutData *about =
-        new KAboutData(I18N_NOOP("kcmkwindecoration"), 0,
-                       ki18n("Window Decoration Control Module"),
-                       0, KLocalizedString(), KAboutData::License_GPL,
-                       ki18n("(c) 2001 Karol Szwed"));
-    about->addAuthor(ki18n("Karol Szwed"), KLocalizedString(), "gallium@kde.org");
+        new KAboutData(i18n("kcmkwindecoration"), QString(),
+                       i18n("Window Decoration Control Module"),
+                       QString(), QString(), KAboutData::License_GPL,
+                       i18n("(c) 2001 Karol Szwed"));
+    about->addAuthor(i18n("Karol Szwed"), QString(), "gallium@kde.org");
     setAboutData(about);
 }
 
@@ -117,13 +118,12 @@ void KWinDecorationModule::init()
         // init already called
         return;
     }
-    const QString mainQmlPath = KStandardDirs::locate("data", "kwin/kcm_kwindecoration/main.qml");
+    const QString mainQmlPath = QStandardPaths::locate(QStandardPaths::GenericDataLocation, "kwin/kcm_kwindecoration/main.qml");
     if (mainQmlPath.isNull()) {
-        // TODO 4.11 i18n this
-        KMessageBox::error(this, "<h1>Installation error</h1>"
+        KMessageBox::error(this, i18n("<h1>Installation error</h1>"
         "The resource<h2>kwin/kcm_kwindecoration/main.qml</h2>could not be located in any application data path."
         "<h2>Please contact your distribution</h2>"
-        "The application will now abort", "Installation Error");
+        "The application will now abort"), i18n("Installation Error"));
         abort();
     }
     KConfigGroup style(kwinConfig, "Style");
@@ -133,41 +133,33 @@ void KWinDecorationModule::init()
     m_proxyModel = new QSortFilterProxyModel(this);
     m_proxyModel->setSourceModel(m_model);
     m_proxyModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
-    m_ui->decorationList->setResizeMode(QDeclarativeView::SizeRootObjectToView);
-    /* use logic from KDeclarative::setupBindings():
-    "addImportPath adds the path at the beginning, so to honour user's
-     paths we need to traverse the list in reverse order" */
-    QStringListIterator paths(KGlobal::dirs()->findDirs("module", "imports"));
-    paths.toBack();
-    while (paths.hasPrevious()) {
-        m_ui->decorationList->engine()->addImportPath(paths.previous());
-    }
-    m_ui->decorationList->rootContext()->setContextProperty("decorationModel", m_proxyModel);
-    m_ui->decorationList->rootContext()->setContextProperty("decorationBaseModel", m_model);
-    m_ui->decorationList->rootContext()->setContextProperty("options", m_decorationButtons);
-    m_ui->decorationList->rootContext()->setContextProperty("highlightColor", m_ui->decorationList->palette().color(QPalette::Highlight));
-    m_ui->decorationList->rootContext()->setContextProperty("sliderWidth", m_ui->decorationList->verticalScrollBar()->width());
-    m_ui->decorationList->rootContext()->setContextProperty("auroraeSource", KStandardDirs::locate("data", "kwin/aurorae/aurorae.qml"));
-    m_ui->decorationList->rootContext()->setContextProperty("decorationActiveCaptionColor", KDecoration::options()->color(ColorFont, true));
-    m_ui->decorationList->rootContext()->setContextProperty("decorationInactiveCaptionColor", KDecoration::options()->color(ColorFont, false));
-    m_ui->decorationList->rootContext()->setContextProperty("decorationActiveTitleBarColor", KDecoration::options()->color(ColorTitleBar, true));
-    m_ui->decorationList->rootContext()->setContextProperty("decorationInactiveTitleBarColor", KDecoration::options()->color(ColorTitleBar, false));
-    m_ui->decorationList->setSource(mainQmlPath);
+
+    m_listView->setResizeMode(QQuickView::SizeRootObjectToView);
+    m_listView->rootContext()->setContextProperty("decorationModel", m_proxyModel);
+    m_listView->rootContext()->setContextProperty("decorationBaseModel", m_model);
+    m_listView->rootContext()->setContextProperty("options", m_decorationButtons);
+    m_listView->rootContext()->setContextProperty("highlightColor", m_ui->decorationList->palette().color(QPalette::Highlight));
+    m_listView->rootContext()->setContextProperty("auroraeSource", QStandardPaths::locate(QStandardPaths::GenericDataLocation, "kwin/aurorae/aurorae.qml"));
+    m_listView->rootContext()->setContextProperty("decorationActiveCaptionColor", KDecoration::options()->color(ColorFont, true));
+    m_listView->rootContext()->setContextProperty("decorationInactiveCaptionColor", KDecoration::options()->color(ColorFont, false));
+    m_listView->rootContext()->setContextProperty("decorationActiveTitleBarColor", KDecoration::options()->color(ColorTitleBar, true));
+    m_listView->rootContext()->setContextProperty("decorationInactiveTitleBarColor", KDecoration::options()->color(ColorTitleBar, false));
+    m_listView->setSource(QUrl::fromLocalFile(mainQmlPath));
 
     readConfig(style);
 
-    connect(m_ui->decorationList->rootObject(), SIGNAL(currentIndexChanged()), SLOT(slotSelectionChanged()));
-    connect(m_ui->decorationList->rootObject(), SIGNAL(widthChanged()), SLOT(updatePreviewWidth()));
+    connect(m_listView->rootObject(), SIGNAL(currentIndexChanged()), SLOT(slotSelectionChanged()));
     connect(m_ui->configureButtonsButton, SIGNAL(clicked(bool)), this, SLOT(slotConfigureButtons()));
+    connect(m_ui->ghnsButton, SIGNAL(clicked(bool)), SLOT(slotGHNSClicked()));
     connect(m_ui->searchEdit, SIGNAL(textChanged(QString)), m_proxyModel, SLOT(setFilterFixedString(QString)));
-    connect(m_ui->searchEdit, SIGNAL(textChanged(QString)), m_ui->decorationList->rootObject(), SLOT(returnToBounds()), Qt::QueuedConnection);
+    connect(m_ui->searchEdit, SIGNAL(textChanged(QString)), m_listView->rootObject(), SLOT(returnToBounds()), Qt::QueuedConnection);
     connect(m_ui->searchEdit, SIGNAL(textChanged(QString)), SLOT(updateScrollbarRange()), Qt::QueuedConnection);
     connect(m_ui->configureDecorationButton, SIGNAL(clicked(bool)), SLOT(slotConfigureDecoration()));
 
     m_ui->decorationList->disconnect(m_ui->decorationList->verticalScrollBar());
     m_ui->decorationList->verticalScrollBar()->disconnect(m_ui->decorationList);
-    connect(m_ui->decorationList->rootObject(), SIGNAL(contentYChanged()), SLOT(updateScrollbarValue()));
-    connect(m_ui->decorationList->rootObject(), SIGNAL(contentHeightChanged()), SLOT(updateScrollbarRange()));
+    connect(m_listView->rootObject(), SIGNAL(contentYChanged()), SLOT(updateScrollbarValue()));
+    connect(m_listView->rootObject(), SIGNAL(contentHeightChanged()), SLOT(updateScrollbarRange()));
     connect(m_ui->decorationList->verticalScrollBar(), SIGNAL(rangeChanged(int,int)), SLOT(updateScrollbarRange()));
     connect(m_ui->decorationList->verticalScrollBar(), SIGNAL(valueChanged(int)), SLOT(updateViewPosition(int)));
 
@@ -177,13 +169,6 @@ void KWinDecorationModule::init()
     updateScrollbarRange();
 }
 
-int KWinDecorationModule::itemWidth() const
-{
-    const int width = m_ui->decorationList->rootObject()->property("width").toInt();
-    const int sliderWidth = m_ui->decorationList->verticalScrollBar()->width();
-    return width - sliderWidth;
-}
-
 // This is the selection handler setting
 void KWinDecorationModule::slotSelectionChanged()
 {
@@ -199,31 +184,31 @@ void KWinDecorationModule::readConfig(const KConfigGroup & conf)
     // Find the corresponding decoration name to that of
     // the current plugin library name
 
-    QString libraryName = conf.readEntry("PluginLib", "kwin3_oxygen");
+    QString libraryName = conf.readEntry("PluginLib", s_defaultPluginName);
 
     if (libraryName.isEmpty()) {
         // Selected decoration doesn't exist, use the default
-        libraryName = "kwin3_oxygen";
+        libraryName = s_defaultPluginName;
     }
 
     const int bsize = conf.readEntry("BorderSize", (int)BorderNormal);
     BorderSize borderSize = BorderNormal;
     if (bsize >= BorderTiny && bsize < BordersCount)
         borderSize = static_cast< BorderSize >(bsize);
-    if (libraryName == "kwin3_aurorae") {
+    if (libraryName == "aurorae") {
         KConfig auroraeConfig("auroraerc");
         KConfigGroup group(&auroraeConfig, "Engine");
         const QString themeName = group.readEntry("ThemeName", "example-deco");
         const QString type = group.readEntry("EngineType", "aurorae");
         const QModelIndex index = m_proxyModel->mapFromSource(m_model->indexOfAuroraeName(themeName, type));
         if (index.isValid()) {
-            m_ui->decorationList->rootObject()->setProperty("currentIndex", index.row());
+            m_listView->rootObject()->setProperty("currentIndex", index.row());
         }
     } else {
         const QModelIndex index = m_proxyModel->mapFromSource(m_model->indexOfLibrary(libraryName));
         if (index.isValid()) {
             m_model->setBorderSize(index, borderSize);
-            m_ui->decorationList->rootObject()->setProperty("currentIndex", index.row());
+            m_listView->rootObject()->setProperty("currentIndex", index.row());
         }
     }
 
@@ -231,9 +216,11 @@ void KWinDecorationModule::readConfig(const KConfigGroup & conf)
     // ============
     m_decorationButtons->setCustomPositions(conf.readEntry("CustomButtonPositions", false));
     // Menu and onAllDesktops buttons are default on LHS
-    m_decorationButtons->setLeftButtons(conf.readEntry("ButtonsOnLeft", KDecorationOptions::defaultTitleButtonsLeft()));
+    m_decorationButtons->setLeftButtons(KDecorationOptions::readDecorationButtons(conf, "ButtonsOnLeft",
+                                                                                  KDecorationOptions::defaultTitleButtonsLeft()));
     // Help, Minimize, Maximize and Close are default on RHS
-    m_decorationButtons->setRightButtons(conf.readEntry("ButtonsOnRight", KDecorationOptions::defaultTitleButtonsRight()));
+    m_decorationButtons->setRightButtons(KDecorationOptions::readDecorationButtons(conf, "ButtonsOnRight",
+                                                                                   KDecorationOptions::defaultTitleButtonsRight()));
     if (m_configLoaded)
         m_model->changeButtons(m_decorationButtons);
     else {
@@ -248,7 +235,7 @@ void KWinDecorationModule::readConfig(const KConfigGroup & conf)
 // Writes the selected user configuration to the kwin config file
 void KWinDecorationModule::writeConfig(KConfigGroup & conf)
 {
-    const QModelIndex index = m_proxyModel->mapToSource(m_proxyModel->index(m_ui->decorationList->rootObject()->property("currentIndex").toInt(), 0));
+    const QModelIndex index = m_proxyModel->mapToSource(m_proxyModel->index(m_listView->rootObject()->property("currentIndex").toInt(), 0));
     const QString libName = m_model->data(index, DecorationModel::LibraryNameRole).toString();
 
     // General settings
@@ -257,8 +244,8 @@ void KWinDecorationModule::writeConfig(KConfigGroup & conf)
     conf.writeEntry("ShowToolTips", m_showTooltips);
 
     // Button settings
-    conf.writeEntry("ButtonsOnLeft", m_decorationButtons->leftButtons());
-    conf.writeEntry("ButtonsOnRight", m_decorationButtons->rightButtons());
+    KDecorationOptions::writeDecorationButtons(conf, "ButtonsOnLeft", m_decorationButtons->leftButtons());
+    KDecorationOptions::writeDecorationButtons(conf, "ButtonsOnRight", m_decorationButtons->rightButtons());
     conf.writeEntry("BorderSize",
                     static_cast<int>(m_model->data(index, DecorationModel::BorderSizeRole).toInt()));
 
@@ -309,7 +296,7 @@ void KWinDecorationModule::defaults()
     m_showTooltips = true;
     const QModelIndex index = m_proxyModel->mapFromSource(m_model->indexOfName(i18n("Oxygen")));
     if (index.isValid())
-        m_ui->decorationList->rootObject()->setProperty("currentIndex", index.row());
+        m_listView->rootObject()->setProperty("currentIndex", index.row());
 
     m_decorationButtons->resetToDefaults();
 
@@ -333,7 +320,7 @@ QString KWinDecorationModule::quickHelp() const
 void KWinDecorationModule::slotConfigureButtons()
 {
     QPointer< KWinDecorationButtonsConfigDialog > configDialog = new KWinDecorationButtonsConfigDialog(m_decorationButtons, m_showTooltips, this);
-    if (configDialog->exec() == KDialog::Accepted) {
+    if (configDialog->exec() == QDialog::Accepted) {
         m_decorationButtons->setCustomPositions(configDialog->customPositions());
         m_showTooltips = configDialog->showTooltips();
         m_decorationButtons->setLeftButtons(configDialog->buttonsLeft());
@@ -345,33 +332,76 @@ void KWinDecorationModule::slotConfigureButtons()
     delete configDialog;
 }
 
+void KWinDecorationModule::slotGHNSClicked()
+{
+    QPointer<KNS3::DownloadDialog> downloadDialog = new KNS3::DownloadDialog("aurorae.knsrc", this);
+    if (downloadDialog->exec() == QDialog::Accepted) {
+        if (!downloadDialog->changedEntries().isEmpty()) {
+            const QModelIndex index = m_proxyModel->mapToSource(m_proxyModel->index(m_listView->rootObject()->property("currentIndex").toInt(), 0));
+            const QString libraryName = m_model->data(index, DecorationModel::LibraryNameRole).toString();
+            bool aurorae = m_model->data(index, DecorationModel::TypeRole).toInt() == DecorationModelData::AuroraeDecoration;
+            bool qml = m_model->data(index, DecorationModel::TypeRole).toInt() == DecorationModelData::QmlDecoration;
+            const QString auroraeName = m_model->data(index, DecorationModel::AuroraeNameRole).toString();
+            m_model->reload();
+            if (aurorae) {
+                const QModelIndex proxyIndex = m_proxyModel->mapFromSource(m_model->indexOfAuroraeName(auroraeName, "aurorae"));
+                if (proxyIndex.isValid())
+                    m_listView->rootObject()->setProperty("currentIndex", proxyIndex.row());
+            } else if (qml) {
+                const QModelIndex proxyIndex = m_proxyModel->mapFromSource(m_model->indexOfAuroraeName(auroraeName, "qml"));
+                if (proxyIndex.isValid())
+                    m_listView->rootObject()->setProperty("currentIndex", proxyIndex.row());
+            } else {
+                const QModelIndex proxyIndex = m_proxyModel->mapFromSource(m_model->indexOfLibrary(libraryName));
+                if (proxyIndex.isValid())
+                    m_listView->rootObject()->setProperty("currentIndex", proxyIndex.row());
+            }
+        }
+    }
+    delete downloadDialog;
+}
+
 void KWinDecorationModule::slotConfigureDecoration()
 {
-    const QModelIndex index = m_proxyModel->mapToSource(m_proxyModel->index(m_ui->decorationList->rootObject()->property("currentIndex").toInt(), 0));
+    const QModelIndex index = m_proxyModel->mapToSource(m_proxyModel->index(m_listView->rootObject()->property("currentIndex").toInt(), 0));
     bool reload = false;
     if (index.data(DecorationModel::TypeRole).toInt() == DecorationModelData::AuroraeDecoration ||
         index.data(DecorationModel::TypeRole).toInt() == DecorationModelData::QmlDecoration) {
-        QPointer< KDialog > dlg = new KDialog(this);
-        dlg->setCaption(i18n("Decoration Options"));
-        dlg->setButtons(KDialog::Ok | KDialog::Cancel);
+        QPointer<QDialog> dlg = new QDialog(this);
+        dlg->setWindowTitle(i18n("Decoration Options"));
         KWinAuroraeConfigForm *form = new KWinAuroraeConfigForm(dlg);
         form->enableNoSideBorderSupport(index.data(DecorationModel::TypeRole).toInt() == DecorationModelData::QmlDecoration);
-        dlg->setMainWidget(form);
+        dlg->setLayout(new QVBoxLayout);
+        QDialogButtonBox* buttons = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel, dlg.data());
+        connect(buttons, SIGNAL(accepted()), dlg, SLOT(accept()));
+        connect(buttons, SIGNAL(rejected()), dlg, SLOT(reject()));
+        dlg->layout()->addWidget(form);
+        dlg->layout()->addWidget(buttons);
+
         form->borderSizesCombo->setCurrentIndex(index.data(DecorationModel::BorderSizeRole).toInt());
         form->buttonSizesCombo->setCurrentIndex(index.data(DecorationModel::ButtonSizeRole).toInt());
         form->closeWindowsDoubleClick->setChecked(index.data(DecorationModel::CloseOnDblClickRole).toBool());
+        form->doubleClickMessage->setCloseButtonVisible(true);
+        form->doubleClickMessage->setText(i18n("Close by double clicking:\n To open the menu, keep the button pressed until it appears."));
+        form->doubleClickMessage->setVisible(false);
+        connect(form->closeWindowsDoubleClick, &QCheckBox::toggled, [form](bool toggled) {
+            if (!toggled) {
+                return;
+            }
+            form->doubleClickMessage->animatedShow();
+        });
         // in case of QmlDecoration look for a config.ui in the package structure
-        KConfigDialogManager *configManager = NULL;
+        KConfigDialogManager *configManager = nullptr;
         if (index.data(DecorationModel::TypeRole).toInt() == DecorationModelData::QmlDecoration) {
             const QString packageName = index.data(DecorationModel::AuroraeNameRole).toString();
-            const QString uiPath = KStandardDirs::locate("data", "kwin/decorations/" + packageName + "/contents/ui/config.ui");
-            const QString configPath = KStandardDirs::locate("data", "kwin/decorations/" + packageName + "/contents/config/main.xml");
+            const QString uiPath = QStandardPaths::locate(QStandardPaths::GenericDataLocation, "kwin/decorations/" + packageName + "/contents/ui/config.ui");
+            const QString configPath = QStandardPaths::locate(QStandardPaths::GenericDataLocation, "kwin/decorations/" + packageName + "/contents/config/main.xml");
             if (!uiPath.isEmpty() && !configPath.isEmpty()) {
                 // load the KConfigSkeleton
                 QFile configFile(configPath);
                 KSharedConfigPtr auroraeConfig = KSharedConfig::openConfig("auroraerc");
                 KConfigGroup configGroup = auroraeConfig->group(packageName);
-                Plasma::ConfigLoader *skeleton = new Plasma::ConfigLoader(&configGroup, &configFile, dlg);
+                KConfigLoader *skeleton = new KConfigLoader(configGroup, &configFile, dlg);
                 // load the ui file
                 QUiLoader *loader = new QUiLoader(dlg);
                 QFile uiFile(uiPath);
@@ -384,7 +414,7 @@ void KWinDecorationModule::slotConfigureDecoration()
                 configManager->updateWidgets();
             }
         }
-        if (dlg->exec() == KDialog::Accepted) {
+        if (dlg->exec() == QDialog::Accepted) {
             m_model->setData(index, form->borderSizesCombo->currentIndex(), DecorationModel::BorderSizeRole);
             m_model->setData(index, form->buttonSizesCombo->currentIndex(), DecorationModel::ButtonSizeRole);
             m_model->setData(index, form->closeWindowsDoubleClick->isChecked(), DecorationModel::CloseOnDblClickRole);
@@ -403,7 +433,7 @@ void KWinDecorationModule::slotConfigureDecoration()
             static_cast<KDecorationDefines::BorderSize>(index.data(DecorationModel::BorderSizeRole).toInt());
         QPointer< KWinDecorationConfigDialog > configDialog =
             new KWinDecorationConfigDialog(name, borderSizes, size, this);
-        if (configDialog->exec() == KDialog::Accepted) {
+        if (configDialog->exec() == QDialog::Accepted) {
             m_model->setData(index, configDialog->borderSize(), DecorationModel::BorderSizeRole);
             reload = true;
         }
@@ -422,7 +452,7 @@ bool KWinDecorationModule::eventFilter(QObject *o, QEvent *e)
             updateScrollbarRange();
         else if (e->type() == QEvent::KeyPress) {
             int d = 0;
-            const int currentRow = m_ui->decorationList->rootObject()->property("currentIndex").toInt();
+            const int currentRow = m_listView->rootObject()->property("currentIndex").toInt();
             const int key = static_cast<QKeyEvent*>(e)->key();
             switch (key) {
             case Qt::Key_Home:
@@ -440,7 +470,7 @@ bool KWinDecorationModule::eventFilter(QObject *o, QEvent *e)
             case Qt::Key_PageUp:
             case Qt::Key_PageDown:
                 d = 150;
-                if (QObject *decoItem = m_ui->decorationList->rootObject()->findChild<QObject*>("decorationItem")) {
+                if (QObject *decoItem = m_listView->rootObject()->findChild<QObject*>("decorationItem")) {
                     QVariant v = decoItem->property("height");
                     if (v.isValid())
                         d = v.toInt();
@@ -455,7 +485,7 @@ bool KWinDecorationModule::eventFilter(QObject *o, QEvent *e)
             }
             if (d) {
                 d = qMin(qMax(0, currentRow + d), m_proxyModel->rowCount());
-                m_ui->decorationList->rootObject()->setProperty("currentIndex", d);
+                m_listView->rootObject()->setProperty("currentIndex", d);
                 return true;
             }
         }
@@ -467,42 +497,12 @@ bool KWinDecorationModule::eventFilter(QObject *o, QEvent *e)
     return KCModule::eventFilter(o, e);
 }
 
-void KWinDecorationModule::updatePreviews()
-{
-    if (!m_model) {
-        return;
-    }
-    m_ui->decorationList->rootContext()->setContextProperty("sliderWidth", m_ui->decorationList->verticalScrollBar()->width());
-    const int newWidth = m_ui->decorationList->rootObject()->property("width").toInt();
-    if (newWidth == m_lastPreviewWidth)
-        return;
-    m_lastPreviewWidth = newWidth;
-    const int h = m_ui->decorationList->rootObject()->property("contentHeight").toInt();
-    const int y = m_ui->decorationList->rootObject()->property("contentY").toInt();
-    // start at first element in sight
-    int row = 0;
-    if (h > 0)
-        row = qMin(qMax(0, y*m_model->rowCount()/h), m_model->rowCount());
-    m_model->regeneratePreviews(row);
-}
-
-void KWinDecorationModule::updatePreviewWidth()
-{
-    if (!m_previewUpdateTimer) {
-        m_previewUpdateTimer = new QTimer(this);
-        m_previewUpdateTimer->setSingleShot(true);
-        connect(m_previewUpdateTimer, SIGNAL(timeout()), this, SLOT(updatePreviews()));
-    }
-    m_model->stopPreviewGeneration();
-    m_previewUpdateTimer->start(100);
-}
-
 void KWinDecorationModule::updateScrollbarRange()
 {
     m_ui->decorationList->verticalScrollBar()->blockSignals(true);
-    const bool atMinimum = m_ui->decorationList->rootObject()->property("atYBeginning").toBool();
-    const int h = m_ui->decorationList->rootObject()->property("contentHeight").toInt();
-    const int y = atMinimum ? m_ui->decorationList->rootObject()->property("contentY").toInt() : 0;
+    const bool atMinimum = m_listView->rootObject()->property("atYBeginning").toBool();
+    const int h = m_listView->rootObject()->property("contentHeight").toInt();
+    const int y = atMinimum ? m_listView->rootObject()->property("contentY").toInt() : 0;
     m_ui->decorationList->verticalScrollBar()->setRange(y, y + h - m_ui->decorationList->height());
     m_ui->decorationList->verticalScrollBar()->setPageStep(m_ui->decorationList->verticalScrollBar()->maximum()/m_model->rowCount());
     m_ui->decorationList->verticalScrollBar()->blockSignals(false);
@@ -510,7 +510,7 @@ void KWinDecorationModule::updateScrollbarRange()
 
 void KWinDecorationModule::updateScrollbarValue()
 {
-    const int v = m_ui->decorationList->rootObject()->property("contentY").toInt();
+    const int v = m_listView->rootObject()->property("contentY").toInt();
     m_ui->decorationList->verticalScrollBar()->blockSignals(true); // skippig this will kill kinetic scrolling but the scrollwidth is too low
     m_ui->decorationList->verticalScrollBar()->setValue(v);
     m_ui->decorationList->verticalScrollBar()->blockSignals(false);
@@ -518,16 +518,17 @@ void KWinDecorationModule::updateScrollbarValue()
 
 void KWinDecorationModule::updateViewPosition(int v)
 {
-    QGraphicsObject *list = m_ui->decorationList->rootObject();
-    list->setProperty("contentY", v);
+    m_listView->rootObject()->setProperty("contentY", v);
 }
 
 DecorationButtons::DecorationButtons(QObject *parent)
     : QObject(parent)
     , m_customPositions(false)
-    , m_leftButtons(KDecorationOptions::defaultTitleButtonsLeft())
-    , m_rightButtons(KDecorationOptions::defaultTitleButtonsRight())
+    , m_leftButtons()
+    , m_rightButtons()
 {
+    setLeftButtons(KDecorationOptions::defaultTitleButtonsLeft());
+    setRightButtons(KDecorationOptions::defaultTitleButtonsRight());
 }
 
 DecorationButtons::~DecorationButtons()
@@ -539,12 +540,12 @@ bool DecorationButtons::customPositions() const
     return m_customPositions;
 }
 
-const QString &DecorationButtons::leftButtons() const
+const QList<KDecorationDefines::DecorationButton> &DecorationButtons::leftButtons() const
 {
     return m_leftButtons;
 }
 
-const QString &DecorationButtons::rightButtons() const
+const QList<KDecorationDefines::DecorationButton> &DecorationButtons::rightButtons() const
 {
     return m_rightButtons;
 }
@@ -558,7 +559,7 @@ void DecorationButtons::setCustomPositions(bool set)
     emit customPositionsChanged();
 }
 
-void DecorationButtons::setLeftButtons(const QString &leftButtons)
+void DecorationButtons::setLeftButtons(const QList<DecorationButton> &leftButtons)
 {
     if (m_leftButtons == leftButtons) {
         return;
@@ -567,7 +568,7 @@ void DecorationButtons::setLeftButtons(const QString &leftButtons)
     emit leftButtonsChanged();
 }
 
-void DecorationButtons::setRightButtons (const QString &rightButtons)
+void DecorationButtons::setRightButtons(const QList<DecorationButton> &rightButtons)
 {
     if (m_rightButtons == rightButtons) {
         return;
@@ -585,4 +586,4 @@ void DecorationButtons::resetToDefaults()
 
 } // namespace KWin
 
-#include "moc_kwindecoration.cpp"
+#include "kwindecoration.moc"
