@@ -32,15 +32,14 @@
 #define KWINDECORATION_H
 
 #include <kcmodule.h>
-#include <kconfig.h>
+#include <KSharedConfig>
 
 #include <kdecoration.h>
 
 #include "ui_decoration.h"
 
-#include <QSortFilterProxyModel>
-#include <QTimer>
-
+class QQuickView;
+class QSortFilterProxyModel;
 namespace KWin
 {
 
@@ -54,23 +53,21 @@ public:
     explicit KWinDecorationForm(QWidget* parent);
 };
 
-class DecorationButtons : public QObject
+class DecorationButtons : public QObject, public KDecorationDefines
 {
     Q_OBJECT
     Q_PROPERTY(bool customButtonPositions READ customPositions WRITE setCustomPositions NOTIFY customPositionsChanged)
-    Q_PROPERTY(QString leftButtons READ leftButtons WRITE setLeftButtons NOTIFY leftButtonsChanged)
-    Q_PROPERTY(QString rightButtons READ rightButtons WRITE setRightButtons NOTIFY rightButtonsChanged)
 public:
-    explicit DecorationButtons(QObject *parent = 0);
+    explicit DecorationButtons(QObject *parent = nullptr);
     virtual ~DecorationButtons();
 
     bool customPositions() const;
-    const QString &leftButtons() const;
-    const QString &rightButtons() const;
+    const QList<DecorationButton> &leftButtons() const;
+    const QList<DecorationButton> &rightButtons() const;
 
     void setCustomPositions(bool set);
-    void setLeftButtons(const QString &leftButtons);
-    void setRightButtons(const QString &rightButtons);
+    void setLeftButtons(const QList<KDecorationDefines::DecorationButton> &buttons);
+    void setRightButtons(const QList<KDecorationDefines::DecorationButton> &buttons);
 
 public Q_SLOTS:
     void resetToDefaults();
@@ -82,8 +79,8 @@ Q_SIGNALS:
 
 private:
     bool m_customPositions;
-    QString m_leftButtons;
-    QString m_rightButtons;
+    QList<DecorationButton> m_leftButtons;
+    QList<DecorationButton> m_rightButtons;
 };
 
 class KWinDecorationModule : public KCModule, public KDecorationDefines
@@ -100,9 +97,7 @@ public:
 
     QString quickHelp() const;
 
-    int itemWidth() const;
-
-signals:
+Q_SIGNALS:
     void pluginLoad(const KConfigGroup& conf);
     void pluginSave(KConfigGroup &conf);
     void pluginDefaults();
@@ -111,19 +106,18 @@ protected:
     bool eventFilter(QObject *o, QEvent *e);
     virtual void showEvent(QShowEvent *ev);
 
-protected slots:
+protected Q_SLOTS:
     // Allows us to turn "save" on
     void slotSelectionChanged();
     void slotConfigureButtons();
+    void slotGHNSClicked();
     void slotConfigureDecoration();
 
 private:
     void init();
     void readConfig(const KConfigGroup& conf);
     void writeConfig(KConfigGroup &conf);
-private slots:
-    void updatePreviews();
-    void updatePreviewWidth();
+private Q_SLOTS:
     void updateScrollbarRange();
     void updateScrollbarValue();
     void updateViewPosition(int v);
@@ -138,8 +132,7 @@ private:
     bool m_configLoaded;
     DecorationButtons *m_decorationButtons;
 
-    int m_lastPreviewWidth;
-    QTimer *m_previewUpdateTimer;
+    QScopedPointer<QQuickView> m_listView;
 };
 
 } //namespace
