@@ -22,12 +22,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <QDBusMessage>
 #include <QDBusConnection>
+#include <QDBusInterface>
 
+#include <KAboutData>
 #include <KAboutApplicationDialog>
 #include <KActionCollection>
-#include <KAction>
 #include <KCModuleProxy>
 #include <KGlobalAccel>
+#include <KLocalizedString>
 #include <KPluginInfo>
 #include <KPluginFactory>
 #include <KConfigGroup>
@@ -35,12 +37,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <KServiceTypeTrader>
 #include <KShortcutsEditor>
 
-#include <QtGui/qx11info_x11.h>
+#include <QX11Info>
+#include <X11/Xlib.h>
+#include <fixx11h.h>
 
 #include <netwm.h>
+#include <QDialogButtonBox>
 
 K_PLUGIN_FACTORY(KWinDesktopConfigFactory, registerPlugin<KWin::KWinDesktopConfig>();)
-K_EXPORT_PLUGIN(KWinDesktopConfigFactory("kcm_kwindesktop"))
 
 namespace KWin
 {
@@ -52,10 +56,10 @@ KWinDesktopConfigForm::KWinDesktopConfigForm(QWidget* parent)
 }
 
 KWinDesktopConfig::KWinDesktopConfig(QWidget* parent, const QVariantList& args)
-    : KCModule(KWinDesktopConfigFactory::componentData(), parent, args)
+    : KCModule(KAboutData::pluginData(QStringLiteral("kcm_kwindesktop")), parent, args)
     , m_config(KSharedConfig::openConfig("kwinrc"))
-    , m_actionCollection(NULL)
-    , m_switchDesktopCollection(NULL)
+    , m_actionCollection(nullptr)
+    , m_switchDesktopCollection(nullptr)
 {
     init();
 }
@@ -79,11 +83,11 @@ void KWinDesktopConfig::init()
     setQuickHelp(i18n("<h1>Multiple Desktops</h1>In this module, you can configure how many virtual desktops you want and how these should be labeled."));
 
     // Shortcut config. The shortcut belongs to the component "kwin"!
-    m_actionCollection = new KActionCollection(this, KComponentData("kwin"));
+    m_actionCollection = new KActionCollection(this, QStringLiteral("kwin"));
     m_actionCollection->setConfigGroup("Desktop Switching");
     m_actionCollection->setConfigGlobal(true);
 
-    m_switchDesktopCollection = new KActionCollection(this, KComponentData("kwin"));
+    m_switchDesktopCollection = new KActionCollection(this, QStringLiteral("kwin"));
     m_switchDesktopCollection->setConfigGroup("Desktop Switching");
     m_switchDesktopCollection->setConfigGlobal(true);
 
@@ -102,14 +106,17 @@ void KWinDesktopConfig::init()
     m_editor->addCollection(m_switchDesktopCollection, i18n("Desktop Switching"));
 
     // get number of desktops
-    NETRootInfo info(QX11Info::display(), NET::NumberOfDesktops | NET::DesktopNames);
-    int n = info.numberOfDesktops();
+    int n = 1;
+    if (QX11Info::isPlatformX11()) {
+        NETRootInfo info(QX11Info::connection(), NET::NumberOfDesktops | NET::DesktopNames);
+        n = info.numberOfDesktops();
+    }
 
     for (int i = 1; i <= n; ++i) {
-        KAction* a = qobject_cast<KAction*>(m_actionCollection->addAction(QString("Switch to Desktop %1").arg(i)));
+        QAction* a = m_actionCollection->addAction(QString("Switch to Desktop %1").arg(i));
         a->setProperty("isConfigurationAction", true);
         a->setText(i18n("Switch to Desktop %1", i));
-        a->setGlobalShortcut(KShortcut(), KAction::ActiveShortcut);
+        KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>());
     }
 
     // This should be after the "Switch to Desktop %1" loop. It HAS to be
@@ -123,18 +130,32 @@ void KWinDesktopConfig::init()
     // TODO: way to recognize if a effect is not found
     KServiceTypeTrader* trader = KServiceTypeTrader::self();
     KService::List services;
-    m_ui->effectComboBox->addItem(i18n("No Animation"));
+    QString slide;
+    QString cube;
+    QString fadedesktop;
     services = trader->query("KWin/Effect", "[X-KDE-PluginInfo-Name] == 'kwin4_effect_slide'");
     if (!services.isEmpty())
-        m_ui->effectComboBox->addItem(services.first()->name());
+        slide = services.first()->name();
+    services = trader->query("KWin/Effect", "[X-KDE-PluginInfo-Name] == 'kwin4_effect_cubeslide'");
+    if (!services.isEmpty())
+        cube = services.first()->name();
+    services = trader->query("KWin/Effect", "[X-KDE-PluginInfo-Name] == 'kwin4_effect_fadedesktop'");
+    if (!services.isEmpty())
+        fadedesktop = services.first()->name();
+
+    m_ui->effectComboBox->addItem(i18n("No Animation"));
+    m_ui->effectComboBox->addItem(slide);
+    m_ui->effectComboBox->addItem(cube);
+    m_ui->effectComboBox->addItem(fadedesktop);
 
     // effect config and info button
-    m_ui->effectInfoButton->setIcon(KIcon("dialog-information"));
-    m_ui->effectConfigButton->setIcon(KIcon("configure"));
+    m_ui->effectInfoButton->setIcon(QIcon::fromTheme("dialog-information"));
+    m_ui->effectConfigButton->setIcon(QIcon::fromTheme("configure"));
 
     connect(m_ui->rowsSpinBox, SIGNAL(valueChanged(int)), SLOT(changed()));
     connect(m_ui->numberSpinBox, SIGNAL(valueChanged(int)), SLOT(changed()));
     connect(m_ui->numberSpinBox, SIGNAL(valueChanged(int)), SLOT(slotChangeShortcuts(int)));
+    connect(m_ui->activityCheckBox, SIGNAL(stateChanged(int)), SLOT(changed()));
     connect(m_ui->desktopNames, SIGNAL(changed()), SLOT(changed()));
     connect(m_ui->popupInfoCheckBox, SIGNAL(toggled(bool)), SLOT(changed()));
     connect(m_ui->popupHideSpinBox, SIGNAL(valueChanged(int)), SLOT(changed()));
@@ -148,7 +169,7 @@ void KWinDesktopConfig::init()
     connect(m_ui->effectConfigButton, SIGNAL(clicked()), SLOT(slotConfigureEffectClicked()));
 
     // Begin check for immutable - taken from old desktops kcm
-    int kwin_screen_number = DefaultScreen(QX11Info::display());
+    int kwin_screen_number = QX11Info::appScreen();
 
     m_config = KSharedConfig::openConfig("kwinrc");
 
@@ -183,10 +204,10 @@ KWinDesktopConfig::~KWinDesktopConfig()
 
 void KWinDesktopConfig::addAction(const QString &name, const QString &label)
 {
-    KAction* a = qobject_cast<KAction*>(m_switchDesktopCollection->addAction(name));
+    QAction* a = m_switchDesktopCollection->addAction(name);
     a->setProperty("isConfigurationAction", true);
     a->setText(label);
-    a->setGlobalShortcut(KShortcut(), KAction::ActiveShortcut);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>());
 }
 
 void KWinDesktopConfig::defaults()
@@ -222,16 +243,20 @@ void KWinDesktopConfig::load()
     // This method is called on reset(). So undo all changes.
     undo();
 
-    // get number of desktops
-    unsigned long properties[] = {NET::NumberOfDesktops | NET::DesktopNames, NET::WM2DesktopLayout };
-    NETRootInfo info(QX11Info::display(), properties, 2);
+    if (QX11Info::isPlatformX11()) {
+        // get number of desktops
+        NETRootInfo info(QX11Info::connection(), NET::NumberOfDesktops | NET::DesktopNames, NET::WM2DesktopLayout);
 
-    for (int i = 1; i <= maxDesktops; i++) {
-        QString name = QString::fromUtf8(info.desktopName(i));
-        m_desktopNames << name;
-        m_ui->desktopNames->setName(i, name);
+        for (int i = 1; i <= maxDesktops; i++) {
+            QString name = QString::fromUtf8(info.desktopName(i));
+            m_desktopNames << name;
+            m_ui->desktopNames->setName(i, name);
+        }
+        m_ui->rowsSpinBox->setValue(info.desktopLayoutColumnsRows().height());
+    } else {
+        // TODO: proper implementation
+        m_ui->rowsSpinBox->setValue(1);
     }
-    m_ui->rowsSpinBox->setValue(info.desktopLayoutColumnsRows().height());
 
     // Popup info
     KConfigGroup effectconfig(m_config, "Plugins");
@@ -249,44 +274,59 @@ void KWinDesktopConfig::load()
     m_ui->effectComboBox->setCurrentIndex(0);
     if (effectEnabled("slide", effectconfig))
         m_ui->effectComboBox->setCurrentIndex(1);
+    if (effectEnabled("cubeslide", effectconfig))
+        m_ui->effectComboBox->setCurrentIndex(2);
+    if (effectEnabled("fadedesktop", effectconfig))
+        m_ui->effectComboBox->setCurrentIndex(3);
     slotEffectSelectionChanged(m_ui->effectComboBox->currentIndex());
     // TODO: plasma stuff
 
+    QDBusInterface interface("org.kde.plasma-desktop", "/App");
+    if (interface.isValid()) {
+        bool perVirtualDesktopViews = interface.call("perVirtualDesktopViews").arguments().first().toBool();
+        m_ui->activityCheckBox->setEnabled(true);
+        m_ui->activityCheckBox->setChecked(perVirtualDesktopViews);
+    } else
+        m_ui->activityCheckBox->setEnabled(false);
+
     emit changed(false);
 }
 
 void KWinDesktopConfig::save()
 {
     // TODO: plasma stuff
-    unsigned long properties[] = {NET::NumberOfDesktops | NET::DesktopNames, NET::WM2DesktopLayout };
-    NETRootInfo info(QX11Info::display(), properties, 2);
-    // set desktop names
-    for (int i = 1; i <= maxDesktops; i++) {
-        QString desktopName = m_desktopNames[ i -1 ];
-        if (i <= m_ui->numberSpinBox->value())
-            desktopName = m_ui->desktopNames->name(i);
-        info.setDesktopName(i, desktopName.toUtf8());
-        info.activate();
-    }
-    // set number of desktops
+
     const int numberDesktops = m_ui->numberSpinBox->value();
-    info.setNumberOfDesktops(numberDesktops);
-    info.activate();
-    int rows =m_ui->rowsSpinBox->value();
+    int rows = m_ui->rowsSpinBox->value();
     rows = qBound(1, rows, numberDesktops);
     // avoid weird cases like having 3 rows for 4 desktops, where the last row is unused
     int columns = numberDesktops / rows;
     if (numberDesktops % rows > 0) {
         columns++;
     }
-    info.setDesktopLayout(NET::OrientationHorizontal, columns, rows, NET::DesktopLayoutCornerTopLeft);
-    info.activate();
 
-    XSync(QX11Info::display(), false);
+    if (QX11Info::isPlatformX11()) {
+        NETRootInfo info(QX11Info::connection(), NET::NumberOfDesktops | NET::DesktopNames, NET::WM2DesktopLayout);
+        // set desktop names
+        for (int i = 1; i <= maxDesktops; i++) {
+            QString desktopName = m_desktopNames[ i -1 ];
+            if (i <= m_ui->numberSpinBox->value())
+                desktopName = m_ui->desktopNames->name(i);
+            info.setDesktopName(i, desktopName.toUtf8());
+            info.activate();
+        }
+        // set number of desktops
+        info.setNumberOfDesktops(numberDesktops);
+        info.activate();
+        info.setDesktopLayout(NET::OrientationHorizontal, columns, rows, NET::DesktopLayoutCornerTopLeft);
+        info.activate();
+
+        XSync(QX11Info::display(), false);
+    }
 
     // save the desktops
     QString groupname;
-    const int screenNumber = DefaultScreen(QX11Info::display());
+    const int screenNumber = QX11Info::appScreen();
     if (screenNumber == 0)
         groupname = "Desktops";
     else
@@ -311,10 +351,26 @@ void KWinDesktopConfig::save()
     case 0:
         // no effect
         effectconfig.writeEntry("kwin4_effect_slideEnabled", false);
+        effectconfig.writeEntry("kwin4_effect_cubeslideEnabled", false);
+        effectconfig.writeEntry("kwin4_effect_fadedesktopEnabled", false);
         break;
     case 1:
         // slide
         effectconfig.writeEntry("kwin4_effect_slideEnabled", true);
+        effectconfig.writeEntry("kwin4_effect_cubeslideEnabled", false);
+        effectconfig.writeEntry("kwin4_effect_fadedesktopEnabled", false);
+        break;
+    case 2:
+        // cube
+        effectconfig.writeEntry("kwin4_effect_slideEnabled", false);
+        effectconfig.writeEntry("kwin4_effect_cubeslideEnabled", true);
+        effectconfig.writeEntry("kwin4_effect_fadedesktopEnabled", false);
+        break;
+    case 3:
+        // fadedesktop
+        effectconfig.writeEntry("kwin4_effect_slideEnabled", false);
+        effectconfig.writeEntry("kwin4_effect_cubeslideEnabled", false);
+        effectconfig.writeEntry("kwin4_effect_fadedesktopEnabled", true);
         break;
     }
 
@@ -325,6 +381,9 @@ void KWinDesktopConfig::save()
     QDBusMessage message = QDBusMessage::createSignal("/KWin", "org.kde.KWin", "reloadConfig");
     QDBusConnection::sessionBus().send(message);
 
+    QDBusInterface interface("org.kde.plasma-desktop", "/App");
+    interface.call("setPerVirtualDesktopViews", (m_ui->activityCheckBox->isChecked()));
+
     emit changed(false);
 }
 
@@ -351,10 +410,15 @@ QString KWinDesktopConfig::extrapolatedShortcut(int desktop) const
     if (desktop == 1)
         return QString("Ctrl+F1");
 
-    KAction *beforeAction = qobject_cast<KAction*>(m_actionCollection->actions().at(qMin(9, desktop - 2)));
-    QString before = beforeAction->globalShortcut(KAction::ActiveShortcut).toString();
-    if (before.isEmpty())
-        before = beforeAction->globalShortcut(KAction::DefaultShortcut).toString();
+    QAction *beforeAction = m_actionCollection->actions().at(qMin(9, desktop - 2));
+    auto shortcuts = KGlobalAccel::self()->shortcut(beforeAction);
+    if (shortcuts.isEmpty()) {
+        shortcuts = KGlobalAccel::self()->defaultShortcut(beforeAction);
+    }
+    QString before;
+    if (!shortcuts.isEmpty()) {
+        before = shortcuts.first().toString(QKeySequence::NativeText);
+    }
 
     QString seq;
     if (before.contains(QRegExp("F[0-9]{1,2}"))) {
@@ -398,30 +462,26 @@ void KWinDesktopConfig::slotChangeShortcuts(int number)
             // Remove the action from the action collection. The action itself
             // will still exist because that's the way kwin currently works.
             // No need to remove/forget it. See kwinbindings.
-            KAction *a = qobject_cast<KAction*>(
-                             m_actionCollection->takeAction(m_actionCollection->actions().last()));
+            QAction *a = m_actionCollection->takeAction(m_actionCollection->actions().last());
             // Remove any associated global shortcut. Set it to ""
-            a->setGlobalShortcut(
-                KShortcut(),
-                KAction::ActiveShortcut,
-                KAction::NoAutoloading);
+            KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>(), KGlobalAccel::NoAutoloading);
             m_ui->messageLabel->hide();
             delete a;
         } else {
             // add desktop
             int desktop = m_actionCollection->count() + 1;
-            KAction* action = qobject_cast<KAction*>(m_actionCollection->addAction(QString("Switch to Desktop %1").arg(desktop)));
+            QAction* action = m_actionCollection->addAction(QString("Switch to Desktop %1").arg(desktop));
             action->setProperty("isConfigurationAction", true);
             action->setText(i18n("Switch to Desktop %1", desktop));
-            action->setGlobalShortcut(KShortcut(), KAction::ActiveShortcut);
+            KGlobalAccel::self()->setShortcut(action, QList<QKeySequence>());
             QString shortcutString = extrapolatedShortcut(desktop);
             if (shortcutString.isEmpty()) {
                 m_ui->messageLabel->setText(i18n("No suitable Shortcut for Desktop %1 found", desktop));
                 m_ui->messageLabel->show();
             } else {
-                KShortcut shortcut(shortcutString);
-                if (!shortcut.primary().isEmpty() || KGlobalAccel::self()->isGlobalShortcutAvailable(shortcut.primary())) {
-                    action->setGlobalShortcut(shortcut, KAction::ActiveShortcut, KAction::NoAutoloading);
+                QKeySequence shortcut(shortcutString);
+                if (!shortcut.isEmpty() || KGlobalAccel::self()->isGlobalShortcutAvailable(shortcut)) {
+                    KGlobalAccel::self()->setShortcut(action, QList<QKeySequence>(), KGlobalAccel::NoAutoloading);
                     m_ui->messageLabel->setText(i18n("Assigned global Shortcut \"%1\" to Desktop %2", shortcutString, desktop));
                     m_ui->messageLabel->show();
                 } else {
@@ -473,6 +533,12 @@ void KWinDesktopConfig::slotAboutEffectClicked()
     case 1:
         effect = "slide";
         break;
+    case 2:
+        effect = "cubeslide";
+        break;
+    case 3:
+        effect = "fadedesktop";
+        break;
     default:
         return;
     }
@@ -490,7 +556,7 @@ void KWinDesktopConfig::slotAboutEffectClicked()
     const QString license = pluginInfo.license();
     const QString icon    = pluginInfo.icon();
 
-    KAboutData aboutData(name.toUtf8(), name.toUtf8(), ki18n(name.toUtf8()), version.toUtf8(), ki18n(comment.toUtf8()), KAboutLicense::byKeyword(license).key(), ki18n(QByteArray()), ki18n(QByteArray()), website.toLatin1());
+    KAboutData aboutData(name, name, name, version, comment, KAboutLicense::byKeyword(license).key(), QString(), QString(), website.toLatin1());
     aboutData.setProgramIconName(icon);
     const QStringList authors = author.split(',');
     const QStringList emails = email.split(',');
@@ -498,20 +564,50 @@ void KWinDesktopConfig::slotAboutEffectClicked()
     if (authors.count() == emails.count()) {
         foreach (const QString & author, authors) {
             if (!author.isEmpty()) {
-                aboutData.addAuthor(ki18n(author.toUtf8()), ki18n(QByteArray()), emails[i].toUtf8(), 0);
+                aboutData.addAuthor(i18n(author.toUtf8()), QString(), emails[i]);
             }
             i++;
         }
     }
-    QPointer<KAboutApplicationDialog> aboutPlugin = new KAboutApplicationDialog(&aboutData, this);
+    QPointer<KAboutApplicationDialog> aboutPlugin = new KAboutApplicationDialog(aboutData, this);
     aboutPlugin->exec();
     delete aboutPlugin;
 }
 
 void KWinDesktopConfig::slotConfigureEffectClicked()
 {
+    QString effect;
+    switch(m_ui->effectComboBox->currentIndex()) {
+    case 2:
+        effect = "cubeslide_config";
+        break;
+    default:
+        return;
+    }
+    KCModuleProxy* proxy = new KCModuleProxy(effect);
+    QPointer<QDialog> configDialog = new QDialog(this);
+    configDialog->setWindowTitle(m_ui->effectComboBox->currentText());
+    configDialog->setLayout(new QVBoxLayout);
+    QDialogButtonBox* buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel|QDialogButtonBox::RestoreDefaults, configDialog);
+    connect(buttons, SIGNAL(accepted()), configDialog, SLOT(accept()));
+    connect(buttons, SIGNAL(rejected()), configDialog, SLOT(reject()));
+    connect(buttons->button(QDialogButtonBox::RestoreDefaults), SIGNAL(clicked(bool)), proxy, SLOT(defaults()));
+
+    QWidget *showWidget = new QWidget(configDialog);
+    QVBoxLayout *layout = new QVBoxLayout;
+    showWidget->setLayout(layout);
+    layout->addWidget(proxy);
+    configDialog->layout()->addWidget(showWidget);
+    configDialog->layout()->addWidget(buttons);
+
+    if (configDialog->exec() == QDialog::Accepted) {
+        proxy->save();
+    } else {
+        proxy->load();
+    }
+    delete configDialog;
 }
 
 } // namespace
 
-#include "moc_main.cpp"
+#include "main.moc"
