@@ -25,12 +25,9 @@
 
 #include <QtDBus/QtDBus>
 
-#include <kapplication.h>
-#include <kglobal.h>
 #include <KLocalizedString>
 #include <kconfig.h>
 #include <kaboutdata.h>
-#include <kdialog.h>
 #include <KPluginFactory>
 #include <KPluginLoader>
 
@@ -44,7 +41,7 @@ class KFocusConfigStandalone : public KFocusConfig
     Q_OBJECT
 public:
     KFocusConfigStandalone(QWidget* parent, const QVariantList &)
-        : KFocusConfig(true, new KConfig("kwinrc"), KWinOptionsFactory::componentData(), parent)
+        : KFocusConfig(true, new KConfig("kwinrc"), parent)
     {}
 };
 
@@ -53,7 +50,7 @@ class KMovingConfigStandalone : public KMovingConfig
     Q_OBJECT
 public:
     KMovingConfigStandalone(QWidget* parent, const QVariantList &)
-        : KMovingConfig(true, new KConfig("kwinrc"), KWinOptionsFactory::componentData(), parent)
+        : KMovingConfig(true, new KConfig("kwinrc"), parent)
     {}
 };
 
@@ -62,59 +59,59 @@ class KAdvancedConfigStandalone : public KAdvancedConfig
     Q_OBJECT
 public:
     KAdvancedConfigStandalone(QWidget* parent, const QVariantList &)
-        : KAdvancedConfig(true, new KConfig("kwinrc"), KWinOptionsFactory::componentData(), parent)
+        : KAdvancedConfig(true, new KConfig("kwinrc"), parent)
     {}
 };
 
 KWinOptions::KWinOptions(QWidget *parent, const QVariantList &)
-    : KCModule(KWinOptionsFactory::componentData(), parent)
+    : KCModule(parent)
 {
     mConfig = new KConfig("kwinrc");
 
     QVBoxLayout *layout = new QVBoxLayout(this);
     layout->setMargin(0);
-    tab = new KTabWidget(this);
+    tab = new QTabWidget(this);
     layout->addWidget(tab);
 
-    mFocus = new KFocusConfig(false, mConfig, componentData(), this);
+    mFocus = new KFocusConfig(false, mConfig, this);
     mFocus->setObjectName(QLatin1String("KWin Focus Config"));
     tab->addTab(mFocus, i18n("&Focus"));
     connect(mFocus, SIGNAL(changed(bool)), this, SLOT(moduleChanged(bool)));
 
-    mTitleBarActions = new KTitleBarActionsConfig(false, mConfig, componentData(), this);
+    mTitleBarActions = new KTitleBarActionsConfig(false, mConfig, this);
     mTitleBarActions->setObjectName(QLatin1String("KWin TitleBar Actions"));
     tab->addTab(mTitleBarActions, i18n("&Titlebar Actions"));
     connect(mTitleBarActions, SIGNAL(changed(bool)), this, SLOT(moduleChanged(bool)));
 
-    mWindowActions = new KWindowActionsConfig(false, mConfig, componentData(), this);
+    mWindowActions = new KWindowActionsConfig(false, mConfig, this);
     mWindowActions->setObjectName(QLatin1String("KWin Window Actions"));
     tab->addTab(mWindowActions, i18n("Window Actio&ns"));
     connect(mWindowActions, SIGNAL(changed(bool)), this, SLOT(moduleChanged(bool)));
 
-    mMoving = new KMovingConfig(false, mConfig, componentData(), this);
+    mMoving = new KMovingConfig(false, mConfig, this);
     mMoving->setObjectName(QLatin1String("KWin Moving"));
     tab->addTab(mMoving, i18n("&Moving"));
     connect(mMoving, SIGNAL(changed(bool)), this, SLOT(moduleChanged(bool)));
 
-    mAdvanced = new KAdvancedConfig(false, mConfig, componentData(), this);
+    mAdvanced = new KAdvancedConfig(false, mConfig, this);
     mAdvanced->setObjectName(QLatin1String("KWin Advanced"));
     tab->addTab(mAdvanced, i18n("Ad&vanced"));
     connect(mAdvanced, SIGNAL(changed(bool)), this, SLOT(moduleChanged(bool)));
 
     KAboutData *about =
-        new KAboutData(I18N_NOOP("kcmkwinoptions"), 0, ki18n("Window Behavior Configuration Module"),
-                       0, KLocalizedString(), KAboutData::License_GPL,
-                       ki18n("(c) 1997 - 2002 KWin and KControl Authors"));
+        new KAboutData(i18n("kcmkwinoptions"), QString(), i18n("Window Behavior Configuration Module"),
+                       QString(), QString(), KAboutData::License_GPL,
+                       i18n("(c) 1997 - 2002 KWin and KControl Authors"));
 
-    about->addAuthor(ki18n("Matthias Ettrich"), KLocalizedString(), "ettrich@kde.org");
-    about->addAuthor(ki18n("Waldo Bastian"), KLocalizedString(), "bastian@kde.org");
-    about->addAuthor(ki18n("Cristian Tibirna"), KLocalizedString(), "tibirna@kde.org");
-    about->addAuthor(ki18n("Matthias Kalle Dalheimer"), KLocalizedString(), "kalle@kde.org");
-    about->addAuthor(ki18n("Daniel Molkentin"), KLocalizedString(), "molkentin@kde.org");
-    about->addAuthor(ki18n("Wynn Wilkes"), KLocalizedString(), "wynnw@caldera.com");
-    about->addAuthor(ki18n("Pat Dowler"), KLocalizedString(), "dowler@pt1B1106.FSH.UVic.CA");
-    about->addAuthor(ki18n("Bernd Wuebben"), KLocalizedString(), "wuebben@kde.org");
-    about->addAuthor(ki18n("Matthias Hoelzer-Kluepfel"), KLocalizedString(), "hoelzer@kde.org");
+    about->addAuthor(i18n("Matthias Ettrich"), QString(), "ettrich@kde.org");
+    about->addAuthor(i18n("Waldo Bastian"), QString(), "bastian@kde.org");
+    about->addAuthor(i18n("Cristian Tibirna"), QString(), "tibirna@kde.org");
+    about->addAuthor(i18n("Matthias Kalle Dalheimer"), QString(), "kalle@kde.org");
+    about->addAuthor(i18n("Daniel Molkentin"), QString(), "molkentin@kde.org");
+    about->addAuthor(i18n("Wynn Wilkes"), QString(), "wynnw@caldera.com");
+    about->addAuthor(i18n("Pat Dowler"), QString(), "dowler@pt1B1106.FSH.UVic.CA");
+    about->addAuthor(i18n("Bernd Wuebben"), QString(), "wuebben@kde.org");
+    about->addAuthor(i18n("Matthias Hoelzer-Kluepfel"), QString(), "hoelzer@kde.org");
     setAboutData(about);
 }
 
@@ -180,21 +177,21 @@ void KWinOptions::moduleChanged(bool state)
 }
 
 KActionsOptions::KActionsOptions(QWidget *parent, const QVariantList &)
-    : KCModule(KWinOptionsFactory::componentData(), parent)
+    : KCModule(parent)
 {
     mConfig = new KConfig("kwinrc");
 
     QVBoxLayout *layout = new QVBoxLayout(this);
     layout->setMargin(0);
-    tab = new KTabWidget(this);
+    tab = new QTabWidget(this);
     layout->addWidget(tab);
 
-    mTitleBarActions = new KTitleBarActionsConfig(false, mConfig, componentData(), this);
+    mTitleBarActions = new KTitleBarActionsConfig(false, mConfig, this);
     mTitleBarActions->setObjectName(QLatin1String("KWin TitleBar Actions"));
     tab->addTab(mTitleBarActions, i18n("&Titlebar Actions"));
     connect(mTitleBarActions, SIGNAL(changed(bool)), this, SLOT(moduleChanged(bool)));
 
-    mWindowActions = new KWindowActionsConfig(false, mConfig, componentData(), this);
+    mWindowActions = new KWindowActionsConfig(false, mConfig, this);
     mWindowActions->setObjectName(QLatin1String("KWin Window Actions"));
     tab->addTab(mWindowActions, i18n("Window Actio&ns"));
     connect(mWindowActions, SIGNAL(changed(bool)), this, SLOT(moduleChanged(bool)));
@@ -247,7 +244,6 @@ K_PLUGIN_FACTORY_DEFINITION(KWinOptionsFactory,
                             registerPlugin<KAdvancedConfigStandalone>("kwinadvanced");
                             registerPlugin<KWinOptions>("kwinoptions");
                            )
-K_EXPORT_PLUGIN(KWinOptionsFactory("kcmkwm"))
 
 #include "main.moc"
 #include "moc_main.cpp"
