@@ -21,8 +21,6 @@
  *
  */
 
-#include <config-workspace.h>
-
 #include <QApplication>
 #include <QCheckBox>
 #include <QRadioButton>
@@ -32,11 +30,11 @@
 #include <QFormLayout>
 #include <QDesktopWidget>
 #include <QtDBus/QtDBus>
+#include <QGroupBox>
 
-#include <KButtonGroup>
+#include <KConfig>
+#include <KConfigGroup>
 #include <KLocalizedString>
-#include <knuminput.h>
-#include <kdialog.h>
 
 #include <X11/X.h>
 #include <X11/Xlib.h>
@@ -94,8 +92,8 @@ KWinFocusConfigForm::KWinFocusConfigForm(QWidget* parent)
 }
 
 // removed the LCD display over the slider - this is not good GUI design :) RNolden 051701
-KFocusConfig::KFocusConfig(bool _standAlone, KConfig *_config, const KComponentData &inst, QWidget * parent)
-    : KCModule(inst, parent), config(_config), standAlone(_standAlone)
+KFocusConfig::KFocusConfig(bool _standAlone, KConfig *_config, QWidget * parent)
+    : KCModule(parent), config(_config), standAlone(_standAlone)
     , m_ui(new KWinFocusConfigForm(this))
 {
     connect(m_ui->focusStealing, SIGNAL(activated(int)), SLOT(changed()));
@@ -357,8 +355,8 @@ KAdvancedConfig::~KAdvancedConfig()
         delete config;
 }
 
-KAdvancedConfig::KAdvancedConfig(bool _standAlone, KConfig *_config, const KComponentData &inst, QWidget *parent)
-    : KCModule(inst, parent), config(_config), standAlone(_standAlone)
+KAdvancedConfig::KAdvancedConfig(bool _standAlone, KConfig *_config, QWidget *parent)
+    : KCModule(parent), config(_config), standAlone(_standAlone)
     , m_ui(new KWinAdvancedConfigForm(this))
 {
     m_ui->placementCombo->setItemData(0, "Smart");
@@ -511,8 +509,8 @@ KMovingConfig::~KMovingConfig()
         delete config;
 }
 
-KMovingConfig::KMovingConfig(bool _standAlone, KConfig *_config, const KComponentData &inst, QWidget *parent)
-    : KCModule(inst, parent), config(_config), standAlone(_standAlone)
+KMovingConfig::KMovingConfig(bool _standAlone, KConfig *_config, QWidget *parent)
+    : KCModule(parent), config(_config), standAlone(_standAlone)
     , m_ui(new KWinMovingConfigForm(this))
 {
     // Any changes goes to slotChanged()
@@ -642,4 +640,4 @@ void KMovingConfig::setCenterSnapZone(int pxls)
     m_ui->centerSnap->setValue(pxls);
 }
 
-#include "moc_windows.cpp"
+#include "windows.moc"
