@@ -18,12 +18,10 @@
 
 #include "kcm.h"
 
-#include <kglobal.h>
 #include <QLayout>
 //Added by qt3to4:
 #include <QVBoxLayout>
 #include <KLocalizedString>
-#include <kapplication.h>
 #include <kaboutdata.h>
 #include <QtDBus/QtDBus>
 
@@ -34,13 +32,12 @@
 K_PLUGIN_FACTORY(KCMRulesFactory,
                  registerPlugin<KWin::KCMRules>();
                 )
-K_EXPORT_PLUGIN(KCMRulesFactory("kcmkwinrules"))
 
 namespace KWin
 {
 
 KCMRules::KCMRules(QWidget *parent, const QVariantList &)
-    : KCModule(KCMRulesFactory::componentData(), parent)
+    : KCModule(parent)
     , config("kwinrulesrc")
 {
     QVBoxLayout *layout = new QVBoxLayout(this);
@@ -49,10 +46,10 @@ KCMRules::KCMRules(QWidget *parent, const QVariantList &)
     widget = new KCMRulesList(this);
     layout->addWidget(widget);
     connect(widget, SIGNAL(changed(bool)), SLOT(moduleChanged(bool)));
-    KAboutData *about = new KAboutData(I18N_NOOP("kcmkwinrules"), 0,
-                                       ki18n("Window-Specific Settings Configuration Module"),
-                                       0, KLocalizedString(), KAboutData::License_GPL, ki18n("(c) 2004 KWin and KControl Authors"));
-    about->addAuthor(ki18n("Lubos Lunak"), KLocalizedString(), "l.lunak@kde.org");
+    KAboutData *about = new KAboutData(i18n("kcmkwinrules"), QString(),
+                                       i18n("Window-Specific Settings Configuration Module"),
+                                       QString(), QString(), KAboutData::License_GPL, i18n("(c) 2004 KWin and KControl Authors"));
+    about->addAuthor(i18n("Lubos Lunak"), QString(), "l.lunak@kde.org");
     setAboutData(about);
 }
 
@@ -105,4 +102,4 @@ I18N_NOOP("Internal setting for remembering")
 #endif
 
 
-#include "moc_kcm.cpp"
+#include "kcm.moc"
