@@ -19,19 +19,18 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 
-#include "kdebug.h"
-
 #include "main.h"
 
 #include <kservicetypetrader.h>
 
+#include <KAboutData>
 #include <KConfigGroup>
+#include <KLocalizedString>
 #include <KPluginFactory>
 #include <KPluginLoader>
 #include <QtDBus/QtDBus>
 
 K_PLUGIN_FACTORY(KWinScreenEdgesConfigFactory, registerPlugin<KWin::KWinScreenEdgesConfig>();)
-K_EXPORT_PLUGIN(KWinScreenEdgesConfigFactory("kcmkwinscreenedges"))
 
 namespace KWin
 {
@@ -43,7 +42,7 @@ KWinScreenEdgesConfigForm::KWinScreenEdgesConfigForm(QWidget* parent)
 }
 
 KWinScreenEdgesConfig::KWinScreenEdgesConfig(QWidget* parent, const QVariantList& args)
-    : KCModule(KWinScreenEdgesConfigFactory::componentData(), parent, args)
+    : KCModule(parent, args)
     , m_config(KSharedConfig::openConfig("kwinrc"))
 {
     m_ui = new KWinScreenEdgesConfigForm(this);
@@ -185,17 +184,50 @@ void KWinScreenEdgesConfig::monitorItemSetEnabled(int index, bool enabled)
 void KWinScreenEdgesConfig::monitorInit()
 {
     monitorAddItem(i18n("No Action"));
+    monitorAddItem(i18n("Show Dashboard"));
     monitorAddItem(i18n("Show Desktop"));
     monitorAddItem(i18n("Lock Screen"));
+    monitorAddItem(i18n("Prevent Screen Locking"));
+    //Prevent Screen Locking is not supported on some edges
+    m_ui->monitor->setEdgeItemEnabled(int(Monitor::Top), 4, false);
+    m_ui->monitor->setEdgeItemEnabled(int(Monitor::Left), 4, false);
+    m_ui->monitor->setEdgeItemEnabled(int(Monitor::Right), 4, false);
+    m_ui->monitor->setEdgeItemEnabled(int(Monitor::Bottom), 4, false);
 
     // Search the effect names
     KServiceTypeTrader* trader = KServiceTypeTrader::self();
-    KService::List services = trader->query("KWin/Effect", "[X-KDE-PluginInfo-Name] == 'kwin4_effect_presentwindows'");
-    if (!services.isEmpty()) {
+    KService::List services;
+    services = trader->query("KWin/Effect", "[X-KDE-PluginInfo-Name] == 'kwin4_effect_presentwindows'");
+    if (services.isEmpty()) {
+        // adding empty strings in case the effect is not found
+        // TODO: after string freeze add a info that the effect is missing
+        monitorAddItem(QString());
+        monitorAddItem(QString());
+    } else {
         monitorAddItem(services.first()->name() + " - " + i18n("All Desktops"));
         monitorAddItem(services.first()->name() + " - " + i18n("Current Desktop"));
         monitorAddItem(services.first()->name() + " - " + i18n("Current Application"));
     }
+    services = trader->query("KWin/Effect", "[X-KDE-PluginInfo-Name] == 'kwin4_effect_desktopgrid'");
+    if (services.isEmpty()) {
+        // adding empty strings in case the effect is not found
+        // TODO: after string freeze add a info that the effect is missing
+        monitorAddItem(QString());
+    } else {
+        monitorAddItem(services.first()->name());
+    }
+    services = trader->query("KWin/Effect", "[X-KDE-PluginInfo-Name] == 'kwin4_effect_cube'");
+    if (services.isEmpty()) {
+        // adding empty strings in case the effect is not found
+        // TODO: after string freeze add a info that the effect is missing
+        monitorAddItem(QString());
+        monitorAddItem(QString());
+        monitorAddItem(QString());
+    } else {
+        monitorAddItem(services.first()->name() + " - " + i18n("Cube"));
+        monitorAddItem(services.first()->name() + " - " + i18n("Cylinder"));
+        monitorAddItem(services.first()->name() + " - " + i18n("Sphere"));
+    }
 
     monitorAddItem(i18n("Toggle window switching"));
     monitorAddItem(i18n("Toggle alternative window switching"));
@@ -207,8 +239,10 @@ void KWinScreenEdgesConfig::monitorLoadAction(ElectricBorder edge, const QString
 {
     KConfigGroup config(m_config, "ElectricBorders");
     QString lowerName = config.readEntry(configName, "None").toLower();
-    if (lowerName == "showdesktop") monitorChangeEdge(edge, ElectricActionShowDesktop);
-    else if (lowerName == "lockscreen") monitorChangeEdge(edge, ElectricActionLockScreen);
+    if (lowerName == "dashboard") monitorChangeEdge(edge, int(ElectricActionDashboard));
+    else if (lowerName == "showdesktop") monitorChangeEdge(edge, int(ElectricActionShowDesktop));
+    else if (lowerName == "lockscreen") monitorChangeEdge(edge, int(ElectricActionLockScreen));
+    else if (lowerName == "preventscreenlocking") monitorChangeEdge(edge, int(ElectricActionPreventScreenLocking));
 }
 
 void KWinScreenEdgesConfig::monitorLoad()
@@ -229,41 +263,71 @@ void KWinScreenEdgesConfig::monitorLoad()
     KConfigGroup presentWindowsConfig(m_config, "Effect-PresentWindows");
     QList<int> list = QList<int>();
     // PresentWindows BorderActivateAll
-    list.append(ElectricTopLeft);
+    list.append(int(ElectricTopLeft));
     list = presentWindowsConfig.readEntry("BorderActivateAll", list);
     foreach (int i, list) {
-        monitorChangeEdge(ElectricBorder(i), PresentWindowsAll);
+        monitorChangeEdge(ElectricBorder(i), int(PresentWindowsAll));
     }
     // PresentWindows BorderActivate
     list.clear();
-    list.append(ElectricNone);
+    list.append(int(ElectricNone));
     list = presentWindowsConfig.readEntry("BorderActivate", list);
     foreach (int i, list) {
-        monitorChangeEdge(ElectricBorder(i), PresentWindowsCurrent);
+        monitorChangeEdge(ElectricBorder(i), int(PresentWindowsCurrent));
     }
     // PresentWindows BorderActivateClass
     list.clear();
-    list.append(ElectricNone);
+    list.append(int(ElectricNone));
     list = presentWindowsConfig.readEntry("BorderActivateClass", list);
     foreach (int i, list) {
-        monitorChangeEdge(ElectricBorder(i), PresentWindowsClass);
+        monitorChangeEdge(ElectricBorder(i), int(PresentWindowsClass));
+    }
+
+    // Desktop Grid
+    KConfigGroup gridConfig(m_config, "Effect-DesktopGrid");
+    list.clear();
+    list.append(int(ElectricNone));
+    list = gridConfig.readEntry("BorderActivate", list);
+    foreach (int i, list) {
+        monitorChangeEdge(ElectricBorder(i), int(DesktopGrid));
+    }
+
+    // Desktop Cube
+    KConfigGroup cubeConfig(m_config, "Effect-Cube");
+    list.clear();
+    list.append(int(ElectricNone));
+    list = cubeConfig.readEntry("BorderActivate", list);
+    foreach (int i, list) {
+        monitorChangeEdge(ElectricBorder(i), int(Cube));
+    }
+    list.clear();
+    list.append(int(ElectricNone));
+    list = cubeConfig.readEntry("BorderActivateCylinder", list);
+    foreach (int i, list) {
+        monitorChangeEdge(ElectricBorder(i), int(Cylinder));
+    }
+    list.clear();
+    list.append(int(ElectricNone));
+    list = cubeConfig.readEntry("BorderActivateSphere", list);
+    foreach (int i, list) {
+        monitorChangeEdge(ElectricBorder(i), int(Sphere));
     }
 
     // TabBox
     KConfigGroup tabBoxConfig(m_config, "TabBox");
     list.clear();
     // TabBox
-    list.append(ElectricNone);
+    list.append(int(ElectricNone));
     list = tabBoxConfig.readEntry("BorderActivate", list);
     foreach (int i, list) {
-        monitorChangeEdge(ElectricBorder(i), TabBox);
+        monitorChangeEdge(ElectricBorder(i), int(TabBox));
     }
     // Alternative TabBox
     list.clear();
-    list.append(ElectricNone);
+    list.append(int(ElectricNone));
     list = tabBoxConfig.readEntry("BorderAlternativeActivate", list);
     foreach (int i, list) {
-        monitorChangeEdge(ElectricBorder(i), TabBoxAlternative);
+        monitorChangeEdge(ElectricBorder(i), int(TabBoxAlternative));
     }
 }
 
@@ -271,53 +335,81 @@ void KWinScreenEdgesConfig::monitorSaveAction(int edge, const QString& configNam
 {
     KConfigGroup config(m_config, "ElectricBorders");
     int item = m_ui->monitor->selectedEdgeItem(edge);
-    if (item == 1)
-        config.writeEntry(configName, "ShowDesktop");
+    if (item == 1)   // Plasma dashboard
+        config.writeEntry(configName, "Dashboard");
     else if (item == 2)
+        config.writeEntry(configName, "ShowDesktop");
+    else if (item == 3)
         config.writeEntry(configName, "LockScreen");
+    else if (item == 4)
+        config.writeEntry(configName, "PreventScreenLocking");
     else // Anything else
         config.writeEntry(configName, "None");
+
+    if ((edge == int(Monitor::TopRight)) ||
+            (edge == int(Monitor::BottomRight)) ||
+            (edge == int(Monitor::BottomLeft)) ||
+            (edge == int(Monitor::TopLeft))) {
+        KConfig scrnConfig("kscreensaverrc");
+        KConfigGroup scrnGroup = scrnConfig.group("ScreenSaver");
+        scrnGroup.writeEntry("Action" + configName, (item == 4) ? 2 /* Prevent Screen Locking */ : 0 /* None */);
+        scrnGroup.sync();
+    }
 }
 
 void KWinScreenEdgesConfig::monitorSave()
 {
     // Save ElectricBorderActions
-    monitorSaveAction(Monitor::Top,         "Top");
-    monitorSaveAction(Monitor::TopRight,    "TopRight");
-    monitorSaveAction(Monitor::Right,       "Right");
-    monitorSaveAction(Monitor::BottomRight, "BottomRight");
-    monitorSaveAction(Monitor::Bottom,      "Bottom");
-    monitorSaveAction(Monitor::BottomLeft,  "BottomLeft");
-    monitorSaveAction(Monitor::Left,        "Left");
-    monitorSaveAction(Monitor::TopLeft,     "TopLeft");
+    monitorSaveAction(int(Monitor::Top),         "Top");
+    monitorSaveAction(int(Monitor::TopRight),    "TopRight");
+    monitorSaveAction(int(Monitor::Right),       "Right");
+    monitorSaveAction(int(Monitor::BottomRight), "BottomRight");
+    monitorSaveAction(int(Monitor::Bottom),      "Bottom");
+    monitorSaveAction(int(Monitor::BottomLeft),  "BottomLeft");
+    monitorSaveAction(int(Monitor::Left),        "Left");
+    monitorSaveAction(int(Monitor::TopLeft),     "TopLeft");
 
     // Save effect-specific actions:
 
     // Present Windows
     KConfigGroup presentWindowsConfig(m_config, "Effect-PresentWindows");
     presentWindowsConfig.writeEntry("BorderActivateAll",
-                                    monitorCheckEffectHasEdge(PresentWindowsAll));
+                                    monitorCheckEffectHasEdge(int(PresentWindowsAll)));
     presentWindowsConfig.writeEntry("BorderActivate",
-                                    monitorCheckEffectHasEdge(PresentWindowsCurrent));
+                                    monitorCheckEffectHasEdge(int(PresentWindowsCurrent)));
     presentWindowsConfig.writeEntry("BorderActivateClass",
-                                    monitorCheckEffectHasEdge(PresentWindowsClass));
+                                    monitorCheckEffectHasEdge(int(PresentWindowsClass)));
+
+    // Desktop Grid
+    KConfigGroup gridConfig(m_config, "Effect-DesktopGrid");
+    gridConfig.writeEntry("BorderActivate",
+                          monitorCheckEffectHasEdge(int(DesktopGrid)));
+
+    // Desktop Cube
+    KConfigGroup cubeConfig(m_config, "Effect-Cube");
+    cubeConfig.writeEntry("BorderActivate",
+                          monitorCheckEffectHasEdge(int(Cube)));
+    cubeConfig.writeEntry("BorderActivateCylinder",
+                          monitorCheckEffectHasEdge(int(Cylinder)));
+    cubeConfig.writeEntry("BorderActivateSphere",
+                          monitorCheckEffectHasEdge(int(Sphere)));
 
     // TabBox
     KConfigGroup tabBoxConfig(m_config, "TabBox");
     tabBoxConfig.writeEntry("BorderActivate",
-                                monitorCheckEffectHasEdge(TabBox));
+                                monitorCheckEffectHasEdge(int(TabBox)));
     tabBoxConfig.writeEntry("BorderAlternativeActivate",
-                                monitorCheckEffectHasEdge(TabBoxAlternative));
+                                monitorCheckEffectHasEdge(int(TabBoxAlternative)));
 }
 
 void KWinScreenEdgesConfig::monitorDefaults()
 {
     // Clear all edges
     for (int i = 0; i < 8; i++)
-        m_ui->monitor->selectEdgeItem(i, 1); // the index will be deduced to 0
+        m_ui->monitor->selectEdgeItem(i, 0);
 
     // Present windows = Top-left
-    m_ui->monitor->selectEdgeItem(Monitor::TopLeft, PresentWindowsAll);
+    m_ui->monitor->selectEdgeItem(int(Monitor::TopLeft), int(PresentWindowsAll));
 }
 
 void KWinScreenEdgesConfig::monitorShowEvent()
@@ -330,46 +422,60 @@ void KWinScreenEdgesConfig::monitorShowEvent()
 
         // Present Windows
         bool enabled = effectEnabled("presentwindows", config);
-        monitorItemSetEnabled(PresentWindowsCurrent, enabled);
-        monitorItemSetEnabled(PresentWindowsAll, enabled);
+        monitorItemSetEnabled(int(PresentWindowsCurrent), enabled);
+        monitorItemSetEnabled(int(PresentWindowsAll), enabled);
+
+        // Desktop Grid
+        enabled = effectEnabled("desktopgrid", config);
+        monitorItemSetEnabled(int(DesktopGrid), enabled);
+
+        // Desktop Cube
+        enabled = effectEnabled("cube", config);
+        monitorItemSetEnabled(int(Cube), enabled);
+        monitorItemSetEnabled(int(Cylinder), enabled);
+        monitorItemSetEnabled(int(Sphere), enabled);
     } else { // Compositing disabled
-        monitorItemSetEnabled(PresentWindowsCurrent, false);
-        monitorItemSetEnabled(PresentWindowsAll, false);
+        monitorItemSetEnabled(int(PresentWindowsCurrent), false);
+        monitorItemSetEnabled(int(PresentWindowsAll), false);
+        monitorItemSetEnabled(int(DesktopGrid), false);
+        monitorItemSetEnabled(int(Cube), false);
+        monitorItemSetEnabled(int(Cylinder), false);
+        monitorItemSetEnabled(int(Sphere), false);
     }
     // tabbox, depends on reasonable focus policy.
     KConfigGroup config2(m_config, "Windows");
     QString focusPolicy = config2.readEntry("FocusPolicy", QString());
     bool reasonable = focusPolicy != "FocusStrictlyUnderMouse" && focusPolicy != "FocusUnderMouse";
-    monitorItemSetEnabled(TabBox, reasonable);
-    monitorItemSetEnabled(TabBoxAlternative, reasonable);
+    monitorItemSetEnabled(int(TabBox), reasonable);
+    monitorItemSetEnabled(int(TabBoxAlternative), reasonable);
 }
 
 void KWinScreenEdgesConfig::monitorChangeEdge(ElectricBorder border, int index)
 {
     switch(border) {
     case ElectricTop:
-        m_ui->monitor->selectEdgeItem(Monitor::Top, index);
+        m_ui->monitor->selectEdgeItem(int(Monitor::Top), index);
         break;
     case ElectricTopRight:
-        m_ui->monitor->selectEdgeItem(Monitor::TopRight, index);
+        m_ui->monitor->selectEdgeItem(int(Monitor::TopRight), index);
         break;
     case ElectricRight:
-        m_ui->monitor->selectEdgeItem(Monitor::Right, index);
+        m_ui->monitor->selectEdgeItem(int(Monitor::Right), index);
         break;
     case ElectricBottomRight:
-        m_ui->monitor->selectEdgeItem(Monitor::BottomRight, index);
+        m_ui->monitor->selectEdgeItem(int(Monitor::BottomRight), index);
         break;
     case ElectricBottom:
-        m_ui->monitor->selectEdgeItem(Monitor::Bottom, index);
+        m_ui->monitor->selectEdgeItem(int(Monitor::Bottom), index);
         break;
     case ElectricBottomLeft:
-        m_ui->monitor->selectEdgeItem(Monitor::BottomLeft, index);
+        m_ui->monitor->selectEdgeItem(int(Monitor::BottomLeft), index);
         break;
     case ElectricLeft:
-        m_ui->monitor->selectEdgeItem(Monitor::Left, index);
+        m_ui->monitor->selectEdgeItem(int(Monitor::Left), index);
         break;
     case ElectricTopLeft:
-        m_ui->monitor->selectEdgeItem(Monitor::TopLeft, index);
+        m_ui->monitor->selectEdgeItem(int(Monitor::TopLeft), index);
         break;
     default: // Nothing
         break;
@@ -380,28 +486,28 @@ void KWinScreenEdgesConfig::monitorHideEdge(ElectricBorder border, bool hidden)
 {
     switch(border) {
     case ElectricTop:
-        m_ui->monitor->setEdgeHidden(Monitor::Top, hidden);
+        m_ui->monitor->setEdgeHidden(int(Monitor::Top), hidden);
         break;
     case ElectricTopRight:
-        m_ui->monitor->setEdgeHidden(Monitor::TopRight, hidden);
+        m_ui->monitor->setEdgeHidden(int(Monitor::TopRight), hidden);
         break;
     case ElectricRight:
-        m_ui->monitor->setEdgeHidden(Monitor::Right, hidden);
+        m_ui->monitor->setEdgeHidden(int(Monitor::Right), hidden);
         break;
     case ElectricBottomRight:
-        m_ui->monitor->setEdgeHidden(Monitor::BottomRight, hidden);
+        m_ui->monitor->setEdgeHidden(int(Monitor::BottomRight), hidden);
         break;
     case ElectricBottom:
-        m_ui->monitor->setEdgeHidden(Monitor::Bottom, hidden);
+        m_ui->monitor->setEdgeHidden(int(Monitor::Bottom), hidden);
         break;
     case ElectricBottomLeft:
-        m_ui->monitor->setEdgeHidden(Monitor::BottomLeft, hidden);
+        m_ui->monitor->setEdgeHidden(int(Monitor::BottomLeft), hidden);
         break;
     case ElectricLeft:
-        m_ui->monitor->setEdgeHidden(Monitor::Left, hidden);
+        m_ui->monitor->setEdgeHidden(int(Monitor::Left), hidden);
         break;
     case ElectricTopLeft:
-        m_ui->monitor->setEdgeHidden(Monitor::TopLeft, hidden);
+        m_ui->monitor->setEdgeHidden(int(Monitor::TopLeft), hidden);
         break;
     default: // Nothing
         break;
@@ -411,21 +517,21 @@ void KWinScreenEdgesConfig::monitorHideEdge(ElectricBorder border, bool hidden)
 QList<int> KWinScreenEdgesConfig::monitorCheckEffectHasEdge(int index) const
 {
     QList<int> list = QList<int>();
-    if (m_ui->monitor->selectedEdgeItem(Monitor::Top) == index)
+    if (m_ui->monitor->selectedEdgeItem(int(Monitor::Top)) == index)
         list.append(int(ElectricTop));
-    if (m_ui->monitor->selectedEdgeItem(Monitor::TopRight) == index)
+    if (m_ui->monitor->selectedEdgeItem(int(Monitor::TopRight)) == index)
         list.append(int(ElectricTopRight));
-    if (m_ui->monitor->selectedEdgeItem(Monitor::Right) == index)
+    if (m_ui->monitor->selectedEdgeItem(int(Monitor::Right)) == index)
         list.append(int(ElectricRight));
-    if (m_ui->monitor->selectedEdgeItem(Monitor::BottomRight) == index)
+    if (m_ui->monitor->selectedEdgeItem(int(Monitor::BottomRight)) == index)
         list.append(int(ElectricBottomRight));
-    if (m_ui->monitor->selectedEdgeItem(Monitor::Bottom) == index)
+    if (m_ui->monitor->selectedEdgeItem(int(Monitor::Bottom)) == index)
         list.append(int(ElectricBottom));
-    if (m_ui->monitor->selectedEdgeItem(Monitor::BottomLeft) == index)
+    if (m_ui->monitor->selectedEdgeItem(int(Monitor::BottomLeft)) == index)
         list.append(int(ElectricBottomLeft));
-    if (m_ui->monitor->selectedEdgeItem(Monitor::Left) == index)
+    if (m_ui->monitor->selectedEdgeItem(int(Monitor::Left)) == index)
         list.append(int(ElectricLeft));
-    if (m_ui->monitor->selectedEdgeItem(Monitor::TopLeft) == index)
+    if (m_ui->monitor->selectedEdgeItem(int(Monitor::TopLeft)) == index)
         list.append(int(ElectricTopLeft));
 
     if (list.isEmpty())
@@ -435,4 +541,4 @@ QList<int> KWinScreenEdgesConfig::monitorCheckEffectHasEdge(int index) const
 
 } // namespace
 
-#include "moc_main.cpp"
+#include "main.moc"
