@@ -28,7 +28,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "ui_main.h"
 
-#include <QShowEvent>
+class QShowEvent;
 
 namespace KWin
 {
@@ -49,14 +49,14 @@ public:
     explicit KWinScreenEdgesConfig(QWidget* parent, const QVariantList& args);
     ~KWinScreenEdgesConfig();
 
-public slots:
+public Q_SLOTS:
     virtual void groupChanged();
     virtual void save();
     virtual void load();
     virtual void defaults();
 protected:
     virtual void showEvent(QShowEvent* e);
-private slots:
+private Q_SLOTS:
     void sanitizeCooldown();
 private:
     KWinScreenEdgesConfigForm* m_ui;
@@ -64,11 +64,14 @@ private:
 
     enum EffectActions {
         PresentWindowsAll = ELECTRIC_ACTION_COUNT, // Start at the end of built in actions
-        PresentWindowsCurrent = 4,
-        PresentWindowsClass = 5,
-        DesktopGrid = 6,
-        TabBox = 7,
-        TabBoxAlternative = 8
+        PresentWindowsCurrent,
+        PresentWindowsClass,
+        DesktopGrid,
+        Cube,
+        Cylinder,
+        Sphere,
+        TabBox,
+        TabBoxAlternative
     };
 
     bool effectEnabled(const QString& effect, const KConfigGroup& cfg) const;
