@@ -23,28 +23,28 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QtDBus/QtDBus>
 #include <QDesktopWidget>
 #include <QVBoxLayout>
-#include <QtCore/qdebug.h>
+#include <QDialogButtonBox>
+#include <QStandardPaths>
+#include <QPointer>
 
 // KDE
-#include <KAction>
 #include <KActionCollection>
 #include <KCModuleProxy>
-//#include <KLocalizedString>
+#include <KGlobalAccel>
+#include <KLocalizedString>
 #include <KPluginFactory>
 #include <KPluginInfo>
 #include <KPluginLoader>
-#include <KTabWidget>
 #include <KTitleWidget>
 #include <KServiceTypeTrader>
 #include <KShortcutsEditor>
-#include <KStandardDirs>
+#include <KNewStuff3/KNS3/DownloadDialog>
 
 // own
 #include "tabboxconfig.h"
 #include "layoutpreview.h"
 
 K_PLUGIN_FACTORY(KWinTabBoxConfigFactory, registerPlugin<KWin::KWinTabBoxConfig>();)
-K_EXPORT_PLUGIN(KWinTabBoxConfigFactory("kcm_kwintabbox"))
 
 namespace KWin
 {
@@ -58,12 +58,10 @@ KWinTabBoxConfigForm::KWinTabBoxConfigForm(QWidget* parent)
 }
 
 KWinTabBoxConfig::KWinTabBoxConfig(QWidget* parent, const QVariantList& args)
-    : KCModule(KWinTabBoxConfigFactory::componentData(), parent, args)
+    : KCModule(parent, args)
     , m_config(KSharedConfig::openConfig("kwinrc"))
-    , m_layoutPreview(NULL)
 {
-    KGlobal::locale()->insertCatalog("kwin_effects");
-    KTabWidget* tabWidget = new KTabWidget(this);
+    QTabWidget* tabWidget = new QTabWidget(this);
     m_primaryTabBoxUi = new KWinTabBoxConfigForm(tabWidget);
     m_alternativeTabBoxUi = new KWinTabBoxConfigForm(tabWidget);
     tabWidget->addTab(m_primaryTabBoxUi, i18n("Main"));
@@ -78,36 +76,37 @@ KWinTabBoxConfig::KWinTabBoxConfig(QWidget* parent, const QVariantList& args)
     setLayout(layout);
 
 #define ADD_SHORTCUT(_NAME_, _CUT_, _BTN_) \
-    a = qobject_cast<KAction*>(m_actionCollection->addAction(_NAME_));\
+    a = m_actionCollection->addAction(_NAME_);\
     a->setProperty("isConfigurationAction", true);\
     _BTN_->setProperty("shortcutAction", _NAME_);\
     a->setText(i18n(_NAME_));\
-    a->setGlobalShortcut(KShortcut(_CUT_)); \
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << _CUT_); \
     connect(_BTN_, SIGNAL(keySequenceChanged(QKeySequence)), SLOT(shortcutChanged(QKeySequence)))
 
     // Shortcut config. The shortcut belongs to the component "kwin"!
-    m_actionCollection = new KActionCollection(this, KComponentData("kwin"));
+    m_actionCollection = new KActionCollection(this, QStringLiteral("kwin"));
     m_actionCollection->setConfigGroup("Navigation");
     m_actionCollection->setConfigGlobal(true);
-    KAction* a;
+    QAction* a;
     ADD_SHORTCUT("Walk Through Windows", Qt::ALT + Qt::Key_Tab, m_primaryTabBoxUi->scAll);
     ADD_SHORTCUT("Walk Through Windows (Reverse)", Qt::ALT + Qt::SHIFT + Qt::Key_Backtab,
                  m_primaryTabBoxUi->scAllReverse);
-    ADD_SHORTCUT("Walk Through Windows Alternative", , m_alternativeTabBoxUi->scAll);
-    ADD_SHORTCUT("Walk Through Windows Alternative (Reverse)", ,m_alternativeTabBoxUi->scAllReverse);
+    ADD_SHORTCUT("Walk Through Windows Alternative", QKeySequence(), m_alternativeTabBoxUi->scAll);
+    ADD_SHORTCUT("Walk Through Windows Alternative (Reverse)", QKeySequence(), m_alternativeTabBoxUi->scAllReverse);
     ADD_SHORTCUT("Walk Through Windows of Current Application", Qt::ALT + Qt::Key_QuoteLeft,
                  m_primaryTabBoxUi->scCurrent);
     ADD_SHORTCUT("Walk Through Windows of Current Application (Reverse)", Qt::ALT + Qt::Key_AsciiTilde,
                  m_primaryTabBoxUi->scCurrentReverse);
-    ADD_SHORTCUT("Walk Through Windows of Current Application Alternative", , m_alternativeTabBoxUi->scCurrent);
-    ADD_SHORTCUT("Walk Through Windows of Current Application Alternative (Reverse)", ,
+    ADD_SHORTCUT("Walk Through Windows of Current Application Alternative", QKeySequence(), m_alternativeTabBoxUi->scCurrent);
+    ADD_SHORTCUT("Walk Through Windows of Current Application Alternative (Reverse)", QKeySequence(),
                  m_alternativeTabBoxUi->scCurrentReverse);
 #undef ADD_SHORTCUT
 
     initLayoutLists();
     KWinTabBoxConfigForm *ui[2] = { m_primaryTabBoxUi, m_alternativeTabBoxUi };
     for (int i = 0; i < 2; ++i) {
-        ui[i]->effectConfigButton->setIcon(KIcon("view-preview"));
+        ui[i]->effectConfigButton->setIcon(QIcon::fromTheme("view-preview"));
+        ui[i]->ghns->setIcon(QIcon::fromTheme("get-hot-new-stuff"));
 
         connect(ui[i]->highlightWindowCheck, SIGNAL(clicked(bool)), SLOT(changed()));
         connect(ui[i]->showTabBox, SIGNAL(clicked(bool)), SLOT(tabBoxToggled(bool)));
@@ -122,6 +121,10 @@ KWinTabBoxConfig::KWinTabBoxConfig(QWidget* parent, const QVariantList& args)
         connect(ui[i]->currentDesktop, SIGNAL(clicked(bool)), SLOT(changed()));
         connect(ui[i]->otherDesktops, SIGNAL(clicked(bool)), SLOT(changed()));
 
+        connect(ui[i]->filterActivities, SIGNAL(clicked(bool)), SLOT(changed()));
+        connect(ui[i]->currentActivity, SIGNAL(clicked(bool)), SLOT(changed()));
+        connect(ui[i]->otherActivities, SIGNAL(clicked(bool)), SLOT(changed()));
+
         connect(ui[i]->filterScreens, SIGNAL(clicked(bool)), SLOT(changed()));
         if (QApplication::desktop()->screenCount() < 2) {
             ui[i]->filterScreens->hide();
@@ -135,6 +138,7 @@ KWinTabBoxConfig::KWinTabBoxConfig(QWidget* parent, const QVariantList& args)
         connect(ui[i]->filterMinimization, SIGNAL(clicked(bool)), SLOT(changed()));
         connect(ui[i]->visibleWindows, SIGNAL(clicked(bool)), SLOT(changed()));
         connect(ui[i]->hiddenWindows, SIGNAL(clicked(bool)), SLOT(changed()));
+        connect(ui[i]->ghns, SIGNAL(clicked(bool)), SLOT(slotGHNS()));
     }
 
     // check focus policy - we don't offer configs for unreasonable focus policies
@@ -156,10 +160,19 @@ void KWinTabBoxConfig::initLayoutLists()
     // search the effect names
     // TODO: way to recognize if a effect is not found
     KServiceTypeTrader* trader = KServiceTypeTrader::self();
+    KService::List services;
+    QString coverswitch;
+    QString flipswitch;
+    services = trader->query("KWin/Effect", "[X-KDE-PluginInfo-Name] == 'kwin4_effect_coverswitch'");
+    if (!services.isEmpty())
+        coverswitch = services.first()->name();
+    services = trader->query("KWin/Effect", "[X-KDE-PluginInfo-Name] == 'kwin4_effect_flipswitch'");
+    if (!services.isEmpty())
+        flipswitch = services.first()->name();
 
     KService::List offers = trader->query("KWin/WindowSwitcher");
     QStringList layoutNames, layoutPlugins, layoutPaths;
-    foreach (const KService::Ptr service, offers) {
+    foreach (KService::Ptr service, offers) {
         const QString pluginName = service->property("X-KDE-PluginInfo-Name").toString();
         if (service->property("X-Plasma-API").toString() != "declarativeappletscript") {
             continue;
@@ -168,7 +181,9 @@ void KWinTabBoxConfig::initLayoutLists()
             continue;
         }
         const QString scriptName = service->property("X-Plasma-MainScript").toString();
-        const QString scriptFile = KStandardDirs::locate("data", "kwin/tabbox/" + pluginName + "/contents/" + scriptName);
+        const QString scriptFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation,
+                                                          "kwin/tabbox/" + pluginName + "/contents/"
+                                                          + scriptName);
         if (scriptFile.isNull()) {
             continue;
         }
@@ -183,6 +198,8 @@ void KWinTabBoxConfig::initLayoutLists()
         int index = ui[i]->effectCombo->currentIndex();
         QVariant data = ui[i]->effectCombo->itemData(index);
         ui[i]->effectCombo->clear();
+        ui[i]->effectCombo->addItem(coverswitch);
+        ui[i]->effectCombo->addItem(flipswitch);
         for (int j = 0; j < layoutNames.count(); ++j) {
             ui[i]->effectCombo->addItem(layoutNames[j], layoutPlugins[j]);
             ui[i]->effectCombo->setItemData(ui[i]->effectCombo->count() - 1, layoutPaths[j], Qt::UserRole+1);
@@ -209,12 +226,21 @@ void KWinTabBoxConfig::load()
 
         updateUiFromConfig(ui[i], *(tabBoxConfig[i]));
 
+        KConfigGroup effectconfig(m_config, "Plugins");
+        if (effectEnabled("coverswitch", effectconfig) && KConfigGroup(m_config, "Effect-CoverSwitch").readEntry(group[i], false))
+            ui[i]->effectCombo->setCurrentIndex(CoverSwitch);
+        else if (effectEnabled("flipswitch", effectconfig) && KConfigGroup(m_config, "Effect-FlipSwitch").readEntry(group[i], false))
+            ui[i]->effectCombo->setCurrentIndex(FlipSwitch);
+
         QString action;
 #define LOAD_SHORTCUT(_BTN_)\
         action = ui[i]->_BTN_->property("shortcutAction").toString();\
         qDebug() << "load shortcut for " << action;\
-        if (KAction *a = qobject_cast<KAction*>(m_actionCollection->action(action)))\
-            ui[i]->_BTN_->setKeySequence(a->globalShortcut().primary())
+        if (QAction *a = m_actionCollection->action(action)) { \
+            auto shortcuts = KGlobalAccel::self()->shortcut(a); \
+            if (!shortcuts.isEmpty()) \
+                ui[i]->_BTN_->setKeySequence(shortcuts.first()); \
+        }
         LOAD_SHORTCUT(scAll);
         LOAD_SHORTCUT(scAllReverse);
         LOAD_SHORTCUT(scCurrent);
@@ -228,6 +254,8 @@ void KWinTabBoxConfig::loadConfig(const KConfigGroup& config, KWin::TabBox::TabB
 {
     tabBoxConfig.setClientDesktopMode(TabBoxConfig::ClientDesktopMode(
                                        config.readEntry<int>("DesktopMode", TabBoxConfig::defaultDesktopMode())));
+    tabBoxConfig.setClientActivitiesMode(TabBoxConfig::ClientActivitiesMode(
+                                       config.readEntry<int>("ActivitiesMode", TabBoxConfig::defaultActivitiesMode())));
     tabBoxConfig.setClientApplicationsMode(TabBoxConfig::ClientApplicationsMode(
                                        config.readEntry<int>("ApplicationsMode", TabBoxConfig::defaultApplicationsMode())));
     tabBoxConfig.setClientMinimizedMode(TabBoxConfig::ClientMinimizedMode(
@@ -249,6 +277,7 @@ void KWinTabBoxConfig::saveConfig(KConfigGroup& config, const KWin::TabBox::TabB
 {
     // combo boxes
     config.writeEntry("DesktopMode",        int(tabBoxConfig.clientDesktopMode()));
+    config.writeEntry("ActivitiesMode",     int(tabBoxConfig.clientActivitiesMode()));
     config.writeEntry("ApplicationsMode",   int(tabBoxConfig.clientApplicationsMode()));
     config.writeEntry("MinimizedMode",      int(tabBoxConfig.clientMinimizedMode()));
     config.writeEntry("ShowDesktopMode",    int(tabBoxConfig.showDesktopMode()));
@@ -278,12 +307,32 @@ void KWinTabBoxConfig::save()
     // effects
     bool highlightWindows = m_primaryTabBoxUi->highlightWindowCheck->isChecked() ||
                             m_alternativeTabBoxUi->highlightWindowCheck->isChecked();
+    const bool coverSwitch = m_primaryTabBoxUi->showTabBox->isChecked() &&
+                             m_primaryTabBoxUi->effectCombo->currentIndex() == CoverSwitch;
+    const bool flipSwitch = m_primaryTabBoxUi->showTabBox->isChecked() &&
+                            m_primaryTabBoxUi->effectCombo->currentIndex() == FlipSwitch;
+    const bool coverSwitchAlternative = m_alternativeTabBoxUi->showTabBox->isChecked() &&
+                                        m_alternativeTabBoxUi->effectCombo->currentIndex() == CoverSwitch;
+    const bool flipSwitchAlternative = m_alternativeTabBoxUi->showTabBox->isChecked() &&
+                                       m_alternativeTabBoxUi->effectCombo->currentIndex() == FlipSwitch;
 
     // activate effects if not active
     KConfigGroup effectconfig(m_config, "Plugins");
+    if (coverSwitch || coverSwitchAlternative)
+        effectconfig.writeEntry("kwin4_effect_coverswitchEnabled", true);
+    if (flipSwitch || flipSwitchAlternative)
+        effectconfig.writeEntry("kwin4_effect_flipswitchEnabled", true);
     if (highlightWindows)
         effectconfig.writeEntry("kwin4_effect_highlightwindowEnabled", true);
     effectconfig.sync();
+    KConfigGroup coverswitchconfig(m_config, "Effect-CoverSwitch");
+    coverswitchconfig.writeEntry("TabBox", coverSwitch);
+    coverswitchconfig.writeEntry("TabBoxAlternative", coverSwitchAlternative);
+    coverswitchconfig.sync();
+    KConfigGroup flipswitchconfig(m_config, "Effect-FlipSwitch");
+    flipswitchconfig.writeEntry("TabBox", flipSwitch);
+    flipswitchconfig.writeEntry("TabBoxAlternative", flipSwitchAlternative);
+    flipswitchconfig.sync();
 
     // Reload KWin.
     QDBusMessage message = QDBusMessage::createSignal("/KWin", "org.kde.KWin", "reloadConfig");
@@ -302,6 +351,9 @@ void KWinTabBoxConfig::defaults()
         CONFIGURE(filterDesktops, Desktop, !=, AllDesktopsClients);
         CONFIGURE(currentDesktop, Desktop, ==, OnlyCurrentDesktopClients);
         CONFIGURE(otherDesktops, Desktop, ==, ExcludeCurrentDesktopClients);
+        CONFIGURE(filterActivities, Activities, !=, AllActivitiesClients);
+        CONFIGURE(currentActivity, Activities, ==, OnlyCurrentActivityClients);
+        CONFIGURE(otherActivities, Activities, ==, ExcludeCurrentActivityClients);
         CONFIGURE(filterScreens, MultiScreen, !=, IgnoreMultiScreen);
         CONFIGURE(currentScreen, MultiScreen, ==, OnlyCurrentScreenClients);
         CONFIGURE(otherScreens, MultiScreen, ==, ExcludeCurrentScreenClients);
@@ -322,20 +374,20 @@ void KWinTabBoxConfig::defaults()
     }
 
     QString action;
-#define RESET_SHORTCUT(_BTN_, _CUT_) \
-    action = _BTN_->property("shortcutAction").toString(); \
-    if (KAction *a = qobject_cast<KAction*>(m_actionCollection->action(action))) \
-        a->setGlobalShortcut(KShortcut(_CUT_), KAction::ActiveShortcut, KAction::NoAutoloading)
+    auto RESET_SHORTCUT = [this](KKeySequenceWidget *widget, const QKeySequence &sequence = QKeySequence()) {
+        const QString action = widget->property("shortcutAction").toString();
+        QAction *a = m_actionCollection->action(action);
+        KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << sequence, KGlobalAccel::NoAutoloading);
+    };
     RESET_SHORTCUT(m_primaryTabBoxUi->scAll, Qt::ALT + Qt::Key_Tab);
     RESET_SHORTCUT(m_primaryTabBoxUi->scAllReverse, Qt::ALT + Qt::SHIFT + Qt::Key_Backtab);
-    RESET_SHORTCUT(m_alternativeTabBoxUi->scAll, );
-    RESET_SHORTCUT(m_alternativeTabBoxUi->scAllReverse, );
+    RESET_SHORTCUT(m_alternativeTabBoxUi->scAll);
+    RESET_SHORTCUT(m_alternativeTabBoxUi->scAllReverse);
     RESET_SHORTCUT(m_primaryTabBoxUi->scCurrent, Qt::ALT + Qt::Key_QuoteLeft);
     RESET_SHORTCUT(m_primaryTabBoxUi->scCurrentReverse, Qt::ALT + Qt::Key_AsciiTilde);
-    RESET_SHORTCUT(m_alternativeTabBoxUi->scCurrent, );
-    RESET_SHORTCUT(m_alternativeTabBoxUi->scCurrentReverse, );
+    RESET_SHORTCUT(m_alternativeTabBoxUi->scCurrent);
+    RESET_SHORTCUT(m_alternativeTabBoxUi->scCurrentReverse);
     m_actionCollection->writeSettings();
-#undef RESET_SHORTCUT
     emit changed(true);
 }
 
@@ -355,6 +407,9 @@ void KWinTabBoxConfig::updateUiFromConfig(KWinTabBoxConfigForm* ui, const KWin::
     CONFIGURE(filterDesktops, clientDesktop, !=, AllDesktopsClients);
     CONFIGURE(currentDesktop, clientDesktop, ==, OnlyCurrentDesktopClients);
     CONFIGURE(otherDesktops, clientDesktop, ==, ExcludeCurrentDesktopClients);
+    CONFIGURE(filterActivities, clientActivities, !=, AllActivitiesClients);
+    CONFIGURE(currentActivity, clientActivities, ==, OnlyCurrentActivityClients);
+    CONFIGURE(otherActivities, clientActivities, ==, ExcludeCurrentActivityClients);
     CONFIGURE(filterScreens, clientMultiScreen, !=, IgnoreMultiScreen);
     CONFIGURE(currentScreen, clientMultiScreen, ==, OnlyCurrentScreenClients);
     CONFIGURE(otherScreens, clientMultiScreen, ==, ExcludeCurrentScreenClients);
@@ -379,6 +434,10 @@ void KWinTabBoxConfig::updateConfigFromUi(const KWin::KWinTabBoxConfigForm* ui,
         config.setClientDesktopMode(ui->currentDesktop->isChecked() ? TabBoxConfig::OnlyCurrentDesktopClients : TabBoxConfig::ExcludeCurrentDesktopClients);
     else
         config.setClientDesktopMode(TabBoxConfig::AllDesktopsClients);
+    if (ui->filterActivities->isChecked())
+        config.setClientActivitiesMode(ui->currentActivity->isChecked() ? TabBoxConfig::OnlyCurrentActivityClients : TabBoxConfig::ExcludeCurrentActivityClients);
+    else
+        config.setClientActivitiesMode(TabBoxConfig::AllActivitiesClients);
     if (ui->filterScreens->isChecked())
         config.setClientMultiScreenMode(ui->currentScreen->isChecked() ? TabBoxConfig::OnlyCurrentScreenClients : TabBoxConfig::ExcludeCurrentScreenClients);
     else
@@ -393,13 +452,15 @@ void KWinTabBoxConfig::updateConfigFromUi(const KWin::KWinTabBoxConfigForm* ui,
 
     config.setShowTabBox(ui->showTabBox->isChecked());
     config.setHighlightWindows(ui->highlightWindowCheck->isChecked());
-    config.setLayoutName(ui->effectCombo->itemData(ui->effectCombo->currentIndex()).toString());
+    if (ui->effectCombo->currentIndex() >= Layout) {
+        config.setLayoutName(ui->effectCombo->itemData(ui->effectCombo->currentIndex()).toString());
+    }
     config.setShowDesktopMode(ui->showDesktop->isChecked() ? TabBoxConfig::ShowDesktopClient : TabBoxConfig::DoNotShowDesktopClient);
 }
 
 #define CHECK_CURRENT_TABBOX_UI \
     Q_ASSERT(sender());\
-    KWinTabBoxConfigForm *ui = 0;\
+    KWinTabBoxConfigForm *ui = nullptr;\
     QObject *dad = sender();\
     while (!ui && (dad = dad->parent()))\
         ui = qobject_cast<KWinTabBoxConfigForm*>(dad);\
@@ -408,18 +469,15 @@ void KWinTabBoxConfig::updateConfigFromUi(const KWin::KWinTabBoxConfigForm* ui,
 void KWinTabBoxConfig::effectSelectionChanged(int index)
 {
     CHECK_CURRENT_TABBOX_UI
-    ui->effectConfigButton->setIcon(KIcon("view-preview"));
+    ui->effectConfigButton->setIcon(QIcon::fromTheme(index < Layout ? "configure" : "view-preview"));
     if (!ui->showTabBox->isChecked())
         return;
-    ui->highlightWindowCheck->setEnabled(index);
-    if (m_layoutPreview && m_layoutPreview->isVisible()) {
-        m_layoutPreview->setLayout(ui->effectCombo->itemData(index, Qt::UserRole+1).toString(), ui->effectCombo->itemText(index));
-    }
+    ui->highlightWindowCheck->setEnabled(index >= Layout);
 }
 
 void KWinTabBoxConfig::tabBoxToggled(bool on) {
     CHECK_CURRENT_TABBOX_UI
-    on = !on || ui->effectCombo->currentIndex() >= 0;
+    on = !on || ui->effectCombo->currentIndex() >= Layout;
     ui->highlightWindowCheck->setEnabled(on);
     emit changed();
 }
@@ -429,13 +487,34 @@ void KWinTabBoxConfig::configureEffectClicked()
     CHECK_CURRENT_TABBOX_UI
 
     const int effect = ui->effectCombo->currentIndex();
-    if (!m_layoutPreview) {
-        m_layoutPreview = new LayoutPreview(this);
-        m_layoutPreview->setWindowTitle(i18n("Tabbox layout preview"));
-        m_layoutPreview->setWindowFlags(Qt::Dialog);
+    if (effect >= Layout) {
+        // TODO: here we need to show the preview
+        new LayoutPreview(ui->effectCombo->itemData(effect, Qt::UserRole+1).toString(), this);
+    } else {
+        QPointer<QDialog> configDialog = new QDialog(this);
+        configDialog->setLayout(new QVBoxLayout);
+        configDialog->setWindowTitle(ui->effectCombo->currentText());
+        QDialogButtonBox* buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel|QDialogButtonBox::RestoreDefaults, configDialog);
+        connect(buttonBox, SIGNAL(accepted()), configDialog, SLOT(accept()));
+        connect(buttonBox, SIGNAL(rejected()), configDialog, SLOT(reject()));
+
+        KCModuleProxy* proxy = new KCModuleProxy(effect == CoverSwitch ? "coverswitch_config" : "flipswitch_config");
+        connect(configDialog, SIGNAL(defaultClicked()), proxy, SLOT(defaults()));
+
+        QWidget *showWidget = new QWidget(configDialog);
+        QVBoxLayout *layout = new QVBoxLayout;
+        showWidget->setLayout(layout);
+        layout->addWidget(proxy);
+        configDialog->layout()->addWidget(showWidget);
+        configDialog->layout()->addWidget(buttonBox);
+
+        if (configDialog->exec() == QDialog::Accepted) {
+            proxy->save();
+        } else {
+            proxy->load();
+        }
+        delete configDialog;
     }
-    m_layoutPreview->setLayout(ui->effectCombo->itemData(effect, Qt::UserRole+1).toString(), ui->effectCombo->itemText(effect));
-    m_layoutPreview->show();
 }
 
 void KWinTabBoxConfig::shortcutChanged(const QKeySequence &seq)
@@ -445,9 +524,22 @@ void KWinTabBoxConfig::shortcutChanged(const QKeySequence &seq)
         action = sender()->property("shortcutAction").toString();
     if (action.isEmpty())
         return;
-    if (KAction *a = qobject_cast<KAction*>(m_actionCollection->action(action)))
-        a->setGlobalShortcut(KShortcut(seq), KAction::ActiveShortcut, KAction::NoAutoloading);
+    QAction *a = m_actionCollection->action(action);
+    KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << seq, KGlobalAccel::NoAutoloading);
     m_actionCollection->writeSettings();
 }
 
+void KWinTabBoxConfig::slotGHNS()
+{
+    QPointer<KNS3::DownloadDialog> downloadDialog = new KNS3::DownloadDialog("kwinswitcher.knsrc", this);
+    if (downloadDialog->exec() == QDialog::Accepted) {
+        if (!downloadDialog->changedEntries().isEmpty()) {
+            initLayoutLists();
+        }
+    }
+    delete downloadDialog;
+}
+
 } // namespace
+
+#include "main.moc"
