@@ -35,7 +35,6 @@ namespace KWin
 namespace TabBox
 {
 
-class LayoutPreview;
 }
 
 
@@ -56,16 +55,17 @@ public:
     explicit KWinTabBoxConfig(QWidget* parent, const QVariantList& args);
     ~KWinTabBoxConfig();
 
-public slots:
+public Q_SLOTS:
     virtual void save();
     virtual void load();
     virtual void defaults();
 
-private slots:
+private Q_SLOTS:
     void effectSelectionChanged(int index);
     void configureEffectClicked();
     void tabBoxToggled(bool on);
     void shortcutChanged(const QKeySequence &seq);
+    void slotGHNS();
 private:
     void updateUiFromConfig(KWinTabBoxConfigForm* ui, const TabBox::TabBoxConfig& config);
     void updateConfigFromUi(const KWinTabBoxConfigForm* ui, TabBox::TabBoxConfig& config);
@@ -74,6 +74,11 @@ private:
     void initLayoutLists();
 
 private:
+    enum Mode {
+        CoverSwitch = 0,
+        FlipSwitch = 1,
+        Layout = 2
+    };
     KWinTabBoxConfigForm* m_primaryTabBoxUi;
     KWinTabBoxConfigForm* m_alternativeTabBoxUi;
     KSharedConfigPtr m_config;
@@ -81,7 +86,6 @@ private:
     KShortcutsEditor* m_editor;
     TabBox::TabBoxConfig m_tabBoxConfig;
     TabBox::TabBoxConfig m_tabBoxAlternativeConfig;
-    TabBox::LayoutPreview *m_layoutPreview;
 
     bool effectEnabled(const QString& effect, const KConfigGroup& cfg) const;
 };
