@@ -19,12 +19,14 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 
-#include "config-kwin.h"
-
 #include "kwineffects.h"
-#include "kwinxrenderutils.h"
 
-#include <QtDBus/QtDBus>
+#include "config-kwin.h"
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
+#include "kwinxrenderutils.h"
+#endif
+
+#include <qmath.h>
 #include <QVariant>
 #include <QList>
 #include <QTimeLine>
@@ -32,22 +34,35 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QtGui/QPainter>
 #include <QtGui/QPixmap>
 #include <QtGui/QVector2D>
-#include <QtGui/qgraphicstransform.h>
-#include <QtCore/qmath.h>
+#include <QGraphicsRotation>
+#include <QGraphicsScale>
 
-#include <kdebug.h>
 #include <ksharedconfig.h>
-#include <kstandarddirs.h>
 #include <kconfiggroup.h>
 
 #include <assert.h>
 
-#ifdef KWIN_BUILD_COMPOSITE
-#include <X11/extensions/Xrender.h>
-#include <X11/extensions/Xfixes.h>
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
 #include <xcb/xfixes.h>
 #endif
 
+#if defined(__GNUC__)
+#  define KWIN_ALIGN(n) __attribute((aligned(n)))
+#  if defined(__SSE2__)
+#    define HAVE_SSE2
+#  endif
+#elif defined(__INTEL_COMPILER)
+#  define KWIN_ALIGN(n) __declspec(align(n))
+#  define HAVE_SSE2
+#else
+#  define KWIN_ALIGN(n)
+#endif
+
+#ifdef HAVE_SSE2
+#  include <emmintrin.h>
+#endif
+
+
 namespace KWin
 {
 
@@ -220,6 +235,7 @@ public:
 
 WindowPaintData::WindowPaintData(EffectWindow* w)
     : PaintData()
+    , shader(nullptr)
     , d(new WindowPaintDataPrivate())
 {
     quads = w->buildQuads();
@@ -234,6 +250,7 @@ WindowPaintData::WindowPaintData(EffectWindow* w)
 WindowPaintData::WindowPaintData(const WindowPaintData &other)
     : PaintData()
     , quads(other.quads)
+    , shader(other.shader)
     , d(new WindowPaintDataPrivate())
 {
     setXScale(other.xScale());
@@ -477,7 +494,7 @@ void Effect::reconfigure(ReconfigureFlags)
 
 void* Effect::proxy()
 {
-    return NULL;
+    return nullptr;
 }
 
 void Effect::windowInputMouseEvent(QEvent*)
@@ -569,16 +586,6 @@ void Effect::setPositionTransformations(WindowPaintData& data, QRect& region, Ef
     data.setYTranslation(y - w->y());
 }
 
-int Effect::displayWidth()
-{
-    return KWin::displayWidth();
-}
-
-int Effect::displayHeight()
-{
-    return KWin::displayHeight();
-}
-
 QPoint Effect::cursorPos()
 {
     return effects->cursorPos();
@@ -596,6 +603,32 @@ double Effect::animationTime(int defaultTime)
     return qMax(defaultTime * effects->animationTimeFactor(), 1.);
 }
 
+int Effect::requestedEffectChainPosition() const
+{
+    return 0;
+}
+
+//****************************************
+// EffectFactory
+//****************************************
+EffectPluginFactory::EffectPluginFactory()
+{
+}
+
+EffectPluginFactory::~EffectPluginFactory()
+{
+}
+
+bool EffectPluginFactory::enabledByDefault() const
+{
+    return true;
+}
+
+bool EffectPluginFactory::isSupported() const
+{
+    return true;
+}
+
 //****************************************
 // EffectsHandler
 //****************************************
@@ -619,20 +652,18 @@ CompositingType EffectsHandler::compositingType() const
     return compositing_type;
 }
 
-void EffectsHandler::sendReloadMessage(const QString& effectname)
+bool EffectsHandler::isOpenGLCompositing() const
 {
-    QDBusMessage message = QDBusMessage::createMethodCall("org.kde.kwin", "/KWin", "org.kde.KWin", "reconfigureEffect");
-    message << QString("kwin4_effect_" + effectname);
-    QDBusConnection::sessionBus().send(message);
+    return compositing_type & OpenGLCompositing;
 }
 
 KConfigGroup EffectsHandler::effectConfig(const QString& effectname)
 {
-    KSharedConfig::Ptr kwinconfig = KSharedConfig::openConfig("kwinrc", KConfig::NoGlobals);
-    return kwinconfig->group("Effect-" + effectname);
+    KSharedConfig::Ptr kwinconfig = KSharedConfig::openConfig(QStringLiteral(KWIN_CONFIG), KConfig::NoGlobals);
+    return kwinconfig->group(QStringLiteral("Effect-") + effectname);
 }
 
-EffectsHandler* effects = 0;
+EffectsHandler* effects = nullptr;
 
 
 //****************************************
@@ -685,11 +716,12 @@ WINDOW_HELPER(bool, isManaged, "managed")
 WINDOW_HELPER(bool, isDeleted, "deleted")
 WINDOW_HELPER(bool, hasOwnShape, "shaped")
 WINDOW_HELPER(QString, windowRole, "windowRole")
+WINDOW_HELPER(QStringList, activities, "activities")
 WINDOW_HELPER(bool, skipsCloseAnimation, "skipsCloseAnimation")
 
 QString EffectWindow::windowClass() const
 {
-    return parent()->property("resourceName").toString() + ' ' + parent()->property("resourceClass").toString();
+    return parent()->property("resourceName").toString() + QStringLiteral(" ") + parent()->property("resourceClass").toString();
 }
 
 QRect EffectWindow::contentsRect() const
@@ -702,6 +734,17 @@ NET::WindowType EffectWindow::windowType() const
     return static_cast<NET::WindowType>(parent()->property("windowType").toInt());
 }
 
+bool EffectWindow::isOnActivity(QString activity) const
+{
+    const QStringList activities = parent()->property("activities").toStringList();
+    return activities.isEmpty() || activities.contains(activity);
+}
+
+bool EffectWindow::isOnAllActivities() const
+{
+    return parent()->property("activities").toStringList().isEmpty();
+}
+
 #undef WINDOW_HELPER
 
 #define WINDOW_HELPER_DEFAULT( rettype, prototype, propertyname, defaultValue ) \
@@ -717,7 +760,7 @@ NET::WindowType EffectWindow::windowType() const
 WINDOW_HELPER_DEFAULT(bool, isMinimized, "minimized", false)
 WINDOW_HELPER_DEFAULT(bool, isMovable, "moveable", false)
 WINDOW_HELPER_DEFAULT(bool, isMovableAcrossScreens, "moveableAcrossScreens", false)
-WINDOW_HELPER_DEFAULT(QString, caption, "caption", "")
+WINDOW_HELPER_DEFAULT(QString, caption, "caption", QString())
 WINDOW_HELPER_DEFAULT(bool, keepAbove, "keepAbove", true)
 WINDOW_HELPER_DEFAULT(bool, isModal, "modal", false)
 WINDOW_HELPER_DEFAULT(QSize, basicUnit, "basicUnit", QSize(1, 1))
@@ -726,7 +769,7 @@ WINDOW_HELPER_DEFAULT(bool, isUserResize, "resize", false)
 WINDOW_HELPER_DEFAULT(QRect, iconGeometry, "iconGeometry", QRect())
 WINDOW_HELPER_DEFAULT(bool, isSpecialWindow, "specialWindow", true)
 WINDOW_HELPER_DEFAULT(bool, acceptsFocus, "wantsInput", true) // We don't actually know...
-WINDOW_HELPER_DEFAULT(QPixmap, icon, "icon", QPixmap())
+WINDOW_HELPER_DEFAULT(QIcon, icon, "icon", QIcon())
 WINDOW_HELPER_DEFAULT(bool, isSkipSwitcher, "skipSwitcher", false)
 WINDOW_HELPER_DEFAULT(bool, isCurrentTab, "isCurrentTab", false)
 WINDOW_HELPER_DEFAULT(bool, decorationHasAlpha, "decorationHasAlpha", false)
@@ -786,6 +829,10 @@ void EffectWindow::addLayerRepaint(const QRect &r)
     QMetaObject::invokeMethod(parent(), "addLayerRepaint", Q_ARG(const QRect&, r));
 }
 
+bool EffectWindow::isOnCurrentActivity() const
+{
+    return isOnActivity(effects->currentActivity());
+}
 
 bool EffectWindow::isOnCurrentDesktop() const
 {
@@ -809,7 +856,9 @@ bool EffectWindow::hasDecoration() const
 
 bool EffectWindow::isVisible() const
 {
-    return !isMinimized() && isOnCurrentDesktop();
+    return !isMinimized()
+           && isOnCurrentDesktop()
+           && isOnCurrentActivity();
 }
 
 
@@ -821,15 +870,6 @@ EffectWindowGroup::~EffectWindowGroup()
 {
 }
 
-//****************************************
-// GlobalShortcutsEditor
-//****************************************
-
-GlobalShortcutsEditor::GlobalShortcutsEditor(QWidget *parent) :
-    KShortcutsEditor(parent, GlobalAction)
-{
-}
-
 /***************************************************************
  WindowQuad
 ***************************************************************/
@@ -839,7 +879,7 @@ WindowQuad WindowQuad::makeSubQuad(double x1, double y1, double x2, double y2) c
     assert(x1 < x2 && y1 < y2 && x1 >= left() && x2 <= right() && y1 >= top() && y2 <= bottom());
 #ifndef NDEBUG
     if (isTransformed())
-        kFatal(1212) << "Splitting quads is allowed only in pre-paint calls!" ;
+        qFatal("Splitting quads is allowed only in pre-paint calls!");
 #endif
     WindowQuad ret(*this);
     // vertices are clockwise starting from topleft
@@ -898,7 +938,7 @@ WindowQuadList WindowQuadList::splitAtX(double x) const
     foreach (const WindowQuad & quad, *this) {
 #ifndef NDEBUG
         if (quad.isTransformed())
-            kFatal(1212) << "Splitting quads is allowed only in pre-paint calls!" ;
+            qFatal("Splitting quads is allowed only in pre-paint calls!");
 #endif
         bool wholeleft = true;
         bool wholeright = true;
@@ -930,7 +970,7 @@ WindowQuadList WindowQuadList::splitAtY(double y) const
     foreach (const WindowQuad & quad, *this) {
 #ifndef NDEBUG
         if (quad.isTransformed())
-            kFatal(1212) << "Splitting quads is allowed only in pre-paint calls!" ;
+            qFatal("Splitting quads is allowed only in pre-paint calls!");
 #endif
         bool wholetop = true;
         bool wholebottom = true;
@@ -970,7 +1010,7 @@ WindowQuadList WindowQuadList::makeGrid(int maxQuadSize) const
     foreach (const WindowQuad &quad, *this) {
 #ifndef NDEBUG
         if (quad.isTransformed())
-            kFatal(1212) << "Splitting quads is allowed only in pre-paint calls!" ;
+            qFatal("Splitting quads is allowed only in pre-paint calls!");
 #endif
         left   = qMin(left,   quad.left());
         right  = qMax(right,  quad.right());
@@ -1021,7 +1061,7 @@ WindowQuadList WindowQuadList::makeRegularGrid(int xSubdivisions, int ySubdivisi
     foreach (const WindowQuad &quad, *this) {
 #ifndef NDEBUG
         if (quad.isTransformed())
-            kFatal(1212) << "Splitting quads is allowed only in pre-paint calls!";
+            qFatal("Splitting quads is allowed only in pre-paint calls!");
 #endif
         left   = qMin(left,   quad.left());
         right  = qMax(right,  quad.right());
@@ -1061,6 +1101,137 @@ WindowQuadList WindowQuadList::makeRegularGrid(int xSubdivisions, int ySubdivisi
     return ret;
 }
 
+#ifndef GL_TRIANGLES
+#  define GL_TRIANGLES      0x0004
+#endif
+
+#ifndef GL_QUADS
+#  define GL_QUADS          0x0007
+#endif
+
+void WindowQuadList::makeInterleavedArrays(unsigned int type, GLVertex2D *vertices, const QMatrix4x4 &textureMatrix) const
+{
+    // Since we know that the texture matrix just scales and translates
+    // we can use this information to optimize the transformation
+    const QVector2D coeff(textureMatrix(0, 0), textureMatrix(1, 1));
+    const QVector2D offset(textureMatrix(0, 3), textureMatrix(1, 3));
+
+    GLVertex2D *vertex = vertices;
+
+    assert(type == GL_QUADS || type == GL_TRIANGLES);
+
+    switch (type)
+    {
+    case GL_QUADS:
+#ifdef HAVE_SSE2
+        if (!(intptr_t(vertex) & 0xf)) {
+            for (int i = 0; i < count(); i++) {
+                const WindowQuad &quad = at(i);
+                KWIN_ALIGN(16) GLVertex2D v[4];
+
+                for (int j = 0; j < 4; j++) {
+                    const WindowVertex &wv = quad[j];
+
+                    v[j].position = QVector2D(wv.x(), wv.y());
+                    v[j].texcoord = QVector2D(wv.u(), wv.v()) * coeff + offset;
+                }
+
+                const __m128i *srcP = (const __m128i *) &v;
+                __m128i *dstP = (__m128i *) vertex;
+
+                _mm_stream_si128(&dstP[0], _mm_load_si128(&srcP[0])); // Top-left
+                _mm_stream_si128(&dstP[1], _mm_load_si128(&srcP[1])); // Top-right
+                _mm_stream_si128(&dstP[2], _mm_load_si128(&srcP[2])); // Bottom-right
+                _mm_stream_si128(&dstP[3], _mm_load_si128(&srcP[3])); // Bottom-left
+
+                vertex += 4;
+            }
+        } else
+#endif // HAVE_SSE2
+        {
+            for (int i = 0; i < count(); i++) {
+                const WindowQuad &quad = at(i);
+
+                for (int j = 0; j < 4; j++) {
+                    const WindowVertex &wv = quad[j];
+
+                    GLVertex2D v;
+                    v.position = QVector2D(wv.x(), wv.y());
+                    v.texcoord = QVector2D(wv.u(), wv.v()) * coeff + offset;
+
+                    *(vertex++) = v;
+                }
+            }
+        }
+        break;
+
+    case GL_TRIANGLES:
+#ifdef HAVE_SSE2
+        if (!(intptr_t(vertex) & 0xf)) {
+            for (int i = 0; i < count(); i++) {
+                const WindowQuad &quad = at(i);
+                KWIN_ALIGN(16) GLVertex2D v[4];
+
+                for (int j = 0; j < 4; j++) {
+                    const WindowVertex &wv = quad[j];
+
+                    v[j].position = QVector2D(wv.x(), wv.y());
+                    v[j].texcoord = QVector2D(wv.u(), wv.v()) * coeff + offset;
+                }
+
+                const __m128i *srcP = (const __m128i *) &v;
+                __m128i *dstP = (__m128i *) vertex;
+
+                __m128i src[4];
+                src[0] = _mm_load_si128(&srcP[0]); // Top-left
+                src[1] = _mm_load_si128(&srcP[1]); // Top-right
+                src[2] = _mm_load_si128(&srcP[2]); // Bottom-right
+                src[3] = _mm_load_si128(&srcP[3]); // Bottom-left
+
+                // First triangle
+                _mm_stream_si128(&dstP[0], src[1]); // Top-right
+                _mm_stream_si128(&dstP[1], src[0]); // Top-left
+                _mm_stream_si128(&dstP[2], src[3]); // Bottom-left
+
+                // Second triangle
+                _mm_stream_si128(&dstP[3], src[3]); // Bottom-left
+                _mm_stream_si128(&dstP[4], src[2]); // Bottom-right
+                _mm_stream_si128(&dstP[5], src[1]); // Top-right
+
+                vertex += 6;
+            }
+        } else
+#endif // HAVE_SSE2
+        {
+            for (int i = 0; i < count(); i++) {
+                const WindowQuad &quad = at(i);
+                GLVertex2D v[4]; // Four unique vertices / quad
+
+                for (int j = 0; j < 4; j++) {
+                    const WindowVertex &wv = quad[j];
+
+                    v[j].position = QVector2D(wv.x(), wv.y());
+                    v[j].texcoord = QVector2D(wv.u(), wv.v()) * coeff + offset;
+                }
+
+                // First triangle
+                *(vertex++) = v[1]; // Top-right
+                *(vertex++) = v[0]; // Top-left
+                *(vertex++) = v[3]; // Bottom-left
+
+                // Second triangle
+                *(vertex++) = v[3]; // Bottom-left
+                *(vertex++) = v[2]; // Bottom-right
+                *(vertex++) = v[1]; // Top-right
+            }
+        }
+        break;
+
+    default:
+        break;
+    }
+}
+
 void WindowQuadList::makeArrays(float **vertices, float **texcoords, const QSizeF &size, bool yInverted) const
 {
     *vertices = new float[count() * 6 * 2];
@@ -1137,7 +1308,7 @@ bool WindowQuadList::isTransformed() const
  PaintClipper
 ***************************************************************/
 
-QStack< QRegion >* PaintClipper::areas = NULL;
+QStack< QRegion >* PaintClipper::areas = nullptr;
 
 PaintClipper::PaintClipper(const QRegion& allowed_area)
     : area(allowed_area)
@@ -1154,7 +1325,7 @@ void PaintClipper::push(const QRegion& allowed_area)
 {
     if (allowed_area == infiniteRegion())  // don't push these
         return;
-    if (areas == NULL)
+    if (areas == nullptr)
         areas = new QStack< QRegion >;
     areas->push(allowed_area);
 }
@@ -1163,23 +1334,23 @@ void PaintClipper::pop(const QRegion& allowed_area)
 {
     if (allowed_area == infiniteRegion())
         return;
-    Q_ASSERT(areas != NULL);
+    Q_ASSERT(areas != nullptr);
     Q_ASSERT(areas->top() == allowed_area);
     areas->pop();
     if (areas->isEmpty()) {
         delete areas;
-        areas = NULL;
+        areas = nullptr;
     }
 }
 
 bool PaintClipper::clip()
 {
-    return areas != NULL;
+    return areas != nullptr;
 }
 
 QRegion PaintClipper::paintArea()
 {
-    assert(areas != NULL);   // can be called only with clip() == true
+    assert(areas != nullptr);   // can be called only with clip() == true
     QRegion ret = QRegion(0, 0, displayWidth(), displayHeight());
     foreach (const QRegion & r, *areas)
     ret &= r;
@@ -1195,7 +1366,12 @@ struct PaintClipper::Iterator::Data {
 PaintClipper::Iterator::Iterator()
     : data(new Data)
 {
-#ifdef KWIN_BUILD_COMPOSITE
+    if (clip() && effects->isOpenGLCompositing()) {
+        data->rects = paintArea().rects();
+        data->index = -1;
+        next(); // move to the first one
+    }
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     if (clip() && effects->compositingType() == XRenderCompositing) {
         XFixesRegion region(paintArea());
         xcb_xfixes_set_picture_clip_region(connection(), effects->xrenderBufferPicture(), region, 0, 0);
@@ -1205,7 +1381,7 @@ PaintClipper::Iterator::Iterator()
 
 PaintClipper::Iterator::~Iterator()
 {
-#ifdef KWIN_BUILD_COMPOSITE
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     if (clip() && effects->compositingType() == XRenderCompositing)
         xcb_xfixes_set_picture_clip_region(connection(), effects->xrenderBufferPicture(), XCB_XFIXES_REGION_NONE, 0, 0);
 #endif
@@ -1216,7 +1392,9 @@ bool PaintClipper::Iterator::isDone()
 {
     if (!clip())
         return data->index == 1; // run once
-#ifdef KWIN_BUILD_COMPOSITE
+    if (effects->isOpenGLCompositing())
+        return data->index >= data->rects.count(); // run once per each area
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     if (effects->compositingType() == XRenderCompositing)
         return data->index == 1; // run once
 #endif
@@ -1232,7 +1410,9 @@ QRect PaintClipper::Iterator::boundingRect() const
 {
     if (!clip())
         return infiniteRegion();
-#ifdef KWIN_BUILD_COMPOSITE
+    if (effects->isOpenGLCompositing())
+        return data->rects[ data->index ];
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
     if (effects->compositingType() == XRenderCompositing)
         return paintArea().boundingRect();
 #endif
@@ -1499,7 +1679,7 @@ EffectWindow* WindowMotionManager::windowAtPoint(QPoint point, bool useStackingO
         ++it;
     }
 
-    return NULL;
+    return nullptr;
 }
 
 /***************************************************************
@@ -1560,4 +1740,4 @@ void EffectFrame::enableCrossFade(bool enable)
 
 } // namespace
 
-#include "moc_kwineffects.cpp"
+#include "kwineffects.moc"
