@@ -23,7 +23,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "kwinconfig.h" // KWIN_HAVE_OPENGL
 
 #include "kwinglplatform.h"
-#include "kwineffects.h"
 #include "kwinglutils_funcs.h"
 #include "kwinglutils.h"
 
@@ -109,6 +108,11 @@ GLTexture::GLTexture(int width, int height)
     }
 }
 
+GLTexture::GLTexture(const QSize &size)
+    : GLTexture(size.width(), size.height())
+{
+}
+
 GLTexture::~GLTexture()
 {
 }
@@ -130,7 +134,7 @@ GLTexturePrivate::GLTexturePrivate()
     m_markedDirty = false;
     m_unnormalizeActive = 0;
     m_normalizeActive = 0;
-    m_vbo = 0;
+    m_vbo = nullptr;
     m_filterChanged = true;
     m_wrapModeChanged = false;
     ++s_textureObjectCounter;
@@ -138,7 +142,7 @@ GLTexturePrivate::GLTexturePrivate()
 
 GLTexturePrivate::~GLTexturePrivate()
 {
-    if (m_vbo != 0) {
+    if (m_vbo != nullptr) {
         delete m_vbo;
     }
     if (m_texture != 0) {
@@ -157,16 +161,16 @@ void GLTexturePrivate::initStatic()
     sNPOTTextureSupported = true;
     sFramebufferObjectSupported = true;
     sSaturationSupported = true;
-    if (hasGLExtension("GL_EXT_texture_format_BGRA8888"))
+    if (hasGLExtension(QStringLiteral("GL_EXT_texture_format_BGRA8888")))
         sTextureFormat = GL_BGRA_EXT;
     else
         sTextureFormat = GL_RGBA;
 #else
-    sNPOTTextureSupported = hasGLExtension("GL_ARB_texture_non_power_of_two");
-    sFramebufferObjectSupported = hasGLExtension("GL_EXT_framebuffer_object");
-    sSaturationSupported = ((hasGLExtension("GL_ARB_texture_env_crossbar")
-                             && hasGLExtension("GL_ARB_texture_env_dot3")) || hasGLVersion(1, 4))
-                           && (glTextureUnitsCount >= 4) && glActiveTexture != NULL;
+    sNPOTTextureSupported = hasGLExtension(QStringLiteral("GL_ARB_texture_non_power_of_two"));
+    sFramebufferObjectSupported = hasGLExtension(QStringLiteral("GL_EXT_framebuffer_object"));
+    sSaturationSupported = ((hasGLExtension(QStringLiteral("GL_ARB_texture_env_crossbar"))
+                             && hasGLExtension(QStringLiteral("GL_ARB_texture_env_dot3"))) || hasGLVersion(1, 4))
+                           && (glTextureUnitsCount >= 4) && glActiveTexture != nullptr;
     sTextureFormat = GL_BGRA;
 #endif
 }
@@ -242,7 +246,7 @@ void GLTexture::update(const QImage &image, const QPoint &offset, const QRect &s
 
     Q_D(GLTexture);
 #ifdef KWIN_HAVE_OPENGLES
-    static bool s_supportsUnpack = hasGLExtension("GL_EXT_unpack_subimage");
+    static bool s_supportsUnpack = hasGLExtension(QStringLiteral("GL_EXT_unpack_subimage"));
 #else
     static bool s_supportsUnpack = true;
 #endif
@@ -297,10 +301,6 @@ void GLTexture::discard()
 
 void GLTexturePrivate::bind()
 {
-#ifdef KWIN_HAVE_OPENGL_1
-    if (!ShaderManager::instance()->isValid())
-        glEnable(m_target);
-#endif
     glBindTexture(m_target, m_texture);
 }
 
@@ -347,10 +347,6 @@ void GLTexture::bind()
 void GLTexturePrivate::unbind()
 {
     glBindTexture(m_target, 0);
-#ifdef KWIN_HAVE_OPENGL_1
-    if (!ShaderManager::instance()->isValid())
-        glDisable(m_target);
-#endif
 }
 
 void GLTexture::unbind()
@@ -393,20 +389,11 @@ void GLTexture::render(QRegion region, const QRect& rect, bool hardwareClipping)
     }
     QMatrix4x4 translation;
     translation.translate(rect.x(), rect.y());
-    if (ShaderManager::instance()->isShaderBound()) {
-        GLShader *shader = ShaderManager::instance()->getBoundShader();
-        shader->setUniform(GLShader::Offset, QVector2D(rect.x(), rect.y()));
-        shader->setUniform(GLShader::WindowTransformation, translation);
-    } else {
-        pushMatrix(translation);
-    }
+    GLShader *shader = ShaderManager::instance()->getBoundShader();
+    shader->setUniform(GLShader::Offset, QVector2D(rect.x(), rect.y()));
+    shader->setUniform(GLShader::WindowTransformation, translation);
     d->m_vbo->render(region, GL_TRIANGLE_STRIP, hardwareClipping);
-    if (ShaderManager::instance()->isShaderBound()) {
-        GLShader *shader = ShaderManager::instance()->getBoundShader();
-        shader->setUniform(GLShader::WindowTransformation, QMatrix4x4());
-    } else {
-        popMatrix();
-    }
+    shader->setUniform(GLShader::WindowTransformation, QMatrix4x4());
 }
 
 GLuint GLTexture::texture() const
