@@ -25,15 +25,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "kwingltexture_p.h"
 
 #include "kwinglcolorcorrection.h"
-#include "kwinglobals.h"
 #include "kwineffects.h"
 #include "kwinglplatform.h"
 
-#include "kdebug.h"
-#include <kstandarddirs.h>
-#include <KDE/KConfig>
-#include <KDE/KConfigGroup>
 
+#include <QDebug>
 #include <QPixmap>
 #include <QImage>
 #include <QHash>
@@ -76,8 +72,8 @@ void initGLX()
     glXQueryVersion(display(), &major, &minor);
     glXVersion = MAKE_GL_VERSION(major, minor, 0);
     // Get list of supported GLX extensions
-    glxExtensions = QString((const char*)glXQueryExtensionsString(
-                                display(), DefaultScreen(display()))).split(' ');
+    glxExtensions = QString::fromUtf8(glXQueryExtensionsString(
+                                display(), DefaultScreen(display()))).split(QStringLiteral(" "));
 
     glxResolveFunctions();
 #endif
@@ -90,7 +86,7 @@ void initEGL()
     int major, minor;
     eglInitialize(dpy, &major, &minor);
     eglVersion = MAKE_GL_VERSION(major, minor, 0);
-    eglExtension = QString((const char*)eglQueryString(dpy, EGL_EXTENSIONS)).split(' ');
+    eglExtension = QString::fromUtf8(eglQueryString(dpy, EGL_EXTENSIONS)).split(QStringLiteral(" "));
 
     eglResolveFunctions();
 #endif
@@ -99,10 +95,10 @@ void initEGL()
 void initGL(OpenGLPlatformInterface platformInterface)
 {
     // Get OpenGL version
-    QString glversionstring = QString((const char*)glGetString(GL_VERSION));
-    QStringList glversioninfo = glversionstring.left(glversionstring.indexOf(' ')).split('.');
+    QString glversionstring = QString::fromUtf8((const char*)glGetString(GL_VERSION));
+    QStringList glversioninfo = glversionstring.left(glversionstring.indexOf(QStringLiteral(" "))).split(QStringLiteral("."));
     while (glversioninfo.count() < 3)
-        glversioninfo << "0";
+        glversioninfo << QStringLiteral("0");
 
 #ifndef KWIN_HAVE_OPENGLES
     glVersion = MAKE_GL_VERSION(glversioninfo[0].toInt(), glversioninfo[1].toInt(), glversioninfo[2].toInt());
@@ -123,11 +119,11 @@ void initGL(OpenGLPlatformInterface platformInterface)
 
         for (int i = 0; i < count; i++) {
             const char *name = (const char *) glGetStringi(GL_EXTENSIONS, i);
-            glExtensions << QString(name);
+            glExtensions << QString::fromUtf8(name);
         }
     } else
 #endif
-        glExtensions = QString((const char*)glGetString(GL_EXTENSIONS)).split(' ');
+        glExtensions = QString::fromUtf8((const char*)glGetString(GL_EXTENSIONS)).split(QStringLiteral(" "));
 
     // handle OpenGL extensions functions
     glResolveFunctions(platformInterface);
@@ -166,16 +162,16 @@ bool hasGLExtension(const QString& extension)
 static QString formatGLError(GLenum err)
 {
     switch(err) {
-    case GL_NO_ERROR:          return "GL_NO_ERROR";
-    case GL_INVALID_ENUM:      return "GL_INVALID_ENUM";
-    case GL_INVALID_VALUE:     return "GL_INVALID_VALUE";
-    case GL_INVALID_OPERATION: return "GL_INVALID_OPERATION";
+    case GL_NO_ERROR:          return QStringLiteral("GL_NO_ERROR");
+    case GL_INVALID_ENUM:      return QStringLiteral("GL_INVALID_ENUM");
+    case GL_INVALID_VALUE:     return QStringLiteral("GL_INVALID_VALUE");
+    case GL_INVALID_OPERATION: return QStringLiteral("GL_INVALID_OPERATION");
 #ifndef KWIN_HAVE_OPENGLES
-    case GL_STACK_OVERFLOW:    return "GL_STACK_OVERFLOW";
-    case GL_STACK_UNDERFLOW:   return "GL_STACK_UNDERFLOW";
+    case GL_STACK_OVERFLOW:    return QStringLiteral("GL_STACK_OVERFLOW");
+    case GL_STACK_UNDERFLOW:   return QStringLiteral("GL_STACK_UNDERFLOW");
 #endif
-    case GL_OUT_OF_MEMORY:     return "GL_OUT_OF_MEMORY";
-    default: return QString("0x") + QString::number(err, 16);
+    case GL_OUT_OF_MEMORY:     return QStringLiteral("GL_OUT_OF_MEMORY");
+    default: return QStringLiteral("0x") + QString::number(err, 16);
     }
 }
 
@@ -184,7 +180,7 @@ bool checkGLError(const char* txt)
     GLenum err = glGetError();
     bool hasError = false;
     while (err != GL_NO_ERROR) {
-        kWarning(1212) << "GL error (" << txt << "): " << formatGLError(err);
+        qWarning() << "GL error (" << txt << "): " << formatGLError(err);
         hasError = true;
         err = glGetError();
     }
@@ -206,78 +202,6 @@ int nearestPowerOfTwo(int x)
     return 1 << last;
 }
 
-void pushMatrix()
-{
-#ifdef KWIN_HAVE_OPENGL_1
-    if (ShaderManager::instance()->isValid()) {
-        return;
-    }
-    glPushMatrix();
-#endif
-}
-
-void pushMatrix(const QMatrix4x4 &matrix)
-{
-#ifndef KWIN_HAVE_OPENGL_1
-    Q_UNUSED(matrix)
-#else
-    if (ShaderManager::instance()->isValid()) {
-        return;
-    }
-    glPushMatrix();
-    multiplyMatrix(matrix);
-#endif
-}
-
-void multiplyMatrix(const QMatrix4x4 &matrix)
-{
-#ifndef KWIN_HAVE_OPENGL_1
-    Q_UNUSED(matrix)
-#else
-    if (ShaderManager::instance()->isValid()) {
-        return;
-    }
-    GLfloat m[16];
-    const qreal *data = matrix.constData();
-    for (int i = 0; i < 4; ++i) {
-        for (int j = 0; j < 4; ++j) {
-            m[i*4+j] = data[i*4+j];
-        }
-    }
-    glMultMatrixf(m);
-#endif
-}
-
-void loadMatrix(const QMatrix4x4 &matrix)
-{
-#ifndef KWIN_HAVE_OPENGL_1
-    Q_UNUSED(matrix)
-#else
-    if (ShaderManager::instance()->isValid()) {
-        return;
-    }
-    GLfloat m[16];
-    const qreal *data = matrix.constData();
-    for (int i = 0; i < 4; ++i) {
-        for (int j = 0; j < 4; ++j) {
-            m[i*4+j] = data[i*4+j];
-        }
-    }
-    glLoadMatrixf(m);
-#endif
-}
-
-void popMatrix()
-{
-#ifdef KWIN_HAVE_OPENGL_1
-    if (ShaderManager::instance()->isValid()) {
-        return;
-    }
-    glPopMatrix();
-#endif
-}
-
-
 //****************************************
 // GLShader
 //****************************************
@@ -312,14 +236,14 @@ bool GLShader::loadFromFiles(const QString &vertexFile, const QString &fragmentF
 {
     QFile vf(vertexFile);
     if (!vf.open(QIODevice::ReadOnly)) {
-        kError(1212) << "Couldn't open" << vertexFile << "for reading!" << endl;
+        qCritical() << "Couldn't open" << vertexFile << "for reading!" << endl;
         return false;
     }
     const QByteArray vertexSource = vf.readAll();
 
     QFile ff(fragmentFile);
     if (!ff.open(QIODevice::ReadOnly)) {
-        kError(1212) << "Couldn't open" << fragmentFile << "for reading!" << endl;
+        qCritical() << "Couldn't open" << fragmentFile << "for reading!" << endl;
         return false;
     }
     const QByteArray fragmentSource = ff.readAll();
@@ -346,10 +270,10 @@ bool GLShader::link()
     glGetProgramiv(mProgram, GL_LINK_STATUS, &status);
 
     if (status == 0) {
-        kError(1212) << "Failed to link shader:" << endl << log << endl;
+        qCritical() << "Failed to link shader:" << endl << log << endl;
         mValid = false;
     } else if (length > 0) {
-        kDebug(1212) << "Shader link log:" << log;
+        qDebug() << "Shader link log:" << log;
     }
 
     return mValid;
@@ -387,7 +311,7 @@ bool GLShader::compile(GLuint program, GLenum shaderType, const QByteArray &sour
 
     QByteArray preparedSource = prepareSource(shaderType, source);
     const char* src = preparedSource.constData();
-    glShaderSource(shader, 1, &src, NULL);
+    glShaderSource(shader, 1, &src, nullptr);
 
     // Compile the shader
     glCompileShader(shader);
@@ -405,9 +329,9 @@ bool GLShader::compile(GLuint program, GLenum shaderType, const QByteArray &sour
 
     if (status == 0) {
         const char *typeName = (shaderType == GL_VERTEX_SHADER ? "vertex" : "fragment");
-        kError(1212) << "Failed to compile" << typeName << "shader:" << endl << log << endl;
+        qCritical() << "Failed to compile" << typeName << "shader:" << endl << log << endl;
     } else if (length > 0)
-        kDebug(1212) << "Shader compile log:" << log;
+        qDebug() << "Shader compile log:" << log;
 
     if (status != 0)
         glAttachShader(program, shader);
@@ -421,7 +345,7 @@ bool GLShader::load(const QByteArray &vertexSource, const QByteArray &fragmentSo
 #ifndef KWIN_HAVE_OPENGLES
     // Make sure shaders are actually supported
     if (!GLPlatform::instance()->supports(GLSL) || GLPlatform::instance()->supports(LimitedNPOT)) {
-        kError(1212) << "Shaders are not supported";
+        qCritical() << "Shaders are not supported";
         return false;
     }
 #endif
@@ -635,7 +559,7 @@ bool GLShader::setUniform(int location, const QMatrix4x4 &value)
 {
     if (location >= 0) {
         GLfloat m[16];
-        const qreal *data = value.constData();
+        const auto *data = value.constData();
         // i is column, j is row for m
         for (int i = 0; i < 16; ++i) {
             m[i] = data[i];
@@ -688,7 +612,7 @@ QMatrix4x4 GLShader::getUniformMatrix4x4(const char* name)
 //****************************************
 // ShaderManager
 //****************************************
-ShaderManager *ShaderManager::s_shaderManager = NULL;
+ShaderManager *ShaderManager::s_shaderManager = nullptr;
 
 ShaderManager *ShaderManager::instance()
 {
@@ -700,21 +624,10 @@ ShaderManager *ShaderManager::instance()
     return s_shaderManager;
 }
 
-void ShaderManager::disable()
-{
-    // for safety do a Cleanup first
-    ShaderManager::cleanup();
-
-    // create a new ShaderManager and set it to inited without calling init
-    // that will ensure that the ShaderManager is not valid
-    s_shaderManager = new ShaderManager();
-    s_shaderManager->m_inited = true;
-}
-
 void ShaderManager::cleanup()
 {
     delete s_shaderManager;
-    s_shaderManager = NULL;
+    s_shaderManager = nullptr;
 }
 
 ShaderManager::ShaderManager()
@@ -740,7 +653,7 @@ ShaderManager::~ShaderManager()
 GLShader *ShaderManager::getBoundShader() const
 {
     if (m_boundShaders.isEmpty()) {
-        return NULL;
+        return nullptr;
     } else {
         return m_boundShaders.top();
     }
@@ -764,7 +677,7 @@ bool ShaderManager::isShaderDebug() const
 GLShader *ShaderManager::pushShader(ShaderType type, bool reset)
 {
     if (m_inited && !m_valid) {
-        return NULL;
+        return nullptr;
     }
 
     pushShader(m_shader[type]);
@@ -841,7 +754,7 @@ GLShader *ShaderManager::loadFragmentShader(ShaderType vertex, const QString &fr
         "scene-color-vertex.glsl"
     };
 
-    GLShader *shader = new GLShader(m_shaderDir + vertexFile[vertex], fragmentFile, GLShader::ExplicitLinking);
+    GLShader *shader = new GLShader(QString::fromUtf8(m_shaderDir + vertexFile[vertex]), fragmentFile, GLShader::ExplicitLinking);
     bindAttributeLocations(shader);
     bindFragDataLocations(shader);
     shader->link();
@@ -864,7 +777,7 @@ GLShader *ShaderManager::loadVertexShader(ShaderType fragment, const QString &ve
         "scene-color-fragment.glsl"
     };
 
-    GLShader *shader = new GLShader(vertexFile, m_shaderDir + fragmentFile[fragment], GLShader::ExplicitLinking);
+    GLShader *shader = new GLShader(vertexFile, QString::fromUtf8(m_shaderDir + fragmentFile[fragment]), GLShader::ExplicitLinking);
     bindAttributeLocations(shader);
     bindFragDataLocations(shader);
     shader->link();
@@ -916,7 +829,8 @@ void ShaderManager::initShaders()
     m_valid = true;
 
     for (int i = 0; i < 3; i++) {
-        m_shader[i] = new GLShader(m_shaderDir + vertexFile[i], m_shaderDir + fragmentFile[i],
+        m_shader[i] = new GLShader(QString::fromUtf8(m_shaderDir + vertexFile[i]),
+                                   QString::fromUtf8(m_shaderDir + fragmentFile[i]),
                                    GLShader::ExplicitLinking);
         bindAttributeLocations(m_shader[i]);
         bindFragDataLocations(m_shader[i]);
@@ -1004,8 +918,8 @@ void GLRenderTarget::initStatic()
     sSupported = true;
     s_blitSupported = false;
 #else
-    sSupported = hasGLVersion(3, 0) || hasGLExtension("GL_ARB_framebuffer_object") || hasGLExtension("GL_EXT_framebuffer_object");
-    s_blitSupported = hasGLVersion(3, 0) || hasGLExtension("GL_ARB_framebuffer_object") || hasGLExtension("GL_EXT_framebuffer_blit");
+    sSupported = hasGLVersion(3, 0) || hasGLExtension(QStringLiteral("GL_ARB_framebuffer_object")) || hasGLExtension(QStringLiteral("GL_EXT_framebuffer_object"));
+    s_blitSupported = hasGLVersion(3, 0) || hasGLExtension(QStringLiteral("GL_ARB_framebuffer_object")) || hasGLExtension(QStringLiteral("GL_EXT_framebuffer_blit"));
 #endif
 }
 
@@ -1050,7 +964,7 @@ GLRenderTarget::GLRenderTarget(const GLTexture& color)
     if (sSupported && !mTexture.isNull()) {
         initFBO();
     } else
-        kError(1212) << "Render targets aren't supported!" << endl;
+        qCritical() << "Render targets aren't supported!" << endl;
 }
 
 GLRenderTarget::~GLRenderTarget()
@@ -1063,7 +977,7 @@ GLRenderTarget::~GLRenderTarget()
 bool GLRenderTarget::enable()
 {
     if (!valid()) {
-        kError(1212) << "Can't enable invalid render target!" << endl;
+        qCritical() << "Can't enable invalid render target!" << endl;
         return false;
     }
 
@@ -1077,7 +991,7 @@ bool GLRenderTarget::enable()
 bool GLRenderTarget::disable()
 {
     if (!valid()) {
-        kError(1212) << "Can't disable invalid render target!" << endl;
+        qCritical() << "Can't disable invalid render target!" << endl;
         return false;
     }
 
@@ -1092,32 +1006,32 @@ static QString formatFramebufferStatus(GLenum status)
     switch(status) {
     case GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT:
         // An attachment is the wrong type / is invalid / has 0 width or height
-        return "GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT";
+        return QStringLiteral("GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT");
     case GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT:
         // There are no images attached to the framebuffer
-        return "GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT";
+        return QStringLiteral("GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT");
     case GL_FRAMEBUFFER_UNSUPPORTED:
         // A format or the combination of formats of the attachments is unsupported
-        return "GL_FRAMEBUFFER_UNSUPPORTED";
+        return QStringLiteral("GL_FRAMEBUFFER_UNSUPPORTED");
 #ifndef KWIN_HAVE_OPENGLES
     case GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT:
         // Not all attached images have the same width and height
-        return "GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT";
+        return QStringLiteral("GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT");
     case GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT:
         // The color attachments don't have the same format
-        return "GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT";
+        return QStringLiteral("GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT");
     case GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT:
         // The attachments don't have the same number of samples
-        return "GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE";
+        return QStringLiteral("GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE");
     case GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT:
         // The draw buffer is missing
-        return "GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER";
+        return QStringLiteral("GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER");
     case GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT:
         // The read buffer is missing
-        return "GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER";
+        return QStringLiteral("GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER");
 #endif
     default:
-        return "Unknown (0x" + QString::number(status, 16) + ')';
+        return QStringLiteral("Unknown (0x") + QString::number(status, 16) + QStringLiteral(")");
     }
 }
 
@@ -1126,14 +1040,14 @@ void GLRenderTarget::initFBO()
 #if DEBUG_GLRENDERTARGET
     GLenum err = glGetError();
     if (err != GL_NO_ERROR)
-        kError(1212) << "Error status when entering GLRenderTarget::initFBO: " << formatGLError(err);
+        qCritical() << "Error status when entering GLRenderTarget::initFBO: " << formatGLError(err);
 #endif
 
     glGenFramebuffers(1, &mFramebuffer);
 
 #if DEBUG_GLRENDERTARGET
     if ((err = glGetError()) != GL_NO_ERROR) {
-        kError(1212) << "glGenFramebuffers failed: " << formatGLError(err);
+        qCritical() << "glGenFramebuffers failed: " << formatGLError(err);
         return;
     }
 #endif
@@ -1142,7 +1056,7 @@ void GLRenderTarget::initFBO()
 
 #if DEBUG_GLRENDERTARGET
     if ((err = glGetError()) != GL_NO_ERROR) {
-        kError(1212) << "glBindFramebuffer failed: " << formatGLError(err);
+        qCritical() << "glBindFramebuffer failed: " << formatGLError(err);
         glDeleteFramebuffers(1, &mFramebuffer);
         return;
     }
@@ -1153,7 +1067,7 @@ void GLRenderTarget::initFBO()
 
 #if DEBUG_GLRENDERTARGET
     if ((err = glGetError()) != GL_NO_ERROR) {
-        kError(1212) << "glFramebufferTexture2D failed: " << formatGLError(err);
+        qCritical() << "glFramebufferTexture2D failed: " << formatGLError(err);
         glBindFramebuffer(GL_FRAMEBUFFER, 0);
         glDeleteFramebuffers(1, &mFramebuffer);
         return;
@@ -1167,9 +1081,9 @@ void GLRenderTarget::initFBO()
     if (status != GL_FRAMEBUFFER_COMPLETE) {
         // We have an incomplete framebuffer, consider it invalid
         if (status == 0)
-            kError(1212) << "glCheckFramebufferStatus failed: " << formatGLError(glGetError());
+            qCritical() << "glCheckFramebufferStatus failed: " << formatGLError(glGetError());
         else
-            kError(1212) << "Invalid framebuffer status: " << formatFramebufferStatus(status);
+            qCritical() << "Invalid framebuffer status: " << formatFramebufferStatus(status);
         glDeleteFramebuffers(1, &mFramebuffer);
         return;
     }
@@ -1452,7 +1366,7 @@ void IndexBuffer::accomodate(int count)
     GLuint buffer;
     glGenBuffers(1, &buffer);
     glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, buffer);
-    glBufferData(GL_ELEMENT_ARRAY_BUFFER, size, NULL, GL_STATIC_DRAW);
+    glBufferData(GL_ELEMENT_ARRAY_BUFFER, size, nullptr, GL_STATIC_DRAW);
 
     // Use the GPU to copy the data from the old object to the new object,
     glBindBuffer(GL_COPY_READ_BUFFER, m_buffer);
@@ -1577,8 +1491,7 @@ public:
         , nextOffset(0)
         , baseAddress(0)
     {
-        if (GLVertexBufferPrivate::supported)
-            glGenBuffers(1, &buffer);
+        glGenBuffers(1, &buffer);
 
         switch(usageHint) {
         case GLVertexBuffer::Dynamic:
@@ -1594,8 +1507,7 @@ public:
     }
 
     ~GLVertexBufferPrivate() {
-        if (GLVertexBufferPrivate::supported)
-            glDeleteBuffers(1, &buffer);
+        glDeleteBuffers(1, &buffer);
     }
 
     void interleaveArrays(float *array, int dim, const float *vertices, const float *texcoords, int count);
@@ -1608,7 +1520,6 @@ public:
     GLenum usage;
     int stride;
     int vertexCount;
-    static bool supported;
     static GLVertexBuffer *streamingBuffer;
     static bool hasMapBufferRange;
     static bool supportsIndexedQuads;
@@ -1626,12 +1537,11 @@ public:
 #endif
 };
 
-bool GLVertexBufferPrivate::supported = false;
 bool GLVertexBufferPrivate::hasMapBufferRange = false;
 bool GLVertexBufferPrivate::supportsIndexedQuads = false;
-GLVertexBuffer *GLVertexBufferPrivate::streamingBuffer = NULL;
+GLVertexBuffer *GLVertexBufferPrivate::streamingBuffer = nullptr;
 #ifndef KWIN_HAVE_OPENGLES
-IndexBuffer *GLVertexBufferPrivate::s_indexBuffer = NULL;
+IndexBuffer *GLVertexBufferPrivate::s_indexBuffer = nullptr;
 #endif
 
 void GLVertexBufferPrivate::interleaveArrays(float *dst, int dim,
@@ -1677,63 +1587,27 @@ void GLVertexBufferPrivate::interleaveArrays(float *dst, int dim,
 
 void GLVertexBufferPrivate::bindArrays()
 {
-#ifndef KWIN_HAVE_OPENGLES
-    if (ShaderManager::instance()->isShaderBound()) {
-#endif
-        if (useColor) {
-            GLShader *shader = ShaderManager::instance()->getBoundShader();
-            shader->setUniform(GLShader::Color, color);
-        }
-
-        glBindBuffer(GL_ARRAY_BUFFER, buffer);
-
-        BitfieldIterator it(enabledArrays);
-        while (it.hasNext()) {
-            const int index = it.next();
-            glVertexAttribPointer(index, attrib[index].size, attrib[index].type, GL_FALSE, stride,
-                                 (const GLvoid *) (baseAddress + attrib[index].offset));
-            glEnableVertexAttribArray(index);
-        }
-#ifndef KWIN_HAVE_OPENGLES
-    } else {
-        if (GLVertexBufferPrivate::supported)
-            glBindBuffer(GL_ARRAY_BUFFER, buffer);
-
-        // FIXME Is there any good reason to not leave this array permanently enabled?
-        //       When do we not use it in the GL 1.x path?
-        glEnableClientState(GL_VERTEX_ARRAY);
-        glVertexPointer(attrib[VA_Position].size, attrib[VA_Position].type, stride,
-                        (const GLvoid *) (baseAddress + attrib[VA_Position].offset));
-
-        if (enabledArrays[VA_TexCoord]) {
-            glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-            glTexCoordPointer(attrib[VA_TexCoord].size, attrib[VA_TexCoord].type, stride,
-                              (const GLvoid *) (baseAddress + attrib[VA_TexCoord].offset));
-        }
-
-        if (useColor)
-            glColor4f(color.x(), color.y(), color.z(), color.w());
+    if (useColor) {
+        GLShader *shader = ShaderManager::instance()->getBoundShader();
+        shader->setUniform(GLShader::Color, color);
+    }
+
+    glBindBuffer(GL_ARRAY_BUFFER, buffer);
+
+    BitfieldIterator it(enabledArrays);
+    while (it.hasNext()) {
+        const int index = it.next();
+        glVertexAttribPointer(index, attrib[index].size, attrib[index].type, GL_FALSE, stride,
+                                (const GLvoid *) (baseAddress + attrib[index].offset));
+        glEnableVertexAttribArray(index);
     }
-#endif
 }
 
 void GLVertexBufferPrivate::unbindArrays()
 {
-#ifndef KWIN_HAVE_OPENGLES
-    if (ShaderManager::instance()->isShaderBound()) {
-#endif
-        BitfieldIterator it(enabledArrays);
-        while (it.hasNext())
-            glDisableVertexAttribArray(it.next());
-#ifndef KWIN_HAVE_OPENGLES
-    } else {
-        // Assume that the conventional arrays were enabled
-        glDisableClientState(GL_VERTEX_ARRAY);
-
-        if (enabledArrays[VA_TexCoord])
-            glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-    }
-#endif
+    BitfieldIterator it(enabledArrays);
+    while (it.hasNext())
+        glDisableVertexAttribArray(it.next());
 }
 
 void GLVertexBufferPrivate::reallocateBuffer(size_t size)
@@ -1809,8 +1683,7 @@ GLvoid *GLVertexBuffer::map(size_t size)
 {
     d->mappedSize = size;
 
-    if (GLVertexBufferPrivate::supported)
-        glBindBuffer(GL_ARRAY_BUFFER, d->buffer);
+    glBindBuffer(GL_ARRAY_BUFFER, d->buffer);
 
     bool preferBufferSubData = GLPlatform::instance()->preferBufferSubData();
 
@@ -1835,7 +1708,7 @@ void GLVertexBuffer::unmap()
 
         d->baseAddress = d->nextOffset;
         d->nextOffset += align(d->mappedSize, 16); // Align to 16 bytes for SSE
-    } else if (GLVertexBufferPrivate::supported) {
+    } else {
         // Upload the data from local memory to the buffer object
         if (preferBufferSubData) {
             if ((d->nextOffset + d->mappedSize) > d->bufferSize) {
@@ -1856,10 +1729,6 @@ void GLVertexBuffer::unmap()
         if (d->usage == GL_STATIC_DRAW)
             d->dataStore = QByteArray();
 
-    } else {
-        // If buffer objects aren't supported we just need to update
-        // the client memory pointer and we're done.
-        d->baseAddress = intptr_t(d->dataStore.data());
     }
 
     d->mappedSize = 0;
@@ -1933,12 +1802,12 @@ void GLVertexBuffer::draw(const QRegion &region, GLenum primitiveMode, int first
         count = count * 6 / 4;
 
         if (!hardwareClipping) {
-            glDrawElementsBaseVertex(GL_TRIANGLES, count, GL_UNSIGNED_SHORT, NULL, first);
+            glDrawElementsBaseVertex(GL_TRIANGLES, count, GL_UNSIGNED_SHORT, nullptr, first);
         } else {
             // Clip using scissoring
             foreach (const QRect &r, region.rects()) {
                 glScissor(r.x(), displayHeight() - r.y() - r.height(), r.width(), r.height());
-                glDrawElementsBaseVertex(GL_TRIANGLES, count, GL_UNSIGNED_SHORT, NULL, first);
+                glDrawElementsBaseVertex(GL_TRIANGLES, count, GL_UNSIGNED_SHORT, nullptr, first);
             }
         }
         return;
@@ -1956,11 +1825,6 @@ void GLVertexBuffer::draw(const QRegion &region, GLenum primitiveMode, int first
     }
 }
 
-bool GLVertexBuffer::isSupported()
-{
-    return GLVertexBufferPrivate::supported;
-}
-
 bool GLVertexBuffer::supportsIndexedQuads()
 {
     return GLVertexBufferPrivate::supportsIndexedQuads;
@@ -1992,14 +1856,12 @@ void GLVertexBuffer::reset()
 void GLVertexBuffer::initStatic()
 {
 #ifdef KWIN_HAVE_OPENGLES
-    GLVertexBufferPrivate::supported = true;
-    GLVertexBufferPrivate::hasMapBufferRange = hasGLExtension("GL_EXT_map_buffer_range");
+    GLVertexBufferPrivate::hasMapBufferRange = hasGLExtension(QStringLiteral("GL_EXT_map_buffer_range"));
     GLVertexBufferPrivate::supportsIndexedQuads = false;
 #else
-    GLVertexBufferPrivate::supported = hasGLVersion(1, 5) || hasGLExtension("GL_ARB_vertex_buffer_object");
-    GLVertexBufferPrivate::hasMapBufferRange = hasGLVersion(3, 0) || hasGLExtension("GL_ARB_map_buffer_range");
+    GLVertexBufferPrivate::hasMapBufferRange = hasGLVersion(3, 0) || hasGLExtension(QStringLiteral("GL_ARB_map_buffer_range"));
     GLVertexBufferPrivate::supportsIndexedQuads = glMapBufferRange && glCopyBufferSubData && glDrawElementsBaseVertex;
-    GLVertexBufferPrivate::s_indexBuffer = 0;
+    GLVertexBufferPrivate::s_indexBuffer = nullptr;
 #endif
     GLVertexBufferPrivate::streamingBuffer = new GLVertexBuffer(GLVertexBuffer::Stream);
 }
@@ -2008,7 +1870,7 @@ void GLVertexBuffer::cleanup()
 {
 #ifndef KWIN_HAVE_OPENGLES
     delete GLVertexBufferPrivate::s_indexBuffer;
-    GLVertexBufferPrivate::s_indexBuffer = 0;
+    GLVertexBufferPrivate::s_indexBuffer = nullptr;
 #endif
 }
 
