@@ -23,6 +23,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define KWIN_GLUTILS_H
 
 // kwin
+#include <kwinglutils_export.h>
 #include "kwinglutils_funcs.h"
 #include "kwingltexture.h"
 
@@ -49,30 +50,30 @@ class GLVertexBufferPrivate;
 
 
 // Initializes GLX function pointers
-void KWIN_EXPORT initGLX();
+void KWINGLUTILS_EXPORT initGLX();
 // Initializes OpenGL stuff. This includes resolving function pointers as
 //  well as checking for GL version and extensions
 //  Note that GL context has to be created by the time this function is called
-void KWIN_EXPORT initGL(OpenGLPlatformInterface platformInterface);
+void KWINGLUTILS_EXPORT initGL(OpenGLPlatformInterface platformInterface);
 // Initializes EGL function pointers
-void KWIN_EXPORT initEGL();
+void KWINGLUTILS_EXPORT initEGL();
 // Cleans up all resources hold by the GL Context
-void KWIN_EXPORT cleanupGL();
+void KWINGLUTILS_EXPORT cleanupGL();
 
 // Number of supported texture units
-extern KWIN_EXPORT int glTextureUnitsCount;
+extern KWINGLUTILS_EXPORT int glTextureUnitsCount;
 
 
-bool KWIN_EXPORT hasGLVersion(int major, int minor, int release = 0);
-bool KWIN_EXPORT hasGLXVersion(int major, int minor, int release = 0);
-bool KWIN_EXPORT hasEGLVersion(int major, int minor, int release = 0);
+bool KWINGLUTILS_EXPORT hasGLVersion(int major, int minor, int release = 0);
+bool KWINGLUTILS_EXPORT hasGLXVersion(int major, int minor, int release = 0);
+bool KWINGLUTILS_EXPORT hasEGLVersion(int major, int minor, int release = 0);
 // use for both OpenGL and GLX extensions
-bool KWIN_EXPORT hasGLExtension(const QString& extension);
+bool KWINGLUTILS_EXPORT hasGLExtension(const QString& extension);
 
 // detect OpenGL error (add to various places in code to pinpoint the place)
-bool KWIN_EXPORT checkGLError(const char* txt);
+bool KWINGLUTILS_EXPORT checkGLError(const char* txt);
 
-inline bool KWIN_EXPORT isPowerOfTwo(int x)
+inline bool KWINGLUTILS_EXPORT isPowerOfTwo(int x)
 {
     return ((x & (x - 1)) == 0);
 }
@@ -80,51 +81,9 @@ inline bool KWIN_EXPORT isPowerOfTwo(int x)
  * @return power of two integer _greater or equal to_ x.
  *  E.g. nearestPowerOfTwo(513) = nearestPowerOfTwo(800) = 1024
  **/
-int KWIN_EXPORT nearestPowerOfTwo(int x);
+int KWINGLUTILS_EXPORT nearestPowerOfTwo(int x);
 
-/**
- * Push a new matrix on the GL matrix stack.
- * In GLES this method is a noop. This method should be preferred over glPushMatrix
- * as it also handles GLES.
- * @since 4.7
- **/
-KWIN_EXPORT void pushMatrix();
-/**
- * Multiplies current matrix on GL stack with @p matrix and pushes the result on the matrix stack.
- * This method is the same as pushMatrix followed by multiplyMatrix.
- * In GLES this method is a noop. This method should be preferred over glPushMatrix
- * as it also handles GLES.
- * @param matrix The matrix the current matrix on the stack should be multiplied with.
- * @see pushMatrix
- * @see multiplyMatrix
- * @since 4.7
- **/
-KWIN_EXPORT void pushMatrix(const QMatrix4x4 &matrix);
-/**
- * Multiplies the current matrix on GL stack with @p matrix.
- * In GLES this method is a noop. This method should be preferred over glMultMatrix
- * as it also handles GLES.
- * @param matrix The matrix the current matrix on the stack should be multiplied with.
- * @since 4.7
- **/
-KWIN_EXPORT void multiplyMatrix(const QMatrix4x4 &matrix);
-/**
- * Replaces the current matrix on GL stack with @p matrix.
- * In GLES this method is a no-op. This method should be preferred over glLoadMatrix
- * as it also handles GLES.
- * @param matrix The new matrix to replace the existing one on the GL stack.
- * @since 4.7
- **/
-KWIN_EXPORT void loadMatrix(const QMatrix4x4 &matrix);
-/**
- * Pops the current matrix from the GL matrix stack.
- * In GLES this method is a noop. This method should be preferred over glPopMatrix
- * as it also handles GLES.
- * @since 4.7
- **/
-KWIN_EXPORT void popMatrix();
-
-class KWIN_EXPORT GLShader
+class KWINGLUTILS_EXPORT GLShader
 {
 public:
     enum Flags {
@@ -253,7 +212,7 @@ private:
  * @author Martin Gräßlin <mgraesslin@kde.org>
  * @since 4.7
  **/
-class KWIN_EXPORT ShaderManager
+class KWINGLUTILS_EXPORT ShaderManager
 {
 public:
     /**
@@ -303,10 +262,6 @@ public:
      * Allows to query whether Shaders are supported by the compositor, that is
      * whether the Shaders compiled successfully.
      *
-     * With OpenGL 1 compositing this method will always return @c false.
-     *
-     * Do not use this method to check whether the compositor uses OpenGL 1 or 2,
-     * use @link EffectsHandler::compositingType instead.
      * @return @c true if the built-in shaders are valid, @c false otherwise
      **/
     bool isValid() const;
@@ -385,15 +340,6 @@ public:
      * @return a pointer to the ShaderManager instance
      **/
     static ShaderManager *instance();
-    /**
-     * @brief Ensures that the ShaderManager is disabled.
-     *
-     * Used only by an OpenGL 1 Scene which does not use OpenGL 2 Shaders.
-     *
-     * @internal
-     * @since 4.10
-     **/
-    static void disable();
 
     /**
      * @internal
@@ -432,12 +378,9 @@ private:
  * // here the Shader is automatically popped as helper does no longer exist.
  * @endcode
  *
- * This class takes care for the case that the Compositor uses OpenGL 1 and the ShaderManager is
- * not valid. In that case the helper does not do anything. So this helper can be used to simplify
- * the code to remove checks for OpenGL 1/2.
  * @since 4.10
  **/
-class KWIN_EXPORT ShaderBinder
+class KWINGLUTILS_EXPORT ShaderBinder
 {
 public:
     /**
@@ -458,7 +401,7 @@ public:
     ~ShaderBinder();
 
     /**
-     * @return The Shader pushed to the Stack. On OpenGL 1 this returns a @c null pointer.
+     * @return The Shader pushed to the Stack.
      **/
     GLShader *shader();
 
@@ -468,13 +411,8 @@ private:
 
 inline
 ShaderBinder::ShaderBinder(ShaderManager::ShaderType type, bool reset)
-    : m_shader(NULL)
+    : m_shader(nullptr)
 {
-#ifdef KWIN_HAVE_OPENGL_1
-    if (!ShaderManager::instance()->isValid()) {
-        return;
-    }
-#endif
     m_shader = ShaderManager::instance()->pushShader(type, reset);
 }
 
@@ -482,22 +420,12 @@ inline
 ShaderBinder::ShaderBinder(GLShader *shader)
     : m_shader(shader)
 {
-#ifdef KWIN_HAVE_OPENGL_1
-    if (!ShaderManager::instance()->isValid()) {
-        return;
-    }
-#endif
     ShaderManager::instance()->pushShader(shader);
 }
 
 inline
 ShaderBinder::~ShaderBinder()
 {
-#ifdef KWIN_HAVE_OPENGL_1
-    if (!ShaderManager::instance()->isValid()) {
-        return;
-    }
-#endif
     ShaderManager::instance()->popShader();
 }
 
@@ -515,7 +443,7 @@ GLShader* ShaderBinder::shader()
  *
  * @author Rivo Laks <rivolaks@hot.ee>
  **/
-class KWIN_EXPORT GLRenderTarget
+class KWINGLUTILS_EXPORT GLRenderTarget
 {
 public:
     /**
@@ -627,7 +555,7 @@ struct GLVertexAttrib
  * @author Martin Gräßlin <mgraesslin@kde.org>
  * @since 4.6
  */
-class KWIN_EXPORT GLVertexBuffer
+class KWINGLUTILS_EXPORT GLVertexBuffer
 {
 public:
     /**
@@ -791,13 +719,6 @@ public:
      */
     static void cleanup();
 
-    /**
-     * Returns true if VBOs are supported, it is save to use this class even if VBOs are not
-     * supported.
-     * @returns true if vertex buffer objects are supported
-     */
-    static bool isSupported();
-
     /**
      * Returns true if indexed quad mode is supported, and false otherwise.
      */
