@@ -24,12 +24,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "client.h"
 
 #include <kstartupinfo.h>
-#include <kglobal.h>
-#include <X11/extensions/shape.h>
 
+#ifdef KWIN_BUILD_ACTIVITIES
+#include "activities.h"
+#endif
 #include "cursor.h"
 #include "decorations.h"
-#include <QtGui/qx11info_x11.h>
+#include <QX11Info>
 #include "rules.h"
 #include "group.h"
 #include "netinfo.h"
@@ -37,6 +38,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "workspace.h"
 #include "xcbutils.h"
 
+#include <xcb/shape.h>
+
 namespace KWin
 {
 
@@ -77,8 +80,7 @@ bool Client::manage(xcb_window_t w, bool isMapped)
     if (isMapped)
         init_minimize = false; // If it's already mapped, ignore hint
 
-    unsigned long properties[2];
-    properties[WinInfo::PROTOCOLS] =
+    const NET::Properties properties =
         NET::WMDesktop |
         NET::WMState |
         NET::WMWindowType |
@@ -87,18 +89,16 @@ bool Client::manage(xcb_window_t w, bool isMapped)
         NET::WMIconGeometry |
         NET::WMIcon |
         NET::WMPid |
-        NET::WMIconName |
-        0;
-    properties[WinInfo::PROTOCOLS2] =
+        NET::WMIconName;
+    const NET::Properties2 properties2 =
         NET::WM2UserTime |
         NET::WM2StartupId |
         NET::WM2ExtendedStrut |
         NET::WM2Opacity |
         NET::WM2FullscreenMonitors |
-        NET::WM2FrameOverlap |
-        0;
+        NET::WM2FrameOverlap;
 
-    info = new WinInfo(this, display(), m_client, rootWindow(), properties, 2);
+    info = new WinInfo(this, m_client, rootWindow(), properties, properties2);
 
     m_colormap = attr.colormap;
 
@@ -115,7 +115,7 @@ bool Client::manage(xcb_window_t w, bool isMapped)
     setCaption(cap_normal, true);
 
     if (Xcb::Extensions::self()->isShapeAvailable())
-        XShapeSelectInput(display(), window(), ShapeNotifyMask);
+        xcb_shape_select_input(connection(), window(), true);
     detectShape(window());
     detectNoBorder();
     fetchIconicName();
@@ -157,11 +157,14 @@ bool Client::manage(xcb_window_t w, bool isMapped)
     init_minimize = rules()->checkMinimize(init_minimize, !isMapped);
     noborder = rules()->checkNoBorder(noborder, !isMapped);
 
+    checkActivities();
+
     // Initial desktop placement
     if (session) {
         desk = session->desktop;
         if (session->onAllDesktops)
             desk = NET::OnAllDesktops;
+        setOnActivities(session->activities);
     } else {
         // If this window is transient, ensure that it is opened on the
         // same window as its parent.  this is necessary when an application
@@ -189,11 +192,26 @@ bool Client::manage(xcb_window_t w, bool isMapped)
                 desk = VirtualDesktopManager::self()->current();
             else if (maincl != NULL)
                 desk = maincl->desktop();
+
+            if (maincl)
+                setOnActivities(maincl->activities());
         }
         if (info->desktop())
             desk = info->desktop(); // Window had the initial desktop property, force it
         if (desktop() == 0 && asn_valid && asn_data.desktop() != 0)
             desk = asn_data.desktop();
+#ifdef KWIN_BUILD_ACTIVITIES
+        if (!isMapped && !noborder && isNormalWindow() && !activitiesDefined) {
+            //a new, regular window, when we're not recovering from a crash,
+            //and it hasn't got an activity. let's try giving it the current one.
+            //TODO: decide whether to keep this before the 4.6 release
+            //TODO: if we are keeping it (at least as an option), replace noborder checking
+            //with a public API for setting windows to be on all activities.
+            //something like KWindowSystem::setOnAllActivities or
+            //KActivityConsumer::setOnAllActivities
+            setOnActivity(Activities::self()->current(), true);
+        }
+#endif
     }
 
     if (desk == 0)   // Assume window wants to be visible on the current desktop
@@ -205,6 +223,11 @@ bool Client::manage(xcb_window_t w, bool isMapped)
     workspace()->updateOnAllDesktopsOfTransients(this);   // SELI TODO
     //onAllDesktopsChange(); // Decoration doesn't exist here yet
 
+    QString activitiesList;
+    activitiesList = rules()->checkActivity(activitiesList, !isMapped);
+    if (!activitiesList.isEmpty())
+        setOnActivities(activitiesList.split(QStringLiteral(",")));
+
     QRect geom(attr.x, attr.y, attr.width, attr.height);
     bool placementDone = false;
 
@@ -308,7 +331,7 @@ bool Client::manage(xcb_window_t w, bool isMapped)
             foreach (Client *other, workspace()->clientList()) {
                 if (other->maximizeMode() != MaximizeFull &&
                     geom == QRect(other->pos(), other->clientSize()) &&
-                    desk == other->desktop()) {
+                    desk == other->desktop() && activities() == other->activities()) {
 
                     tabBehind(other, autogroupInFg);
                     break;
@@ -372,7 +395,7 @@ bool Client::manage(xcb_window_t w, bool isMapped)
             // Window is too large for the screen, maximize in the
             // directions necessary
             const QSize ss = workspace()->clientArea(ScreenArea, area.center(), desktop()).size();
-            const QRect fsa = workspace()->clientArea(FullArea, geom.center(), desktop());
+            const QSize fs = workspace()->clientArea(FullArea, geom.center(), desktop()).size();
             const QSize cs = clientSize();
             int pseudo_max = Client::MaximizeRestore;
             if (width() >= area.width())
@@ -388,15 +411,10 @@ bool Client::manage(xcb_window_t w, bool isMapped)
             // i intended a second check on cs < area.size() ("the managed client ("minus border") is smaller
             // than the workspace") but gtk / gimp seems to store it's size including the decoration,
             // thus a former maximized window wil become non-maximized
-            bool keepInFsArea = false;
-            if (width() < fsa.width() && (cs.width() > ss.width()+1)) {
+            if (width() < fs.width() && (cs.width() > ss.width()+1))
                 pseudo_max &= ~Client::MaximizeHorizontal;
-                keepInFsArea = true;
-            }
-            if (height() < fsa.height() && (cs.height() > ss.height()+1)) {
+            if (height() < fs.height() && (cs.height() > ss.height()+1))
                 pseudo_max &= ~Client::MaximizeVertical;
-                keepInFsArea = true;
-            }
 
             if (pseudo_max != Client::MaximizeRestore) {
                 maximize((MaximizeMode)pseudo_max);
@@ -412,8 +430,6 @@ bool Client::manage(xcb_window_t w, bool isMapped)
                     geom_restore.setWidth(width());
                 }
             }
-            if (keepInFsArea)
-                keepInArea(fsa, partial_keep_in_area);
         }
     }
 
@@ -500,7 +516,7 @@ bool Client::manage(xcb_window_t w, bool isMapped)
             demandAttention();
         if (info->state() & NET::Modal)
             setModal(true);
-        if (fullscreen_mode != FullScreenHack)
+        if (fullscreen_mode != FullScreenHack && isFullScreenable())
             setFullScreen(rules()->checkFullScreen(info->state() & NET::FullScreen, !isMapped), false);
     }
 
@@ -512,7 +528,7 @@ bool Client::manage(xcb_window_t w, bool isMapped)
 
     // This should avoid flicker, because real restacking is done
     // only after manage() finishes because of blocking, but the window is shown sooner
-    XLowerWindow(display(), frameId());
+    m_frame.lower();
     if (session && session->stackingOrder != -1) {
         sm_stacking_order = session->stackingOrder;
         workspace()->restoreSessionStackingOrder(this);
@@ -556,6 +572,9 @@ bool Client::manage(xcb_window_t w, bool isMapped)
                 if (!isOnCurrentDesktop() && options->focusPolicyIsReasonable()) {
                     VirtualDesktopManager::self()->setCurrent(desktop());
                 }
+                /*if (!isOnCurrentActivity()) {
+                    workspace()->setCurrentActivity( activities().first() );
+                } FIXME no such method*/
             }
         }
 
@@ -599,6 +618,8 @@ bool Client::manage(xcb_window_t w, bool isMapped)
     updateWindowRules(Rules::All); // Was blocked while !isManaged()
 
     updateCompositeBlocking(true);
+    updateColorScheme();
+    updateShowOnScreenEdge();
 
     // TODO: there's a small problem here - isManaged() depends on the mapping state,
     // but this client is not yet in Workspace's client list at this point, will
@@ -613,7 +634,7 @@ void Client::embedClient(xcb_window_t w, const XWindowAttributes& attr)
     assert(m_client == XCB_WINDOW_NONE);
     assert(frameId() == XCB_WINDOW_NONE);
     assert(m_wrapper == XCB_WINDOW_NONE);
-    m_client = w;
+    m_client.reset(w, false);
 
     const xcb_visualid_t visualid = XVisualIDFromVisual(attr.visual);
     const uint32_t zero_value = 0;
@@ -623,9 +644,9 @@ void Client::embedClient(xcb_window_t w, const XWindowAttributes& attr)
     // We don't want the window to be destroyed when we quit
     xcb_change_save_set(conn, XCB_SET_MODE_INSERT, m_client);
 
-    xcb_change_window_attributes(conn, m_client, XCB_CW_EVENT_MASK, &zero_value);
-    xcb_unmap_window(conn, m_client);
-    xcb_configure_window(conn, m_client, XCB_CONFIG_WINDOW_BORDER_WIDTH, &zero_value);
+    m_client.selectInput(zero_value);
+    m_client.unmap();
+    m_client.setBorderWidth(zero_value);
 
     // Note: These values must match the order in the xcb_cw_t enum
     const uint32_t cw_values[] = {
@@ -659,8 +680,9 @@ void Client::embedClient(xcb_window_t w, const XWindowAttributes& attr)
     xcb_window_t frame = xcb_generate_id(conn);
     xcb_create_window(conn, attr.depth, frame, rootWindow(), 0, 0, 1, 1, 0,
                       XCB_WINDOW_CLASS_INPUT_OUTPUT, visualid, cw_mask, cw_values);
+    m_frame.reset(frame);
 
-    setWindowHandles(m_client, frame);
+    setWindowHandles(m_client);
 
     // Create the wrapper window
     xcb_window_t wrapperId = xcb_generate_id(conn);
@@ -668,14 +690,14 @@ void Client::embedClient(xcb_window_t w, const XWindowAttributes& attr)
                       XCB_WINDOW_CLASS_INPUT_OUTPUT, visualid, cw_mask, cw_values);
     m_wrapper.reset(wrapperId);
 
-    xcb_reparent_window(conn, m_client, m_wrapper, 0, 0);
+    m_client.reparent(m_wrapper);
 
     // We could specify the event masks when we create the windows, but the original
     // Xlib code didn't.  Let's preserve that behavior here for now so we don't end up
     // receiving any unexpected events from the wrapper creation or the reparenting.
-    xcb_change_window_attributes(conn, frame,   XCB_CW_EVENT_MASK, &frame_event_mask);
-    xcb_change_window_attributes(conn, m_wrapper, XCB_CW_EVENT_MASK, &wrapper_event_mask);
-    xcb_change_window_attributes(conn, m_client,  XCB_CW_EVENT_MASK, &client_event_mask);
+    m_frame.selectInput(frame_event_mask);
+    m_wrapper.selectInput(wrapper_event_mask);
+    m_client.selectInput(client_event_mask);
 
     updateMouseGrab();
 }
@@ -701,6 +723,8 @@ Client* Client::findAutogroupCandidate() const
     QString wGId = rules()->checkAutogroupById(QString());
     if (!wGId.isEmpty()) {
         foreach (Client *c, workspace()->clientList()) {
+            if (activities() != c->activities())
+                continue; // don't cross activities
             if (wGId == c->rules()->checkAutogroupById(QString())) {
                 if (found && found->tabGroup() != c->tabGroup()) { // We've found two, ignore both
                     found = NULL;
@@ -721,7 +745,7 @@ Client* Client::findAutogroupCandidate() const
     if (rules()->checkAutogrouping(options->isAutogroupSimilarWindows())) {
         QByteArray wRole = truncatedWindowRole(windowRole());
         foreach (Client *c, workspace()->clientList()) {
-            if (desktop() != c->desktop())
+            if (desktop() != c->desktop() || activities() != c->activities())
                 continue;
             QByteArray wRoleB = truncatedWindowRole(c->windowRole());
             if (resourceClass() == c->resourceClass() &&  // Same resource class
