@@ -25,19 +25,18 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #ifndef KCMRULES
 
-#include <QPalette>
-#include <QPixmap>
-#include <kapplication.h>
-#include <kconfig.h>
-#include <kglobal.h>
-#include <kglobalsettings.h>
-#include <klocale.h>
+#include <QProcess>
 
-#include "client.h"
+#include "compositingprefs.h"
 #include "settings.h"
 #include "xcbutils.h"
+#include <kwinglplatform.h>
 
-#include <X11/extensions/Xrandr.h>
+#ifndef KWIN_HAVE_OPENGLES
+#ifndef KWIN_NO_XF86VM
+#include <X11/extensions/xf86vmode.h>
+#endif
+#endif
 
 #endif //KCMRULES
 
@@ -51,10 +50,49 @@ int currentRefreshRate()
     int rate = -1;
     if (options->refreshRate() > 0)   // use manually configured refresh rate
         rate = options->refreshRate();
+#ifndef KWIN_HAVE_OPENGLES
+    else if (GLPlatform::instance()->driver() == Driver_NVidia) {
+#ifndef KWIN_NO_XF86VM
+        int major, event, error;
+        if (XQueryExtension(display(), "XFree86-VidModeExtension", &major, &event, &error)) {
+            XF86VidModeModeLine modeline;
+            int dotclock, vtotal;
+            if (XF86VidModeGetModeLine(display(), 0, &dotclock, &modeline)) {
+                vtotal = modeline.vtotal;
+                if (modeline.flags & 0x0010) // V_INTERLACE
+                    dotclock *= 2;
+                if (modeline.flags & 0x0020) // V_DBLSCAN
+                    vtotal *= 2;
+                if (modeline.htotal*vtotal) // BUG 313996
+                    rate = 1000*dotclock/(modeline.htotal*vtotal); // WTF was wikipedia 1998 when I nedded it?
+                qDebug() << "Vertical Refresh Rate (as detected by XF86VM): " << rate << "Hz";
+            }
+        }
+        if (rate < 1)
+#endif
+        { // modeline approach failed
+            QProcess nvidia_settings;
+            QStringList env = QProcess::systemEnvironment();
+            env << QStringLiteral("LC_ALL=C");
+            nvidia_settings.setEnvironment(env);
+            nvidia_settings.start(QStringLiteral("nvidia-settings"), QStringList() << QStringLiteral("-t") << QStringLiteral("-q") << QStringLiteral("RefreshRate"), QIODevice::ReadOnly);
+            nvidia_settings.waitForFinished();
+            if (nvidia_settings.exitStatus() == QProcess::NormalExit) {
+                QString reply = QString::fromLocal8Bit(nvidia_settings.readAllStandardOutput()).split(QStringLiteral(" ")).first();
+                bool ok;
+                float frate = QLocale::c().toFloat(reply, &ok);
+                if (!ok)
+                    rate = -1;
+                else
+                    rate = qRound(frate);
+                qDebug() << "Vertical Refresh Rate (as detected by nvidia-settings): " << rate << "Hz";
+            }
+        }
+    }
+#endif
     else if (Xcb::Extensions::self()->isRandrAvailable()) {
-        XRRScreenConfiguration *config = XRRGetScreenInfo(display(), rootWindow());
-        rate = XRRConfigCurrentRate(config);
-        XRRFreeScreenConfigInfo(config);
+        Xcb::RandR::ScreenInfo screenInfo(rootWindow());
+        rate = screenInfo->rate;
     }
 
     // 0Hz or less is invalid, so we fallback to a default rate
@@ -64,13 +102,13 @@ int currentRefreshRate()
     // however, additional throttling prevents very high rates from taking place anyway
     else if (rate > 1000)
         rate = 1000;
-    kDebug(1212) << "Vertical Refresh rate " << rate << "Hz";
+    qDebug() << "Vertical Refresh rate " << rate << "Hz";
     return rate;
 }
 
 Options::Options(QObject *parent)
-    : QObject(parent)
-    , m_settings(new Settings(KGlobal::config()))
+    : KDecorationOptions(parent)
+    , m_settings(new Settings(KSharedConfig::openConfig()))
     , m_focusPolicy(ClickToFocus)
     , m_nextFocusPrefersMouse(false)
     , m_clickRaise(false)
@@ -99,10 +137,16 @@ Options::Options(QObject *parent)
     , m_compositingInitialized(Options::defaultCompositingInitialized())
     , m_hiddenPreviews(Options::defaultHiddenPreviews())
     , m_unredirectFullscreen(Options::defaultUnredirectFullscreen())
+    , m_glSmoothScale(Options::defaultGlSmoothScale())
+    , m_colorCorrected(Options::defaultColorCorrected())
     , m_xrenderSmoothScale(Options::defaultXrenderSmoothScale())
     , m_maxFpsInterval(Options::defaultMaxFpsInterval())
     , m_refreshRate(Options::defaultRefreshRate())
     , m_vBlankTime(Options::defaultVBlankTime())
+    , m_glStrictBinding(Options::defaultGlStrictBinding())
+    , m_glStrictBindingFollowsDriver(Options::defaultGlStrictBindingFollowsDriver())
+    , m_glCoreProfile(Options::defaultGLCoreProfile())
+    , m_glPreferBufferSwap(Options::defaultGlPreferBufferSwap())
     , OpTitlebarDblClick(Options::defaultOperationTitlebarDblClick())
     , CmdActiveTitlebar1(Options::defaultCommandActiveTitlebar1())
     , CmdActiveTitlebar2(Options::defaultCommandActiveTitlebar2())
@@ -595,13 +639,36 @@ void Options::setHiddenPreviews(int hiddenPreviews)
 
 void Options::setUnredirectFullscreen(bool unredirectFullscreen)
 {
+    if (GLPlatform::instance()->driver() == Driver_Intel)
+        unredirectFullscreen = false; // bug #252817
     if (m_unredirectFullscreen == unredirectFullscreen) {
         return;
     }
+    if (GLPlatform::instance()->driver() == Driver_Intel) { // write back the value
+        KConfigGroup(m_settings->config(), "Compositing").writeEntry("UnredirectFullscreen", false);
+    }
     m_unredirectFullscreen = unredirectFullscreen;
     emit unredirectFullscreenChanged();
 }
 
+void Options::setGlSmoothScale(int glSmoothScale)
+{
+    if (m_glSmoothScale == glSmoothScale) {
+        return;
+    }
+    m_glSmoothScale = glSmoothScale;
+    emit glSmoothScaleChanged();
+}
+
+void Options::setColorCorrected(bool colorCorrected)
+{
+    if (m_colorCorrected == colorCorrected) {
+        return;
+    }
+    m_colorCorrected = colorCorrected;
+    emit colorCorrectedChanged();
+}
+
 void Options::setXrenderSmoothScale(bool xrenderSmoothScale)
 {
     if (m_xrenderSmoothScale == xrenderSmoothScale) {
@@ -638,14 +705,59 @@ void Options::setVBlankTime(qint64 vBlankTime)
     emit vBlankTimeChanged();
 }
 
-void Options::reparseConfiguration()
+void Options::setGlStrictBinding(bool glStrictBinding)
 {
-    KGlobal::config()->reparseConfiguration();
+    if (m_glStrictBinding == glStrictBinding) {
+        return;
+    }
+    m_glStrictBinding = glStrictBinding;
+    emit glStrictBindingChanged();
 }
 
-unsigned long Options::updateSettings()
+void Options::setGlStrictBindingFollowsDriver(bool glStrictBindingFollowsDriver)
 {
-    unsigned long changed = loadConfig();
+    if (m_glStrictBindingFollowsDriver == glStrictBindingFollowsDriver) {
+        return;
+    }
+    m_glStrictBindingFollowsDriver = glStrictBindingFollowsDriver;
+    emit glStrictBindingFollowsDriverChanged();
+}
+
+void Options::setGLCoreProfile(bool value)
+{
+    if (m_glCoreProfile == value) {
+        return;
+    }
+    m_glCoreProfile = value;
+    emit glCoreProfileChanged();
+}
+
+void Options::setGlPreferBufferSwap(char glPreferBufferSwap)
+{
+    if (glPreferBufferSwap == 'a') {
+        // buffer cpying is very fast with the nvidia blob
+        // but due to restrictions in DRI2 *incredibly* slow for all MESA drivers
+        // see http://www.x.org/releases/X11R7.7/doc/dri2proto/dri2proto.txt, item 2.5
+        if (GLPlatform::instance()->driver() == Driver_NVidia)
+            glPreferBufferSwap = CopyFrontBuffer;
+        else if (GLPlatform::instance()->driver() != Driver_Unknown) // undetected, finally resolved when context is initialized
+            glPreferBufferSwap = ExtendDamage;
+    }
+    if (m_glPreferBufferSwap == (GlSwapStrategy)glPreferBufferSwap) {
+        return;
+    }
+    m_glPreferBufferSwap = (GlSwapStrategy)glPreferBufferSwap;
+    emit glPreferBufferSwapChanged();
+}
+
+void Options::reparseConfiguration()
+{
+    m_settings->config()->reparseConfiguration();
+}
+
+void Options::updateSettings()
+{
+    loadConfig();
     // Read button tooltip animation effect from kdeglobals
     // Since we want to allow users to enable window decoration tooltips
     // and not kstyle tooltips and vise-versa, we don't read the
@@ -660,31 +772,27 @@ unsigned long Options::updateSettings()
     reloadCompositingSettings();
 
     emit configChanged();
-
-    return changed;
 }
 
-unsigned long Options::loadConfig()
+void Options::loadConfig()
 {
-    KSharedConfig::Ptr _config = KGlobal::config();
-    m_settings->readConfig();
-    unsigned long changed = 0;
-    changed |= KDecorationOptions::updateSettings(m_settings->config());   // read decoration settings
+    m_settings->load();
+    KDecorationOptions::updateSettings(m_settings->config());   // read decoration settings
 
     syncFromKcfgc();
 
     // Electric borders
-    KConfigGroup config(_config, "Windows");
+    KConfigGroup config(m_settings->config(), "Windows");
     OpTitlebarDblClick = windowOperation(config.readEntry("TitlebarDoubleClickCommand", "Maximize"), true);
     setOpMaxButtonLeftClick(windowOperation(config.readEntry("MaximizeButtonLeftClickCommand", "Maximize"), true));
     setOpMaxButtonMiddleClick(windowOperation(config.readEntry("MaximizeButtonMiddleClickCommand", "Maximize (vertical only)"), true));
     setOpMaxButtonRightClick(windowOperation(config.readEntry("MaximizeButtonRightClickCommand", "Maximize (horizontal only)"), true));
 
     // Mouse bindings
-    config = KConfigGroup(_config, "MouseBindings");
+    config = KConfigGroup(m_settings->config(), "MouseBindings");
     // TODO: add properties for missing options
     CmdTitlebarWheel = mouseWheelCommand(config.readEntry("CommandTitlebarWheel", "Switch to Window Tab to the Left/Right"));
-    CmdAllModKey = (config.readEntry("CommandAllKey", "Alt") == "Meta") ? Qt::Key_Meta : Qt::Key_Alt;
+    CmdAllModKey = (config.readEntry("CommandAllKey", "Alt") == QStringLiteral("Meta")) ? Qt::Key_Meta : Qt::Key_Alt;
     CmdAllWheel = mouseWheelCommand(config.readEntry("CommandAllWheel", "Nothing"));
     setCommandActiveTitlebar1(mouseCommand(config.readEntry("CommandActiveTitlebar1", "Raise"), true));
     setCommandActiveTitlebar2(mouseCommand(config.readEntry("CommandActiveTitlebar2", "Start Window Tab Drag"), true));
@@ -701,12 +809,10 @@ unsigned long Options::loadConfig()
     setCommandAll3(mouseCommand(config.readEntry("CommandAll3", "Resize"), false));
 
     // TODO: should they be moved into reloadCompositingSettings?
-    config = KConfigGroup(_config, "Compositing");
+    config = KConfigGroup(m_settings->config(), "Compositing");
     setMaxFpsInterval(1 * 1000 * 1000 * 1000 / config.readEntry("MaxFPS", Options::defaultMaxFps()));
     setRefreshRate(config.readEntry("RefreshRate", Options::defaultRefreshRate()));
     setVBlankTime(config.readEntry("VBlankTime", Options::defaultVBlankTime()) * 1000); // config in micro, value in nano resolution
-
-    return changed;
 }
 
 void Options::syncFromKcfgc()
@@ -719,7 +825,13 @@ void Options::syncFromKcfgc()
     setRollOverDesktops(m_settings->rollOverDesktops());
     setLegacyFullscreenSupport(m_settings->legacyFullscreenSupport());
     setFocusStealingPreventionLevel(m_settings->focusStealingPreventionLevel());
+
+#ifdef KWIN_BUILD_DECORATIONS
     setPlacement(m_settings->placement());
+#else
+    setPlacement(Placement::Maximizing);
+#endif
+
     setAutoRaise(m_settings->autoRaise());
     setAutoRaiseInterval(m_settings->autoRaiseInterval());
     setDelayFocusInterval(m_settings->delayFocusInterval());
@@ -745,31 +857,44 @@ void Options::syncFromKcfgc()
 
 bool Options::loadCompositingConfig (bool force)
 {
-    KSharedConfig::Ptr _config = KGlobal::config();
-    KConfigGroup config(_config, "Compositing");
+    KConfigGroup config(m_settings->config(), "Compositing");
 
     bool useCompositing = false;
     CompositingType compositingMode = NoCompositing;
-    QString compositingBackend = config.readEntry("Backend", "XRender");
-    if (compositingBackend == "XRender")
+    QString compositingBackend = config.readEntry("Backend", "OpenGL");
+    if (compositingBackend == QStringLiteral("XRender"))
         compositingMode = XRenderCompositing;
+    else if (compositingBackend == "QPainter")
+        compositingMode = QPainterCompositing;
+    else
+        compositingMode = OpenGLCompositing;
 
     if (const char *c = getenv("KWIN_COMPOSE")) {
         switch(c[0]) {
+        case 'O':
+            qDebug() << "Compositing forced to OpenGL mode by environment variable";
+            compositingMode = OpenGLCompositing;
+            useCompositing = true;
+            break;
         case 'X':
-            kDebug(1212) << "Compositing forced to XRender mode by environment variable";
+            qDebug() << "Compositing forced to XRender mode by environment variable";
             compositingMode = XRenderCompositing;
             useCompositing = true;
             break;
+        case 'Q':
+            qDebug() << "Compositing forced to QPainter mode by environment variable";
+            compositingMode = QPainterCompositing;
+            useCompositing = true;
+            break;
         case 'N':
             if (getenv("KDE_FAILSAFE"))
-                kDebug(1212) << "Compositing disabled forcefully by KDE failsafe mode";
+                qDebug() << "Compositing disabled forcefully by KDE failsafe mode";
             else
-                kDebug(1212) << "Compositing disabled forcefully by environment variable";
+                qDebug() << "Compositing disabled forcefully by environment variable";
             compositingMode = NoCompositing;
             break;
         default:
-            kDebug(1212) << "Unknown KWIN_COMPOSE mode set, ignoring";
+            qDebug() << "Unknown KWIN_COMPOSE mode set, ignoring";
             break;
         }
     }
@@ -797,8 +922,25 @@ void Options::reloadCompositingSettings(bool force)
     // see Workspace::setupCompositing(), composite.cpp
     setCompositingInitialized(true);
 
-    KSharedConfig::Ptr _config = KGlobal::config();
-    KConfigGroup config(_config, "Compositing");
+    // Compositing settings
+    KConfigGroup config(m_settings->config(), "Compositing");
+
+    setGlSmoothScale(qBound(-1, config.readEntry("GLTextureFilter", Options::defaultGlSmoothScale()), 2));
+    setGlStrictBindingFollowsDriver(!config.hasKey("GLStrictBinding"));
+    if (!isGlStrictBindingFollowsDriver()) {
+        setGlStrictBinding(config.readEntry("GLStrictBinding", Options::defaultGlStrictBinding()));
+    }
+    setGLCoreProfile(config.readEntry("GLCore", Options::defaultGLCoreProfile()));
+
+    char c = 0;
+    const QString s = config.readEntry("GLPreferBufferSwap", QString(Options::defaultGlPreferBufferSwap()));
+    if (!s.isEmpty())
+        c = s.at(0).toAscii();
+    if (c != 'a' && c != 'c' && c != 'p' && c != 'e')
+        c = 0;
+    setGlPreferBufferSwap(c);
+
+    setColorCorrected(config.readEntry("GLColorCorrection", Options::defaultColorCorrected()));
 
     m_xrenderSmoothScale = config.readEntry("XRenderSmoothScale", false);
 
@@ -824,27 +966,27 @@ void Options::reloadCompositingSettings(bool force)
 // may not be able to move it back, unless they know about Alt+LMB)
 Options::WindowOperation Options::windowOperation(const QString &name, bool restricted)
 {
-    if (name == "Move")
+    if (name == QStringLiteral("Move"))
         return restricted ? MoveOp : UnrestrictedMoveOp;
-    else if (name == "Resize")
+    else if (name == QStringLiteral("Resize"))
         return restricted ? ResizeOp : UnrestrictedResizeOp;
-    else if (name == "Maximize")
+    else if (name == QStringLiteral("Maximize"))
         return MaximizeOp;
-    else if (name == "Minimize")
+    else if (name == QStringLiteral("Minimize"))
         return MinimizeOp;
-    else if (name == "Close")
+    else if (name == QStringLiteral("Close"))
         return CloseOp;
-    else if (name == "OnAllDesktops")
+    else if (name == QStringLiteral("OnAllDesktops"))
         return OnAllDesktopsOp;
-    else if (name == "Shade")
+    else if (name == QStringLiteral("Shade"))
         return ShadeOp;
-    else if (name == "Operations")
+    else if (name == QStringLiteral("Operations"))
         return OperationsOp;
-    else if (name == "Maximize (vertical only)")
+    else if (name == QStringLiteral("Maximize (vertical only)"))
         return VMaximizeOp;
-    else if (name == "Maximize (horizontal only)")
+    else if (name == QStringLiteral("Maximize (horizontal only)"))
         return HMaximizeOp;
-    else if (name == "Lower")
+    else if (name == QStringLiteral("Lower"))
         return LowerOp;
     return NoOp;
 }
@@ -852,43 +994,43 @@ Options::WindowOperation Options::windowOperation(const QString &name, bool rest
 Options::MouseCommand Options::mouseCommand(const QString &name, bool restricted)
 {
     QString lowerName = name.toLower();
-    if (lowerName == "raise") return MouseRaise;
-    if (lowerName == "lower") return MouseLower;
-    if (lowerName == "operations menu") return MouseOperationsMenu;
-    if (lowerName == "toggle raise and lower") return MouseToggleRaiseAndLower;
-    if (lowerName == "activate and raise") return MouseActivateAndRaise;
-    if (lowerName == "activate and lower") return MouseActivateAndLower;
-    if (lowerName == "activate") return MouseActivate;
-    if (lowerName == "activate, raise and pass click") return MouseActivateRaiseAndPassClick;
-    if (lowerName == "activate and pass click") return MouseActivateAndPassClick;
-    if (lowerName == "scroll") return MouseNothing;
-    if (lowerName == "activate and scroll") return MouseActivateAndPassClick;
-    if (lowerName == "activate, raise and scroll") return MouseActivateRaiseAndPassClick;
-    if (lowerName == "activate, raise and move")
+    if (lowerName == QStringLiteral("raise")) return MouseRaise;
+    if (lowerName == QStringLiteral("lower")) return MouseLower;
+    if (lowerName == QStringLiteral("operations menu")) return MouseOperationsMenu;
+    if (lowerName == QStringLiteral("toggle raise and lower")) return MouseToggleRaiseAndLower;
+    if (lowerName == QStringLiteral("activate and raise")) return MouseActivateAndRaise;
+    if (lowerName == QStringLiteral("activate and lower")) return MouseActivateAndLower;
+    if (lowerName == QStringLiteral("activate")) return MouseActivate;
+    if (lowerName == QStringLiteral("activate, raise and pass click")) return MouseActivateRaiseAndPassClick;
+    if (lowerName == QStringLiteral("activate and pass click")) return MouseActivateAndPassClick;
+    if (lowerName == QStringLiteral("scroll")) return MouseNothing;
+    if (lowerName == QStringLiteral("activate and scroll")) return MouseActivateAndPassClick;
+    if (lowerName == QStringLiteral("activate, raise and scroll")) return MouseActivateRaiseAndPassClick;
+    if (lowerName == QStringLiteral("activate, raise and move"))
         return restricted ? MouseActivateRaiseAndMove : MouseActivateRaiseAndUnrestrictedMove;
-    if (lowerName == "move") return restricted ? MouseMove : MouseUnrestrictedMove;
-    if (lowerName == "resize") return restricted ? MouseResize : MouseUnrestrictedResize;
-    if (lowerName == "shade") return MouseShade;
-    if (lowerName == "minimize") return MouseMinimize;
-    if (lowerName == "start window tab drag") return MouseDragTab;
-    if (lowerName == "close") return MouseClose;
-    if (lowerName == "increase opacity") return MouseOpacityMore;
-    if (lowerName == "decrease opacity") return MouseOpacityLess;
-    if (lowerName == "nothing") return MouseNothing;
+    if (lowerName == QStringLiteral("move")) return restricted ? MouseMove : MouseUnrestrictedMove;
+    if (lowerName == QStringLiteral("resize")) return restricted ? MouseResize : MouseUnrestrictedResize;
+    if (lowerName == QStringLiteral("shade")) return MouseShade;
+    if (lowerName == QStringLiteral("minimize")) return MouseMinimize;
+    if (lowerName == QStringLiteral("start window tab drag")) return MouseDragTab;
+    if (lowerName == QStringLiteral("close")) return MouseClose;
+    if (lowerName == QStringLiteral("increase opacity")) return MouseOpacityMore;
+    if (lowerName == QStringLiteral("decrease opacity")) return MouseOpacityLess;
+    if (lowerName == QStringLiteral("nothing")) return MouseNothing;
     return MouseNothing;
 }
 
 Options::MouseWheelCommand Options::mouseWheelCommand(const QString &name)
 {
     QString lowerName = name.toLower();
-    if (lowerName == "raise/lower") return MouseWheelRaiseLower;
-    if (lowerName == "shade/unshade") return MouseWheelShadeUnshade;
-    if (lowerName == "maximize/restore") return MouseWheelMaximizeRestore;
-    if (lowerName == "above/below") return MouseWheelAboveBelow;
-    if (lowerName == "previous/next desktop") return MouseWheelPreviousNextDesktop;
-    if (lowerName == "change opacity") return MouseWheelChangeOpacity;
-    if (lowerName == "switch to window tab to the left/right") return MouseWheelChangeCurrentTab;
-    if (lowerName == "nothing") return MouseWheelNothing;
+    if (lowerName == QStringLiteral("raise/lower")) return MouseWheelRaiseLower;
+    if (lowerName == QStringLiteral("shade/unshade")) return MouseWheelShadeUnshade;
+    if (lowerName == QStringLiteral("maximize/restore")) return MouseWheelMaximizeRestore;
+    if (lowerName == QStringLiteral("above/below")) return MouseWheelAboveBelow;
+    if (lowerName == QStringLiteral("previous/next desktop")) return MouseWheelPreviousNextDesktop;
+    if (lowerName == QStringLiteral("change opacity")) return MouseWheelChangeOpacity;
+    if (lowerName == QStringLiteral("switch to window tab to the left/right")) return MouseWheelChangeCurrentTab;
+    if (lowerName == QStringLiteral("nothing")) return MouseWheelNothing;
     return MouseWheelChangeCurrentTab;
 }
 
