@@ -23,23 +23,35 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef KWIN_OPTIONS_H
 #define KWIN_OPTIONS_H
 
-#include <QObject>
-#include <QFont>
-#include <QPalette>
 #include <kdecoration.h>
 
+#include "main.h"
 #include "placement.h"
-#include "utils.h"
 
 namespace KWin
 {
 
+// Whether to keep all windows mapped when compositing (i.e. whether to have
+// actively updated window pixmaps).
+enum HiddenPreviews {
+    // The normal mode with regard to mapped windows. Hidden (minimized, etc.)
+    // and windows on inactive virtual desktops are not mapped, their pixmaps
+    // are only their icons.
+    HiddenPreviewsNever,
+    // Like normal mode, but shown windows (i.e. on inactive virtual desktops)
+    // are kept mapped, only hidden windows are unmapped.
+    HiddenPreviewsShown,
+    // All windows are kept mapped regardless of their state.
+    HiddenPreviewsAlways
+};
+
 class Settings;
 
-class Options : public QObject, public KDecorationOptions
+class Options : public KDecorationOptions
 {
     Q_OBJECT
     Q_ENUMS(FocusPolicy)
+    Q_ENUMS(GlSwapStrategy)
     Q_ENUMS(MouseCommand)
     Q_ENUMS(MouseWheelCommand)
 
@@ -75,7 +87,7 @@ class Options : public QObject, public KDecorationOptions
      **/
     Q_PROPERTY(bool separateScreenFocus READ isSeparateScreenFocus WRITE setSeparateScreenFocus NOTIFY separateScreenFocusChanged)
     Q_PROPERTY(int placement READ placement WRITE setPlacement NOTIFY placementChanged)
-    Q_PROPERTY(bool focusPolicyIsReasonable READ focusPolicyIsReasonable NOTIFY configChanged)
+    Q_PROPERTY(bool focusPolicyIsReasonable READ focusPolicyIsReasonable NOTIFY focusPolicyIsResonableChanged)
     /**
      * the size of the zone that triggers snapping on desktop borders
      */
@@ -162,16 +174,27 @@ class Options : public QObject, public KDecorationOptions
      * 2 = try trilinear when transformed; else 1,
      * -1 = auto
      **/
+    Q_PROPERTY(int glSmoothScale READ glSmoothScale WRITE setGlSmoothScale NOTIFY glSmoothScaleChanged)
+    Q_PROPERTY(bool colorCorrected READ isColorCorrected WRITE setColorCorrected NOTIFY colorCorrectedChanged)
     Q_PROPERTY(bool xrenderSmoothScale READ isXrenderSmoothScale WRITE setXrenderSmoothScale NOTIFY xrenderSmoothScaleChanged)
     Q_PROPERTY(qint64 maxFpsInterval READ maxFpsInterval WRITE setMaxFpsInterval NOTIFY maxFpsIntervalChanged)
     Q_PROPERTY(uint refreshRate READ refreshRate WRITE setRefreshRate NOTIFY refreshRateChanged)
     Q_PROPERTY(qint64 vBlankTime READ vBlankTime WRITE setVBlankTime NOTIFY vBlankTimeChanged)
+    Q_PROPERTY(bool glStrictBinding READ isGlStrictBinding WRITE setGlStrictBinding NOTIFY glStrictBindingChanged)
+    /**
+     * Whether strict binding follows the driver or has been overwritten by a user defined config value.
+     * If @c true @link glStrictBinding is set by the OpenGL Scene during initialization.
+     * If @c false @link glStrictBinding is set from a config value and not updated during scene initialization.
+     **/
+    Q_PROPERTY(bool glStrictBindingFollowsDriver READ isGlStrictBindingFollowsDriver WRITE setGlStrictBindingFollowsDriver NOTIFY glStrictBindingFollowsDriverChanged)
+    Q_PROPERTY(bool glCoreProfile READ glCoreProfile WRITE setGLCoreProfile NOTIFY glCoreProfileChanged)
+    Q_PROPERTY(GlSwapStrategy glPreferBufferSwap READ glPreferBufferSwap WRITE setGlPreferBufferSwap NOTIFY glPreferBufferSwapChanged)
 public:
 
     explicit Options(QObject *parent = NULL);
     ~Options();
 
-    virtual unsigned long updateSettings();
+    void updateSettings();
 
     /*!
       Different focus policies:
@@ -470,7 +493,7 @@ public:
     }
     // Separate to mode so the user can toggle
     bool isUseCompositing() const {
-        return m_useCompositing;
+        return m_useCompositing || kwinApp()->requiresCompositing();
     }
     bool isCompositingInitialized() const {
         return m_compositingInitialized;
@@ -481,7 +504,17 @@ public:
         return m_hiddenPreviews;
     }
     bool isUnredirectFullscreen() const {
-        return m_unredirectFullscreen;
+        return m_unredirectFullscreen && !kwinApp()->requiresCompositing();
+    }
+    // OpenGL
+    // 0 = no, 1 = yes when transformed,
+    // 2 = try trilinear when transformed; else 1,
+    // -1 = auto
+    int glSmoothScale() const {
+        return m_glSmoothScale;
+    }
+    bool isColorCorrected() const {
+        return m_colorCorrected;
     }
     // XRender
     bool isXrenderSmoothScale() const {
@@ -498,6 +531,20 @@ public:
     qint64 vBlankTime() const {
         return m_vBlankTime;
     }
+    bool isGlStrictBinding() const {
+        return m_glStrictBinding;
+    }
+    bool isGlStrictBindingFollowsDriver() const {
+        return m_glStrictBindingFollowsDriver;
+    }
+    bool glCoreProfile() const {
+        return m_glCoreProfile;
+    }
+
+    enum GlSwapStrategy { NoSwapEncourage = 0, CopyFrontBuffer = 'c', PaintFullScreen = 'p', ExtendDamage = 'e', AutoSwapStrategy = 'a' };
+    GlSwapStrategy glPreferBufferSwap() const {
+        return m_glPreferBufferSwap;
+    }
 
     // setters
     void setFocusPolicy(FocusPolicy focusPolicy);
@@ -549,10 +596,15 @@ public:
     void setCompositingInitialized(bool compositingInitialized);
     void setHiddenPreviews(int hiddenPreviews);
     void setUnredirectFullscreen(bool unredirectFullscreen);
+    void setGlSmoothScale(int glSmoothScale);
     void setXrenderSmoothScale(bool xrenderSmoothScale);
     void setMaxFpsInterval(qint64 maxFpsInterval);
     void setRefreshRate(uint refreshRate);
     void setVBlankTime(qint64 vBlankTime);
+    void setGlStrictBinding(bool glStrictBinding);
+    void setGlStrictBindingFollowsDriver(bool glStrictBindingFollowsDriver);
+    void setGLCoreProfile(bool glCoreProfile);
+    void setGlPreferBufferSwap(char glPreferBufferSwap);
 
     // default values
     static WindowOperation defaultOperationTitlebarDblClick() {
@@ -610,7 +662,7 @@ public:
         return true;
     }
     static CompositingType defaultCompositingMode() {
-        return XRenderCompositing;
+        return OpenGLCompositing;
     }
     static bool defaultUseCompositing() {
         return true;
@@ -624,6 +676,12 @@ public:
     static bool defaultUnredirectFullscreen() {
         return false;
     }
+    static int defaultGlSmoothScale() {
+        return 2;
+    }
+    static bool defaultColorCorrected() {
+        return false;
+    }
     static bool defaultXrenderSmoothScale() {
         return false;
     }
@@ -639,6 +697,18 @@ public:
     static uint defaultVBlankTime() {
         return 6000; // 6ms
     }
+    static bool defaultGlStrictBinding() {
+        return true;
+    }
+    static bool defaultGlStrictBindingFollowsDriver() {
+        return true;
+    }
+    static bool defaultGLCoreProfile() {
+        return false;
+    }
+    static GlSwapStrategy defaultGlPreferBufferSwap() {
+        return AutoSwapStrategy;
+    }
     static int defaultAnimationSpeed() {
         return 3;
     }
@@ -646,19 +716,18 @@ public:
     /**
      * Performs loading all settings except compositing related.
      **/
-    unsigned long loadConfig();
+    void loadConfig();
     /**
-     * Performs loading of compositing settings
+     * Performs loading of compositing settings which do not depend on OpenGL.
      **/
     bool loadCompositingConfig(bool force);
     void reparseConfiguration();
 
     //----------------------
 Q_SIGNALS:
-    void configChanged();
-
     // for properties
     void focusPolicyChanged();
+    void focusPolicyIsResonableChanged();
     void nextFocusPrefersMouseChanged();
     void clickRaiseChanged();
     void autoRaiseChanged();
@@ -707,10 +776,19 @@ Q_SIGNALS:
     void compositingInitializedChanged();
     void hiddenPreviewsChanged();
     void unredirectFullscreenChanged();
+    void glSmoothScaleChanged();
+    void colorCorrectedChanged();
     void xrenderSmoothScaleChanged();
     void maxFpsIntervalChanged();
     void refreshRateChanged();
     void vBlankTimeChanged();
+    void glStrictBindingChanged();
+    void glStrictBindingFollowsDriverChanged();
+    void glCoreProfileChanged();
+    void glPreferBufferSwapChanged();
+
+public Q_SLOTS:
+    void setColorCorrected(bool colorCorrected = false);
 
 private:
     void setElectricBorders(int borders);
@@ -745,11 +823,17 @@ private:
     bool m_compositingInitialized;
     HiddenPreviews m_hiddenPreviews;
     bool m_unredirectFullscreen;
+    int m_glSmoothScale;
+    bool m_colorCorrected;
     bool m_xrenderSmoothScale;
     qint64 m_maxFpsInterval;
     // Settings that should be auto-detected
     uint m_refreshRate;
     qint64 m_vBlankTime;
+    bool m_glStrictBinding;
+    bool m_glStrictBindingFollowsDriver;
+    bool m_glCoreProfile;
+    GlSwapStrategy m_glPreferBufferSwap;
 
     WindowOperation OpTitlebarDblClick;
 
