@@ -28,8 +28,9 @@ DEALINGS IN THE SOFTWARE.
 #include "client.h"
 #include "deleted.h"
 #include "effects.h"
+#include <kwinglplatform.h>
+#include <kwinglutils.h>
 #include <kwinxrenderutils.h>
-#include <kdebug.h>
 #include <QPaintEngine>
 #include <qevent.h>
 #include <qpainter.h>
@@ -38,19 +39,27 @@ DEALINGS IN THE SOFTWARE.
 namespace KWin
 {
 
-PaintRedirector *PaintRedirector::create(Client *c, QWidget *widget)
+PaintRedirector *PaintRedirector::create(Client *c, KDecoration *deco)
 {
-    return new RasterXRenderPaintRedirector(c, widget);
+    if (effects->isOpenGLCompositing()) {
+        return new OpenGLPaintRedirector(c, deco);
+    } else if (effects->compositingType() == XRenderCompositing) {
+        return new RasterXRenderPaintRedirector(c, deco);
+    } else if (effects->compositingType() == QPainterCompositing) {
+        return new QImagePaintRedirector(c, deco);
+    }
+    return NULL;
 }
 
-PaintRedirector::PaintRedirector(Client *c, QWidget* w)
-    : QObject(w)
-    , widget(w)
+PaintRedirector::PaintRedirector(Client *c, KDecoration *deco)
+    : QObject(deco)
+    , widget(deco->widget())
     , recursionCheck(false)
     , m_client(c)
+    , m_decoration(deco)
     , m_requiresRepaint(false)
 {
-    added(w);
+    added(deco->widget());
 }
 
 PaintRedirector::~PaintRedirector()
@@ -62,6 +71,7 @@ void PaintRedirector::reparent(Deleted *d)
     setParent(d);
     widget = NULL;
     m_client = NULL;
+    m_decoration = NULL;
 }
 
 static int align(int value, int align)
@@ -71,7 +81,7 @@ static int align(int value, int align)
 
 void PaintRedirector::performPendingPaint()
 {
-    if (!widget) {
+    if (!widget && !m_decoration->window()) {
         return;
     }
     //qDebug() << "### performing paint, pending:" << pending.boundingRect();
@@ -84,8 +94,7 @@ void PaintRedirector::performPendingPaint()
     }
     fillScratch(Qt::transparent);
     recursionCheck = true;
-    // do not use DrawWindowBackground, it's ok to be transparent
-    widget->render(scratch, QPoint(), pending.boundingRect(), QWidget::DrawChildren);
+    m_decoration->render(scratch, pending.boundingRect());
     recursionCheck = false;
     cleanupTimer.start(2000, this);
 }
@@ -136,6 +145,19 @@ bool PaintRedirector::eventFilter(QObject* o, QEvent* e)
     return false;
 }
 
+void PaintRedirector::addRepaint(const QRegion &region)
+{
+    pending |= region;
+    scheduled = pending;
+
+    // schedule repaint
+    const int paddingLeft = m_client->paddingLeft();
+    const int paddingTop = m_client->paddingTop();
+    const bool needsTranslate = (paddingLeft != 0 || paddingTop != 0);
+    m_client->addRepaint(needsTranslate ? pending.translated(-paddingLeft, -paddingTop) : pending);
+    m_requiresRepaint = true;
+}
+
 QRegion PaintRedirector::pendingRegion() const
 {
     return pending;
@@ -150,6 +172,9 @@ QRegion PaintRedirector::scheduledRepaintRegion()
 
 void PaintRedirector::added(QWidget* w)
 {
+    if (!w) {
+        return;
+    }
     w->installEventFilter(this);
     foreach (QObject * o, w->children()) {
         if (o->isWidgetType() && !isToolTip(static_cast< QWidget* >(o)))
@@ -233,12 +258,24 @@ void PaintRedirector::resizePixmaps(const QRect *rects)
     }
 }
 
+GLTexture *PaintRedirector::texture(PaintRedirector::DecorationPixmap border) const
+{
+    Q_UNUSED(border)
+    return NULL;
+}
+
 xcb_render_picture_t PaintRedirector::picture(PaintRedirector::DecorationPixmap border) const
 {
     Q_UNUSED(border)
     return XCB_RENDER_PICTURE_NONE;
 }
 
+const QImage *PaintRedirector::image(PaintRedirector::DecorationPixmap border) const
+{
+    Q_UNUSED(border)
+    return NULL;
+}
+
 void PaintRedirector::resize(DecorationPixmap border, const QSize &size)
 {
     Q_UNUSED(border)
@@ -255,10 +292,129 @@ void PaintRedirector::paint(DecorationPixmap border, const QRect& r, const QRect
 
 
 
+
 // ------------------------------------------------------------------
 
-RasterXRenderPaintRedirector::RasterXRenderPaintRedirector(Client *c, QWidget *widget)
-    : PaintRedirector(c, widget)
+
+
+
+ImageBasedPaintRedirector::ImageBasedPaintRedirector(Client *c, KDecoration *deco)
+    : PaintRedirector(c, deco)
+{
+}
+
+ImageBasedPaintRedirector::~ImageBasedPaintRedirector()
+{
+}
+
+QPaintDevice *ImageBasedPaintRedirector::recreateScratch(const QSize &size)
+{
+    m_scratchImage = QImage(size, QImage::Format_ARGB32_Premultiplied);
+    return &m_scratchImage;
+}
+
+QPaintDevice *ImageBasedPaintRedirector::scratch()
+{
+    return &m_scratchImage;
+}
+
+void ImageBasedPaintRedirector::fillScratch(Qt::GlobalColor color)
+{
+    m_scratchImage.fill(color);
+}
+
+void ImageBasedPaintRedirector::discardScratch()
+{
+    m_scratchImage = QImage();
+}
+
+
+
+// ------------------------------------------------------------------
+
+
+OpenGLPaintRedirector::OpenGLPaintRedirector(Client *c, KDecoration *deco)
+    : ImageBasedPaintRedirector(c, deco)
+{
+    for (int i = 0; i < TextureCount; ++i)
+        m_textures[i] = NULL;
+
+    PaintRedirector::resizePixmaps();
+}
+
+OpenGLPaintRedirector::~OpenGLPaintRedirector()
+{
+    for (int i = 0; i < TextureCount; ++i)
+        delete m_textures[i];
+}
+
+void OpenGLPaintRedirector::resizePixmaps(const QRect *rects)
+{
+    QSize size[2];
+    size[LeftRight] = QSize(rects[LeftPixmap].width() + rects[RightPixmap].width(),
+                            align(qMax(rects[LeftPixmap].height(), rects[RightPixmap].height()), 128));
+    size[TopBottom] = QSize(align(qMax(rects[TopPixmap].width(), rects[BottomPixmap].width()), 128),
+                            rects[TopPixmap].height() + rects[BottomPixmap].height());
+
+    effects->makeOpenGLContextCurrent();
+    if (!GLTexture::NPOTTextureSupported()) {
+        for (int i = 0; i < 2; i++) {
+            size[i].rwidth()  = nearestPowerOfTwo(size[i].width());
+            size[i].rheight() = nearestPowerOfTwo(size[i].height());
+        }
+    }
+
+    for (int i = 0; i < 2; i++) {
+        if (m_textures[i] && m_textures[i]->size() == size[i])
+            continue;
+
+        delete m_textures[i];
+        m_textures[i] = NULL;
+
+        if (size[i].isEmpty())
+            continue;
+
+        m_textures[i] = new GLTexture(size[i].width(), size[i].height());
+        m_textures[i]->setYInverted(true);
+        m_textures[i]->setWrapMode(GL_CLAMP_TO_EDGE);
+        m_textures[i]->clear();
+    }
+}
+
+void OpenGLPaintRedirector::updatePixmaps(const QRect *rects, const QRegion &region)
+{
+    const QImage &image = scratchImage();
+    const QRect bounding = region.boundingRect();
+
+    const int leftWidth = rects[LeftPixmap].width();
+    const int topHeight = rects[TopPixmap].height();
+
+    // Top, Right, Bottom, Left
+    GLTexture *textures[4] = { m_textures[TopBottom], m_textures[LeftRight], m_textures[TopBottom], m_textures[LeftRight] };
+    QPoint offsets[4] = { QPoint(0, 0), QPoint(leftWidth, 0), QPoint(0, topHeight), QPoint(0, 0) };
+
+    for (int i = 0; i < 4; i++) {
+        const QRect dirty = (region & rects[i]).boundingRect();
+        if (!textures[i] || dirty.isEmpty())
+            continue;
+
+        const QPoint dst = dirty.topLeft() - rects[i].topLeft() + offsets[i];
+        const QRect src(dirty.topLeft() - bounding.topLeft(), dirty.size());
+
+        textures[i]->update(image, dst, src);
+    }
+}
+
+
+
+
+// ------------------------------------------------------------------
+
+
+
+
+RasterXRenderPaintRedirector::RasterXRenderPaintRedirector(Client *c, KDecoration *deco)
+    : ImageBasedPaintRedirector(c, deco)
     , m_gc(0)
 {
     for (int i=0; i<PixmapCount; ++i) {
@@ -317,32 +473,43 @@ void RasterXRenderPaintRedirector::paint(PaintRedirector::DecorationPixmap borde
         xcb_create_gc(connection(), m_gc, m_pixmaps[border], 0, NULL);
     }
 
-    const QImage img(m_scratchImage.copy(QRect(bounding.topLeft() - b.topLeft(), bounding.size())));
+    const QImage img(scratchImage().copy(QRect(bounding.topLeft() - b.topLeft(), bounding.size())));
     xcb_put_image(connection(), XCB_IMAGE_FORMAT_Z_PIXMAP, m_pixmaps[border], m_gc,
                   img.width(), img.height(), offset.x(), offset.y(), 0, 32, img.byteCount(), img.constBits());
 }
 
-QPaintDevice *RasterXRenderPaintRedirector::recreateScratch(const QSize &size)
+QImagePaintRedirector::QImagePaintRedirector(Client *c, KDecoration *deco)
+    : ImageBasedPaintRedirector(c, deco)
 {
-    m_scratchImage = QImage(size, QImage::Format_ARGB32_Premultiplied);
-    return &m_scratchImage;
+    resizePixmaps();
 }
 
-QPaintDevice *RasterXRenderPaintRedirector::scratch()
+QImagePaintRedirector::~QImagePaintRedirector()
 {
-    return &m_scratchImage;
 }
 
-void RasterXRenderPaintRedirector::fillScratch(Qt::GlobalColor color)
+void QImagePaintRedirector::paint(PaintRedirector::DecorationPixmap border, const QRect &r, const QRect &b, const QRegion &reg)
 {
-    m_scratchImage.fill(color);
+    // clip the sub area
+    const QRect bounding = reg.boundingRect();
+    const QPoint offset = bounding.topLeft() - r.topLeft();
+
+    QPainter painter(&m_images[border]);
+    painter.setCompositionMode(QPainter::CompositionMode_Source);
+    painter.drawImage(offset, scratchImage(), QRect(bounding.topLeft() - b.topLeft(), bounding.size()));
 }
 
-void RasterXRenderPaintRedirector::discardScratch()
+void QImagePaintRedirector::resize(PaintRedirector::DecorationPixmap border, const QSize &size)
 {
-    m_scratchImage = QImage();
+    m_images[border] = QImage(size, QImage::Format_ARGB32_Premultiplied);
+    m_images[border].fill(Qt::transparent);
+}
+
+const QImage *QImagePaintRedirector::image(PaintRedirector::DecorationPixmap border) const
+{
+    return &m_images[border];
 }
 
 } // namespace
 
-#include "moc_paintredirector.cpp"
+#include "paintredirector.moc"
