@@ -24,9 +24,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <kconfig.h>
 #include <KXMessages>
 #include <QRegExp>
-#include <ktemporaryfile.h>
+#include <QTemporaryFile>
 #include <QFile>
+#include <QFileInfo>
 #include <ktoolinvocation.h>
+#include <QDebug>
 
 #ifndef KCMRULES
 #include "client.h"
@@ -69,6 +71,7 @@ Rules::Rules()
     , belowrule(UnusedSetRule)
     , fullscreenrule(UnusedSetRule)
     , noborderrule(UnusedSetRule)
+    , decocolorrule(UnusedForceRule)
     , blockcompositingrule(UnusedForceRule)
     , fsplevelrule(UnusedForceRule)
     , acceptfocusrule(UnusedForceRule)
@@ -85,7 +88,7 @@ Rules::Rules()
 Rules::Rules(const QString& str, bool temporary)
     : temporary_state(temporary ? 2 : 0)
 {
-    KTemporaryFile file;
+    QTemporaryFile file;
     if (file.open()) {
         QByteArray s = str.toUtf8();
         file.write(s.data(), s.length());
@@ -94,7 +97,7 @@ Rules::Rules(const QString& str, bool temporary)
     KConfig cfg(file.fileName(), KConfig::SimpleConfig);
     readFromCfg(cfg.group(QString()));
     if (description.isEmpty())
-        description = "temporary";
+        description = QStringLiteral("temporary");
 }
 
 #define READ_MATCH_STRING( var, func ) \
@@ -104,19 +107,19 @@ Rules::Rules(const QString& str, bool temporary)
 
 #define READ_SET_RULE( var, func, def ) \
     var = func ( cfg.readEntry( #var, def)); \
-    var##rule = readSetRule( cfg, #var "rule" );
+    var##rule = readSetRule( cfg, QStringLiteral( #var "rule" ) );
 
 #define READ_SET_RULE_DEF( var , func, def ) \
     var = func ( cfg.readEntry( #var, def )); \
-    var##rule = readSetRule( cfg, #var "rule" );
+    var##rule = readSetRule( cfg, QStringLiteral( #var "rule" ) );
 
 #define READ_FORCE_RULE( var, func, def) \
     var = func ( cfg.readEntry( #var, def)); \
-    var##rule = readForceRule( cfg, #var "rule" );
+    var##rule = readForceRule( cfg, QStringLiteral( #var "rule" ) );
 
 #define READ_FORCE_RULE2( var, def, func, funcarg ) \
     var = func ( cfg.readEntry( #var, def),funcarg ); \
-    var##rule = readForceRule( cfg, #var "rule" );
+    var##rule = readForceRule( cfg, QStringLiteral( #var "rule" ) );
 
 
 
@@ -141,7 +144,7 @@ void Rules::readFromCfg(const KConfigGroup& cfg)
     READ_MATCH_STRING(windowrole, .toLower().toLatin1());
     READ_MATCH_STRING(title,);
     READ_MATCH_STRING(clientmachine, .toLower().toLatin1());
-    types = cfg.readEntry("types", uint(NET::AllTypesMask));
+    types = NET::WindowTypeMask(cfg.readEntry<uint>("types", NET::AllTypesMask));
     READ_FORCE_RULE2(placement, QString(), Placement::policyFromString, false);
     READ_SET_RULE_DEF(position, , invalidPoint);
     READ_SET_RULE(size, , QSize());
@@ -163,8 +166,8 @@ void Rules::readFromCfg(const KConfigGroup& cfg)
     READ_SET_RULE(desktop, , 0);
     READ_SET_RULE(screen, , 0);
     READ_SET_RULE(activity, , QString());
-    type = readType(cfg, "type");
-    typerule = type != NET::Unknown ? readForceRule(cfg, "typerule") : UnusedForceRule;
+    type = readType(cfg, QStringLiteral("type"));
+    typerule = type != NET::Unknown ? readForceRule(cfg, QStringLiteral("typerule")) : UnusedForceRule;
     READ_SET_RULE(maximizevert, , false);
     READ_SET_RULE(maximizehoriz, , false);
     READ_SET_RULE(minimize, , false);
@@ -176,6 +179,8 @@ void Rules::readFromCfg(const KConfigGroup& cfg)
     READ_SET_RULE(below, , false);
     READ_SET_RULE(fullscreen, , false);
     READ_SET_RULE(noborder, , false);
+    decocolor = readDecoColor(cfg);
+    decocolorrule = decocolor.isEmpty() ? UnusedForceRule : readForceRule(cfg, QStringLiteral("decocolorrule"));
     READ_FORCE_RULE(blockcompositing, , false);
     READ_FORCE_RULE(fsplevel, limit0to4, 0); // fsp is 0-4
     READ_FORCE_RULE(acceptfocus, , false);
@@ -193,10 +198,10 @@ void Rules::readFromCfg(const KConfigGroup& cfg)
 #undef READ_FORCE_RULE
 #undef READ_FORCE_RULE2
 
-#define WRITE_MATCH_STRING( var, cast, force ) \
+#define WRITE_MATCH_STRING( var, force ) \
     if ( !var.isEmpty() || force ) \
     { \
-        cfg.writeEntry( #var, cast var ); \
+        cfg.writeEntry( #var, var ); \
         cfg.writeEntry( #var "match", (int)var##match ); \
     } \
     else \
@@ -233,11 +238,11 @@ void Rules::write(KConfigGroup& cfg) const
 {
     cfg.writeEntry("Description", description);
     // always write wmclass
-    WRITE_MATCH_STRING(wmclass, (const char*), true);
+    WRITE_MATCH_STRING(wmclass, true);
     cfg.writeEntry("wmclasscomplete", wmclasscomplete);
-    WRITE_MATCH_STRING(windowrole, (const char*), false);
-    WRITE_MATCH_STRING(title, , false);
-    WRITE_MATCH_STRING(clientmachine, (const char*), false);
+    WRITE_MATCH_STRING(windowrole, false);
+    WRITE_MATCH_STRING(title, false);
+    WRITE_MATCH_STRING(clientmachine, false);
     if (types != NET::AllTypesMask)
         cfg.writeEntry("types", uint(types));
     else
@@ -265,6 +270,14 @@ void Rules::write(KConfigGroup& cfg) const
     WRITE_SET_RULE(below,);
     WRITE_SET_RULE(fullscreen,);
     WRITE_SET_RULE(noborder,);
+    auto colorToString = [](const QString &value) -> QString {
+        if (value.endsWith(QStringLiteral(".colors"))) {
+            return QFileInfo(value).baseName();
+        } else {
+            return value;
+        }
+    };
+    WRITE_FORCE_RULE(decocolor, colorToString);
     WRITE_FORCE_RULE(blockcompositing,);
     WRITE_FORCE_RULE(fsplevel,);
     WRITE_FORCE_RULE(acceptfocus,);
@@ -307,6 +320,7 @@ bool Rules::isEmpty() const
            && belowrule == UnusedSetRule
            && fullscreenrule == UnusedSetRule
            && noborderrule == UnusedSetRule
+           && decocolorrule == UnusedForceRule
            && blockcompositingrule == UnusedForceRule
            && fsplevelrule == UnusedForceRule
            && acceptfocusrule == UnusedForceRule
@@ -343,6 +357,17 @@ NET::WindowType Rules::readType(const KConfigGroup& cfg, const QString& key)
     return NET::Unknown;
 }
 
+QString Rules::readDecoColor(const KConfigGroup &cfg)
+{
+    QString themeName = cfg.readEntry("decocolor", QString());
+    if (themeName.isEmpty()) {
+        return QString();
+    }
+    // find the actual scheme file
+    return QStandardPaths::locate(QStandardPaths::GenericDataLocation,
+                                  QStringLiteral("color-schemes/") + themeName + QStringLiteral(".colors"));
+}
+
 bool Rules::matchType(NET::WindowType match_type) const
 {
     if (types != NET::AllTypesMask) {
@@ -360,7 +385,7 @@ bool Rules::matchWMClass(const QByteArray& match_class, const QByteArray& match_
         // TODO optimize?
         QByteArray cwmclass = wmclasscomplete
                               ? match_name + ' ' + match_class : match_class;
-        if (wmclassmatch == RegExpMatch && QRegExp(wmclass).indexIn(cwmclass) == -1)
+        if (wmclassmatch == RegExpMatch && QRegExp(QString::fromUtf8(wmclass)).indexIn(QString::fromUtf8(cwmclass)) == -1)
             return false;
         if (wmclassmatch == ExactMatch && wmclass != cwmclass)
             return false;
@@ -373,7 +398,7 @@ bool Rules::matchWMClass(const QByteArray& match_class, const QByteArray& match_
 bool Rules::matchRole(const QByteArray& match_role) const
 {
     if (windowrolematch != UnimportantMatch) {
-        if (windowrolematch == RegExpMatch && QRegExp(windowrole).indexIn(match_role) == -1)
+        if (windowrolematch == RegExpMatch && QRegExp(QString::fromUtf8(windowrole)).indexIn(QString::fromUtf8(match_role)) == -1)
             return false;
         if (windowrolematch == ExactMatch && windowrole != match_role)
             return false;
@@ -404,7 +429,7 @@ bool Rules::matchClientMachine(const QByteArray& match_machine, bool local) cons
                 && matchClientMachine("localhost", true))
             return true;
         if (clientmachinematch == RegExpMatch
-                && QRegExp(clientmachine).indexIn(match_machine) == -1)
+                && QRegExp(QString::fromUtf8(clientmachine)).indexIn(QString::fromUtf8(match_machine)) == -1)
             return false;
         if (clientmachinematch == ExactMatch
                 && clientmachine != match_machine)
@@ -470,6 +495,12 @@ bool Rules::update(Client* c, int selection)
         updated = updated || screen != c->screen();
         screen = c->screen();
     }
+    if NOW_REMEMBER(Activity, activity) {
+        // TODO: ivan - multiple activities support
+        const QString & joinedActivities = c->activities().join(QStringLiteral(","));
+        updated = updated || activity != joinedActivities;
+        activity = joinedActivities;
+    }
     if NOW_REMEMBER(MaximizeVert, maximizevert) {
         updated = updated || maximizevert != bool(c->maximizeMode() & MaximizeVertical);
         maximizevert = c->maximizeMode() & MaximizeVertical;
@@ -618,6 +649,7 @@ APPLY_RULE(above, KeepAbove, bool)
 APPLY_RULE(below, KeepBelow, bool)
 APPLY_RULE(fullscreen, FullScreen, bool)
 APPLY_RULE(noborder, NoBorder, bool)
+APPLY_FORCE_RULE(decocolor, DecoColor, QString)
 APPLY_FORCE_RULE(blockcompositing, BlockCompositing, bool)
 APPLY_FORCE_RULE(fsplevel, FSP, int)
 APPLY_FORCE_RULE(acceptfocus, AcceptFocus, bool)
@@ -685,6 +717,7 @@ void Rules::discardUsed(bool withdrawn)
     DISCARD_USED_SET_RULE(below);
     DISCARD_USED_SET_RULE(fullscreen);
     DISCARD_USED_SET_RULE(noborder);
+    DISCARD_USED_FORCE_RULE(decocolor);
     DISCARD_USED_FORCE_RULE(blockcompositing);
     DISCARD_USED_FORCE_RULE(fsplevel);
     DISCARD_USED_FORCE_RULE(acceptfocus);
@@ -817,6 +850,7 @@ CHECK_RULE(KeepAbove, bool)
 CHECK_RULE(KeepBelow, bool)
 CHECK_RULE(FullScreen, bool)
 CHECK_RULE(NoBorder, bool)
+CHECK_FORCE_RULE(DecoColor, QString)
 CHECK_FORCE_RULE(BlockCompositing, bool)
 CHECK_FORCE_RULE(FSP, int)
 CHECK_FORCE_RULE(AcceptFocus, bool)
@@ -854,6 +888,7 @@ void Client::applyWindowRules()
     // IgnoreGeometry
     setDesktop(desktop());
     workspace()->sendClientToScreen(this, screen());
+    setOnActivities(activities());
     // Type
     maximize(maximizeMode());
     // Minimize : functions don't check, and there are two functions
@@ -869,6 +904,7 @@ void Client::applyWindowRules()
     setKeepBelow(keepBelow());
     setFullScreen(isFullScreen(), true);
     setNoBorder(noBorder());
+    updateColorScheme();
     // FSP
     // AcceptFocus :
     if (workspace()->mostRecentlyActivatedClient() == this
@@ -913,7 +949,7 @@ RuleBook::RuleBook(QObject *parent)
     : QObject(parent)
     , m_updateTimer(new QTimer(this))
     , m_updatesDisabled(false)
-    , m_temporaryRulesMessages(new KXMessages("_KDE_NET_WM_TEMPORARY_RULES"))
+    , m_temporaryRulesMessages(new KXMessages("_KDE_NET_WM_TEMPORARY_RULES", NULL))
 {
     connect(m_temporaryRulesMessages.data(), SIGNAL(gotMessage(QString)), SLOT(temporaryRulesMessage(QString)));
     connect(m_updateTimer, SIGNAL(timeout()), SLOT(save()));
@@ -945,7 +981,7 @@ WindowRules RuleBook::find(const Client* c, bool ignore_temporary)
         }
         if ((*it)->match(c)) {
             Rules* rule = *it;
-            kDebug(1212) << "Rule found:" << rule << ":" << c;
+            qDebug() << "Rule found:" << rule << ":" << c;
             if (rule->isTemporary())
                 it = m_rules.erase(it);
             else
@@ -962,16 +998,16 @@ void RuleBook::edit(Client* c, bool whole_app)
 {
     save();
     QStringList args;
-    args << "--wid" << QString::number(c->window());
+    args << QStringLiteral("--wid") << QString::number(c->window());
     if (whole_app)
-        args << "--whole-app";
-    KToolInvocation::kdeinitExec("kwin_rules_dialog", args);
+        args << QStringLiteral("--whole-app");
+    KToolInvocation::kdeinitExec(QStringLiteral("kwin_rules_dialog"), args);
 }
 
 void RuleBook::load()
 {
     deleteAll();
-    KConfig cfg("kwinrulesrc", KConfig::NoGlobals);
+    KConfig cfg(QStringLiteral(KWIN_NAME) + QStringLiteral("rulesrc"), KConfig::NoGlobals);
     int count = cfg.group("General").readEntry("count", 0);
     for (int i = 1;
             i <= count;
@@ -985,7 +1021,7 @@ void RuleBook::load()
 void RuleBook::save()
 {
     m_updateTimer->stop();
-    KConfig cfg("kwinrulesrc", KConfig::NoGlobals);
+    KConfig cfg(QStringLiteral(KWIN_NAME) + QStringLiteral("rulesrc"), KConfig::NoGlobals);
     QStringList groups = cfg.groupList();
     for (QStringList::ConstIterator it = groups.constBegin();
             it != groups.constEnd();
