@@ -20,7 +20,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 /*
  The base class for compositing, implementing shared functionality
- between the OpenGL (that is no more) and XRender backends.
+ between the OpenGL and XRender backends.
 
  Design:
 
@@ -68,10 +68,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "scene.h"
 
-#include <X11/extensions/shape.h>
-
-#include <QGraphicsScene>
-#include <QGraphicsView>
+#include <QQuickWindow>
 #include <QVector2D>
 
 #include "client.h"
@@ -93,6 +90,7 @@ namespace KWin
 
 Scene::Scene(Workspace* ws)
     : QObject(ws)
+    , wspace(ws)
 {
     last_time.invalidate(); // Initialize the timer
     connect(Workspace::self(), SIGNAL(deletedRemoved(KWin::Deleted*)), SLOT(windowDeleted(KWin::Deleted*)));
@@ -100,6 +98,9 @@ Scene::Scene(Workspace* ws)
 
 Scene::~Scene()
 {
+    foreach (Window *w, m_windows) {
+        delete w;
+    }
 }
 
 // returns mask and possibly modified region
@@ -223,7 +224,7 @@ void Scene::paintGenericScreen(int orig_mask, ScreenPaintData)
         effects->prePaintWindow(effectWindow(w), data, time_diff);
 #ifndef NDEBUG
         if (data.quads.isTransformed()) {
-            kFatal(1212) << "Pre-paint calls are not allowed to transform quads!" ;
+            qFatal("Pre-paint calls are not allowed to transform quads!");
         }
 #endif
         if (!w->isPaintingEnabled()) {
@@ -253,7 +254,10 @@ void Scene::paintSimpleScreen(int orig_mask, QRegion region)
 
     QRegion dirtyArea = region;
     bool opaqueFullscreen(false);
-    foreach (Window *w, stacking_order) { // do prePaintWindow bottom to top
+    for (int i = 0;  // do prePaintWindow bottom to top
+            i < stacking_order.count();
+            ++i) {
+        Window* w = stacking_order[ i ];
         Toplevel* topw = w->window();
         WindowPrePaintData data;
         data.mask = orig_mask | (w->isOpaque() ? PAINT_WINDOW_OPAQUE : PAINT_WINDOW_TRANSLUCENT);
@@ -298,7 +302,7 @@ void Scene::paintSimpleScreen(int orig_mask, QRegion region)
         effects->prePaintWindow(effectWindow(w), data, time_diff);
 #ifndef NDEBUG
         if (data.quads.isTransformed()) {
-            kFatal(1212) << "Pre-paint calls are not allowed to transform quads!" ;
+            qFatal("Pre-paint calls are not allowed to transform quads!");
         }
 #endif
         if (!w->isPaintingEnabled()) {
@@ -390,6 +394,64 @@ void Scene::paintSimpleScreen(int orig_mask, QRegion region)
     }
 }
 
+void Scene::windowAdded(Toplevel *c)
+{
+    assert(!m_windows.contains(c));
+    Scene::Window *w = createWindow(c);
+    m_windows[ c ] = w;
+    connect(c, SIGNAL(geometryShapeChanged(KWin::Toplevel*,QRect)), SLOT(windowGeometryShapeChanged(KWin::Toplevel*)));
+    connect(c, SIGNAL(windowClosed(KWin::Toplevel*,KWin::Deleted*)), SLOT(windowClosed(KWin::Toplevel*,KWin::Deleted*)));
+    c->effectWindow()->setSceneWindow(w);
+    c->getShadow();
+    w->updateShadow(c->shadow());
+}
+
+void Scene::windowClosed(Toplevel *c, Deleted *deleted)
+{
+    assert(m_windows.contains(c));
+    if (deleted != NULL) {
+        // replace c with deleted
+        Window* w = m_windows.take(c);
+        w->updateToplevel(deleted);
+        if (w->shadow()) {
+            w->shadow()->setToplevel(deleted);
+        }
+        m_windows[ deleted ] = w;
+    } else {
+        delete m_windows.take(c);
+        c->effectWindow()->setSceneWindow(NULL);
+    }
+}
+
+void Scene::windowDeleted(Deleted *c)
+{
+    assert(m_windows.contains(c));
+    delete m_windows.take(c);
+    c->effectWindow()->setSceneWindow(NULL);
+}
+
+void Scene::windowGeometryShapeChanged(Toplevel *c)
+{
+    if (!m_windows.contains(c))    // this is ok, shape is not valid by default
+        return;
+    Window *w = m_windows[ c ];
+    w->discardShape();
+}
+
+void Scene::createStackingOrder(ToplevelList toplevels)
+{
+    // TODO: cache the stacking_order in case it has not changed
+    foreach (Toplevel *c, toplevels) {
+        assert(m_windows.contains(c));
+        stacking_order.append(m_windows[ c ]);
+    }
+}
+
+void Scene::clearStackingOrder()
+{
+    stacking_order.clear();
+}
+
 static Scene::Window *s_recursionCheck = NULL;
 
 void Scene::paintWindow(Window* w, int mask, QRegion region, WindowQuadList quads)
@@ -416,6 +478,26 @@ void Scene::paintWindow(Window* w, int mask, QRegion region, WindowQuadList quad
     paintDesktopThumbnails(w);
 }
 
+static void adjustClipRegion(AbstractThumbnailItem *item, QRegion &clippingRegion)
+{
+    if (item->clip() && item->clipTo()) {
+        // the x/y positions of the parent item are not correct. The margins are added, though the size seems fine
+        // that's why we have to get the offset by inspecting the anchors properties
+        QQuickItem *parentItem = item->clipTo();
+        QPointF offset;
+        QVariant anchors = parentItem->property("anchors");
+        if (anchors.isValid()) {
+            if (QObject *anchorsObject = anchors.value<QObject*>()) {
+                offset.setX(anchorsObject->property("leftMargin").toReal());
+                offset.setY(anchorsObject->property("topMargin").toReal());
+            }
+        }
+        const QRectF rect = parentItem->mapRectToScene(QRectF(parentItem->position() - offset,
+                                                                QSizeF(parentItem->width(), parentItem->height())));
+        clippingRegion &= rect.adjusted(0,0,-1,-1).translated(item->window()->position()).toRect();
+    }
+}
+
 void Scene::paintWindowThumbnails(Scene::Window *w, QRegion region, qreal opacity, qreal brightness, qreal saturation)
 {
     EffectWindowImpl *wImpl = static_cast<EffectWindowImpl*>(effectWindow(w));
@@ -444,18 +526,11 @@ void Scene::paintWindowThumbnails(Scene::Window *w, QRegion region, qreal opacit
         }
         thumbData.setXScale(size.width() / static_cast<qreal>(visualThumbRect.width()));
         thumbData.setYScale(size.height() / static_cast<qreal>(visualThumbRect.height()));
-        // it can happen in the init/closing phase of the tabbox
-        // that the corresponding QGraphicsScene is not available
-        if (item->scene() == 0) {
-            continue;
-        }
 
-        QGraphicsView* declview = findViewForThumbnailItem(item, w);
-        if (declview == 0) {
+        if (!item->window()) {
             continue;
         }
-        QPoint viewPos = findOffsetInWindow(declview, w->window()->window());
-        const QPoint point = viewPos + declview->mapFromScene(item->scenePos());
+        const QPointF point = item->mapToScene(item->position());
         qreal x = point.x() + w->x() + (item->width() - size.width())/2;
         qreal y = point.y() + w->y() + (item->height() - size.height()) / 2;
         x -= thumb->x();
@@ -465,7 +540,7 @@ void Scene::paintWindowThumbnails(Scene::Window *w, QRegion region, qreal opacit
         y += (thumb->y()-visualThumbRect.y())*thumbData.yScale();
         thumbData.setXTranslation(x);
         thumbData.setYTranslation(y);
-        int thumbMask = PAINT_WINDOW_TRANSFORMED;
+        int thumbMask = PAINT_WINDOW_TRANSFORMED | PAINT_WINDOW_LANCZOS;
         if (thumbData.opacity() == 1.0) {
             thumbMask |= PAINT_WINDOW_OPAQUE;
         } else {
@@ -473,14 +548,7 @@ void Scene::paintWindowThumbnails(Scene::Window *w, QRegion region, qreal opacit
         }
         QRegion clippingRegion = region;
         clippingRegion &= QRegion(wImpl->x(), wImpl->y(), wImpl->width(), wImpl->height());
-        QPainterPath path = item->clipPath();
-        if (!path.isEmpty()) {
-            // here we assume that the clippath consists of a single rectangle
-            const QPolygonF sceneBounds = item->mapToScene(path.boundingRect());
-            const QRect viewBounds = declview->mapFromScene(sceneBounds).boundingRect();
-            // shrinking the rect due to rounding errors
-            clippingRegion &= viewBounds.adjusted(0,0,-1,-1).translated(viewPos + w->pos());
-        }
+        adjustClipRegion(item, clippingRegion);
         effects->drawWindow(thumb, thumbMask, clippingRegion, thumbData);
     }
 }
@@ -495,16 +563,9 @@ void Scene::paintDesktopThumbnails(Scene::Window *w)
         if (!item->isVisible()) {
             continue;
         }
-        // it can happen in the init/closing phase of the tabbox
-        // that the corresponding QGraphicsScene is not available
-        if (item->scene() == 0) {
+        if (!item->window()) {
             continue;
         }
-        QGraphicsView* declview = findViewForThumbnailItem(item, w);
-        if (declview == 0) {
-            continue;
-        }
-        QPoint viewPos = findOffsetInWindow(declview, w->window()->window());
         s_recursionCheck = w;
 
         ScreenPaintData data;
@@ -513,20 +574,13 @@ void Scene::paintDesktopThumbnails(Scene::Window *w)
         size.scale(item->width(), item->height(), Qt::KeepAspectRatio);
         data *= QVector2D(size.width() / double(displayWidth()),
                           size.height() / double(displayHeight()));
-        const QPoint point = viewPos + declview->mapFromScene(item->scenePos());
+        const QPointF point = item->mapToScene(item->position());
         const qreal x = point.x() + w->x() + (item->width() - size.width())/2;
         const qreal y = point.y() + w->y() + (item->height() - size.height()) / 2;
         const QRect region = QRect(x, y, item->width(), item->height());
         QRegion clippingRegion = region;
         clippingRegion &= QRegion(wImpl->x(), wImpl->y(), wImpl->width(), wImpl->height());
-        QPainterPath path = item->clipPath();
-        if (!path.isEmpty()) {
-            // here we assume that the clippath consists of a single rectangle
-            const QPolygonF sceneBounds = item->mapToScene(path.boundingRect());
-            const QRect viewBounds = declview->mapFromScene(sceneBounds).boundingRect();
-            // shrinking the rect due to rounding errors
-            clippingRegion &= viewBounds.adjusted(0,0,-1,-1).translated(viewPos + w->pos());
-        }
+        adjustClipRegion(item, clippingRegion);
         data += QPointF(x, y);
         const int desktopMask = PAINT_SCREEN_TRANSFORMED | PAINT_WINDOW_TRANSFORMED | PAINT_SCREEN_BACKGROUND_FIRST;
         paintDesktop(item->desktop(), desktopMask, clippingRegion, data);
@@ -534,41 +588,6 @@ void Scene::paintDesktopThumbnails(Scene::Window *w)
     }
 }
 
-QGraphicsView *Scene::findViewForThumbnailItem(AbstractThumbnailItem *item, Scene::Window *w)
-{
-    // in principle there could be more than one QGraphicsView per QGraphicsScene,
-    // although TabBox does not make use of it so far
-    QList<QGraphicsView*> views = item->scene()->views();
-    foreach (QGraphicsView* view, views) {
-        if (view->winId() == w->window()->window()) {
-            return view;
-        }
-        QWidget *parent = view;
-        while ((parent = parent->parentWidget())) {
-            // if the graphicsview is not the topmost widget we try to go up to the
-            // toplevel widget and check whether that is the window we are looking for.
-            if (parent->winId() == w->window()->window()) {
-                return view;
-            }
-        }
-    }
-    return NULL;
-}
-
-QPoint Scene::findOffsetInWindow(QWidget *view, xcb_window_t idOfTopmostWindow)
-{
-    if (view->winId() == idOfTopmostWindow) {
-        return QPoint();
-    }
-    QWidget *parent = view;
-    while ((parent = parent->parentWidget())) {
-        if (parent->winId() == idOfTopmostWindow) {
-            return view->mapTo(parent, QPoint());
-        }
-    }
-    return QPoint();
-}
-
 void Scene::paintDesktop(int desktop, int mask, const QRegion &region, ScreenPaintData &data)
 {
     static_cast<EffectsHandlerImpl*>(effects)->paintDesktop(desktop, mask, region, data);
@@ -592,11 +611,33 @@ void Scene::extendPaintRegion(QRegion &region, bool opaqueFullscreen)
     Q_UNUSED(opaqueFullscreen);
 }
 
+bool Scene::blocksForRetrace() const
+{
+    return false;
+}
+
+bool Scene::syncsToVBlank() const
+{
+    return false;
+}
+
 void Scene::screenGeometryChanged(const QSize &size)
 {
+    if (!overlayWindow()) {
+        return;
+    }
     overlayWindow()->resize(size);
 }
 
+bool Scene::makeOpenGLContextCurrent()
+{
+    return false;
+}
+
+void Scene::doneOpenGLContextCurrent()
+{
+}
+
 //****************************************
 // Scene::Window
 //****************************************
@@ -660,19 +701,18 @@ void Scene::Window::discardShape()
 const QRegion &Scene::Window::shape() const
 {
     if (!shape_valid) {
-        Client* c = qobject_cast< Client* >(toplevel);
+        Client* c = dynamic_cast< Client* >(toplevel);
         if (toplevel->shape() || (c != NULL && !c->mask().isEmpty())) {
-            int count, order;
-            XRectangle* rects = XShapeGetRectangles(display(), toplevel->frameId(),
-                                                    ShapeBounding, &count, &order);
-            if (rects) {
+            auto cookie = xcb_shape_get_rectangles_unchecked(connection(), toplevel->frameId(), XCB_SHAPE_SK_BOUNDING);
+            ScopedCPointer<xcb_shape_get_rectangles_reply_t> reply(xcb_shape_get_rectangles_reply(connection(), cookie, nullptr));
+            if (!reply.isNull()) {
                 shape_region = QRegion();
+                auto *rects = xcb_shape_get_rectangles_rectangles(reply.data());
                 for (int i = 0;
-                        i < count;
+                        i < xcb_shape_get_rectangles_rectangles_length(reply.data());
                         ++i)
                     shape_region += QRegion(rects[ i ].x, rects[ i ].y,
                                             rects[ i ].width, rects[ i ].height);
-                XFree(rects);
                 // make sure the shape is sane (X is async, maybe even XShape is broken)
                 shape_region &= QRegion(0, 0, width(), height());
             } else
@@ -703,6 +743,8 @@ bool Scene::Window::isVisible() const
         return false;
     if (!toplevel->isOnCurrentDesktop())
         return false;
+    if (!toplevel->isOnCurrentActivity())
+        return false;
     if (toplevel->isClient())
         return (static_cast< Client *>(toplevel))->isShown(true);
     return true; // Unmanaged is always visible
@@ -731,6 +773,8 @@ void Scene::Window::resetPaintingEnabled()
         if (!toplevel->isOnCurrentDesktop())
             disable_painting |= PAINT_DISABLED_BY_DESKTOP;
     }
+    if (!toplevel->isOnCurrentActivity())
+        disable_painting |= PAINT_DISABLED_BY_ACTIVITY;
     if (toplevel->isClient()) {
         Client *c = static_cast<Client*>(toplevel);
         if (c->isMinimized())
@@ -760,7 +804,7 @@ WindowQuadList Scene::Window::buildQuads(bool force) const
     if (toplevel->clientPos() == QPoint(0, 0) && toplevel->clientSize() == toplevel->decorationRect().size())
         ret = makeQuads(WindowQuadContents, shape());  // has no decoration
     else {
-        Client *client = qobject_cast<Client*>(toplevel);
+        Client *client = dynamic_cast<Client*>(toplevel);
         QRegion contents = clientShape();
         QRegion center = toplevel->transparentRect();
         QRegion decoration = (client && decorationPlugin()->hasAlpha() ?
@@ -879,20 +923,20 @@ void WindowPixmap::create()
     Xcb::WindowAttributes windowAttributes(toplevel()->frameId());
     Xcb::WindowGeometry windowGeometry(toplevel()->frameId());
     if (xcb_generic_error_t *error = xcb_request_check(connection(), namePixmapCookie)) {
-        kDebug(1212) << "Creating window pixmap failed: " << error->error_code;
+        qDebug() << "Creating window pixmap failed: " << error->error_code;
         free(error);
         return;
     }
     // check that the received pixmap is valid and actually matches what we
     // know about the window (i.e. size)
     if (!windowAttributes || windowAttributes->map_state != XCB_MAP_STATE_VIEWABLE) {
-        kDebug(1212) << "Creating window pixmap failed: " << this;
+        qDebug() << "Creating window pixmap failed: " << this;
         xcb_free_pixmap(connection(), pix);
         return;
     }
     if (!windowGeometry ||
         windowGeometry->width != toplevel()->width() || windowGeometry->height != toplevel()->height()) {
-        kDebug(1212) << "Creating window pixmap failed: " << this;
+        qDebug() << "Creating window pixmap failed: " << this;
         xcb_free_pixmap(connection(), pix);
         return;
     }
