@@ -48,12 +48,16 @@ public:
     virtual bool initFailed() const;
     virtual bool hasPendingFlush() const;
     virtual qint64 paint(QRegion damage, ToplevelList windows);
-    virtual void windowAdded(Toplevel*);
-    virtual void windowDeleted(Deleted*);
+    virtual Scene::EffectFrame *createEffectFrame(EffectFrameImpl *frame);
+    virtual Shadow *createShadow(Toplevel *toplevel);
     virtual void screenGeometryChanged(const QSize &size);
     virtual OverlayWindow *overlayWindow();
+    virtual bool usesOverlayWindow() const;
     virtual bool blocksForRetrace() const;
     virtual bool syncsToVBlank() const;
+    virtual bool makeOpenGLContextCurrent() override;
+    virtual void doneOpenGLContextCurrent() override;
+    virtual bool isLastFrameRendered() const override;
 
     void idle();
 
@@ -86,21 +90,15 @@ protected:
     void handleGraphicsReset(GLenum status);
 
     virtual void doPaintBackground(const QVector<float> &vertices) = 0;
-    virtual SceneOpenGL::Window *createWindow(Toplevel *t) = 0;
 
 Q_SIGNALS:
     void resetCompositing();
 
-public Q_SLOTS:
-    virtual void windowOpacityChanged(KWin::Toplevel* c);
-    virtual void windowGeometryShapeChanged(KWin::Toplevel* c);
-    virtual void windowClosed(KWin::Toplevel* c, KWin::Deleted* deleted);
 protected:
     bool init_ok;
 private:
     bool viewportLimitsMatched(const QSize &size) const;
 private:
-    QHash< Toplevel*, Window* > windows;
     bool m_debug;
     OpenGLBackend *m_backend;
 };
@@ -122,7 +120,7 @@ public:
 protected:
     virtual void paintGenericScreen(int mask, ScreenPaintData data);
     virtual void doPaintBackground(const QVector< float >& vertices);
-    virtual SceneOpenGL::Window *createWindow(Toplevel *t);
+    virtual Scene::Window *createWindow(Toplevel *t);
     virtual void finalDrawWindow(EffectWindowImpl* w, int mask, QRegion region, WindowPaintData& data);
     virtual void paintDesktop(int desktop, int mask, const QRegion &region, ScreenPaintData &data);
 
@@ -139,31 +137,6 @@ private:
     GLuint vao;
 };
 
-#ifdef KWIN_HAVE_OPENGL_1
-class SceneOpenGL1 : public SceneOpenGL
-{
-public:
-    explicit SceneOpenGL1(OpenGLBackend *backend);
-    virtual ~SceneOpenGL1();
-    virtual void screenGeometryChanged(const QSize &size);
-    virtual qint64 paint(QRegion damage, ToplevelList windows);
-    virtual CompositingType compositingType() const {
-        return OpenGL1Compositing;
-    }
-
-    static bool supported(OpenGLBackend *backend);
-
-protected:
-    virtual void paintGenericScreen(int mask, ScreenPaintData data);
-    virtual void doPaintBackground(const QVector< float >& vertices);
-    virtual SceneOpenGL::Window *createWindow(Toplevel *t);
-
-private:
-    void setupModelViewProjectionMatrix();
-    bool m_resetModelViewProjectionMatrix;
-};
-#endif
-
 class SceneOpenGL::TexturePrivate
     : public GLTexturePrivate
 {
@@ -322,23 +295,6 @@ private:
     bool m_blendingEnabled;
 };
 
-#ifdef KWIN_HAVE_OPENGL_1
-class SceneOpenGL1Window : public SceneOpenGL::Window
-{
-public:
-    explicit SceneOpenGL1Window(Toplevel *c);
-    virtual ~SceneOpenGL1Window();
-
-protected:
-    virtual void performPaint(int mask, QRegion region, WindowPaintData data);
-    virtual void prepareStates(TextureType type, qreal opacity, qreal brightness, qreal saturation, int screen);
-    virtual void restoreStates(TextureType type, qreal opacity, qreal brightness, qreal saturation);
-private:
-    void paintContent(SceneOpenGL::Texture* content, const QRegion& region, int mask, qreal opacity,
-                      const WindowPaintData& data, const WindowQuadList &contentQuads, bool normalized);
-};
-#endif
-
 class OpenGLWindowPixmap : public WindowPixmap
 {
 public:
@@ -461,21 +417,37 @@ public:
     }
     virtual void screenGeometryChanged(const QSize &size) = 0;
     virtual SceneOpenGL::TexturePrivate *createBackendTexture(SceneOpenGL::Texture *texture) = 0;
+
     /**
      * @brief Backend specific code to prepare the rendering of a frame including flushing the
      * previously rendered frame to the screen if the backend works this way.
+     *
+     * @return A region that if not empty will be repainted in addition to the damaged region
      **/
-    virtual void prepareRenderingFrame() = 0;
+    virtual QRegion prepareRenderingFrame() = 0;
+
     /**
      * @brief Backend specific code to handle the end of rendering a frame.
      *
-     * @param damage The actual updated region in this frame
+     * @param renderedRegion The possibly larger region that has been rendered
+     * @param damagedRegion The damaged region that should be posted
      **/
-    virtual void endRenderingFrame(const QRegion &damage) = 0;
+    virtual void endRenderingFrame(const QRegion &damage, const QRegion &damagedRegion) = 0;
+    virtual bool makeCurrent() = 0;
+    virtual void doneCurrent() = 0;
+    /**
+     * @brief Backend specific code to determine whether the last frame got rendered.
+     *
+     * Default implementation always returns @c true. That is it's always assumed that the last
+     * frame got rendered. If a backend needs more control it needs to implement this method.
+     */
+    virtual bool isLastFrameRendered() const;
+    virtual bool usesOverlayWindow() const = 0;
     /**
      * @brief Compositor is going into idle mode, flushes any pending paints.
      **/
     void idle();
+
     /**
      * @return bool Whether the scene needs to flush a frame.
      **/
@@ -492,9 +464,7 @@ public:
      *
      * @return :OverlayWindow*
      **/
-    OverlayWindow *overlayWindow() {
-        return m_overlayWindow;
-    }
+    virtual OverlayWindow *overlayWindow();
     /**
      * @brief Whether the creation of the Backend failed.
      *
@@ -536,6 +506,21 @@ public:
     bool isDirectRendering() const {
         return m_directRendering;
     }
+
+    bool supportsBufferAge() const {
+        return m_haveBufferAge;
+    }
+
+    /**
+     * Returns the damage that has accumulated since a buffer of the given age was presented.
+     */
+    QRegion accumulatedDamageHistory(int bufferAge) const;
+
+    /**
+     * Saves the given region to damage history.
+     */
+    void addToDamageHistory(const QRegion &region);
+
 protected:
     /**
      * @brief Backend specific flushing of frame to screen.
@@ -582,6 +567,11 @@ protected:
     void setIsDirectRendering(bool direct) {
         m_directRendering = direct;
     }
+
+    void setSupportsBufferAge(bool value) {
+        m_haveBufferAge = value;
+    }
+
     /**
      * @return const QRegion& Damage of previously rendered frame
      **/
@@ -603,10 +593,6 @@ protected:
     SwapProfiler m_swapProfiler;
 
 private:
-    /**
-     * @brief The OverlayWindow used by this Backend.
-     **/
-    OverlayWindow *m_overlayWindow;
     /**
      * @brief Whether VSync is available and used, defaults to @c false.
      **/
@@ -619,6 +605,10 @@ private:
      * @brief Whether direct rendering is used, defaults to @c false.
      **/
     bool m_directRendering;
+    /**
+     * @brief Whether the backend supports GLX_EXT_buffer_age / EGL_EXT_buffer_age.
+     */
+    bool m_haveBufferAge;
     /**
      * @brief Whether the initialization failed, of course default to @c false.
      **/
@@ -627,6 +617,10 @@ private:
      * @brief Damaged region of previously rendered frame.
      **/
     QRegion m_lastDamage;
+    /**
+     * @brief The damage history for the past 10 frames.
+     */
+    QList<QRegion> m_damageHistory;
     /**
      * @brief Timer to measure how long a frame renders.
      **/
@@ -638,6 +632,16 @@ inline bool SceneOpenGL::hasPendingFlush() const
     return m_backend->hasPendingFlush();
 }
 
+inline bool SceneOpenGL::isLastFrameRendered() const
+{
+    return m_backend->isLastFrameRendered();
+}
+
+inline bool SceneOpenGL::usesOverlayWindow() const
+{
+    return m_backend->usesOverlayWindow();
+}
+
 inline SceneOpenGL::Texture* OpenGLWindowPixmap::texture() const
 {
     return m_texture.data();
