@@ -19,38 +19,41 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
-
-#include "config-kwin.h"
-
 #include "scene_xrender.h"
 
-#ifdef KWIN_BUILD_COMPOSITE
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
 
 #include "toplevel.h"
 #include "client.h"
+#include "composite.h"
 #include "decorations.h"
 #include "deleted.h"
 #include "effects.h"
+#include "main.h"
 #include "overlaywindow.h"
 #include "paintredirector.h"
+#include "workspace.h"
 #include "xcbutils.h"
 #include "kwinxrenderutils.h"
+#if HAVE_WAYLAND
+#include "wayland_backend.h"
+#endif
 
 #include <xcb/xfixes.h>
 
+#include <QDebug>
 #include <QtGui/QPainter>
 #include <qmath.h>
 
 namespace KWin
 {
 
-//****************************************
-// SceneXrender
-//****************************************
-
-xcb_render_picture_t SceneXrender::buffer = XCB_RENDER_PICTURE_NONE;
 ScreenPaintData SceneXrender::screen_paint;
 
+#define DOUBLE_TO_FIXED(d) ((xcb_render_fixed_t) ((d) * 65536))
+#define FIXED_TO_DOUBLE(f) ((double) ((f) / 65536.0))
+
+
 static xcb_render_pictformat_t findFormatForVisual(xcb_visualid_t visual)
 {
     static QHash<xcb_visualid_t, xcb_render_pictformat_t> s_cache;
@@ -87,94 +90,323 @@ static xcb_render_pictformat_t findFormatForVisual(xcb_visualid_t visual)
     return s_cache.value(visual, 0);
 }
 
-SceneXrender::SceneXrender(Workspace* ws)
-    : Scene(ws)
-    , format(0)
-    , front(XCB_RENDER_PICTURE_NONE)
-    , m_overlayWindow(new OverlayWindow())
-    , init_ok(false)
+//****************************************
+// XRenderBackend
+//****************************************
+XRenderBackend::XRenderBackend()
+    : m_buffer(XCB_RENDER_PICTURE_NONE)
+    , m_failed(false)
 {
     if (!Xcb::Extensions::self()->isRenderAvailable()) {
-        kError(1212) << "No XRender extension available";
+        setFailed("No XRender extension available");
         return;
     }
     if (!Xcb::Extensions::self()->isFixesRegionAvailable()) {
-        kError(1212) << "No XFixes v3+ extension available";
+        setFailed("No XFixes v3+ extension available");
         return;
     }
-    initXRender(true);
 }
 
-SceneXrender::~SceneXrender()
+XRenderBackend::~XRenderBackend()
 {
-    if (!init_ok) {
-        // TODO this probably needs to clean up whatever has been created until the failure
-        m_overlayWindow->destroy();
-        return;
+    if (m_buffer) {
+        xcb_render_free_picture(connection(), m_buffer);
+    }
+}
+
+OverlayWindow* XRenderBackend::overlayWindow()
+{
+    return NULL;
+}
+
+void XRenderBackend::showOverlay()
+{
+}
+
+void XRenderBackend::setBuffer(xcb_render_picture_t buffer)
+{
+    if (m_buffer != XCB_RENDER_PICTURE_NONE) {
+        xcb_render_free_picture(connection(), m_buffer);
+    }
+    m_buffer = buffer;
+}
+
+void XRenderBackend::setFailed(const QString& reason)
+{
+    qCritical() << "Creating the XRender backend failed: " << reason;
+    m_failed = true;
+}
+
+void XRenderBackend::screenGeometryChanged(const QSize &size)
+{
+    Q_UNUSED(size)
+}
+
+bool XRenderBackend::isLastFrameRendered() const
+{
+    return true;
+}
+
+//****************************************
+// X11XRenderBackend
+//****************************************
+X11XRenderBackend::X11XRenderBackend()
+    : XRenderBackend()
+    , m_overlayWindow(new OverlayWindow())
+    , m_front(XCB_RENDER_PICTURE_NONE)
+    , m_format(0)
+{
+    init(true);
+}
+
+X11XRenderBackend::~X11XRenderBackend()
+{
+    if (m_front) {
+        xcb_render_free_picture(connection(), m_front);
     }
-    SceneXrender::Window::cleanup();
-    SceneXrender::EffectFrame::cleanup();
-    xcb_render_free_picture(connection(), front);
-    xcb_render_free_picture(connection(), buffer);
-    buffer = XCB_RENDER_PICTURE_NONE;
     m_overlayWindow->destroy();
-    foreach (Window * w, windows)
-        delete w;
-    delete m_overlayWindow;
 }
 
-void SceneXrender::initXRender(bool createOverlay)
+OverlayWindow* X11XRenderBackend::overlayWindow()
 {
-    init_ok = false;
-    if (front != XCB_RENDER_PICTURE_NONE)
-        xcb_render_free_picture(connection(), front);
+    return m_overlayWindow.data();
+}
+
+void X11XRenderBackend::showOverlay()
+{
+    if (m_overlayWindow->window())  // show the window only after the first pass, since
+        m_overlayWindow->show();   // that pass may take long
+}
+
+void X11XRenderBackend::init(bool createOverlay)
+{
+    if (m_front != XCB_RENDER_PICTURE_NONE)
+        xcb_render_free_picture(connection(), m_front);
     bool haveOverlay = createOverlay ? m_overlayWindow->create() : (m_overlayWindow->window() != XCB_WINDOW_NONE);
     if (haveOverlay) {
         m_overlayWindow->setup(XCB_WINDOW_NONE);
         ScopedCPointer<xcb_get_window_attributes_reply_t> attribs(xcb_get_window_attributes_reply(connection(),
             xcb_get_window_attributes_unchecked(connection(), m_overlayWindow->window()), NULL));
         if (!attribs) {
-            kError(1212) << "Failed getting window attributes for overlay window";
+            setFailed("Failed getting window attributes for overlay window");
             return;
         }
-        format = findFormatForVisual(attribs->visual);
-        if (format == 0) {
-            kError(1212) << "Failed to find XRender format for overlay window";
+        m_format = findFormatForVisual(attribs->visual);
+        if (m_format == 0) {
+            setFailed("Failed to find XRender format for overlay window");
             return;
         }
-        front = xcb_generate_id(connection());
-        xcb_render_create_picture(connection(), front, m_overlayWindow->window(), format, 0, NULL);
+        m_front = xcb_generate_id(connection());
+        xcb_render_create_picture(connection(), m_front, m_overlayWindow->window(), m_format, 0, NULL);
     } else {
         // create XRender picture for the root window
-        format = findFormatForVisual(defaultScreen()->root_visual);
-        if (format == 0) {
-            kError(1212) << "Failed to find XRender format for root window";
+        m_format = findFormatForVisual(defaultScreen()->root_visual);
+        if (m_format == 0) {
+            setFailed("Failed to find XRender format for root window");
             return; // error
         }
-        front = xcb_generate_id(connection());
+        m_front = xcb_generate_id(connection());
         const uint32_t values[] = {XCB_SUBWINDOW_MODE_INCLUDE_INFERIORS};
-        xcb_render_create_picture(connection(), front, rootWindow(), format, XCB_RENDER_CP_SUBWINDOW_MODE, values);
+        xcb_render_create_picture(connection(), m_front, rootWindow(), m_format, XCB_RENDER_CP_SUBWINDOW_MODE, values);
     }
     createBuffer();
-    init_ok = true;
+}
+
+void X11XRenderBackend::createBuffer()
+{
+    xcb_pixmap_t pixmap = xcb_generate_id(connection());
+    xcb_create_pixmap(connection(), Xcb::defaultDepth(), pixmap, rootWindow(), displayWidth(), displayHeight());
+    xcb_render_picture_t b = xcb_generate_id(connection());
+    xcb_render_create_picture(connection(), b, pixmap, m_format, 0, NULL);
+    xcb_free_pixmap(connection(), pixmap);   // The picture owns the pixmap now
+    setBuffer(b);
+}
+
+void X11XRenderBackend::present(int mask, const QRegion &damage)
+{
+    if (mask & Scene::PAINT_SCREEN_REGION) {
+        // Use the damage region as the clip region for the root window
+        XFixesRegion frontRegion(damage);
+        xcb_xfixes_set_picture_clip_region(connection(), m_front, frontRegion, 0, 0);
+        // copy composed buffer to the root window
+        xcb_xfixes_set_picture_clip_region(connection(), buffer(), XCB_XFIXES_REGION_NONE, 0, 0);
+        xcb_render_composite(connection(), XCB_RENDER_PICT_OP_SRC, buffer(), XCB_RENDER_PICTURE_NONE,
+                             m_front, 0, 0, 0, 0, 0, 0, displayWidth(), displayHeight());
+        xcb_xfixes_set_picture_clip_region(connection(), m_front, XCB_XFIXES_REGION_NONE, 0, 0);
+        xcb_flush(connection());
+    } else {
+        // copy composed buffer to the root window
+        xcb_render_composite(connection(), XCB_RENDER_PICT_OP_SRC, buffer(), XCB_RENDER_PICTURE_NONE,
+                             m_front, 0, 0, 0, 0, 0, 0, displayWidth(), displayHeight());
+        xcb_flush(connection());
+    }
+}
+
+void X11XRenderBackend::screenGeometryChanged(const QSize &size)
+{
+    Q_UNUSED(size)
+    init(false);
+}
+
+bool X11XRenderBackend::usesOverlayWindow() const
+{
+    return true;
+}
+
+//****************************************
+// WaylandXRenderBackend
+//****************************************
+#if HAVE_WAYLAND
+static void handleFrameCallback(void *data, wl_callback *callback, uint32_t time)
+{
+    Q_UNUSED(data)
+    Q_UNUSED(time)
+    reinterpret_cast<WaylandXRenderBackend*>(data)->lastFrameRendered();
+
+    if (callback) {
+            wl_callback_destroy(callback);
+    }
+}
+
+static const struct wl_callback_listener s_surfaceFrameListener = {
+        handleFrameCallback
+};
+
+WaylandXRenderBackend::WaylandXRenderBackend()
+    : m_shm(new Xcb::Shm)
+    , m_lastFrameRendered(true)
+    , m_format(0)
+{
+    if (!m_shm->isValid()) {
+        setFailed("Could not create XShm");
+    }
+    if (m_shm->pixmapFormat() != XCB_IMAGE_FORMAT_Z_PIXMAP) {
+        setFailed("XShm pixmap does not have Z format");
+    }
+    init();
+    connect(Wayland::WaylandBackend::self(), &Wayland::WaylandBackend::shellSurfaceSizeChanged,
+            this, &WaylandXRenderBackend::createBuffer);
+}
+
+WaylandXRenderBackend::~WaylandXRenderBackend()
+{
+}
+
+void WaylandXRenderBackend::init()
+{
+    m_format = findFormatForVisual(defaultScreen()->root_visual);
+    if (m_format == 0) {
+        setFailed("Failed to find XRender format for root window");
+        return; // error
+    }
+    createBuffer();
+}
+
+void WaylandXRenderBackend::createBuffer()
+{
+    xcb_pixmap_t pixmap = xcb_generate_id(connection());
+    const QSize &size = Wayland::WaylandBackend::self()->shellSurfaceSize();
+    xcb_void_cookie_t cookie = xcb_shm_create_pixmap_checked(connection(), pixmap, rootWindow(),
+                                                             size.width(), size.height(),
+                                                             24, m_shm->segment(), 0);
+    // let's check whether the pixmap got created
+    ScopedCPointer<xcb_generic_error_t> error(xcb_request_check(connection(), cookie));
+    if (error) {
+        setFailed("Creating shm pixmap failed: " + QString::number(error->error_code));
+        return;
+    }
+
+    // create the render picture
+    xcb_render_picture_t b = xcb_generate_id(connection());
+    xcb_render_create_picture(connection(), b, pixmap, m_format, 0, NULL);
+    xcb_free_pixmap(connection(), pixmap);   // The picture owns the pixmap now
+    setBuffer(b);
+
+    qDebug() << "Offscreen shm pixmap created";
+}
+
+void WaylandXRenderBackend::present(int mask, const QRegion &damage)
+{
+    Q_UNUSED(mask)
+
+    // we need to sync
+    ScopedCPointer<xcb_get_input_focus_reply_t> sync(xcb_get_input_focus_reply(
+        connection(), xcb_get_input_focus_unchecked(connection()), NULL));
+
+    Wayland::WaylandBackend *wl = Wayland::WaylandBackend::self();
+    const QSize &size = wl->shellSurfaceSize();
+    wl_buffer *buffer = wl->shmPool()->createBuffer(size, size.width() * 4, m_shm->buffer());
+    if (!buffer) {
+        qDebug() << "Did not get a buffer";
+        return;
+    }
+    m_lastFrameRendered = false;
+    wl_surface *surface = wl->surface();
+    wl_callback *callback = wl_surface_frame(surface);
+    wl_callback_add_listener(callback, &s_surfaceFrameListener, this);
+    wl_surface_attach(surface, buffer, 0, 0);
+    Q_FOREACH (const QRect &rect, damage.rects()) {
+        wl_surface_damage(surface, rect.x(), rect.y(), rect.width(), rect.height());
+    }
+    wl_surface_commit(surface);
+    wl->dispatchEvents();
+}
+
+bool WaylandXRenderBackend::isLastFrameRendered() const
+{
+    return m_lastFrameRendered;
+}
+
+void WaylandXRenderBackend::lastFrameRendered()
+{
+    m_lastFrameRendered = true;
+    Compositor::self()->lastFrameRendered();
+}
+
+bool WaylandXRenderBackend::usesOverlayWindow() const
+{
+    return false;
+}
+
+#endif
+
+//****************************************
+// SceneXrender
+//****************************************
+SceneXrender* SceneXrender::createScene()
+{
+    QScopedPointer<XRenderBackend> backend;
+#if HAVE_WAYLAND
+    if (kwinApp()->shouldUseWaylandForCompositing()) {
+        backend.reset(new WaylandXRenderBackend);
+        if (backend->isFailed()) {
+            return NULL;
+        }
+        return new SceneXrender(backend.take());
+    }
+#endif
+    backend.reset(new X11XRenderBackend);
+    if (backend->isFailed()) {
+        return NULL;
+    }
+    return new SceneXrender(backend.take());
+}
+
+SceneXrender::SceneXrender(XRenderBackend *backend)
+    : Scene(Workspace::self())
+    , m_backend(backend)
+{
+}
+
+SceneXrender::~SceneXrender()
+{
+    SceneXrender::Window::cleanup();
+    SceneXrender::EffectFrame::cleanup();
 }
 
 bool SceneXrender::initFailed() const
 {
-    return !init_ok;
-}
-
-// Create the compositing buffer. The root window is not double-buffered,
-// so it is done manually using this buffer,
-void SceneXrender::createBuffer()
-{
-    if (buffer != XCB_RENDER_PICTURE_NONE)
-        xcb_render_free_picture(connection(), buffer);
-    xcb_pixmap_t pixmap = xcb_generate_id(connection());
-    xcb_create_pixmap(connection(), Xcb::defaultDepth(), pixmap, rootWindow(), displayWidth(), displayHeight());
-    buffer = xcb_generate_id(connection());
-    xcb_render_create_picture(connection(), buffer, pixmap, format, 0, NULL);
-    xcb_free_pixmap(connection(), pixmap);   // The picture owns the pixmap now
+    return false;
 }
 
 // the entry point for painting
@@ -183,45 +415,21 @@ qint64 SceneXrender::paint(QRegion damage, ToplevelList toplevels)
     QElapsedTimer renderTimer;
     renderTimer.start();
 
-    foreach (Toplevel * c, toplevels) {
-        assert(windows.contains(c));
-        stacking_order.append(windows[ c ]);
-    }
+    createStackingOrder(toplevels);
 
     int mask = 0;
     QRegion updateRegion, validRegion;
     paintScreen(&mask, damage, QRegion(), &updateRegion, &validRegion);
 
-    if (m_overlayWindow->window())  // show the window only after the first pass, since
-        m_overlayWindow->show();   // that pass may take long
+    m_backend->showOverlay();
 
-    present(mask, updateRegion);
+    m_backend->present(mask, updateRegion);
     // do cleanup
-    stacking_order.clear();
+    clearStackingOrder();
 
     return renderTimer.nsecsElapsed();
 }
 
-void SceneXrender::present(int mask, QRegion damage)
-{
-    if (mask & PAINT_SCREEN_REGION) {
-        // Use the damage region as the clip region for the root window
-        XFixesRegion frontRegion(damage);
-        xcb_xfixes_set_picture_clip_region(connection(), front, frontRegion, 0, 0);
-        // copy composed buffer to the root window
-        xcb_xfixes_set_picture_clip_region(connection(), buffer, XCB_XFIXES_REGION_NONE, 0, 0);
-        xcb_render_composite(connection(), XCB_RENDER_PICT_OP_SRC, buffer, XCB_RENDER_PICTURE_NONE,
-                             front, 0, 0, 0, 0, 0, 0, displayWidth(), displayHeight());
-        xcb_xfixes_set_picture_clip_region(connection(), front, XCB_XFIXES_REGION_NONE, 0, 0);
-        xcb_flush(connection());
-    } else {
-        // copy composed buffer to the root window
-        xcb_render_composite(connection(), XCB_RENDER_PICT_OP_SRC, buffer, XCB_RENDER_PICTURE_NONE,
-                             front, 0, 0, 0, 0, 0, 0, displayWidth(), displayHeight());
-        xcb_flush(connection());
-    }
-}
-
 void SceneXrender::paintGenericScreen(int mask, ScreenPaintData data)
 {
     screen_paint = data; // save, transformations will be done when painting windows
@@ -240,55 +448,22 @@ void SceneXrender::paintBackground(QRegion region)
 {
     xcb_render_color_t col = { 0, 0, 0, 0xffff }; // black
     const QVector<xcb_rectangle_t> &rects = Xcb::regionToRects(region);
-    xcb_render_fill_rectangles(connection(), XCB_RENDER_PICT_OP_SRC, buffer, col, rects.count(), rects.data());
+    xcb_render_fill_rectangles(connection(), XCB_RENDER_PICT_OP_SRC, bufferPicture(), col, rects.count(), rects.data());
 }
 
-void SceneXrender::windowGeometryShapeChanged(KWin::Toplevel* c)
+Scene::Window *SceneXrender::createWindow(Toplevel *toplevel)
 {
-    if (!windows.contains(c))    // this is ok, shape is not valid by default
-        return;
-    Window* w = windows[ c ];
-    w->discardShape();
+    return new Window(toplevel, this);
 }
 
-void SceneXrender::windowOpacityChanged(KWin::Toplevel* c)
+Scene::EffectFrame *SceneXrender::createEffectFrame(EffectFrameImpl *frame)
 {
-    Q_UNUSED(c)
+    return new SceneXrender::EffectFrame(frame);
 }
 
-void SceneXrender::windowClosed(KWin::Toplevel* c, KWin::Deleted* deleted)
+Shadow *SceneXrender::createShadow(Toplevel *toplevel)
 {
-    assert(windows.contains(c));
-    if (deleted != NULL) {
-        // replace c with deleted
-        Window* w = windows.take(c);
-        w->updateToplevel(deleted);
-        if (w->shadow()) {
-            w->shadow()->setToplevel(deleted);
-        }
-        windows[ deleted ] = w;
-    } else {
-        delete windows.take(c);
-        c->effectWindow()->setSceneWindow(NULL);
-    }
-}
-
-void SceneXrender::windowDeleted(Deleted* c)
-{
-    assert(windows.contains(c));
-    delete windows.take(c);
-    c->effectWindow()->setSceneWindow(NULL);
-}
-
-void SceneXrender::windowAdded(Toplevel* c)
-{
-    assert(!windows.contains(c));
-    windows[ c ] = new Window(c);
-    connect(c, SIGNAL(geometryShapeChanged(KWin::Toplevel*,QRect)), SLOT(windowGeometryShapeChanged(KWin::Toplevel*)));
-    connect(c, SIGNAL(windowClosed(KWin::Toplevel*,KWin::Deleted*)), SLOT(windowClosed(KWin::Toplevel*,KWin::Deleted*)));
-    c->effectWindow()->setSceneWindow(windows[ c ]);
-    c->getShadow();
-    windows[ c ]->updateShadow(c->shadow());
+    return new SceneXRenderShadow(toplevel);
 }
 
 //****************************************
@@ -298,9 +473,11 @@ void SceneXrender::windowAdded(Toplevel* c)
 XRenderPicture *SceneXrender::Window::s_tempPicture = 0;
 QRect SceneXrender::Window::temp_visibleRect;
 
-SceneXrender::Window::Window(Toplevel* c)
+SceneXrender::Window::Window(Toplevel* c, SceneXrender *scene)
     : Scene::Window(c)
+    , m_scene(scene)
     , format(findFormatForVisual(c->visual()->visualid))
+    , alpha_cached_opacity(0.0)
 {
 }
 
@@ -430,8 +607,8 @@ void SceneXrender::Window::performPaint(int mask, QRegion region, WindowPaintDat
     qreal yscale = 1;
     bool scaled = false;
 
-    Client *client = qobject_cast<Client*>(toplevel);
-    Deleted *deleted = qobject_cast<Deleted*>(toplevel);
+    Client *client = dynamic_cast<Client*>(toplevel);
+    Deleted *deleted = dynamic_cast<Deleted*>(toplevel);
     const QRect decorationRect = toplevel->decorationRect();
     if (((client && !client->noBorder()) || (deleted && !deleted->noBorder())) &&
                                                         decorationPlugin()->hasAlpha()) {
@@ -449,14 +626,14 @@ void SceneXrender::Window::performPaint(int mask, QRegion region, WindowPaintDat
         transformed_shape |= toplevel->shadow()->shadowRegion();
 
     xcb_render_transform_t xform = {
-        KWIN_DOUBLE_TO_FIXED(1), KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(0),
-        KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(1), KWIN_DOUBLE_TO_FIXED(0),
-        KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(1)
+        DOUBLE_TO_FIXED(1), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0),
+        DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(1), DOUBLE_TO_FIXED(0),
+        DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(1)
     };
     static xcb_render_transform_t identity = {
-        KWIN_DOUBLE_TO_FIXED(1), KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(0),
-        KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(1), KWIN_DOUBLE_TO_FIXED(0),
-        KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(1)
+        DOUBLE_TO_FIXED(1), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0),
+        DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(1), DOUBLE_TO_FIXED(0),
+        DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(1)
     };
 
     if (mask & PAINT_WINDOW_TRANSFORMED) {
@@ -469,8 +646,8 @@ void SceneXrender::Window::performPaint(int mask, QRegion region, WindowPaintDat
     }
     if (!qFuzzyCompare(xscale, 1.0) || !qFuzzyCompare(yscale, 1.0)) {
         scaled = true;
-        xform.matrix11 = KWIN_DOUBLE_TO_FIXED(1.0 / xscale);
-        xform.matrix22 = KWIN_DOUBLE_TO_FIXED(1.0 / yscale);
+        xform.matrix11 = DOUBLE_TO_FIXED(1.0 / xscale);
+        xform.matrix22 = DOUBLE_TO_FIXED(1.0 / yscale);
 
         // transform the shape for clipping in paintTransformedScreen()
         QVector<QRect> rects = transformed_shape.rects();
@@ -499,7 +676,7 @@ void SceneXrender::Window::performPaint(int mask, QRegion region, WindowPaintDat
     const bool blitInTempPixmap = xRenderOffscreen() || (data.crossFadeProgress() < 1.0 && !opaque) ||
                                  (scaled && (wantShadow || (client && !client->noBorder()) || (deleted && !deleted->noBorder())));
 
-    xcb_render_picture_t renderTarget = buffer;
+    xcb_render_picture_t renderTarget = m_scene->bufferPicture();
     if (blitInTempPixmap) {
         if (scene_xRenderOffscreenTarget()) {
             temp_visibleRect = toplevel->visibleRect().translated(-toplevel->pos());
@@ -648,9 +825,9 @@ xcb_render_composite(connection(), XCB_RENDER_PICT_OP_OVER, m_xrenderShadow->pic
                     }
                     if (previous->size() != pixmap->size()) {
                         xcb_render_transform_t xform2 = {
-                            KWIN_DOUBLE_TO_FIXED(KWIN_FIXED_TO_DOUBLE(xform.matrix11) * previous->size().width() / pixmap->size().width()), KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(0),
-                            KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(KWIN_FIXED_TO_DOUBLE(xform.matrix22) * previous->size().height() / pixmap->size().height()), KWIN_DOUBLE_TO_FIXED(0),
-                            KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(0), KWIN_DOUBLE_TO_FIXED(1)
+                            DOUBLE_TO_FIXED(FIXED_TO_DOUBLE(xform.matrix11) * previous->size().width() / pixmap->size().width()), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0),
+                            DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(FIXED_TO_DOUBLE(xform.matrix22) * previous->size().height() / pixmap->size().height()), DOUBLE_TO_FIXED(0),
+                            DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(1)
                             };
                         xcb_render_set_picture_transform(connection(), previous->picture(), xform2);
                     }
@@ -710,7 +887,7 @@ xcb_render_composite(connection(), XCB_RENDER_PICT_OP_OVER, _PART_, decorationAl
             xcb_render_set_picture_transform(connection(), *s_tempPicture, xform);
             setPictureFilter(*s_tempPicture, filter);
             xcb_render_composite(connection(), XCB_RENDER_PICT_OP_OVER, *s_tempPicture,
-                                 XCB_RENDER_PICTURE_NONE, buffer,
+                                 XCB_RENDER_PICTURE_NONE, m_scene->bufferPicture(),
                                  0, 0, 0, 0, r.x(), r.y(), r.width(), r.height());
             xcb_render_set_picture_transform(connection(), *s_tempPicture, identity);
         }
@@ -750,7 +927,7 @@ WindowPixmap* SceneXrender::Window::createWindowPixmap()
 void SceneXrender::screenGeometryChanged(const QSize &size)
 {
     Scene::screenGeometryChanged(size);
-    initXRender(false);
+    m_backend->screenGeometryChanged(size);
 }
 
 //****************************************
@@ -796,6 +973,7 @@ SceneXrender::EffectFrame::EffectFrame(EffectFrameImpl* frame)
     m_picture = NULL;
     m_textPicture = NULL;
     m_iconPicture = NULL;
+    m_selectionPicture = NULL;
 }
 
 SceneXrender::EffectFrame::~EffectFrame()
@@ -803,6 +981,7 @@ SceneXrender::EffectFrame::~EffectFrame()
     delete m_picture;
     delete m_textPicture;
     delete m_iconPicture;
+    delete m_selectionPicture;
 }
 
 void SceneXrender::EffectFrame::cleanup()
@@ -819,6 +998,8 @@ void SceneXrender::EffectFrame::free()
     m_textPicture = NULL;
     delete m_iconPicture;
     m_iconPicture = NULL;
+    delete m_selectionPicture;
+    m_selectionPicture = NULL;
 }
 
 void SceneXrender::EffectFrame::freeIconFrame()
@@ -833,6 +1014,12 @@ void SceneXrender::EffectFrame::freeTextFrame()
     m_textPicture = NULL;
 }
 
+void SceneXrender::EffectFrame::freeSelection()
+{
+    delete m_selectionPicture;
+    m_selectionPicture = NULL;
+}
+
 void SceneXrender::EffectFrame::crossFadeIcon()
 {
     // TODO: implement me
@@ -851,7 +1038,34 @@ void SceneXrender::EffectFrame::render(QRegion region, double opacity, double fr
     }
 
     // Render the actual frame
-    renderUnstyled(effects->xrenderBufferPicture(), m_effectFrame->geometry(), opacity * frameOpacity);
+    if (m_effectFrame->style() == EffectFrameUnstyled) {
+        renderUnstyled(effects->xrenderBufferPicture(), m_effectFrame->geometry(), opacity * frameOpacity);
+    } else if (m_effectFrame->style() == EffectFrameStyled) {
+        if (!m_picture) { // Lazy creation
+            updatePicture();
+        }
+        if (m_picture) {
+            qreal left, top, right, bottom;
+            m_effectFrame->frame().getMargins(left, top, right, bottom);   // m_geometry is the inner geometry
+            QRect geom = m_effectFrame->geometry().adjusted(-left, -top, right, bottom);
+            xcb_render_composite(connection(), XCB_RENDER_PICT_OP_OVER, *m_picture,
+                                 XCB_RENDER_PICTURE_NONE, effects->xrenderBufferPicture(),
+                                 0, 0, 0, 0, geom.x(), geom.y(), geom.width(), geom.height());
+        }
+    }
+    if (!m_effectFrame->selection().isNull()) {
+        if (!m_selectionPicture) { // Lazy creation
+            const QPixmap pix = m_effectFrame->selectionFrame().framePixmap();
+            if (!pix.isNull()) // don't try if there's no content
+                m_selectionPicture = new XRenderPicture(m_effectFrame->selectionFrame().framePixmap().toImage());
+        }
+        if (m_selectionPicture) {
+            const QRect geom = m_effectFrame->selection();
+            xcb_render_composite(connection(), XCB_RENDER_PICT_OP_OVER, *m_selectionPicture,
+                                 XCB_RENDER_PICTURE_NONE, effects->xrenderBufferPicture(),
+                                 0, 0, 0, 0, geom.x(), geom.y(), geom.width(), geom.height());
+        }
+    }
 
     XRenderPicture fill = xRenderBlendPicture(opacity);
 
@@ -860,7 +1074,7 @@ void SceneXrender::EffectFrame::render(QRegion region, double opacity, double fr
         QPoint topLeft(m_effectFrame->geometry().x(), m_effectFrame->geometry().center().y() - m_effectFrame->iconSize().height() / 2);
 
         if (!m_iconPicture)   // lazy creation
-            m_iconPicture = new XRenderPicture(m_effectFrame->icon().toImage());
+            m_iconPicture = new XRenderPicture(m_effectFrame->icon().pixmap(m_effectFrame->iconSize()).toImage());
         QRect geom = QRect(topLeft, m_effectFrame->iconSize());
         xcb_render_composite(connection(), XCB_RENDER_PICT_OP_OVER, *m_iconPicture, fill,
                              effects->xrenderBufferPicture(),
@@ -925,12 +1139,12 @@ void SceneXrender::EffectFrame::renderUnstyled(xcb_render_picture_t pict, const
 
         QVector<xcb_render_pointfix_t> points;
         xcb_render_pointfix_t point;
-        point.x = KWIN_DOUBLE_TO_FIXED(roundness);
-        point.y = KWIN_DOUBLE_TO_FIXED(roundness);
+        point.x = DOUBLE_TO_FIXED(roundness);
+        point.y = DOUBLE_TO_FIXED(roundness);
         points << point;
         for (int ii = 0; ii <= num_segments; ++ii) {
-            point.x = KWIN_DOUBLE_TO_FIXED(x + roundness);
-            point.y = KWIN_DOUBLE_TO_FIXED(y + roundness);
+            point.x = DOUBLE_TO_FIXED(x + roundness);
+            point.y = DOUBLE_TO_FIXED(y + roundness);
             points << point;
             //apply the rotation matrix
             t = x;
@@ -970,10 +1184,16 @@ void SceneXrender::EffectFrame::updatePicture()
 {
     delete m_picture;
     m_picture = 0L;
+    if (m_effectFrame->style() == EffectFrameStyled) {
+        const QPixmap pix = m_effectFrame->frame().framePixmap();
+        if (!pix.isNull())
+            m_picture = new XRenderPicture(pix.toImage());
+    }
 }
 
 void SceneXrender::EffectFrame::updateTextPicture()
 {
+    // Mostly copied from SceneOpenGL::EffectFrame::updateTextTexture() above
     delete m_textPicture;
     m_textPicture = 0L;
 
@@ -994,15 +1214,19 @@ void SceneXrender::EffectFrame::updateTextPicture()
         text = metrics.elidedText(text, Qt::ElideRight, rect.width());
     }
 
-    QImage image(m_effectFrame->geometry().size(), QImage::Format_ARGB32_Premultiplied);
-    image.fill(Qt::transparent);
-    QPainter p(&image);
+    QPixmap pixmap(m_effectFrame->geometry().size());
+    pixmap.fill(Qt::transparent);
+    QPainter p(&pixmap);
     p.setFont(m_effectFrame->font());
-    // TODO: What about no frame? Custom color setting required
-    p.setPen(Qt::white);
+    if (m_effectFrame->style() == EffectFrameStyled) {
+        p.setPen(m_effectFrame->styledTextColor());
+    } else {
+        // TODO: What about no frame? Custom color setting required
+        p.setPen(Qt::white);
+    }
     p.drawText(rect, m_effectFrame->alignment(), text);
     p.end();
-    m_textPicture = new XRenderPicture(image);
+    m_textPicture = new XRenderPicture(pixmap.toImage());
 }
 
 SceneXRenderShadow::SceneXRenderShadow(Toplevel *toplevel)
@@ -1067,7 +1291,7 @@ bool SceneXRenderShadow::prepareBackend()
     const uint32_t values[] = {XCB_RENDER_REPEAT_NORMAL};
     for (int i=0; i<ShadowElementsCount; ++i) {
         delete m_pictures[i];
-        m_pictures[i] = new XRenderPicture(shadowImage(ShadowElements(i)));
+        m_pictures[i] = new XRenderPicture(shadowPixmap(ShadowElements(i)).toImage());
         xcb_render_change_picture(connection(), *m_pictures[i], XCB_RENDER_CP_REPEAT, values);
     }
     return true;
@@ -1081,5 +1305,8 @@ xcb_render_picture_t SceneXRenderShadow::picture(Shadow::ShadowElements element)
     return *m_pictures[element];
 }
 
+#undef DOUBLE_TO_FIXED
+#undef FIXED_TO_DOUBLE
+
 } // namespace
 #endif
