@@ -23,8 +23,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "screens.h"
 #include "workspace.h"
 
-#include <KDE/KDebug>
-
 namespace KWin {
 namespace ScriptingClientModel {
 
@@ -253,7 +251,7 @@ Client *ClientLevel::clientForId(quint32 child) const
 {
     QMap<quint32, Client*>::const_iterator it = m_clients.constFind(child);
     if (it == m_clients.constEnd()) {
-        return NULL;
+        return nullptr;
     }
     return it.value();
 }
@@ -275,7 +273,7 @@ const AbstractLevel *ClientLevel::levelForId(quint32 id) const
     if (id == AbstractLevel::id()) {
         return this;
     }
-    return NULL;
+    return nullptr;
 }
 
 AbstractLevel *ClientLevel::parentForId(quint32 child) const
@@ -286,7 +284,7 @@ AbstractLevel *ClientLevel::parentForId(quint32 child) const
     if (m_clients.contains(child)) {
         return const_cast<ClientLevel*>(this);
     }
-    return NULL;
+    return nullptr;
 }
 
 AbstractLevel *AbstractLevel::create(const QList< ClientModel::LevelRestriction > &restrictions, ClientModel::LevelRestrictions parentRestrictions, ClientModel *model, AbstractLevel *parent)
@@ -323,7 +321,7 @@ AbstractLevel *AbstractLevel::create(const QList< ClientModel::LevelRestriction
         }
         break;
 #else
-        return NULL;
+        return nullptr;
 #endif
     }
     case ClientModel::ScreenRestriction:
@@ -348,7 +346,7 @@ AbstractLevel *AbstractLevel::create(const QList< ClientModel::LevelRestriction
         break;
     default:
         // invalid
-        return NULL;
+        return nullptr;
     }
 
     return currentLevel;
@@ -582,7 +580,7 @@ const AbstractLevel *ForkLevel::levelForId(quint32 id) const
         }
     }
     // not found
-    return NULL;
+    return nullptr;
 }
 
 AbstractLevel *ForkLevel::parentForId(quint32 child) const
@@ -596,7 +594,7 @@ AbstractLevel *ForkLevel::parentForId(quint32 child) const
         }
     }
     // not found
-    return NULL;
+    return nullptr;
 }
 
 int ForkLevel::rowForId(quint32 child) const
@@ -628,12 +626,12 @@ Client *ForkLevel::clientForId(quint32 child) const
         }
     }
     // not found
-    return NULL;
+    return nullptr;
 }
 
 ClientModel::ClientModel(QObject *parent)
     : QAbstractItemModel(parent)
-    , m_root(NULL)
+    , m_root(nullptr)
     , m_exclusions(NoExclusion)
 {
     QHash<int, QByteArray> roleNames;
@@ -828,7 +826,7 @@ CLIENT_MODEL_WRAPPER(ClientModelByScreenAndDesktop, QList<LevelRestriction>() <<
 
 ClientFilterModel::ClientFilterModel(QObject *parent)
     : QSortFilterProxyModel(parent)
-    , m_clientModel(NULL)
+    , m_clientModel(nullptr)
 {
 }
 
@@ -885,15 +883,15 @@ bool ClientFilterModel::filterAcceptsRow(int sourceRow, const QModelIndex &sourc
     if (client->caption().contains(m_filter, Qt::CaseInsensitive)) {
         return true;
     }
-    const QString windowRole(client->windowRole());
+    const QString windowRole(QString::fromUtf8(client->windowRole()));
     if (windowRole.contains(m_filter, Qt::CaseInsensitive)) {
         return true;
     }
-    const QString resourceName(client->resourceName());
+    const QString resourceName(QString::fromUtf8(client->resourceName()));
     if (resourceName.contains(m_filter, Qt::CaseInsensitive)) {
         return true;
     }
-    const QString resourceClass(client->resourceClass());
+    const QString resourceClass(QString::fromUtf8(client->resourceClass()));
     if (resourceClass.contains(m_filter, Qt::CaseInsensitive)) {
         return true;
     }
