@@ -20,20 +20,34 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 // own
 #include "tabboxhandler.h"
+#include <kwinglobals.h>
+#include "xcbutils.h"
 // tabbox
 #include "clientmodel.h"
-#include "declarative.h"
 #include "desktopmodel.h"
 #include "tabboxconfig.h"
+#include "thumbnailitem.h"
+#include "scripting/scripting.h"
+#include "switcheritem.h"
 // Qt
-#include <QProcess>
-#include <QtGui/qevent.h>
-#include <QtCore/qabstractitemmodel.h>
+#include <QApplication>
+#include <QDebug>
+#include <QDesktopWidget>
+#include <QKeyEvent>
+#include <QModelIndex>
+#include <QStandardPaths>
 #include <QTimer>
-#include <QtGui/qx11info_x11.h>
+#include <QQmlContext>
+#include <QQmlComponent>
+#include <QQmlEngine>
+#include <QQuickItem>
+#include <QQuickWindow>
+#include <QX11Info>
 #include <X11/Xlib.h>
 // KDE
-#include <KDebug>
+#include <KLocalizedString>
+#include <KProcess>
+#include <KServiceTypeTrader>
 #include <KWindowSystem>
 
 namespace KWin
@@ -57,14 +71,21 @@ public:
     */
     void endHighlightWindows(bool abort = false);
 
+    void show();
+    QQuickWindow *window() const;
+    SwitcherItem *switcherItem() const;
+
     ClientModel* clientModel() const;
     DesktopModel* desktopModel() const;
 
     TabBoxHandler *q; // public pointer
     // members
     TabBoxConfig config;
-    DeclarativeView *m_declarativeView;
-    DeclarativeView *m_declarativeDesktopView;
+    QScopedPointer<QQmlContext> m_qmlContext;
+    QScopedPointer<QQmlComponent> m_qmlComponent;
+    QObject *m_mainItem;
+    QMap<QString, QObject*> m_clientTabBoxes;
+    QMap<QString, QObject*> m_desktopTabBoxes;
     ClientModel* m_clientModel;
     DesktopModel* m_desktopModel;
     QModelIndex index;
@@ -77,19 +98,25 @@ public:
     QPoint m_embeddedOffset;
     QSize m_embeddedSize;
     Qt::Alignment m_embeddedAlignment;
+    Xcb::Atom m_highlightWindowsAtom;
+
+private:
+    QObject *createSwitcherItem(bool desktopMode);
 };
 
 TabBoxHandlerPrivate::TabBoxHandlerPrivate(TabBoxHandler *q)
-    : m_declarativeView(NULL)
-    , m_declarativeDesktopView(NULL)
+    : m_qmlContext()
+    , m_qmlComponent()
+    , m_mainItem(nullptr)
     , m_embedded(0)
     , m_embeddedOffset(QPoint(0, 0))
     , m_embeddedSize(QSize(0, 0))
+    , m_highlightWindowsAtom(QByteArrayLiteral("_KDE_WINDOW_HIGHLIGHT"))
 {
     this->q = q;
     isShown = false;
-    lastRaisedClient = 0;
-    lastRaisedClientSucc = 0;
+    lastRaisedClient = nullptr;
+    lastRaisedClientSucc = nullptr;
     config = TabBoxConfig();
     m_clientModel = new ClientModel(q);
     m_desktopModel = new DesktopModel(q);
@@ -97,8 +124,36 @@ TabBoxHandlerPrivate::TabBoxHandlerPrivate(TabBoxHandler *q)
 
 TabBoxHandlerPrivate::~TabBoxHandlerPrivate()
 {
-    delete m_declarativeView;
-    delete m_declarativeDesktopView;
+    for (auto it = m_clientTabBoxes.constBegin(); it != m_clientTabBoxes.constBegin(); ++it) {
+        delete it.value();
+    }
+    for (auto it = m_desktopTabBoxes.constBegin(); it != m_desktopTabBoxes.constBegin(); ++it) {
+        delete it.value();
+    }
+}
+
+QQuickWindow *TabBoxHandlerPrivate::window() const
+{
+    if (!m_mainItem) {
+        return nullptr;
+    }
+    if (QQuickWindow *w = qobject_cast<QQuickWindow*>(m_mainItem)) {
+        return w;
+    }
+    return m_mainItem->findChild<QQuickWindow*>();
+}
+
+SwitcherItem *TabBoxHandlerPrivate::switcherItem() const
+{
+    if (!m_mainItem) {
+        return nullptr;
+    }
+    if (SwitcherItem *i = qobject_cast<SwitcherItem*>(m_mainItem)) {
+        return i;
+    } else if (QQuickWindow *w = qobject_cast<QQuickWindow*>(m_mainItem)) {
+        return w->contentItem()->findChild<SwitcherItem*>();
+    }
+    return m_mainItem->findChild<SwitcherItem*>();
 }
 
 ClientModel* TabBoxHandlerPrivate::clientModel() const
@@ -116,19 +171,15 @@ void TabBoxHandlerPrivate::updateHighlightWindows()
     if (!isShown || config.tabBoxMode() != TabBoxConfig::ClientTabBox)
         return;
 
-    Display *dpy = QX11Info::display();
     TabBoxClient *currentClient = q->client(index);
-    QWidget *w = NULL;
-    if (m_declarativeView && m_declarativeView->isVisible()) {
-        w = m_declarativeView;
-    }
+    QWindow *w = window();
 
     if (q->isKWinCompositing()) {
         if (lastRaisedClient)
-            q->elevateClient(lastRaisedClient, m_declarativeView ? m_declarativeView->winId() : 0, false);
+            q->elevateClient(lastRaisedClient, w ? w->winId() : 0, false);
         lastRaisedClient = currentClient;
         if (currentClient)
-            q->elevateClient(currentClient, m_declarativeView ? m_declarativeView->winId() : 0, true);
+            q->elevateClient(currentClient, w ? w->winId() : 0, true);
     } else {
         if (lastRaisedClient) {
             if (lastRaisedClientSucc)
@@ -148,13 +199,13 @@ void TabBoxHandlerPrivate::updateHighlightWindows()
                     break;
                 }
             }
-            lastRaisedClientSucc = (succIdx < order.count()) ? order.at(succIdx).data() : 0;
+            lastRaisedClientSucc = (succIdx < order.count()) ? order.at(succIdx).data() : nullptr;
             q->raiseClient(lastRaisedClient);
         }
     }
 
-    WId wId;
-    QVector< WId > data;
+    xcb_window_t wId;
+    QVector< xcb_window_t > data;
     if (config.isShowTabBox() && w) {
         wId = w->winId();
         data.resize(2);
@@ -164,24 +215,119 @@ void TabBoxHandlerPrivate::updateHighlightWindows()
         data.resize(1);
     }
     data[ 0 ] = currentClient ? currentClient->window() : 0L;
-    Atom atom = XInternAtom(dpy, "_KDE_WINDOW_HIGHLIGHT", False);
-    XChangeProperty(dpy, wId, atom, atom, 32, PropModeReplace,
-                    reinterpret_cast<unsigned char *>(data.data()), data.size());
+    xcb_change_property(connection(), XCB_PROP_MODE_REPLACE, wId, m_highlightWindowsAtom,
+                        m_highlightWindowsAtom, 32, data.size(), data.constData());
 }
 
 void TabBoxHandlerPrivate::endHighlightWindows(bool abort)
 {
     TabBoxClient *currentClient = q->client(index);
+    QWindow *w = window();
     if (currentClient)
-        q->elevateClient(currentClient, m_declarativeView ? m_declarativeView->winId() : 0, false);
+        q->elevateClient(currentClient, w ? w->winId() : 0, false);
     if (abort && lastRaisedClient && lastRaisedClientSucc)
         q->restack(lastRaisedClient, lastRaisedClientSucc);
-    lastRaisedClient = 0;
-    lastRaisedClientSucc = 0;
+    lastRaisedClient = nullptr;
+    lastRaisedClientSucc = nullptr;
     // highlight windows
-    Display *dpy = QX11Info::display();
-    Atom atom = XInternAtom(dpy, "_KDE_WINDOW_HIGHLIGHT", False);
-    XDeleteProperty(dpy, config.isShowTabBox() && m_declarativeView ? m_declarativeView->winId() : QX11Info::appRootWindow(), atom);
+    xcb_delete_property(connection(), config.isShowTabBox() && w ? w->winId() : rootWindow(), m_highlightWindowsAtom);
+}
+
+QObject *TabBoxHandlerPrivate::createSwitcherItem(bool desktopMode)
+{
+    auto findSwitcher = [this, desktopMode] {
+        QString constraint = QStringLiteral("[X-KDE-PluginInfo-Name] == '%1'").arg(config.layoutName());
+        const QString type = desktopMode ? QStringLiteral("KWin/DesktopSwitcher") : QStringLiteral("KWin/WindowSwitcher");
+        KService::List offers = KServiceTypeTrader::self()->query(type, constraint);
+        if (offers.isEmpty()) {
+            // load default
+            constraint = QStringLiteral("[X-KDE-PluginInfo-Name] == '%1'").arg(QStringLiteral("informative"));
+            offers = KServiceTypeTrader::self()->query(type, constraint);
+            if (offers.isEmpty()) {
+                qDebug() << "could not find default window switcher layout";
+                return KService::Ptr();
+            }
+        }
+        return offers.first();
+    };
+    KService::Ptr service = findSwitcher();
+    if (!service) {
+        return nullptr;
+    }
+    if (service->property(QStringLiteral("X-Plasma-API")).toString() != QStringLiteral("declarativeappletscript")) {
+        qDebug() << "Window Switcher Layout is no declarativeappletscript";
+        return nullptr;
+    }
+    auto findScriptFile = [desktopMode, service] {
+        const QString pluginName = service->property(QStringLiteral("X-KDE-PluginInfo-Name")).toString();
+        const QString scriptName = service->property(QStringLiteral("X-Plasma-MainScript")).toString();
+        const QString type = desktopMode ? QStringLiteral("/desktoptabbox/") : QStringLiteral("/tabbox/");
+        return QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral(KWIN_NAME) + type + pluginName + QStringLiteral("/contents/") + scriptName);
+    };
+    const QString file = findScriptFile();
+    if (file.isNull()) {
+        qDebug() << "Could not find QML file for window switcher";
+        return nullptr;
+    }
+    m_qmlComponent->loadUrl(QUrl::fromLocalFile(file));
+    if (m_qmlComponent->isError()) {
+        qDebug() << "Component failed to load: " << m_qmlComponent->errors();
+        QStringList args;
+        args << QStringLiteral("--passivepopup") << i18n("The Window Switcher installation is broken, resources are missing.\n"
+                                            "Contact your distribution about this.") << QStringLiteral("20");
+        KProcess::startDetached(QStringLiteral("kdialog"), args);
+    } else {
+        QObject *object = m_qmlComponent->create(m_qmlContext.data());
+        if (desktopMode) {
+            m_desktopTabBoxes.insert(config.layoutName(), object);
+        } else {
+            m_clientTabBoxes.insert(config.layoutName(), object);
+        }
+        return object;
+    }
+    return nullptr;
+}
+
+void TabBoxHandlerPrivate::show()
+{
+    if (m_qmlContext.isNull()) {
+        qmlRegisterType<SwitcherItem>("org.kde.kwin", 2, 0, "Switcher");
+        m_qmlContext.reset(new QQmlContext(Scripting::self()->qmlEngine()));
+    }
+    if (m_qmlComponent.isNull()) {
+        m_qmlComponent.reset(new QQmlComponent(Scripting::self()->qmlEngine()));
+    }
+    const bool desktopMode = (config.tabBoxMode() == TabBoxConfig::DesktopTabBox);
+    auto findMainItem = [this](const QMap<QString, QObject *> &tabBoxes) -> QObject* {
+        auto it = tabBoxes.constFind(config.layoutName());
+        if (it != tabBoxes.constEnd()) {
+            return it.value();
+        }
+        return nullptr;
+    };
+    m_mainItem = nullptr;
+    m_mainItem = desktopMode ? findMainItem(m_desktopTabBoxes) : findMainItem(m_clientTabBoxes);
+    if (!m_mainItem) {
+        m_mainItem = createSwitcherItem(desktopMode);
+        if (!m_mainItem) {
+            return;
+        }
+    }
+    if (SwitcherItem *item = switcherItem()) {
+        if (!item->model()) {
+            QAbstractItemModel *model = nullptr;
+            if (desktopMode) {
+                model = desktopModel();
+            } else {
+                model = clientModel();
+            }
+            item->setModel(model);
+        }
+        item->setAllDesktops(config.clientDesktopMode() == TabBoxConfig::AllDesktopsClients);
+        item->setCurrentIndex(index.row());
+        // everything is prepared, so let's make the whole thing visible
+        item->setVisible(true);
+    }
 }
 
 /***********************************************
@@ -214,36 +360,13 @@ void TabBoxHandler::setConfig(const TabBoxConfig& config)
 void TabBoxHandler::show()
 {
     d->isShown = true;
-    d->lastRaisedClient = 0;
-    d->lastRaisedClientSucc = 0;
+    d->lastRaisedClient = nullptr;
+    d->lastRaisedClientSucc = nullptr;
     if (d->config.isShowTabBox()) {
-        DeclarativeView *dv(NULL);
-        if (d->config.tabBoxMode() == TabBoxConfig::ClientTabBox) {
-            // use declarative view
-            if (!d->m_declarativeView) {
-                d->m_declarativeView = new DeclarativeView(d->clientModel(), TabBoxConfig::ClientTabBox);
-            }
-            dv = d->m_declarativeView;
-        } else {
-            if (!d->m_declarativeDesktopView) {
-                d->m_declarativeDesktopView = new DeclarativeView(d->desktopModel(), TabBoxConfig::DesktopTabBox);
-            }
-            dv = d->m_declarativeDesktopView;
-        }
-        if (dv->status() == QDeclarativeView::Ready && dv->rootObject()) {
-            dv->show();
-            dv->setCurrentIndex(d->index, d->config.tabBoxMode() == TabBoxConfig::ClientTabBox);
-        } else {
-            QStringList args;
-            args << "--passivepopup" << /*i18n*/("The Window Switcher installation is broken, resources are missing.\n"
-                                                 "Contact your distribution about this.") << "20";
-            QProcess::startDetached("kdialog", args);
-            hide();
-            return;
-        }
+        d->show();
     }
     if (d->config.isHighlightWindows()) {
-        XSync(QX11Info::display(), false);
+        Xcb::sync();
         // TODO this should be
         // QMetaObject::invokeMethod(this, "updateHighlightWindows", Qt::QueuedConnection);
         // but we somehow need to cross > 1 event cycle (likely because of queued invocation in the effects)
@@ -263,12 +386,14 @@ void TabBoxHandler::hide(bool abort)
     if (d->config.isHighlightWindows()) {
         d->endHighlightWindows(abort);
     }
-    if (d->m_declarativeView) {
-        d->m_declarativeView->hide();
+    if (SwitcherItem *item = d->switcherItem()) {
+        item->setVisible(false);
     }
-    if (d->m_declarativeDesktopView) {
-        d->m_declarativeDesktopView->hide();
+    if (QQuickWindow *w = d->window()) {
+        w->hide();
+        w->destroy();
     }
+    d->m_mainItem = nullptr;
 }
 
 QModelIndex TabBoxHandler::nextPrev(bool forward) const
@@ -355,12 +480,6 @@ void TabBoxHandler::setCurrentIndex(const QModelIndex& index)
     if (!index.isValid()) {
         return;
     }
-    if (d->m_declarativeView) {
-        d->m_declarativeView->setCurrentIndex(index);
-    }
-    if (d->m_declarativeDesktopView) {
-        d->m_declarativeDesktopView->setCurrentIndex(index);
-    }
     d->index = index;
     if (d->config.tabBoxMode() == TabBoxConfig::ClientTabBox) {
         if (d->config.isHighlightWindows()) {
@@ -377,24 +496,22 @@ const QModelIndex& TabBoxHandler::currentIndex() const
 
 void TabBoxHandler::grabbedKeyEvent(QKeyEvent* event) const
 {
-    if (d->m_declarativeView && d->m_declarativeView->isVisible()) {
-        d->m_declarativeView->sendKeyEvent(event);
-    } else if (d->m_declarativeDesktopView && d->m_declarativeDesktopView->isVisible()) {
-        d->m_declarativeDesktopView->sendKeyEvent(event);
+    if (!d->m_mainItem) {
+        return;
     }
+    QApplication::sendEvent(d->window(), event);
 }
 
 bool TabBoxHandler::containsPos(const QPoint& pos) const
 {
-    QWidget *w = NULL;
-    if (d->m_declarativeView && d->m_declarativeView->isVisible()) {
-        w = d->m_declarativeView;
-    } else if (d->m_declarativeDesktopView && d->m_declarativeDesktopView->isVisible()) {
-        w = d->m_declarativeDesktopView;
-    } else {
+    if (!d->m_mainItem) {
         return false;
     }
-    return w->geometry().contains(pos);
+    QWindow *w = d->window();
+    if (w) {
+        return w->geometry().contains(pos);
+    }
+    return false;
 }
 
 QModelIndex TabBoxHandler::index(QWeakPointer<KWin::TabBox::TabBoxClient> client) const
@@ -413,7 +530,7 @@ TabBoxClient* TabBoxHandler::client(const QModelIndex& index) const
 {
     if ((!index.isValid()) ||
             (d->config.tabBoxMode() != TabBoxConfig::ClientTabBox))
-        return NULL;
+        return nullptr;
     TabBoxClient* c = static_cast< TabBoxClient* >(
                           d->clientModel()->data(index, ClientModel::ClientRole).value<void *>());
     return c;
@@ -440,9 +557,9 @@ void TabBoxHandler::createModel(bool partialReset)
             }
         }
         if (d->lastRaisedClient && !lastRaised)
-            d->lastRaisedClient = 0;
+            d->lastRaisedClient = nullptr;
         if (d->lastRaisedClientSucc && !lastRaisedSucc)
-            d->lastRaisedClientSucc = 0;
+            d->lastRaisedClientSucc = nullptr;
         break;
     }
     case TabBoxConfig::DesktopTabBox:
@@ -519,7 +636,7 @@ void TabBoxHandler::resetEmbedded()
     emit embeddedChanged(false);
 }
 
-TabBoxHandler* tabBox = 0;
+TabBoxHandler* tabBox = nullptr;
 
 TabBoxClient::TabBoxClient()
 {
