@@ -20,8 +20,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "toplevel.h"
 
-#include <kxerrorhandler.h>
-
+#ifdef KWIN_BUILD_ACTIVITIES
+#include "activities.h"
+#endif
 #include "atoms.h"
 #include "client.h"
 #include "client_machine.h"
@@ -30,6 +31,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "shadow.h"
 #include "xcbutils.h"
 
+#include <QDebug>
+
 namespace KWin
 {
 
@@ -38,8 +41,7 @@ Toplevel::Toplevel()
     , info(NULL)
     , ready_for_painting(true)
     , m_isDamaged(false)
-    , client(None)
-    , frame(None)
+    , m_client()
     , damage_handle(None)
     , is_shape(false)
     , effect_window(NULL)
@@ -108,8 +110,7 @@ void Toplevel::copyToDeleted(Toplevel* c)
     vis = c->vis;
     bit_depth = c->bit_depth;
     info = c->info;
-    client = c->client;
-    frame = c->frame;
+    m_client.reset(c->m_client, false);
     ready_for_painting = c->ready_for_painting;
     damage_handle = None;
     damage_region = c->damage_region;
@@ -153,7 +154,7 @@ void Toplevel::getWindowRole()
 /*!
   Returns SM_CLIENT_ID property for a given window.
  */
-QByteArray Toplevel::staticSessionId(WId w)
+QByteArray Toplevel::staticSessionId(xcb_window_t w)
 {
     return getStringProperty(w, atoms->sm_client_id);
 }
@@ -161,32 +162,23 @@ QByteArray Toplevel::staticSessionId(WId w)
 /*!
   Returns WM_COMMAND property for a given window.
  */
-QByteArray Toplevel::staticWmCommand(WId w)
+QByteArray Toplevel::staticWmCommand(xcb_window_t w)
 {
-    return getStringProperty(w, XA_WM_COMMAND, ' ');
+    return getStringProperty(w, XCB_ATOM_WM_COMMAND, ' ');
 }
 
 /*!
   Returns WM_CLIENT_LEADER property for a given window.
  */
-Window Toplevel::staticWmClientLeader(WId w)
+xcb_window_t Toplevel::staticWmClientLeader(xcb_window_t w)
 {
-    Atom type;
-    int format, status;
-    unsigned long nitems = 0;
-    unsigned long extra = 0;
-    unsigned char *data = 0;
-    Window result = w;
-    KXErrorHandler err;
-    status = XGetWindowProperty(display(), w, atoms->wm_client_leader, 0, 10000,
-                                false, XA_WINDOW, &type, &format,
-                                &nitems, &extra, &data);
-    if (status == Success && !err.error(false)) {
-        if (data && nitems > 0)
-            result = *((Window*) data);
-        XFree(data);
+    xcb_connection_t *c = connection();
+    auto cookie = xcb_get_property_unchecked(c, false, w, atoms->wm_client_leader, XCB_ATOM_WINDOW, 0, 10000);
+    ScopedCPointer<xcb_get_property_reply_t> prop(xcb_get_property_reply(c, cookie, nullptr));
+    if (prop.isNull() || prop->value_len <= 0) {
+        return w;
     }
-    return result;
+    return static_cast<xcb_window_t*>(xcb_get_property_value(prop.data()))[0];
 }
 
 
@@ -294,7 +286,7 @@ void Toplevel::setReadyForPainting()
         if (compositing()) {
             addRepaintFull();
             emit windowShown(this);
-            if (Client *cl = qobject_cast<Client*>(this)) {
+            if (Client *cl = dynamic_cast<Client*>(this)) {
                 if (cl->tabGroup() && cl->tabGroup()->current() == cl)
                     cl->tabGroup()->setCurrent(cl, true);
             }
@@ -408,11 +400,11 @@ void Toplevel::getWmOpaqueRegion()
         Atom type;
         int rformat;
         unsigned long nitems;
-        if (XGetWindowProperty(display(), client,
-                               atoms->net_wm_opaque_region, 0, length, false, XA_CARDINAL,
+        if (XGetWindowProperty(display(), m_client,
+                               atoms->net_wm_opaque_region, 0, length, false, XCB_ATOM_CARDINAL,
                                &type, &rformat, &nitems, &bytes_after_return,
                                reinterpret_cast< unsigned char** >(&data)) == Success) {
-            if (type != XA_CARDINAL || rformat != 32 || nitems%4) {
+            if (type != XCB_ATOM_CARDINAL || rformat != 32 || nitems%4) {
                 // it can happen, that the window does not provide this property
                 XFree(data);
                 break;
@@ -428,7 +420,7 @@ void Toplevel::getWmOpaqueRegion()
             }
             XFree(data);
         } else {
-            kWarning(1212) << "XGetWindowProperty failed";
+            qWarning() << "XGetWindowProperty failed";
             break;
         }
     } while (bytes_after_return > 0);
@@ -446,6 +438,15 @@ bool Toplevel::isDeleted() const
     return false;
 }
 
+bool Toplevel::isOnCurrentActivity() const
+{
+#ifdef KWIN_BUILD_ACTIVITIES
+    return isOnActivity(Activities::self()->current());
+#else
+    return true;
+#endif
+}
+
 void Toplevel::elevate(bool elevate)
 {
     if (!effectWindow()) {
@@ -460,10 +461,15 @@ pid_t Toplevel::pid() const
     return info->pid();
 }
 
+xcb_window_t Toplevel::frameId() const
+{
+    return m_client;
+}
+
 void Toplevel::getSkipCloseAnimation()
 {
-    xcb_get_property_cookie_t cookie = xcb_get_property_unchecked(connection(), false, window(), atoms->kde_skip_close_animation, XCB_ATOM_CARDINAL, 0, 1);
-    ScopedCPointer<xcb_get_property_reply_t> reply(xcb_get_property_reply(connection(), cookie, NULL));
+    auto cookie = xcb_get_property_unchecked(connection(), false, window(), atoms->kde_skip_close_animation, XCB_ATOM_CARDINAL, 0, 1);
+    ScopedCPointer<xcb_get_property_reply_t> reply(xcb_get_property_reply(connection(), cookie, nullptr));
     bool newValue = false;
     if (!reply.isNull()) {
         if (reply->format == 32 && reply->type == XCB_ATOM_CARDINAL && reply->value_len == 1) {
@@ -488,6 +494,39 @@ void Toplevel::setSkipCloseAnimation(bool set)
     emit skipCloseAnimationChanged();
 }
 
+void Toplevel::sendPointerEnterEvent(const QPointF &globalPos)
+{
+    Q_UNUSED(globalPos)
+}
+
+void Toplevel::sendPointerLeaveEvent(const QPointF &globalPos)
+{
+    Q_UNUSED(globalPos)
+}
+
+void Toplevel::sendPointerMoveEvent(const QPointF &globalPos)
+{
+    Q_UNUSED(globalPos)
+}
+
+void Toplevel::sendPointerButtonEvent(uint32_t button, InputRedirection::PointerButtonState state)
+{
+    Q_UNUSED(button)
+    Q_UNUSED(state)
+}
+
+void Toplevel::sendPointerAxisEvent(InputRedirection::PointerAxis axis, qreal delta)
+{
+    Q_UNUSED(axis)
+    Q_UNUSED(delta)
+}
+
+void Toplevel::sendKeybordKeyEvent(uint32_t key, InputRedirection::KeyboardKeyState state)
+{
+    Q_UNUSED(key)
+    Q_UNUSED(state)
+}
+
 } // namespace
 
-#include "moc_toplevel.cpp"
+#include "toplevel.moc"
