@@ -24,15 +24,20 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // kwin libs
 #include <kdecoration.h>
 // kwin
+#include "input.h"
 #include "utils.h"
 #include "virtualdesktops.h"
+#include "xcbutils.h"
 // KDE
-#include <NETWinInfo>
+#include <NETWM>
 // Qt
 #include <QObject>
 // xcb
 #include <xcb/damage.h>
 #include <xcb/xfixes.h>
+// XLib
+#include <X11/Xlib.h>
+#include <fixx11h.h>
 // system
 #include <assert.h>
 
@@ -149,6 +154,7 @@ class Toplevel
      * See http://standards.freedesktop.org/wm-spec/wm-spec-latest.html .
      */
     Q_PROPERTY(int windowType READ windowType)
+    Q_PROPERTY(QStringList activities READ activities NOTIFY activitiesChanged)
     /**
      * Whether this Toplevel is managed by KWin (it has control over its placement and other
      * aspects, as opposed to override-redirect windows that are entirely handled by the application).
@@ -170,8 +176,8 @@ class Toplevel
     Q_PROPERTY(bool skipsCloseAnimation READ skipsCloseAnimation WRITE setSkipCloseAnimation NOTIFY skipCloseAnimationChanged)
 public:
     explicit Toplevel();
-    Window frameId() const;
-    Window window() const;
+    virtual xcb_window_t frameId() const;
+    xcb_window_t window() const;
     QRect geometry() const;
     QSize size() const;
     QPoint pos() const;
@@ -212,9 +218,13 @@ public:
     bool isDNDIcon() const;
 
     virtual int desktop() const = 0;
+    virtual QStringList activities() const = 0;
     bool isOnDesktop(int d) const;
+    bool isOnActivity(const QString &activity) const;
     bool isOnCurrentDesktop() const;
+    bool isOnCurrentActivity() const;
     bool isOnAllDesktops() const;
+    bool isOnAllActivities() const;
 
     QByteArray windowRole() const;
     QByteArray sessionId() const;
@@ -310,7 +320,26 @@ public:
     bool skipsCloseAnimation() const;
     void setSkipCloseAnimation(bool set);
 
-signals:
+    virtual void sendPointerMoveEvent(const QPointF &globalPos);
+    virtual void sendPointerEnterEvent(const QPointF &globalPos);
+    virtual void sendPointerLeaveEvent(const QPointF &globalPos);
+    virtual void sendPointerButtonEvent(uint32_t button, InputRedirection::PointerButtonState state);
+    virtual void sendPointerAxisEvent(InputRedirection::PointerAxis axis, qreal delta);
+    virtual void sendKeybordKeyEvent(uint32_t key, InputRedirection::KeyboardKeyState state);
+
+    /**
+     * @brief Finds the Toplevel matching the condition expressed in @p func in @p list.
+     *
+     * The method is templated to operate on either a list of Toplevels or on a list of
+     * a subclass type of Toplevel.
+     * @param list The list to search in
+     * @param func The condition function (compare std::find_if)
+     * @return T* The found Toplevel or @c null if there is no matching Toplevel
+     */
+    template <class T>
+    static T *findInList(const QList<T*> &list, std::function<bool (const T*)> func);
+
+Q_SIGNALS:
     void opacityChanged(KWin::Toplevel* toplevel, qreal oldOpacity);
     void damaged(KWin::Toplevel* toplevel, const QRect& damage);
     void propertyNotify(KWin::Toplevel* toplevel, long a);
@@ -330,6 +359,7 @@ signals:
      * schedule a repaint of the scene.
      **/
     void needsRepaint();
+    void activitiesChanged(KWin::Toplevel* toplevel);
     /**
      * Emitted whenever the Toplevel's screen changes. This can happen either in consequence to
      * a screen being removed/added or if the Toplevel's geometry changes.
@@ -350,9 +380,9 @@ protected Q_SLOTS:
 
 protected:
     virtual ~Toplevel();
-    void setWindowHandles(Window client, Window frame);
+    void setWindowHandles(xcb_window_t client);
     void detectShape(Window id);
-    virtual void propertyNotifyEvent(XPropertyEvent* e);
+    virtual void propertyNotifyEvent(xcb_property_notify_event_t *e);
     virtual void damageNotifyEvent();
     void discardWindowPixmap();
     void addDamageFull();
@@ -390,14 +420,11 @@ protected:
     bool m_isDamaged;
 
 private:
-    static QByteArray staticWindowRole(WId);
-    static QByteArray staticSessionId(WId);
-    static QByteArray staticWmCommand(WId);
-    static QByteArray staticWmClientMachine(WId);
-    static Window staticWmClientLeader(WId);
+    static QByteArray staticSessionId(xcb_window_t);
+    static QByteArray staticWmCommand(xcb_window_t);
+    static xcb_window_t staticWmClientLeader(xcb_window_t);
     // when adding new data members, check also copyToDeleted()
-    Window client;
-    Window frame;
+    Xcb::Window m_client;
     xcb_damage_damage_t damage_handle;
     QRegion damage_region; // damage is really damaged window (XDamage) and texture needs
     bool is_shape;
@@ -417,22 +444,15 @@ private:
     // when adding new data members, check also copyToDeleted()
 };
 
-inline Window Toplevel::window() const
+inline xcb_window_t Toplevel::window() const
 {
-    return client;
+    return m_client;
 }
 
-inline Window Toplevel::frameId() const
+inline void Toplevel::setWindowHandles(xcb_window_t w)
 {
-    return frame;
-}
-
-inline void Toplevel::setWindowHandles(Window w, Window f)
-{
-    assert(client == None && w != None);
-    client = w;
-    assert(frame == None && f != None);
-    frame = f;
+    assert(!m_client.isValid() && w != XCB_WINDOW_NONE);
+    m_client.reset(w, false);
 }
 
 inline QRect Toplevel::geometry() const
@@ -607,11 +627,21 @@ inline bool Toplevel::isOnAllDesktops() const
     return desktop() == NET::OnAllDesktops;
 }
 
+inline bool Toplevel::isOnAllActivities() const
+{
+    return activities().isEmpty();
+}
+
 inline bool Toplevel::isOnDesktop(int d) const
 {
     return desktop() == d || /*desk == 0 ||*/ isOnAllDesktops();
 }
 
+inline bool Toplevel::isOnActivity(const QString &activity) const
+{
+    return activities().isEmpty() || activities().contains(activity);
+}
+
 inline bool Toplevel::isOnCurrentDesktop() const
 {
     return isOnDesktop(VirtualDesktopManager::self()->current());
@@ -642,12 +672,19 @@ inline const ClientMachine *Toplevel::clientMachine() const
     return m_clientMachine;
 }
 
+template <class T>
+inline T *Toplevel::findInList(const QList<T*> &list, std::function<bool (const T*)> func)
+{
+    const auto it = std::find_if(list.begin(), list.end(), func);
+    if (it == list.end()) {
+        return nullptr;
+    }
+    return *it;
+}
+
 QDebug& operator<<(QDebug& stream, const Toplevel*);
 QDebug& operator<<(QDebug& stream, const ToplevelList&);
 
-KWIN_COMPARE_PREDICATE(WindowMatchPredicate, Toplevel, Window, cl->window() == value);
-KWIN_COMPARE_PREDICATE(FrameIdMatchPredicate, Toplevel, Window, cl->frameId() == value);
-
 } // namespace
 Q_DECLARE_METATYPE(KWin::Toplevel*)
 
