@@ -28,7 +28,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QTimer>
 #include <QDebug>
 
-#include <X11/extensions/shape.h>
+#include <xcb/shape.h>
+#include <xcb/xtest.h>
 
 namespace KWin
 {
@@ -57,27 +58,20 @@ bool Unmanaged::track(Window w)
         ungrabXServer();
         return false;
     }
-    setWindowHandles(w, w);   // the window is also the frame
-    XSelectInput(display(), w, attr.your_event_mask | StructureNotifyMask | PropertyChangeMask);
+    setWindowHandles(w);   // the window is also the frame
+    Xcb::selectInput(w, attr.your_event_mask | XCB_EVENT_MASK_STRUCTURE_NOTIFY | XCB_EVENT_MASK_PROPERTY_CHANGE);
     geom = QRect(attr.x, attr.y, attr.width, attr.height);
     checkScreen();
     vis = attr.visual;
     bit_depth = attr.depth;
-    unsigned long properties[ 2 ];
-    properties[ NETWinInfo::PROTOCOLS ] =
-        NET::WMWindowType |
-        NET::WMPid |
-        0;
-    properties[ NETWinInfo::PROTOCOLS2 ] =
-        NET::WM2Opacity |
-        0;
-    info = new NETWinInfo2(display(), w, rootWindow(), properties, 2);
+    info = new NETWinInfo(connection(), w, rootWindow(),
+                          NET::WMWindowType | NET::WMPid, NET::WM2Opacity);
     getResourceClass();
     getWindowRole();
     getWmClientLeader();
     getWmClientMachine();
     if (Xcb::Extensions::self()->isShapeAvailable())
-        XShapeSelectInput(display(), w, ShapeNotifyMask);
+        xcb_shape_select_input(connection(), w, true);
     detectShape(w);
     getWmOpaqueRegion();
     getSkipCloseAnimation();
@@ -98,8 +92,8 @@ void Unmanaged::release(bool on_shutdown)
     finishCompositing();
     if (!QWidget::find(window())) { // don't affect our own windows
         if (Xcb::Extensions::self()->isShapeAvailable())
-            XShapeSelectInput(display(), window(), NoEventMask);
-        XSelectInput(display(), window(), NoEventMask);
+            xcb_shape_select_input(connection(), window(), false);
+        Xcb::selectInput(window(), XCB_EVENT_MASK_NO_EVENT);
     }
     if (!on_shutdown) {
         workspace()->removeUnmanaged(this);
@@ -153,7 +147,41 @@ NET::WindowType Unmanaged::windowType(bool direct, int supportedTypes) const
     if (supportedTypes == 0) {
         supportedTypes = SUPPORTED_UNMANAGED_WINDOW_TYPES_MASK;
     }
-    return info->windowType(supportedTypes);
+    return info->windowType(NET::WindowTypes(supportedTypes));
+}
+
+void Unmanaged::sendPointerButtonEvent(uint32_t button, InputRedirection::PointerButtonState state)
+{
+    // TODO: don't use xtest
+    uint8_t type = XCB_BUTTON_PRESS;
+    if (state == KWin::InputRedirection::PointerButtonReleased) {
+        type = XCB_BUTTON_RELEASE;
+    }
+    xcb_test_fake_input(connection(), type, InputRedirection::toXPointerButton(button), XCB_TIME_CURRENT_TIME, window(), 0, 0, 0);
+}
+
+void Unmanaged::sendPointerAxisEvent(InputRedirection::PointerAxis axis, qreal delta)
+{
+    // TODO: don't use xtest
+    const int val = qRound(delta);
+    if (val == 0) {
+        return;
+    }
+    const uint8_t button = InputRedirection::toXPointerButton(axis, delta);
+    for (int i = 0; i < qAbs(val); ++i) {
+        xcb_test_fake_input(connection(), XCB_BUTTON_PRESS, button, XCB_TIME_CURRENT_TIME, window(), 0, 0, 0);
+        xcb_test_fake_input(connection(), XCB_BUTTON_RELEASE, button, XCB_TIME_CURRENT_TIME, window(), 0, 0, 0);
+    }
+}
+
+void Unmanaged::sendKeybordKeyEvent(uint32_t key, InputRedirection::KeyboardKeyState state)
+{
+    // TODO: don't use xtest
+    uint8_t type = XCB_KEY_PRESS;
+    if (state == InputRedirection::KeyboardKeyReleased) {
+        type = XCB_KEY_RELEASE;
+    }
+    xcb_test_fake_input(connection(), type, key + 8, XCB_TIME_CURRENT_TIME, window(), 0, 0, 0);
 }
 
 } // namespace
