@@ -34,11 +34,12 @@ class Unmanaged
     Q_OBJECT
 public:
     explicit Unmanaged();
-    bool windowEvent(XEvent* e);
+    bool windowEvent(xcb_generic_event_t *e);
     void release(bool on_shutdown = false);
     bool track(Window w);
     static void deleteUnmanaged(Unmanaged* c);
     virtual int desktop() const;
+    virtual QStringList activities() const;
     virtual QPoint clientPos() const;
     virtual QSize clientSize() const;
     virtual QRect transparentRect() const;
@@ -46,15 +47,17 @@ public:
         return UnmanagedLayer;
     }
     NET::WindowType windowType(bool direct = false, int supported_types = 0) const;
+
+    void sendPointerButtonEvent(uint32_t button, InputRedirection::PointerButtonState state) override;
+    void sendPointerAxisEvent(InputRedirection::PointerAxis axis, qreal delta) override;
+    void sendKeybordKeyEvent(uint32_t key, InputRedirection::KeyboardKeyState state) override;
 protected:
     virtual void debug(QDebug& stream) const;
     virtual bool shouldUnredirect() const;
 private:
     virtual ~Unmanaged(); // use release()
     // handlers for X11 events
-    void mapNotifyEvent(XMapEvent* e);
-    void unmapNotifyEvent(XUnmapEvent*e);
-    void configureNotifyEvent(XConfigureEvent* e);
+    void configureNotifyEvent(xcb_configure_notify_event_t *e);
 };
 
 } // namespace
