@@ -22,7 +22,15 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "workspace.h"
 // kwin libs
 #include <kdecorationfactory.h>
+#include <kwinglplatform.h>
+#include <kwinxrenderutils.h>
 // kwin
+#ifdef KWIN_BUILD_ACTIVITIES
+#include "activities.h"
+#endif
+#ifdef KWIN_BUILD_KAPPMENU
+#include "appmenu.h"
+#endif
 #include "atoms.h"
 #include "client.h"
 #include "composite.h"
@@ -33,39 +41,41 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "effects.h"
 #include "focuschain.h"
 #include "group.h"
+#include "input.h"
 #include "killwindow.h"
 #include "netinfo.h"
+#include "outline.h"
 #include "placement.h"
 #include "rules.h"
 #ifdef KWIN_BUILD_SCREENEDGES
 #include "screenedge.h"
 #endif
 #include "screens.h"
+#include "scripting/scripting.h"
 #ifdef KWIN_BUILD_TABBOX
 #include "tabbox.h"
 #endif
 #include "unmanaged.h"
 #include "useractions.h"
 #include "virtualdesktops.h"
+#if HAVE_WAYLAND
+#include "wayland_backend.h"
+#endif
 #include "xcbutils.h"
+#include "main.h"
 // KDE
-#include <kdeversion.h>
-#include <KActionCollection>
-#include <KCmdLineArgs>
 #include <KConfig>
 #include <KConfigGroup>
-#include <KGlobal>
-#include <KGlobalSettings>
+#include <KLocalizedString>
 #include <KStartupInfo>
 #include <KWindowInfo>
 #include <KWindowSystem>
 // Qt
-#include <QtCore/qmetaobject.h>
+#include <QtConcurrentRun>
 
 namespace KWin
 {
 
-// main.cpp
 extern int screen_number;
 extern bool is_multihead;
 
@@ -106,7 +116,6 @@ Workspace::Workspace(bool restore)
     , last_active_client(0)
     , most_recently_raised(0)
     , movingClient(0)
-    , pending_take_activity(NULL)
     , delayfocus_client(0)
     , force_restacking(false)
     , x_stacking_dirty(true)
@@ -116,9 +125,6 @@ Workspace::Workspace(bool restore)
     , session_saving(false)
     , block_focus(0)
     , m_userActionsMenu(new UserActionsMenu(this))
-    , keys(0)
-    , client_keys(NULL)
-    , disable_shortcuts_keys(NULL)
     , client_keys_dialog(NULL)
     , client_keys_client(NULL)
     , global_shortcuts_disabled_for_client(false)
@@ -128,17 +134,37 @@ Workspace::Workspace(bool restore)
     , block_stacking_updates(0)
     , forced_global_mouse_grab(false)
 {
+    // If KWin was already running it saved its configuration after loosing the selection -> Reread
+    QFuture<void> reparseConfigFuture = QtConcurrent::run(options, &Options::reparseConfiguration);
+
+#ifdef KWIN_BUILD_KAPPMENU
+    ApplicationMenu::create(this);
+#endif
+
     _self = this;
 
     // first initialize the extensions
-    Extensions::init();
     Xcb::Extensions::self();
 
+    InputRedirection::create(this);
+
+    // start the Wayland Backend - will only be created if WAYLAND_DISPLAY is present
+#if HAVE_WAYLAND
+    Wayland::WaylandBackend::create(this);
+    if (kwinApp()->operationMode() != Application::OperationModeX11) {
+        connect(this, SIGNAL(stackingOrderChanged()), input(), SLOT(updatePointerWindow()));
+    }
+#endif
     // start the cursor support
     Cursor::create(this);
 
-    // PluginMgr needs access to the config file, so we need to wait for
-    options->reparseConfiguration();
+#ifdef KWIN_BUILD_ACTIVITIES
+    Activities *activities = Activities::create(this);
+    connect(activities, SIGNAL(currentChanged(QString)), SLOT(updateCurrentActivity(QString)));
+#endif
+
+    // PluginMgr needs access to the config file, so we need to wait for it for finishing
+    reparseConfigFuture.waitForFinished();
 
     // get screen support
     Screens *screens = Screens::create(this);
@@ -150,8 +176,6 @@ Workspace::Workspace(bool restore)
     ColorMapper *colormaps = new ColorMapper(this);
     connect(this, SIGNAL(clientActivated(KWin::Client*)), colormaps, SLOT(update()));
 
-    updateXTime(); // Needed for proper initialization of user_time in Client ctor
-
     delayFocusTimer = 0;
 
     if (restore)
@@ -164,15 +188,15 @@ Workspace::Workspace(bool restore)
         KStartupInfo::DisableKWinModule | KStartupInfo::AnnounceSilenceChanges, this);
 
     // Select windowmanager privileges
-    XSelectInput(display(), rootWindow(),
-                 KeyPressMask |
-                 PropertyChangeMask |
-                 ColormapChangeMask |
-                 SubstructureRedirectMask |
-                 SubstructureNotifyMask |
-                 FocusChangeMask | // For NotifyDetailNone
-                 ExposureMask
-                );
+    Xcb::selectInput(rootWindow(),
+                     XCB_EVENT_MASK_KEY_PRESS |
+                     XCB_EVENT_MASK_PROPERTY_CHANGE |
+                     XCB_EVENT_MASK_COLOR_MAP_CHANGE |
+                     XCB_EVENT_MASK_SUBSTRUCTURE_REDIRECT |
+                     XCB_EVENT_MASK_SUBSTRUCTURE_NOTIFY |
+                     XCB_EVENT_MASK_FOCUS_CHANGE | // For NotifyDetailNone
+                     XCB_EVENT_MASK_EXPOSURE
+    );
 
 #ifdef KWIN_BUILD_SCREENEDGES
     ScreenEdges::create(this);
@@ -188,27 +212,21 @@ Workspace::Workspace(bool restore)
     TabBox::TabBox::create(this);
 #endif
 
+    // init XRenderUtils
+    XRenderUtils::init(connection(), rootWindow());
     m_compositor = Compositor::create(this);
     connect(this, SIGNAL(currentDesktopChanged(int,KWin::Client*)), m_compositor, SLOT(addRepaintFull()));
-    connect(m_compositor, SIGNAL(compositingToggled(bool)), decorationPlugin(), SLOT(resetCompositing()));
+    connect(m_compositor, &Compositor::compositingToggled, decorationPlugin(), &DecorationPlugin::compositingToggled);
 
     new DBusInterface(this);
 
     // Compatibility
-    long data = 1;
+    int32_t data = 1;
 
-    XChangeProperty(
-        display(),
-        rootWindow(),
-        atoms->kwin_running,
-        atoms->kwin_running,
-        32,
-        PropModeAppend,
-        (unsigned char*)(&data),
-        1
-    );
+    xcb_change_property(connection(), XCB_PROP_MODE_APPEND, rootWindow(), atoms->kwin_running,
+                        atoms->kwin_running, 32, 1, &data);
 
-    client_keys = new KActionCollection(this);
+    Outline::create(this);
 
     initShortcuts();
 
@@ -217,13 +235,15 @@ Workspace::Workspace(bool restore)
 
 void Workspace::init()
 {
+    updateXTime(); // Needed for proper initialization of user_time in Client ctor
+    KSharedConfigPtr config = KSharedConfig::openConfig();
     Screens *screens = Screens::self();
-    screens->setConfig(KGlobal::config());
+    screens->setConfig(config);
     screens->reconfigure();
     connect(options, SIGNAL(configChanged()), screens, SLOT(reconfigure()));
 #ifdef KWIN_BUILD_SCREENEDGES
     ScreenEdges *screenEdges = ScreenEdges::self();
-    screenEdges->setConfig(KGlobal::config());
+    screenEdges->setConfig(config);
     screenEdges->init();
     connect(options, SIGNAL(configChanged()), screenEdges, SLOT(reconfigure()));
     connect(VirtualDesktopManager::self(), SIGNAL(layoutChanged(int,int)), screenEdges, SLOT(updateLayout()));
@@ -252,7 +272,7 @@ void Workspace::init()
     vds->setNavigationWrappingAround(options->isRollOverDesktops());
     connect(options, SIGNAL(rollOverDesktopsChanged(bool)), vds, SLOT(setNavigationWrappingAround(bool)));
     vds->setRootInfo(rootInfo);
-    vds->setConfig(KGlobal::config());
+    vds->setConfig(config);
 
     // Now we know how many desktops we'll have, thus we initialize the positioning object
     Placement::create(this);
@@ -262,16 +282,23 @@ void Workspace::init()
     vds->updateLayout();
 
     // Extra NETRootInfo instance in Client mode is needed to get the values of the properties
-    NETRootInfo client_info(display(), NET::ActiveWindow | NET::CurrentDesktop);
+    NETRootInfo client_info(connection(), NET::ActiveWindow | NET::CurrentDesktop);
     int initial_desktop;
-    if (!kapp->isSessionRestored())
+    if (!qApp->isSessionRestored())
         initial_desktop = client_info.currentDesktop();
     else {
+#if KWIN_QT5_PORTING
         KConfigGroup group(kapp->sessionConfig(), "Session");
         initial_desktop = group.readEntry("desktop", 1);
+#else
+        initial_desktop = 1;
+#endif
     }
     if (!VirtualDesktopManager::self()->setCurrent(initial_desktop))
         VirtualDesktopManager::self()->setCurrent(1);
+#ifdef KWIN_BUILD_ACTIVITIES
+    Activities::self()->update(false, true);
+#endif
 
     reconfigureTimer.setSingleShot(true);
     updateToolWindowsTimer.setSingleShot(true);
@@ -279,22 +306,24 @@ void Workspace::init()
     connect(&reconfigureTimer, SIGNAL(timeout()), this, SLOT(slotReconfigure()));
     connect(&updateToolWindowsTimer, SIGNAL(timeout()), this, SLOT(slotUpdateToolWindows()));
 
-    connect(KGlobalSettings::self(), SIGNAL(appearanceChanged()), this, SLOT(reconfigure()));
-    connect(KGlobalSettings::self(), SIGNAL(settingsChanged(int)), this, SLOT(slotSettingsChanged(int)));
-    connect(KGlobalSettings::self(), SIGNAL(blockShortcuts(int)), this, SLOT(slotBlockShortcuts(int)));
+    // TODO: do we really need to reconfigure everything when fonts change?
+    // maybe just reconfigure the decorations? Move this into libkdecoration?
+    QDBusConnection::sessionBus().connect(QString(),
+                                          QStringLiteral("/KDEPlatformTheme"),
+                                          QStringLiteral("org.kde.KDEPlatformTheme"),
+                                          QStringLiteral("refreshFonts"),
+                                          this, SLOT(reconfigure()));
 
     active_client = NULL;
     rootInfo->setActiveWindow(None);
     focusToNull();
-    if (!kapp->isSessionRestored())
+    if (!qApp->isSessionRestored())
         ++block_focus; // Because it will be set below
 
     {
         // Begin updates blocker block
         StackingUpdatesBlocker blocker(this);
 
-        bool fixoffset = KCmdLineArgs::parsedArgs()->getOption("crashes").toInt() > 0;
-
         Xcb::Tree tree(rootWindow());
         xcb_window_t *wins = xcb_query_tree_children(tree.data());
 
@@ -321,7 +350,7 @@ void Workspace::init()
                     // ### This will request the attributes again
                     createUnmanaged(wins[i]);
             } else if (attr->map_state != XCB_MAP_STATE_UNMAPPED) {
-                if (fixoffset) {
+                if (Application::wasCrash()) {
                     fixPositionAfterCrash(wins[i], windowGeometries[i].data());
                 }
 
@@ -347,15 +376,15 @@ void Workspace::init()
         NETSize desktop_geometry;
         desktop_geometry.width = geom.width();
         desktop_geometry.height = geom.height();
-        rootInfo->setDesktopGeometry(-1, desktop_geometry);
+        rootInfo->setDesktopGeometry(desktop_geometry);
         setShowingDesktop(false);
 
     } // End updates blocker block
 
     Client* new_active_client = NULL;
-    if (!kapp->isSessionRestored()) {
+    if (!qApp->isSessionRestored()) {
         --block_focus;
-        new_active_client = findClient(WindowMatchPredicate(client_info.activeWindow()));
+        new_active_client = findClient(Predicate::WindowMatch, client_info.activeWindow());
     }
     if (new_active_client == NULL
             && activeClient() == NULL && should_get_focus.count() == 0) {
@@ -368,7 +397,7 @@ void Workspace::init()
     if (new_active_client != NULL)
         activateClient(new_active_client);
 
-
+    Scripting::create(this);
 
     // SELI TODO: This won't work with unreasonable focus policies,
     // and maybe in rare cases also if the selected client doesn't
@@ -410,17 +439,17 @@ Workspace::~Workspace()
     }
     for (UnmanagedList::iterator it = unmanaged.begin(), end = unmanaged.end(); it != end; ++it)
         (*it)->release(true);
-    XDeleteProperty(display(), rootWindow(), atoms->kwin_running);
+    xcb_delete_property(connection(), rootWindow(), atoms->kwin_running);
 
     delete RuleBook::self();
-    KGlobal::config()->sync();
+    KSharedConfig::openConfig()->sync();
 
     RootInfo::destroy();
     delete startup;
     delete Placement::self();
     delete client_keys_dialog;
     foreach (SessionInfo * s, session)
-        delete s;
+    delete s;
 
     // TODO: ungrabXServer();
 
@@ -469,7 +498,7 @@ void Workspace::addClient(Client* c)
 {
     Group* grp = findGroup(c->window());
 
-    KWindowInfo info = KWindowSystem::windowInfo(c->window(), -1U, NET::WM2WindowClass);
+    KWindowInfo info(c->window(), NET::WMAllProperties, NET::WM2WindowClass);
 
     emit clientAdded(c);
 
@@ -507,6 +536,10 @@ void Workspace::addClient(Client* c)
     if (TabBox::TabBox::self()->isDisplayed())
         TabBox::TabBox::self()->reset(true);
 #endif
+#ifdef KWIN_BUILD_KAPPMENU
+        if (ApplicationMenu::self()->hasMenu(c->window()))
+            c->setAppMenuAvailable();
+#endif
 }
 
 void Workspace::addUnmanaged(Unmanaged* c)
@@ -560,8 +593,6 @@ void Workspace::removeClient(Client* c)
     Q_ASSERT(c != active_client);
     if (c == last_active_client)
         last_active_client = 0;
-    if (c == pending_take_activity)
-        pending_take_activity = NULL;
     if (c == delayfocus_client)
         cancelDelayFocus();
 
@@ -579,6 +610,7 @@ void Workspace::removeUnmanaged(Unmanaged* c)
 {
     assert(unmanaged.contains(c));
     unmanaged.removeAll(c);
+    emit unmanagedRemoved(c);
     x_stacking_dirty = true;
 }
 
@@ -740,54 +772,34 @@ bool Workspace::waitForCompositingSetup()
     return false;
 }
 
-void Workspace::slotSettingsChanged(int category)
-{
-    kDebug(1212) << "Workspace::slotSettingsChanged()";
-    if (category == KGlobalSettings::SETTINGS_SHORTCUTS)
-        m_userActionsMenu->discard();
-}
-
 /**
  * Reread settings
  */
-KWIN_PROCEDURE(CheckBorderSizesProcedure, Client, cl->checkBorderSizes(true));
 
 void Workspace::slotReconfigure()
 {
-    kDebug(1212) << "Workspace::slotReconfigure()";
+    qDebug() << "Workspace::slotReconfigure()";
     reconfigureTimer.stop();
 
     bool borderlessMaximizedWindows = options->borderlessMaximizedWindows();
 
-    KGlobal::config()->reparseConfiguration();
-    unsigned long changed = options->updateSettings();
+    KSharedConfig::openConfig()->reparseConfiguration();
+    options->updateSettings();
 
     emit configChanged();
     m_userActionsMenu->discard();
     updateToolWindows(true);
 
     DecorationPlugin *deco = DecorationPlugin::self();
-    if (!deco->isDisabled() && deco->reset(changed)) {
-        // Decorations need to be recreated
-
-        // This actually seems to make things worse now
-        //QWidget curtain;
-        //curtain.setBackgroundMode( NoBackground );
-        //curtain.setGeometry( Kephal::ScreenUtils::desktopGeometry() );
-        //curtain.show();
-
-        for (ClientList::ConstIterator it = clients.constBegin(); it != clients.constEnd(); ++it)
-            (*it)->updateDecoration(true, true);
-        // If the new decoration doesn't supports tabs then ungroup clients
-        if (!decorationPlugin()->supportsTabbing()) {
-            foreach (Client * c, clients)
-                c->untab();
-        }
+    if (!deco->isDisabled() && deco->reset()) {
+        deco->recreateDecorations();
         deco->destroyPreviousPlugin();
+        connect(deco->factory(), &KDecorationFactory::recreateDecorations, deco, &DecorationPlugin::recreateDecorations);
     } else {
-        forEachClient(CheckBorderSizesProcedure());
-        foreach (Client * c, clients)
+        foreach (Client * c, clients) {
+            c->checkBorderSizes(true);
             c->triggerDecorationRepaint();
+        }
     }
 
     RuleBook::self()->load();
@@ -906,7 +918,7 @@ void Workspace::updateClientVisibilityOnDesktopChange(uint oldDesktop, uint newD
         if (!c) {
             continue;
         }
-        if (!c->isOnDesktop(newDesktop) && c != movingClient) {
+        if (!c->isOnDesktop(newDesktop) && c != movingClient && c->isOnCurrentActivity()) {
             if (c->isShown(true) && c->isOnDesktop(oldDesktop) && !compositing())
                 obs_wins.create(c);
             (c)->updateVisibility();
@@ -924,7 +936,7 @@ void Workspace::updateClientVisibilityOnDesktopChange(uint oldDesktop, uint newD
         if (!c) {
             continue;
         }
-        if (c->isOnDesktop(newDesktop))
+        if (c->isOnDesktop(newDesktop) && c->isOnCurrentActivity())
             c->updateVisibility();
     }
     --block_showing_desktop;
@@ -975,7 +987,8 @@ Client *Workspace::findClientToActivateOnDesktop(uint desktop)
                 continue;
             }
 
-            if (!(client->isShown(false) && client->isOnDesktop(desktop)))
+            if (!(client->isShown(false) && client->isOnDesktop(desktop) &&
+                client->isOnCurrentActivity() && client->isOnActiveScreen()))
                 continue;
 
             if (client->geometry().contains(Cursor::pos())) {
@@ -988,6 +1001,106 @@ Client *Workspace::findClientToActivateOnDesktop(uint desktop)
     return FocusChain::self()->getForActivation(desktop);
 }
 
+/**
+ * Updates the current activity when it changes
+ * do *not* call this directly; it does not set the activity.
+ *
+ * Shows/Hides windows according to the stacking order
+ */
+
+void Workspace::updateCurrentActivity(const QString &new_activity)
+{
+#ifdef KWIN_BUILD_ACTIVITIES
+    //closeActivePopup();
+    ++block_focus;
+    // TODO: Q_ASSERT( block_stacking_updates == 0 ); // Make sure stacking_order is up to date
+    StackingUpdatesBlocker blocker(this);
+
+    ++block_showing_desktop; //FIXME should I be using that?
+    // Optimized Desktop switching: unmapping done from back to front
+    // mapping done from front to back => less exposure events
+    //Notify::raise((Notify::Event) (Notify::DesktopChange+new_desktop));
+
+    ObscuringWindows obs_wins;
+
+    const QString &old_activity = Activities::self()->previous();
+
+    for (ToplevelList::ConstIterator it = stacking_order.constBegin();
+            it != stacking_order.constEnd();
+            ++it) {
+        Client *c = qobject_cast<Client*>(*it);
+        if (!c) {
+            continue;
+        }
+        if (!c->isOnActivity(new_activity) && c != movingClient && c->isOnCurrentDesktop()) {
+            if (c->isShown(true) && c->isOnActivity(old_activity) && !compositing())
+                obs_wins.create(c);
+            c->updateVisibility();
+        }
+    }
+
+    // Now propagate the change, after hiding, before showing
+    //rootInfo->setCurrentDesktop( currentDesktop() );
+
+    /* TODO someday enable dragging windows to other activities
+    if ( movingClient && !movingClient->isOnDesktop( new_desktop ))
+        {
+        movingClient->setDesktop( new_desktop );
+        */
+
+    for (int i = stacking_order.size() - 1; i >= 0 ; --i) {
+        Client *c = qobject_cast<Client*>(stacking_order.at(i));
+        if (!c) {
+            continue;
+        }
+        if (c->isOnActivity(new_activity))
+            c->updateVisibility();
+    }
+
+    --block_showing_desktop;
+    //FIXME not sure if I should do this either
+    if (showingDesktop())   // Do this only after desktop change to avoid flicker
+        resetShowingDesktop(false);
+
+    // Restore the focus on this desktop
+    --block_focus;
+    Client* c = 0;
+
+    //FIXME below here is a lot of focuschain stuff, probably all wrong now
+    if (options->focusPolicyIsReasonable()) {
+        // Search in focus chain
+        c = FocusChain::self()->getForActivation(VirtualDesktopManager::self()->current());
+    }
+    // If "unreasonable focus policy" and active_client is on_all_desktops and
+    // under mouse (Hence == old_active_client), conserve focus.
+    // (Thanks to Volker Schatz <V.Schatz at thphys.uni-heidelberg.de>)
+    else if (active_client && active_client->isShown(true) && active_client->isOnCurrentDesktop() && active_client->isOnCurrentActivity())
+        c = active_client;
+
+    if (c == NULL && !desktops.isEmpty())
+        c = findDesktop(true, VirtualDesktopManager::self()->current());
+
+    if (c != active_client)
+        setActiveClient(NULL);
+
+    if (c)
+        requestFocus(c);
+    else if (!desktops.isEmpty())
+        requestFocus(findDesktop(true, VirtualDesktopManager::self()->current()));
+    else
+        focusToNull();
+
+    // Not for the very first time, only if something changed and there are more than 1 desktops
+
+    //if ( effects != NULL && old_desktop != 0 && old_desktop != new_desktop )
+    //    static_cast<EffectsHandlerImpl*>( effects )->desktopChanged( old_desktop );
+    if (compositing() && m_compositor)
+        m_compositor->addRepaintFull();
+#else
+    Q_UNUSED(new_activity)
+#endif
+}
+
 void Workspace::moveClientsFromRemovedDesktops()
 {
     for (ClientList::ConstIterator it = clients.constBegin(); it != clients.constEnd(); ++it) {
@@ -1060,6 +1173,7 @@ void Workspace::sendClientToDesktop(Client* c, int desk, bool dont_activate)
  * this is NOT in any way related to XRandR multiscreen
  *
  */
+extern bool is_multihead; // main.cpp
 bool Workspace::isOnCurrentHead()
 {
     if (!is_multihead) {
@@ -1128,12 +1242,6 @@ void Workspace::focusToNull()
     m_nullFocus->focus();
 }
 
-void Workspace::sendTakeActivity(KWin::Client *c, xcb_timestamp_t timestamp, long int flags)
-{
-    rootInfo()->takeActivity(c->window(), timestamp, flags);
-    pending_take_activity = c;
-}
-
 void Workspace::setShowingDesktop(bool showing)
 {
     rootInfo()->setShowingDesktop(showing);
@@ -1152,7 +1260,7 @@ void Workspace::setShowingDesktop(bool showing)
             if (!c) {
                 continue;
             }
-            if (c->isOnCurrentDesktop() && c->isShown(true) && !c->isSpecialWindow())
+            if (c->isOnCurrentActivity() && c->isOnCurrentDesktop() && c->isShown(true) && !c->isSpecialWindow())
                 showing_desktop_clients.prepend(c);   // Topmost first to reduce flicker
         }
         for (ClientList::ConstIterator it = showing_desktop_clients.constBegin();
@@ -1202,26 +1310,18 @@ void Workspace::resetShowingDesktop(bool keep_hidden)
     --block_showing_desktop;
 }
 
-static bool pending_dfc = false;
-
 void Workspace::disableGlobalShortcutsForClient(bool disable)
 {
     if (global_shortcuts_disabled_for_client == disable)
         return;
-    if (disable)
-        pending_dfc = true;
-    KGlobalSettings::self()->emitChange(KGlobalSettings::BlockShortcuts, disable);
-    // KWin will get the kipc message too
-}
+    QDBusMessage message = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kglobalaccel"),
+                                                          QStringLiteral("/kglobalaccel"),
+                                                          QStringLiteral("org.kde.KGlobalAccel"),
+                                                          QStringLiteral("blockGlobalShortcuts"));
+    message.setArguments(QList<QVariant>() << disable);
+    QDBusConnection::sessionBus().asyncCall(message);
 
-void Workspace::slotBlockShortcuts(int data)
-{
-    if (pending_dfc && data) {
-        global_shortcuts_disabled_for_client = true;
-        pending_dfc = false;
-    } else {
-        global_shortcuts_disabled_for_client = false;
-    }
+    global_shortcuts_disabled_for_client = disable;
     // Update also Alt+LMB actions etc.
     for (ClientList::ConstIterator it = clients.constBegin();
             it != clients.constEnd();
@@ -1240,103 +1340,185 @@ QString Workspace::supportInformation() const
         "what OpenGL driver and which effects are running.\n"
         "Please post the information provided underneath this introductory text to a paste bin service\n"
         "like http://paste.kde.org instead of pasting into support threads.\n").toString());
-    support.append("\n==========================\n\n");
+    support.append(QStringLiteral("\n==========================\n\n"));
     // all following strings are intended for support. They need to be pasted to e.g forums.kde.org
     // it is expected that the support will happen in English language or that the people providing
     // help understand English. Because of that all texts are not translated
-    support.append("Version\n");
-    support.append("=======\n");
-    support.append("KWin version: ");
-    support.append(KWIN_VERSION_STRING);
-    support.append('\n');
-    support.append("KDE SC version (runtime): ");
-    support.append(KDE::versionString());
-    support.append('\n');
-    support.append("KDE SC version (compile): ");
-    support.append(KDE_VERSION_STRING);
-    support.append('\n');
-    support.append("Qt Version: ");
-    support.append(qVersion());
-    support.append("\n\n");
-    support.append("Options\n");
-    support.append("=======\n");
+    support.append(QStringLiteral("Version\n"));
+    support.append(QStringLiteral("=======\n"));
+    support.append(QStringLiteral("KWin version: "));
+    support.append(QStringLiteral(KWIN_VERSION_STRING));
+    support.append(QStringLiteral("\n"));
+    support.append(QStringLiteral("Qt Version: "));
+    support.append(QString::fromUtf8(qVersion()));
+    support.append(QStringLiteral("\n\n"));
+    support.append(QStringLiteral("Operation Mode: "));
+    switch (kwinApp()->operationMode()) {
+    case Application::OperationModeX11:
+        support.append(QStringLiteral("X11 only"));
+        break;
+    case Application::OperationModeWaylandAndX11:
+        support.append(QStringLiteral("Wayland and X11"));
+        break;
+    }
+    support.append(QStringLiteral("Options\n"));
+    support.append(QStringLiteral("=======\n"));
     const QMetaObject *metaOptions = options->metaObject();
     for (int i=0; i<metaOptions->propertyCount(); ++i) {
         const QMetaProperty property = metaOptions->property(i);
-        if (QLatin1String(property.name()) == "objectName") {
+        if (QLatin1String(property.name()) == QLatin1String("objectName")) {
             continue;
         }
-        support.append(QLatin1String(property.name()) + ": " + options->property(property.name()).toString() + '\n');
+        support.append(QLatin1String(property.name()) + QStringLiteral(": ") + options->property(property.name()).toString() + QStringLiteral("\n"));
     }
 #ifdef KWIN_BUILD_SCREENEDGES
-    support.append("\nScreen Edges\n");
-    support.append(  "============\n");
+    support.append(QStringLiteral("\nScreen Edges\n"));
+    support.append(QStringLiteral(  "============\n"));
     const QMetaObject *metaScreenEdges = ScreenEdges::self()->metaObject();
     for (int i=0; i<metaScreenEdges->propertyCount(); ++i) {
         const QMetaProperty property = metaScreenEdges->property(i);
-        if (QLatin1String(property.name()) == "objectName") {
+        if (QLatin1String(property.name()) == QLatin1String("objectName")) {
             continue;
         }
-        support.append(QLatin1String(property.name()) + ": " + ScreenEdges::self()->property(property.name()).toString() + '\n');
+        support.append(QLatin1String(property.name()) + QStringLiteral(": ") + ScreenEdges::self()->property(property.name()).toString() + QStringLiteral("\n"));
     }
 #endif
-    support.append("\nScreens\n");
-    support.append(  "=======\n");
-    support.append("Multi-Head: ");
+    support.append(QStringLiteral("\nScreens\n"));
+    support.append(QStringLiteral(  "=======\n"));
+    support.append(QStringLiteral("Multi-Head: "));
     if (is_multihead) {
-        support.append("yes\n");
-        support.append(QString("Head: %1\n").arg(screen_number));
+        support.append(QStringLiteral("yes\n"));
+        support.append(QStringLiteral("Head: %1\n").arg(screen_number));
     } else {
-        support.append("no\n");
+        support.append(QStringLiteral("no\n"));
     }
-    support.append("Active screen follows mouse: ");
+    support.append(QStringLiteral("Active screen follows mouse: "));
     if (screens()->isCurrentFollowsMouse())
-        support.append(" yes\n");
+        support.append(QStringLiteral(" yes\n"));
     else
-        support.append(" no\n");
-    support.append(QString("Number of Screens: %1\n").arg(screens()->count()));
+        support.append(QStringLiteral(" no\n"));
+    support.append(QStringLiteral("Number of Screens: %1\n").arg(screens()->count()));
     for (int i=0; i<screens()->count(); ++i) {
         const QRect geo = screens()->geometry(i);
-        support.append(QString("Screen %1 Geometry: %2,%3,%4x%5\n")
+        support.append(QStringLiteral("Screen %1 Geometry: %2,%3,%4x%5\n")
                               .arg(i)
                               .arg(geo.x())
                               .arg(geo.y())
                               .arg(geo.width())
                               .arg(geo.height()));
     }
-    support.append("\nDecoration\n");
-    support.append(  "==========\n");
+    support.append(QStringLiteral("\nDecoration\n"));
+    support.append(QStringLiteral(  "==========\n"));
     support.append(decorationPlugin()->supportInformation());
-    support.append("\nCompositing\n");
-    support.append(  "===========\n");
+    support.append(QStringLiteral("\nCompositing\n"));
+    support.append(QStringLiteral(  "===========\n"));
     if (effects) {
-        support.append("Compositing is active\n");
+        support.append(QStringLiteral("Compositing is active\n"));
         switch (effects->compositingType()) {
+        case OpenGL2Compositing:
+        case OpenGLCompositing: {
+#ifdef KWIN_HAVE_OPENGLES
+            support.append(QStringLiteral("Compositing Type: OpenGL ES 2.0\n"));
+#else
+            support.append(QStringLiteral("Compositing Type: OpenGL\n"));
+#endif
+
+            GLPlatform *platform = GLPlatform::instance();
+            support.append(QStringLiteral("OpenGL vendor string: ") +   QString::fromUtf8(platform->glVendorString()) + QStringLiteral("\n"));
+            support.append(QStringLiteral("OpenGL renderer string: ") + QString::fromUtf8(platform->glRendererString()) + QStringLiteral("\n"));
+            support.append(QStringLiteral("OpenGL version string: ") +  QString::fromUtf8(platform->glVersionString()) + QStringLiteral("\n"));
+
+            if (platform->supports(LimitedGLSL) || platform->supports(GLSL))
+                support.append(QStringLiteral("OpenGL shading language version string: ") + QString::fromUtf8(platform->glShadingLanguageVersionString()) + QStringLiteral("\n"));
+
+            support.append(QStringLiteral("Driver: ") + GLPlatform::driverToString(platform->driver()) + QStringLiteral("\n"));
+            if (!platform->isMesaDriver())
+                support.append(QStringLiteral("Driver version: ") + GLPlatform::versionToString(platform->driverVersion()) + QStringLiteral("\n"));
+
+            support.append(QStringLiteral("GPU class: ") + GLPlatform::chipClassToString(platform->chipClass()) + QStringLiteral("\n"));
+
+            support.append(QStringLiteral("OpenGL version: ") + GLPlatform::versionToString(platform->glVersion()) + QStringLiteral("\n"));
+
+            if (platform->supports(LimitedGLSL) || platform->supports(GLSL))
+                support.append(QStringLiteral("GLSL version: ") + GLPlatform::versionToString(platform->glslVersion()) + QStringLiteral("\n"));
+
+            if (platform->isMesaDriver())
+                support.append(QStringLiteral("Mesa version: ") + GLPlatform::versionToString(platform->mesaVersion()) + QStringLiteral("\n"));
+            if (platform->serverVersion() > 0)
+                support.append(QStringLiteral("X server version: ") + GLPlatform::versionToString(platform->serverVersion()) + QStringLiteral("\n"));
+            if (platform->kernelVersion() > 0)
+                support.append(QStringLiteral("Linux kernel version: ") + GLPlatform::versionToString(platform->kernelVersion()) + QStringLiteral("\n"));
+
+            support.append(QStringLiteral("Direct rendering: "));
+            support.append(QStringLiteral("Requires strict binding: "));
+            if (!platform->isLooseBinding()) {
+                support.append(QStringLiteral("yes\n"));
+            } else {
+                support.append(QStringLiteral("no\n"));
+            }
+            support.append(QStringLiteral("GLSL shaders: "));
+            if (platform->supports(GLSL)) {
+                if (platform->supports(LimitedGLSL)) {
+                    support.append(QStringLiteral(" limited\n"));
+                } else {
+                    support.append(QStringLiteral(" yes\n"));
+                }
+            } else {
+                support.append(QStringLiteral(" no\n"));
+            }
+            support.append(QStringLiteral("Texture NPOT support: "));
+            if (platform->supports(TextureNPOT)) {
+                if (platform->supports(LimitedNPOT)) {
+                    support.append(QStringLiteral(" limited\n"));
+                } else {
+                    support.append(QStringLiteral(" yes\n"));
+                }
+            } else {
+                support.append(QStringLiteral(" no\n"));
+            }
+            support.append(QStringLiteral("Virtual Machine: "));
+            if (platform->isVirtualMachine()) {
+                support.append(QStringLiteral(" yes\n"));
+            } else {
+                support.append(QStringLiteral(" no\n"));
+            }
+
+            support.append(QStringLiteral("OpenGL 2 Shaders are used\n"));
+            support.append(QStringLiteral("Painting blocks for vertical retrace: "));
+            if (m_compositor->scene()->blocksForRetrace())
+                support.append(QStringLiteral(" yes\n"));
+            else
+                support.append(QStringLiteral(" no\n"));
+            break;
+        }
         case XRenderCompositing:
-            support.append("Compositing Type: XRender\n");
+            support.append(QStringLiteral("Compositing Type: XRender\n"));
+            break;
+        case QPainterCompositing:
+            support.append("Compositing Type: QPainter\n");
             break;
         case NoCompositing:
         default:
-            support.append("Something is really broken, neither OpenGL nor XRender is used");
+            support.append(QStringLiteral("Something is really broken, neither OpenGL nor XRender is used"));
         }
-        support.append("\nLoaded Effects:\n");
-        support.append(  "---------------\n");
+        support.append(QStringLiteral("\nLoaded Effects:\n"));
+        support.append(QStringLiteral(  "---------------\n"));
         foreach (const QString &effect, static_cast<EffectsHandlerImpl*>(effects)->loadedEffects()) {
-            support.append(effect + '\n');
+            support.append(effect + QStringLiteral("\n"));
         }
-        support.append("\nCurrently Active Effects:\n");
-        support.append(  "-------------------------\n");
+        support.append(QStringLiteral("\nCurrently Active Effects:\n"));
+        support.append(QStringLiteral(  "-------------------------\n"));
         foreach (const QString &effect, static_cast<EffectsHandlerImpl*>(effects)->activeEffects()) {
-            support.append(effect + '\n');
+            support.append(effect + QStringLiteral("\n"));
         }
-        support.append("\nEffect Settings:\n");
-        support.append(  "----------------\n");
+        support.append(QStringLiteral("\nEffect Settings:\n"));
+        support.append(QStringLiteral(  "----------------\n"));
         foreach (const QString &effect, static_cast<EffectsHandlerImpl*>(effects)->loadedEffects()) {
             support.append(static_cast<EffectsHandlerImpl*>(effects)->supportInformation(effect));
-            support.append('\n');
+            support.append(QStringLiteral("\n"));
         }
     } else {
-        support.append("Compositing is not active\n");
+        support.append(QStringLiteral("Compositing is not active\n"));
     }
     return support;
 }
@@ -1348,6 +1530,52 @@ void Workspace::slotToggleCompositing()
     }
 }
 
+Client *Workspace::findClient(std::function<bool (const Client*)> func) const
+{
+    if (Client *ret = Toplevel::findInList(clients, func)) {
+        return ret;
+    }
+    if (Client *ret = Toplevel::findInList(desktops, func)) {
+        return ret;
+    }
+    return nullptr;
+}
+
+Unmanaged *Workspace::findUnmanaged(std::function<bool (const Unmanaged*)> func) const
+{
+    return Toplevel::findInList(unmanaged, func);
+}
+
+Unmanaged *Workspace::findUnmanaged(xcb_window_t w) const
+{
+    return findUnmanaged([w](const Unmanaged *u) {
+        return u->window() == w;
+    });
+}
+
+Client *Workspace::findClient(Predicate predicate, xcb_window_t w) const
+{
+    switch (predicate) {
+    case Predicate::WindowMatch:
+        return findClient([w](const Client *c) {
+            return c->window() == w;
+        });
+    case Predicate::WrapperIdMatch:
+        return findClient([w](const Client *c) {
+            return c->wrapperId() == w;
+        });
+    case Predicate::FrameIdMatch:
+        return findClient([w](const Client *c) {
+            return c->frameId() == w;
+        });
+    case Predicate::InputIdMatch:
+        return findClient([w](const Client *c) {
+            return c->inputId() == w;
+        });
+    }
+    return nullptr;
+}
+
 } // namespace
 
-#include "moc_workspace.cpp"
+#include "workspace.moc"
