@@ -30,15 +30,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // Qt
 #include <QTimer>
 #include <QVector>
-// X
-#include <X11/Xlib.h>
+// std
+#include <functional>
 
 // TODO: Cleanup the order of things in this .h file
 
-#include <QStringList>
+class QStringList;
 class KConfig;
-class KActionCollection;
-class KShortcut;
+class KConfigGroup;
 class KStartupInfo;
 class KStartupInfoId;
 class KStartupInfoData;
@@ -56,6 +55,7 @@ class KillWindow;
 class ShortcutDialog;
 class UserActionsMenu;
 class Compositor;
+enum class Predicate;
 
 class Workspace : public QObject, public KDecorationDefines
 {
@@ -68,17 +68,59 @@ public:
         return _self;
     }
 
-    bool workspaceEvent(XEvent*);
+    bool workspaceEvent(xcb_generic_event_t*);
     bool workspaceEvent(QEvent*);
 
     bool hasClient(const Client*);
 
-    template<typename T> Client* findClient(T predicate) const;
-    template<typename T1, typename T2> void forEachClient(T1 procedure, T2 predicate);
-    template<typename T> void forEachClient(T procedure);
-    template<typename T> Unmanaged* findUnmanaged(T predicate) const;
-    template<typename T1, typename T2> void forEachUnmanaged(T1 procedure, T2 predicate);
-    template<typename T> void forEachUnmanaged(T procedure);
+    /**
+     * @brief Finds the first Client matching the condition expressed by passed in @p func.
+     *
+     * Internally findClient uses the std::find_if algorithm and that determines how the function
+     * needs to be implemented. An example usage for finding a Client with a matching windowId
+     * @code
+     * xcb_window_t w; // our test window
+     * Client *client = findClient([w](const Client *c) -> bool {
+     *     return c->window() == w;
+     * });
+     * @endcode
+     *
+     * For the standard cases of matching the window id with one of the Client's windows use
+     * the simplified overload method findClient(Predicate, xcb_window_t). Above example
+     * can be simplified to:
+     * @code
+     * xcb_window_t w; // our test window
+     * Client *client = findClient(Predicate::WindowMatch, w);
+     * @endcode
+     *
+     * @param func Unary function that accepts a Client* as argument and
+     * returns a value convertible to bool. The value returned indicates whether the
+     * Client* is considered a match in the context of this function.
+     * The function shall not modify its argument.
+     * This can either be a function pointer or a function object.
+     * @return KWin::Client* The found Client or @c null
+     * @see findClient(Predicate, xcb_window_t)
+     */
+    Client *findClient(std::function<bool (const Client*)> func) const;
+    /**
+     * @brief Finds the Client matching the given match @p predicate for the given window.
+     *
+     * @param predicate Which window should be compared
+     * @param w The window id to test against
+     * @return KWin::Client* The found Client or @c null
+     * @see findClient(std::function<bool (const Client*)>)
+     */
+    Client *findClient(Predicate predicate, xcb_window_t w) const;
+    void forEachClient(std::function<void (Client*)> func);
+    Unmanaged *findUnmanaged(std::function<bool (const Unmanaged*)> func) const;
+    /**
+     * @brief Finds the Unmanaged with the given window id.
+     *
+     * @param w The window id to search for
+     * @return KWin::Unmanaged* Found Unmanaged or @c null if there is no Unmanaged with given Id.
+     */
+    Unmanaged *findUnmanaged(xcb_window_t w) const;
+    void forEachUnmanaged(std::function<void (Unmanaged*)> func);
 
     QRect clientArea(clientAreaOption, const QPoint& p, int desktop) const;
     QRect clientArea(clientAreaOption, const Client* c) const;
@@ -104,8 +146,13 @@ public:
 
     void activateClient(Client*, bool force = false);
     void requestFocus(Client* c, bool force = false);
-    void takeActivity(Client* c, int flags, bool handled);   // Flags are ActivityFlags
-    void handleTakeActivity(Client* c, xcb_timestamp_t timestamp, int flags);   // Flags are ActivityFlags
+    enum ActivityFlag {
+        ActivityFocus = 1 << 0, // focus the window
+        ActivityFocusForce = 1 << 1 | ActivityFocus, // focus even if Dock etc.
+        ActivityRaise = 1 << 2 // raise the window
+    };
+    Q_DECLARE_FLAGS(ActivityFlags, ActivityFlag)
+    void takeActivity(Client* c, ActivityFlags flags);
     bool allowClientActivation(const Client* c, xcb_timestamp_t time = -1U, bool focus_in = false,
                                bool ignore_desktop = false);
     void restoreFocus();
@@ -187,16 +234,6 @@ public:
     int oldDisplayWidth() const;
     int oldDisplayHeight() const;
 
-    KActionCollection* actionCollection() const {
-        return keys;
-    }
-    KActionCollection* disableShortcutsKeys() const {
-        return disable_shortcuts_keys;
-    }
-    KActionCollection* clientKeys() const {
-        return client_keys;
-    }
-
     /**
      * Returns the list of clients sorted in stacking order, with topmost client
      * at the last position
@@ -226,8 +263,6 @@ public:
     void updateOnAllDesktopsOfTransients(Client*);
     void checkTransients(xcb_window_t w);
 
-    void performWindowOperation(Client* c, WindowOperation op);
-
     void storeSession(KConfig* config, SMSavePhase phase);
     void storeClient(KConfigGroup &cg, int num, Client *c);
     void storeSubSession(const QString &name, QSet<QByteArray> sessionIds);
@@ -246,7 +281,6 @@ public:
     bool showingDesktop() const;
 
     void sendPingToWindow(xcb_window_t w, xcb_timestamp_t timestamp);   // Called from Client::pingWindow()
-    void sendTakeActivity(Client* c, xcb_timestamp_t timestamp, long flags);   // Called from Client::takeActivity()
 
     void removeClient(Client*);   // Only called from Client::destroyClient() or Client::releaseWindow()
     void setActiveClient(Client*);
@@ -265,7 +299,7 @@ public:
 
     bool forcedGlobalMouseGrab() const;
     void clientShortcutUpdated(Client* c);
-    bool shortcutAvailable(const KShortcut& cut, Client* ignore = NULL) const;
+    bool shortcutAvailable(const QKeySequence &cut, Client* ignore = NULL) const;
     bool globalShortcutsDisabled() const;
     void disableGlobalShortcutsForClient(bool disable);
 
@@ -294,7 +328,8 @@ public:
      **/
     bool compositing() const;
 
-public slots:
+public Q_SLOTS:
+    void performWindowOperation(KWin::Client* c, KDecorationDefines::WindowOperation op);
     // Keybindings
     //void slotSwitchToWindow( int );
     void slotWindowToDesktop();
@@ -357,8 +392,6 @@ public slots:
     void slotWindowToDesktopUp();
     void slotWindowToDesktopDown();
 
-    void slotSettingsChanged(int category);
-
     void reconfigure();
     void slotReconfigure();
 
@@ -375,12 +408,12 @@ public slots:
     void slotActivatePrevTab(); // Slot to move right the active Client.
     void slotUntab(); // Slot to remove the active client from its group.
 
-private slots:
+private Q_SLOTS:
     void desktopResized();
     void slotUpdateToolWindows();
     void delayFocus();
-    void slotBlockShortcuts(int data);
     void slotReloadConfig();
+    void updateCurrentActivity(const QString &new_activity);
     // virtual desktop handling
     void moveClientsFromRemovedDesktops();
     void slotDesktopCountChanged(uint previousCount, uint newCount);
@@ -394,7 +427,6 @@ Q_SIGNALS:
     void workspaceInitialized();
 
     //Signals required for the scripting interface
-signals:
     void desktopPresenceChanged(KWin::Client*, int);
     void currentDesktopChanged(int, KWin::Client*);
     void clientAdded(KWin::Client*);
@@ -403,6 +435,7 @@ signals:
     void clientDemandsAttentionChanged(KWin::Client*, bool);
     void groupAdded(KWin::Group*);
     void unmanagedAdded(KWin::Unmanaged*);
+    void unmanagedRemoved(KWin::Unmanaged*);
     void deletedRemoved(KWin::Deleted*);
     void propertyNotify(long a);
     void configChanged();
@@ -416,6 +449,9 @@ signals:
 private:
     void init();
     void initShortcuts();
+    template <typename Slot>
+    void initShortcut(const QString &actionName, const QString &description, const QKeySequence &shortcut,
+                      Slot slot, const QVariant &data = QVariant());
     void setupWindowShortcut(Client* c);
     enum Direction {
         DirectionNorth,
@@ -442,8 +478,6 @@ private:
     Unmanaged* createUnmanaged(xcb_window_t w);
     void addUnmanaged(Unmanaged* c);
 
-    xcb_window_t findSpecialEventWindow(XEvent* e);
-
     //---------------------------------------------------------------------
 
     void closeActivePopup();
@@ -468,7 +502,6 @@ private:
     Client* last_active_client;
     Client* most_recently_raised; // Used ONLY by raiseOrLowerClient()
     Client* movingClient;
-    Client* pending_take_activity;
 
     // Delay(ed) window focus timer and client
     QTimer* delayFocusTimer;
@@ -509,9 +542,6 @@ private:
 
     void modalActionsSwitch(bool enabled);
 
-    KActionCollection* keys;
-    KActionCollection* client_keys;
-    KActionCollection* disable_shortcuts_keys;
     ShortcutDialog* client_keys_dialog;
     Client* client_keys_client;
     bool global_shortcuts_disabled_for_client;
@@ -667,58 +697,24 @@ inline QPoint Workspace::focusMousePosition() const
     return focusMousePos;
 }
 
-template< typename T >
-inline Client* Workspace::findClient(T predicate) const
+inline
+void Workspace::forEachClient(std::function< void (Client*) > func)
 {
-    if (Client* ret = findClientInList(clients, predicate))
-        return ret;
-    if (Client* ret = findClientInList(desktops, predicate))
-        return ret;
-    return NULL;
+    std::for_each(clients.constBegin(), clients.constEnd(), func);
+    std::for_each(desktops.constBegin(), desktops.constEnd(), func);
 }
 
-template< typename T1, typename T2 >
-inline void Workspace::forEachClient(T1 procedure, T2 predicate)
+inline
+void Workspace::forEachUnmanaged(std::function< void (Unmanaged*) > func)
 {
-    for (ClientList::ConstIterator it = clients.constBegin(); it != clients.constEnd(); ++it)
-        if (predicate(const_cast<const Client*>(*it)))
-            procedure(*it);
-    for (ClientList::ConstIterator it = desktops.constBegin(); it != desktops.constEnd(); ++it)
-        if (predicate(const_cast<const Client*>(*it)))
-            procedure(*it);
+    std::for_each(unmanaged.constBegin(), unmanaged.constEnd(), func);
 }
 
-template< typename T >
-inline void Workspace::forEachClient(T procedure)
-{
-    return forEachClient(procedure, TruePredicate());
-}
-
-template< typename T >
-inline Unmanaged* Workspace::findUnmanaged(T predicate) const
-{
-    return findUnmanagedInList(unmanaged, predicate);
-}
-
-template< typename T1, typename T2 >
-inline void Workspace::forEachUnmanaged(T1 procedure, T2 predicate)
-{
-    for (UnmanagedList::ConstIterator it = unmanaged.constBegin(); it != unmanaged.constEnd(); ++it)
-        if (predicate(const_cast<const Unmanaged*>(*it)))
-            procedure(*it);
-}
-
-template< typename T >
-inline void Workspace::forEachUnmanaged(T procedure)
-{
-    return forEachUnmanaged(procedure, TruePredicate());
-}
-
-KWIN_COMPARE_PREDICATE(ClientMatchPredicate, Client, const Client*, cl == value);
-
 inline bool Workspace::hasClient(const Client* c)
 {
-    return findClient(ClientMatchPredicate(c));
+    return findClient([c](const Client *test) {
+        return test == c;
+    });
 }
 
 inline Workspace *workspace()
@@ -727,5 +723,6 @@ inline Workspace *workspace()
 }
 
 } // namespace
+Q_DECLARE_OPERATORS_FOR_FLAGS(KWin::Workspace::ActivityFlags)
 
 #endif
