@@ -20,8 +20,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 #include "xcbutils.h"
 #include "utils.h"
-// KDE
-#include <KDebug>
+// Qt
+#include <QDebug>
 // xcb
 #include <xcb/composite.h>
 #include <xcb/damage.h>
@@ -30,6 +30,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <xcb/shape.h>
 #include <xcb/sync.h>
 #include <xcb/xfixes.h>
+// system
+#include <sys/shm.h>
+#include <sys/types.h>
 
 namespace KWin {
 
@@ -129,6 +132,7 @@ void Extensions::init()
     }
     if (m_randr.present) {
         randrVersion = xcb_randr_query_version_unchecked(c, RANDR_MAX_MAJOR, RANDR_MAX_MINOR);
+        xcb_randr_select_input(connection(), rootWindow(), XCB_RANDR_NOTIFY_MASK_SCREEN_CHANGE);
     }
     if (m_damage.present) {
         damageVersion = xcb_damage_query_version_unchecked(c, DAMAGE_MAX_MAJOR, DAMAGE_MIN_MAJOR);
@@ -167,13 +171,13 @@ void Extensions::init()
     if (m_sync.present) {
         initVersion<xcb_sync_initialize_reply_t>(syncVersion, &xcb_sync_initialize_reply, &m_sync);
     }
-    kDebug(1212) << "Extensions: shape: 0x" << QString::number(m_shape.version, 16)
+    qDebug() << "Extensions: shape: 0x" << QString::number(m_shape.version, 16)
                  << " composite: 0x" << QString::number(m_composite.version, 16)
                  << " render: 0x" << QString::number(m_render.version, 16)
                  << " fixes: 0x" << QString::number(m_fixes.version, 16)
                  << " randr: 0x" << QString::number(m_randr.version, 16)
                  << " sync: 0x" << QString::number(m_sync.version, 16)
-                 << " damage: 0x " << QString::number(m_damage.version, 16);
+                 << " damage: 0x " << QString::number(m_damage.version, 16) << endl;
 }
 
 void Extensions::extensionQueryReply(const xcb_query_extension_reply_t *extension, ExtensionData *dataToFill)
@@ -253,5 +257,66 @@ QVector<ExtensionData> Extensions::extensions() const
     return extensions;
 }
 
+//****************************************
+// Shm
+//****************************************
+Shm::Shm()
+    : m_shmId(-1)
+    , m_buffer(NULL)
+    , m_segment(XCB_NONE)
+    , m_valid(false)
+    , m_pixmapFormat(XCB_IMAGE_FORMAT_XY_BITMAP)
+{
+    m_valid = init();
+}
+
+Shm::~Shm()
+{
+    if (m_valid) {
+        xcb_shm_detach(connection(), m_segment);
+        shmdt(m_buffer);
+    }
+}
+
+bool Shm::init()
+{
+    const xcb_query_extension_reply_t *ext = xcb_get_extension_data(connection(), &xcb_shm_id);
+    if (!ext || !ext->present) {
+        qDebug() << "SHM extension not available";
+        return false;
+    }
+    ScopedCPointer<xcb_shm_query_version_reply_t> version(xcb_shm_query_version_reply(connection(),
+        xcb_shm_query_version_unchecked(connection()), NULL));
+    if (version.isNull()) {
+        qDebug() << "Failed to get SHM extension version information";
+        return false;
+    }
+    m_pixmapFormat = version->pixmap_format;
+    const int MAXSIZE = 4096 * 2048 * 4; // TODO check there are not larger windows
+    m_shmId = shmget(IPC_PRIVATE, MAXSIZE, IPC_CREAT | 0600);
+    if (m_shmId < 0) {
+        qDebug() << "Failed to allocate SHM segment";
+        return false;
+    }
+    m_buffer = shmat(m_shmId, NULL, 0 /*read/write*/);
+    if (-1 == reinterpret_cast<long>(m_buffer)) {
+        qDebug() << "Failed to attach SHM segment";
+        shmctl(m_shmId, IPC_RMID, NULL);
+        return false;
+    }
+    shmctl(m_shmId, IPC_RMID, NULL);
+
+    m_segment = xcb_generate_id(connection());
+    const xcb_void_cookie_t cookie = xcb_shm_attach_checked(connection(), m_segment, m_shmId, false);
+    ScopedCPointer<xcb_generic_error_t> error(xcb_request_check(connection(), cookie));
+    if (!error.isNull()) {
+        qDebug() << "xcb_shm_attach error: " << error->error_code;
+        shmdt(m_buffer);
+        return false;
+    }
+
+    return true;
+}
+
 } // namespace Xcb
 } // namespace KWin
