@@ -29,9 +29,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <xcb/xcb.h>
 #include <xcb/composite.h>
+#include <xcb/randr.h>
 
-#define KWIN_DOUBLE_TO_FIXED(d) ((xcb_render_fixed_t) ((d) * 65536))
-#define KWIN_FIXED_TO_DOUBLE(f) ((double) ((f) / 65536.0))
+#include <xcb/shm.h>
 
 namespace KWin {
 
@@ -44,6 +44,8 @@ static void defineCursor(xcb_window_t window, xcb_cursor_t cursor);
 static void setInputFocus(xcb_window_t window, uint8_t revertTo = XCB_INPUT_FOCUS_POINTER_ROOT, xcb_timestamp_t time = xTime());
 static void moveWindow(xcb_window_t window, const QPoint &pos);
 static void moveWindow(xcb_window_t window, uint32_t x, uint32_t y);
+static void lowerWindow(xcb_window_t window);
+static void selectInput(xcb_window_t window, uint32_t events);
 
 template <typename Reply,
     typename Cookie,
@@ -160,6 +162,51 @@ private:
     Reply *m_reply;
 };
 
+class Atom
+{
+public:
+    explicit Atom(const QByteArray &name, bool onlyIfExists = false)
+        : m_retrieved(false)
+        , m_cookie(xcb_intern_atom_unchecked(connection(), onlyIfExists, name.length(), name.constData()))
+        , m_atom(XCB_ATOM_NONE)
+        , m_name(name)
+        {
+        }
+    Atom() = delete;
+    Atom(const Atom &) = delete;
+
+    ~Atom() {
+        if (!m_retrieved && m_cookie.sequence) {
+            xcb_discard_reply(connection(), m_cookie.sequence);
+        }
+    }
+
+    operator xcb_atom_t() const {
+        (const_cast<Atom*>(this))->getReply();
+        return m_atom;
+    }
+
+    inline const QByteArray &name() const {
+        return m_name;
+    }
+
+private:
+    void getReply() {
+        if (m_retrieved || !m_cookie.sequence) {
+            return;
+        }
+        ScopedCPointer<xcb_intern_atom_reply_t> reply(xcb_intern_atom_reply(connection(), m_cookie, nullptr));
+        if (!reply.isNull()) {
+            m_atom = reply->atom;
+        }
+        m_retrieved = true;
+    }
+    bool m_retrieved;
+    xcb_intern_atom_cookie_t m_cookie;
+    xcb_atom_t m_atom;
+    QByteArray m_name;
+};
+
 typedef Wrapper<xcb_get_window_attributes_reply_t, xcb_get_window_attributes_cookie_t, &xcb_get_window_attributes_reply, &xcb_get_window_attributes_unchecked> WindowAttributes;
 typedef Wrapper<xcb_composite_get_overlay_window_reply_t, xcb_composite_get_overlay_window_cookie_t, &xcb_composite_get_overlay_window_reply, &xcb_composite_get_overlay_window_unchecked> OverlayWindow;
 
@@ -238,6 +285,41 @@ public:
     }
 };
 
+namespace RandR
+{
+typedef Wrapper<xcb_randr_get_screen_info_reply_t, xcb_randr_get_screen_info_cookie_t, &xcb_randr_get_screen_info_reply, &xcb_randr_get_screen_info_unchecked> ScreenInfo;
+
+class ScreenResources : public Wrapper<xcb_randr_get_screen_resources_reply_t, xcb_randr_get_screen_resources_cookie_t, &xcb_randr_get_screen_resources_reply, &xcb_randr_get_screen_resources_unchecked>
+{
+public:
+    explicit ScreenResources(WindowId window) : Wrapper<xcb_randr_get_screen_resources_reply_t, xcb_randr_get_screen_resources_cookie_t, &xcb_randr_get_screen_resources_reply, &xcb_randr_get_screen_resources_unchecked>(window) {}
+
+    inline xcb_randr_crtc_t *crtcs() {
+        if (isNull()) {
+            return nullptr;
+        }
+        return xcb_randr_get_screen_resources_crtcs(data());
+    }
+};
+
+class CrtcGamma : public Wrapper<xcb_randr_get_crtc_gamma_reply_t, xcb_randr_get_crtc_gamma_cookie_t, &xcb_randr_get_crtc_gamma_reply, &xcb_randr_get_crtc_gamma_unchecked>
+{
+public:
+    explicit CrtcGamma(xcb_randr_crtc_t c) : Wrapper<xcb_randr_get_crtc_gamma_reply_t, xcb_randr_get_crtc_gamma_cookie_t, &xcb_randr_get_crtc_gamma_reply, &xcb_randr_get_crtc_gamma_unchecked>(c) {}
+
+    inline uint16_t *red() {
+        return xcb_randr_get_crtc_gamma_red(data());
+    }
+    inline uint16_t *green() {
+        return xcb_randr_get_crtc_gamma_green(data());
+    }
+    inline uint16_t *blue() {
+        return xcb_randr_get_crtc_gamma_blue(data());
+    }
+};
+
+}
+
 class ExtensionData
 {
 public:
@@ -311,6 +393,9 @@ private:
  * will be freed when the instance gets destroyed.
  *
  * Furthermore the class provides wrappers around some xcb methods operating on an xcb_window_t.
+ *
+ * For the cases that one is more interested in wrapping the xcb methods the constructor which takes
+ * an existing window and the @link reset method allow to disable the RAII functionality.
  **/
 class Window
 {
@@ -318,9 +403,16 @@ public:
     /**
      * Takes over responsibility of @p window. If @p window is not provided an invalid Window is
      * created. Use @link create to set an xcb_window_t later on.
+     *
+     * If @p destroy is @c true the window will be destroyed together with this object, if @c false
+     * the window will be kept around. This is useful if you are not interested in the RAII capabilities
+     * but still want to use a window like an object.
+     *
      * @param window The window to manage.
+     * @param destroy Whether the window should be destroyed together with the object.
+     * @see reset
      **/
-    Window(xcb_window_t window = XCB_WINDOW_NONE);
+    Window(xcb_window_t window = XCB_WINDOW_NONE, bool destroy = true);
     /**
      * Creates an xcb_window_t and manages it. It's a convenient method to create a window with
      * depth, class and visual being copied from parent and border being @c 0.
@@ -340,6 +432,7 @@ public:
      * @param parent The parent window
      **/
     Window(const QRect &geometry, uint16_t windowClass, uint32_t mask = 0, const uint32_t *values = NULL, xcb_window_t parent = rootWindow());
+    Window(const Window &other) = delete;
     ~Window();
 
     /**
@@ -367,8 +460,11 @@ public:
     void create(const QRect &geometry, uint16_t windowClass, uint32_t mask = 0, const uint32_t *values = NULL, xcb_window_t parent = rootWindow());
     /**
      * Frees the existing window and starts to manage the new @p window.
+     * If @p destroy is @c true the new managed window will be destroyed together with this
+     * object or when reset is called again. If @p destroy is @c false the window will not
+     * be destroyed. It is then the responsibility of the caller to destroy the window.
      **/
-    void reset(xcb_window_t window = XCB_WINDOW_NONE);
+    void reset(xcb_window_t window = XCB_WINDOW_NONE, bool destroy = true);
     /**
      * @returns @c true if a window is managed, @c false otherwise.
      **/
@@ -384,8 +480,22 @@ public:
     void resize(const QSize &size);
     void resize(uint32_t width, uint32_t height);
     void raise();
+    void lower();
     void map();
     void unmap();
+    void reparent(xcb_window_t parent, int x = 0, int y = 0);
+    void changeProperty(xcb_atom_t property, xcb_atom_t type, uint8_t format, uint32_t lenght,
+                        const void *data, uint8_t mode = XCB_PROP_MODE_REPLACE);
+    void deleteProperty(xcb_atom_t property);
+    void setBorderWidth(uint32_t width);
+    void grabButton(uint8_t pointerMode, uint8_t keyboardmode,
+                    uint16_t modifiers = XCB_MOD_MASK_ANY,
+                    uint8_t button = XCB_BUTTON_INDEX_ANY,
+                    uint16_t eventMask = XCB_EVENT_MASK_BUTTON_PRESS,
+                    xcb_window_t confineTo = XCB_WINDOW_NONE,
+                    xcb_cursor_t cursor = XCB_CURSOR_NONE,
+                    bool ownerEvents = false);
+    void ungrabButton(uint16_t modifiers = XCB_MOD_MASK_ANY, uint8_t button = XCB_BUTTON_INDEX_ANY);
     /**
      * Clears the window area. Same as xcb_clear_area with x, y, width, height being @c 0.
      **/
@@ -393,29 +503,34 @@ public:
     void setBackgroundPixmap(xcb_pixmap_t pixmap);
     void defineCursor(xcb_cursor_t cursor);
     void focus(uint8_t revertTo = XCB_INPUT_FOCUS_POINTER_ROOT, xcb_timestamp_t time = xTime());
+    void selectInput(uint32_t events);
+    void kill();
     operator xcb_window_t() const;
 private:
-    Window(const Window &other);
     xcb_window_t doCreate(const QRect &geometry, uint16_t windowClass, uint32_t mask = 0, const uint32_t *values = NULL, xcb_window_t parent = rootWindow());
     void destroy();
     xcb_window_t m_window;
+    bool m_destroy;
 };
 
 inline
-Window::Window(xcb_window_t window)
+Window::Window(xcb_window_t window, bool destroy)
     : m_window(window)
+    , m_destroy(destroy)
 {
 }
 
 inline
 Window::Window(const QRect &geometry, uint32_t mask, const uint32_t *values, xcb_window_t parent)
     : m_window(doCreate(geometry, XCB_COPY_FROM_PARENT, mask, values, parent))
+    , m_destroy(true)
 {
 }
 
 inline
 Window::Window(const QRect &geometry, uint16_t windowClass, uint32_t mask, const uint32_t *values, xcb_window_t parent)
     : m_window(doCreate(geometry, windowClass, mask, values, parent))
+    , m_destroy(true)
 {
 }
 
@@ -428,7 +543,7 @@ Window::~Window()
 inline
 void Window::destroy()
 {
-    if (!isValid()) {
+    if (!isValid() || !m_destroy) {
         return;
     }
     xcb_destroy_window(connection(), m_window);
@@ -471,10 +586,11 @@ xcb_window_t Window::doCreate(const QRect &geometry, uint16_t windowClass, uint3
 }
 
 inline
-void Window::reset(xcb_window_t window)
+void Window::reset(xcb_window_t window, bool shouldDestroy)
 {
     destroy();
     m_window = window;
+    m_destroy = shouldDestroy;
 }
 
 inline
@@ -533,6 +649,12 @@ void Window::raise()
     xcb_configure_window(connection(), m_window, XCB_CONFIG_WINDOW_STACK_MODE, values);
 }
 
+inline
+void Window::lower()
+{
+    lowerWindow(m_window);
+}
+
 inline
 void Window::map()
 {
@@ -551,6 +673,63 @@ void Window::unmap()
     xcb_unmap_window(connection(), m_window);
 }
 
+inline
+void Window::reparent(xcb_window_t parent, int x, int y)
+{
+    if (!isValid()) {
+        return;
+    }
+    xcb_reparent_window(connection(), m_window, parent, x, y);
+}
+
+inline
+void Window::changeProperty(xcb_atom_t property, xcb_atom_t type, uint8_t format, uint32_t lenght, const void *data, uint8_t mode)
+{
+    if (!isValid()) {
+        return;
+    }
+    xcb_change_property(connection(), mode, m_window, property, type, format, lenght, data);
+}
+
+inline
+void Window::deleteProperty(xcb_atom_t property)
+{
+    if (!isValid()) {
+        return;
+    }
+    xcb_delete_property(connection(), m_window, property);
+}
+
+inline
+void Window::setBorderWidth(uint32_t width)
+{
+    if (!isValid()) {
+        return;
+    }
+    xcb_configure_window(connection(), m_window, XCB_CONFIG_WINDOW_BORDER_WIDTH, &width);
+}
+
+inline
+void Window::grabButton(uint8_t pointerMode, uint8_t keyboardmode, uint16_t modifiers,
+                        uint8_t button, uint16_t eventMask, xcb_window_t confineTo,
+                        xcb_cursor_t cursor, bool ownerEvents)
+{
+    if (!isValid()) {
+        return;
+    }
+    xcb_grab_button(connection(), ownerEvents, m_window, eventMask,
+                    pointerMode, keyboardmode, confineTo, cursor, button, modifiers);
+}
+
+inline
+void Window::ungrabButton(uint16_t modifiers, uint8_t button)
+{
+    if (!isValid()) {
+        return;
+    }
+    xcb_ungrab_button(connection(), button, m_window, modifiers);
+}
+
 inline
 void Window::clear()
 {
@@ -582,6 +761,18 @@ void Window::focus(uint8_t revertTo, xcb_timestamp_t time)
     setInputFocus(m_window, revertTo, time);
 }
 
+inline
+void Window::selectInput(uint32_t events)
+{
+    Xcb::selectInput(m_window, events);
+}
+
+inline
+void Window::kill()
+{
+    xcb_kill_client(connection(), m_window);
+}
+
 // helper functions
 static inline void moveResizeWindow(WindowId window, const QRect &geometry)
 {
@@ -607,6 +798,12 @@ static inline void moveWindow(xcb_window_t window, uint32_t x, uint32_t y)
     xcb_configure_window(connection(), window, mask, values);
 }
 
+static inline void lowerWindow(xcb_window_t window)
+{
+    const uint32_t values[] = { XCB_STACK_MODE_BELOW };
+    xcb_configure_window(connection(), window, XCB_CONFIG_WINDOW_STACK_MODE, values);
+}
+
 static inline WindowId createInputWindow(const QRect &geometry, uint32_t mask, const uint32_t *values)
 {
     WindowId window = xcb_generate_id(connection());
@@ -697,6 +894,75 @@ static inline void setTransientFor(xcb_window_t window, xcb_window_t transient_f
                         XCB_ATOM_WINDOW, 32, 1, &transient_for_window);
 }
 
+static inline void sync()
+{
+    auto *c = connection();
+    const auto cookie = xcb_get_input_focus(c);
+    xcb_generic_error_t *error = nullptr;
+    ScopedCPointer<xcb_get_input_focus_reply_t> sync(xcb_get_input_focus_reply(c, cookie, &error));
+    if (error) {
+        free(error);
+    }
+}
+
+void selectInput(xcb_window_t window, uint32_t events)
+{
+    xcb_change_window_attributes(connection(), window, XCB_CW_EVENT_MASK, &events);
+}
+
+/**
+ * @brief Small helper class to encapsulate SHM related functionality.
+ *
+ */
+class Shm
+{
+public:
+    Shm();
+    ~Shm();
+    int shmId() const;
+    void *buffer() const;
+    xcb_shm_seg_t segment() const;
+    bool isValid() const;
+    uint8_t pixmapFormat() const;
+private:
+    bool init();
+    int m_shmId;
+    void *m_buffer;
+    xcb_shm_seg_t m_segment;
+    bool m_valid;
+    uint8_t m_pixmapFormat;
+};
+
+inline
+void *Shm::buffer() const
+{
+    return m_buffer;
+}
+
+inline
+bool Shm::isValid() const
+{
+    return m_valid;
+}
+
+inline
+xcb_shm_seg_t Shm::segment() const
+{
+    return m_segment;
+}
+
+inline
+int Shm::shmId() const
+{
+    return m_shmId;
+}
+
+inline
+uint8_t Shm::pixmapFormat() const
+{
+    return m_pixmapFormat;
+}
+
 } // namespace X11
 
 } // namespace KWin
