@@ -24,10 +24,11 @@
 #include <QSlider>
 
 #include <KPluginFactory>
+#include <KSharedConfig>
+#include <KConfig>
 #include <KLocalizedString>
 
 K_PLUGIN_FACTORY(PowerDevilBrightnessControlConfigFactory, registerPlugin<PowerDevil::BundledActions::BrightnessControlConfig>(); )
-K_EXPORT_PLUGIN(PowerDevilBrightnessControlConfigFactory("powerdevilbrightnesscontrolaction_config"))
 
 namespace PowerDevil {
 namespace BundledActions {
