@@ -24,10 +24,10 @@
 
 #include <KIntSpinBox>
 #include <KPluginFactory>
+#include <KSharedConfig>
 #include <KLocalizedString>
 
 K_PLUGIN_FACTORY(PowerDevilDimDisplayConfigFactory, registerPlugin<PowerDevil::BundledActions::DimDisplayConfig>(); )
-K_EXPORT_PLUGIN(PowerDevilDimDisplayConfigFactory("powerdevildimdisplayaction_config"))
 
 namespace PowerDevil {
 namespace BundledActions {
