@@ -18,6 +18,7 @@
  ***************************************************************************/
 
 #include "handlebuttonevents.h"
+#include "handlebuttoneventsadaptor.h"
 
 #include "suspendsession.h"
 
@@ -35,6 +36,8 @@
 #include "PowerDevilSettings.h"
 #include "screensaver_interface.h"
 
+#include <kglobalaccel.h>
+
 namespace PowerDevil {
 namespace BundledActions {
 
@@ -45,26 +48,28 @@ HandleButtonEvents::HandleButtonEvents(QObject* parent)
     , m_sleepButtonAction(1)
     , m_hibernateButtonAction(2)
 {
+    new HandleButtonEventsAdaptor(this);
     // We enforce no policies here - after all, we just call other actions - which have their policies.
     setRequiredPolicies(PowerDevil::PolicyAgent::None);
     connect(backend(), SIGNAL(buttonPressed(PowerDevil::BackendInterface::ButtonType)),
             this, SLOT(onButtonPressed(PowerDevil::BackendInterface::ButtonType)));
 
     KActionCollection* actionCollection = new KActionCollection( this );
+    KGlobalAccel *accel = KGlobalAccel::self();
 
-    KAction *globalAction = actionCollection->addAction("Sleep");
+    QAction *globalAction = actionCollection->addAction("Sleep");
     globalAction->setText(i18nc("@action:inmenu Global shortcut", "Sleep"));
-    globalAction->setGlobalShortcut(KShortcut(Qt::Key_Sleep));
+    accel->setDefaultShortcut(globalAction, QList<QKeySequence>() << Qt::Key_Sleep);
     connect(globalAction, SIGNAL(triggered(bool)), SLOT(suspendToRam()));
 
     globalAction = actionCollection->addAction("Hibernate");
     globalAction->setText(i18nc("@action:inmenu Global shortcut", "Hibernate"));
-    globalAction->setGlobalShortcut(KShortcut(Qt::Key_Hibernate));
+    accel->setDefaultShortcut(globalAction, QList<QKeySequence>() << Qt::Key_Hibernate);
     connect(globalAction, SIGNAL(triggered(bool)), SLOT(suspendToDisk()));
 
     globalAction = actionCollection->addAction("PowerOff");
     //globalAction->setText(i18nc("Global shortcut", "Power Off button"));
-    globalAction->setGlobalShortcut(KShortcut(Qt::Key_PowerOff));
+    accel->setDefaultShortcut(globalAction, QList<QKeySequence>() << Qt::Key_PowerOff);
     connect(globalAction, SIGNAL(triggered(bool)), SLOT(powerOffButtonTriggered()));
 }
 
@@ -179,6 +184,11 @@ bool HandleButtonEvents::loadAction(const KConfigGroup& config)
     return true;
 }
 
+int HandleButtonEvents::lidAction() const
+{
+    return m_lidAction;
+}
+
 void HandleButtonEvents::powerOffButtonTriggered()
 {
     onButtonPressed(BackendInterface::PowerButton);
