@@ -22,9 +22,9 @@
 #include <KIntSpinBox>
 #include <KLocalizedString>
 #include <KPluginFactory>
+#include <KSharedConfig>
 
-K_PLUGIN_FACTORY(PowerDevilDPMSConfigFactory, registerPlugin<PowerDevilDPMSActionConfig>(); )
-K_EXPORT_PLUGIN(PowerDevilDPMSConfigFactory("powerdevildpmsaction_config"))
+K_PLUGIN_FACTORY(PowerDevilDPMSConfigFactory, registerPlugin<PowerDevilDPMSActionConfig>();)
 
 PowerDevilDPMSActionConfig::PowerDevilDPMSActionConfig(QObject* parent, const QVariantList& )
         : ActionConfig(parent)
@@ -65,3 +65,4 @@ QList< QPair< QString, QWidget* > > PowerDevilDPMSActionConfig::buildUi()
     return retlist;
 }
 
+#include "powerdevildpmsactionconfig.moc"
\ No newline at end of file
