@@ -26,10 +26,12 @@
 #include <QtDBus>
 
 #include <KDebug>
+#include <kauthexecutejob.h>
 #include <KPluginFactory>
+#include <KSharedConfig>
 #include <KAuth/Action>
 
-#include "xrandrx11helper.h"
+#include "xrandrxcbhelper.h"
 #include "xrandrbrightness.h"
 #include "upowersuspendjob.h"
 #include "login1suspendjob.h"
@@ -61,11 +63,9 @@ bool checkSystemdVersion(uint requiredVersion)
                                                   QLatin1String("/com/ubuntu/Upstart"),
                                                   QDBusConnection::sessionBus());
 
-    QRegExp exp("(?:init \\()?upstart ([0-9.]+)(?:\\))?");
+    QRegExp exp("init \\(upstart ([0-9.]+)\\)");
     if(exp.exactMatch(upstartInterface.version())) {
-        // Only keep the X.Y part of a X.Y.Z version
-        QStringList items = exp.cap(1).split('.').mid(0, 2);
-        const float upstartVersion = items.join(QString('.')).toFloat();
+        const float upstartVersion = exp.cap(1).toFloat();
         return upstartVersion >= 1.1;
     }
 
@@ -75,8 +75,8 @@ bool checkSystemdVersion(uint requiredVersion)
 
 PowerDevilUPowerBackend::PowerDevilUPowerBackend(QObject* parent)
     : BackendInterface(parent),
-      m_brightnessControl(0), m_kbdMaxBrightness(0),
-      m_lidIsPresent(false), m_lidIsClosed(false), m_onBattery(false)
+      m_brightnessControl(0),
+      m_lidIsPresent(false), m_lidIsClosed(false), m_onBattery(false), m_kbdMaxBrightness(0)
 {
 
 }
@@ -165,10 +165,11 @@ void PowerDevilUPowerBackend::init()
     if (!m_brightnessControl->isSupported()) {
         kDebug() << "Using helper";
         KAuth::Action action("org.kde.powerdevil.backlighthelper.syspath");
-        action.setHelperID(HELPER_ID);
-        KAuth::ActionReply reply = action.execute();
-        if (reply.succeeded()) {
-            m_syspath = reply.data()["syspath"].toString();
+        action.setHelperId(HELPER_ID);
+        KAuth::ExecuteJob* job = action.execute();
+        job->exec();
+        if (!job->error()) {
+            m_syspath = job->data()["syspath"].toString();
             m_syspath = QFileInfo(m_syspath).readLink();
 
             UdevQt::Client *client =  new UdevQt::Client(QStringList("backlight"), this);
@@ -177,7 +178,8 @@ void PowerDevilUPowerBackend::init()
         }
     } else {
         kDebug() << "Using XRandR";
-        m_randrHelper = new XRandRX11Helper();
+        m_randrHelper = XRandRXCBHelper::self();
+        Q_ASSERT(m_randrHelper);
         connect(m_randrHelper, SIGNAL(brightnessChanged()), this, SLOT(slotScreenBrightnessChanged()));
         screenBrightnessAvailable = true;
     }
@@ -361,10 +363,10 @@ float PowerDevilUPowerBackend::brightness(PowerDevil::BackendInterface::Brightne
         } else {
             //kDebug() << "Falling back to helper to get brightness";
             KAuth::Action action("org.kde.powerdevil.backlighthelper.brightness");
-            action.setHelperID(HELPER_ID);
-            KAuth::ActionReply reply = action.execute();
-            if (reply.succeeded()) {
-                result = reply.data()["brightness"].toFloat();
+            action.setHelperId(HELPER_ID);
+            KAuth::ExecuteJob *job = action.execute();
+            if (!job->error()) {
+                result = job->data()["brightness"].toFloat();
                 //kDebug() << "org.kde.powerdevil.backlighthelper.brightness succeeded: " << reply.data()["brightness"];
             }
             else
@@ -390,10 +392,10 @@ bool PowerDevilUPowerBackend::setBrightness(float brightnessValue, PowerDevil::B
         } else {
             //kDebug() << "Falling back to helper to set brightness";
             KAuth::Action action("org.kde.powerdevil.backlighthelper.setbrightness");
-            action.setHelperID(HELPER_ID);
+            action.setHelperId(HELPER_ID);
             action.addArgument("brightness", brightnessValue);
-            KAuth::ActionReply reply = action.execute();
-            if (reply.failed()) {
+            KAuth::ExecuteJob *job = action.execute();
+            if (job->error()) {
                 kWarning() << "org.kde.powerdevil.backlighthelper.setbrightness failed";
                 return false;
             }
@@ -467,7 +469,7 @@ void PowerDevilUPowerBackend::slotDeviceAdded(const QString & device)
 
     // for UPower >= 0.99.0 which doesn't emit the DeviceChanged(QString) signal
     QDBusConnection::systemBus().connect(UPOWER_SERVICE, device, "org.freedesktop.DBus.Properties", "PropertiesChanged", this,
-                                         SLOT(onDevicePropertiesChanged(QString,QVariantMap,QStringList)));
+                                         SLOT(onPropertiesChanged(QString,QVariantMap,QStringList)));
 
     updateDeviceProps();
 }
