@@ -22,12 +22,12 @@
 
 #include <QTimer>
 #include <QRadioButton>
+#include <QVariantList>
+#include <QLoggingCategory>
+#include <QMenu>
 #include <QMenuBar>
 
-#include <KMenu>
-#include <KDebug>
-#include <KToolBar>
-#include <KAboutData>
+#include <kaboutdata.h>
 #include <KMessageBox>
 #include <KConfigGroup>
 #include <KCModuleInfo>
@@ -35,6 +35,8 @@
 #include <KStandardAction>
 #include <KActionCollection>
 #include <KServiceTypeTrader>
+#include <KToolBar>
+#include <kwindowconfig.h>
 
 #include "BaseData.h"
 #include "ModuleView.h"
@@ -49,13 +51,15 @@ SettingsBase::SettingsBase( QWidget * parent )
     lostFound = 0;
     // Prepare the view area
     stackedWidget = new QStackedWidget( this );
+    setWindowTitle(i18n("System Settings"));
+    setWindowIcon(QIcon::fromTheme("preferences-system"));
     setCentralWidget(stackedWidget);
     setWindowFlags( windowFlags() | Qt::WindowContextHelpButtonHint );
     // Initialise search
     searchText = new KLineEdit( this );
     searchText->setClearButtonShown( true );
-    searchText->setClickMessage( i18nc( "Search through a list of control modules", "Search" ) );
-    searchText->setCompletionMode( KGlobalSettings::CompletionPopup );
+    searchText->setPlaceholderText( i18nc( "Search through a list of control modules", "Search" ) );
+    searchText->setCompletionMode( KCompletion::CompletionPopup );
 
     spacerWidget = new QWidget( this );
     spacerWidget->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Maximum );
@@ -89,7 +93,7 @@ void SettingsBase::initApplication()
             const KService::Ptr entry = modules.at(i);
             MenuItem * infoItem = new MenuItem(false, lostFound);
             infoItem->setService( entry );
-            kDebug() << "Added " << entry->name();
+            qDebug() << "Added " << entry->name();
         }
     }
 
@@ -110,7 +114,7 @@ void SettingsBase::initApplication()
             connect(searchText, SIGNAL(textChanged(QString)), controller, SLOT(searchChanged(QString)));
             connect(controller, SIGNAL(viewChanged(bool)), this, SLOT(viewChange(bool)));
         } else {
-            kWarning() << "View load error: " + error;
+            qWarning() << "View load error: " + error;
         }
     }
     searchText->completionObject()->setIgnoreCase( true );
@@ -125,26 +129,26 @@ void SettingsBase::initToolBar()
     quitAction = actionCollection()->addAction( KStandardAction::Quit, "quit_action", this, SLOT(close()) );
     // Configure goes at the end
     configureAction = actionCollection()->addAction( KStandardAction::Preferences, this, SLOT(configShow()) );
-    configureAction->setShortcut(KShortcut(QKeySequence(Qt::CTRL + Qt::Key_M)));
+    configureAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_M));
     configureAction->setText( i18n("Configure") );
     // Help after it
     initHelpMenu();
     // Then a spacer so the search line-edit is kept separate
-    spacerAction = new KAction( this );
+    spacerAction = new QWidgetAction( this );
     spacerAction->setDefaultWidget(spacerWidget);
     actionCollection()->addAction( "spacer", spacerAction );
     // Finally the search line-edit
-    searchAction = new KAction( this );
+    searchAction = new QWidgetAction( this );
     searchAction->setDefaultWidget(searchText);
-    searchAction->setShortcut(KShortcut(QKeySequence(Qt::CTRL + Qt::Key_F)));
-    connect( searchAction, SIGNAL(triggered(Qt::MouseButtons,Qt::KeyboardModifiers)),
-         searchText, SLOT(setFocus()));
+    searchAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_F));
+    connect( searchAction, SIGNAL(triggered(bool)), searchText, SLOT(setFocus()));
     actionCollection()->addAction( "searchText", searchAction );
     // Initialise the Window
     setupGUI(Save|Create,QString());
     menuBar()->hide();
+
     // Toolbar & Configuration
-    helpActionMenu->setMenu( qobject_cast<KMenu*>( factory()->container("help", this) ) );
+    helpActionMenu->setMenu( dynamic_cast<QMenu*>( factory()->container("help", this) ) );
     setMinimumSize(620,430);
     toolBar()->setMovable(false); // We don't allow any changes
     changeToolBar( BaseMode::Search | BaseMode::Configure | BaseMode::Quit );
@@ -152,7 +156,7 @@ void SettingsBase::initToolBar()
 
 void SettingsBase::initHelpMenu()
 {
-    helpActionMenu = new KActionMenu( KIcon("system-help"), i18n("Help"), this );
+    helpActionMenu = new KActionMenu( QIcon::fromTheme("system-help"), i18n("Help"), this );
     helpActionMenu->setDelayed( false );
     actionCollection()->addAction( "help_toolbar_menu", helpActionMenu );
     // Add the custom actions
@@ -165,25 +169,25 @@ void SettingsBase::initConfig()
 {
     // Prepare dialog first
     configDialog = new KConfigDialog( this, "systemsettingsconfig", BaseConfig::self() );
-    configDialog->setButtons( KDialog::Ok | KDialog::Cancel );
+    configDialog->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
 
     // Add our page
     QWidget * configPage = new QWidget( configDialog );
     configWidget.setupUi(configPage);
-    QString iconName = KGlobal::activeComponent().aboutData()->programIconName();
+    QString iconName = KAboutData::applicationData().programIconName();
     configDialog->addPage( configPage, i18nc("General config for System Settings", "General"), iconName );
     QVBoxLayout * configLayout = new QVBoxLayout;
     // Get the list of modules
     foreach( BaseMode * mode, possibleViews ) {
         mode->addConfiguration( configDialog );
         QRadioButton * radioButton = new QRadioButton( mode->service()->name(), configWidget.GbViewStyle );
-        radioButton->setIcon( KIcon(mode->service()->icon()) );
+        radioButton->setIcon( QIcon::fromTheme(mode->service()->icon()) );
         configLayout->addWidget( radioButton );
         viewSelection.addButton( radioButton, possibleViews.values().indexOf(mode) );
     }
     configWidget.GbViewStyle->setLayout( configLayout );
-    configDialog->restoreDialogSize( KGlobal::config()->group("ConfigDialog") );
-    connect(configDialog, SIGNAL(okClicked()), this, SLOT(configUpdated()));
+    KWindowConfig::restoreWindowSize(configDialog->windowHandle(), KSharedConfig::openConfig()->group("ConfigDialog"));
+    connect(configDialog, SIGNAL(accepted()), this, SLOT(configUpdated()));
 }
 
 
@@ -231,8 +235,8 @@ void SettingsBase::initMenuList(MenuItem * parent)
 
 void SettingsBase::configUpdated()
 {
-    KConfigGroup dialogConfig = KGlobal::config()->group("ConfigDialog");
-    configDialog->saveDialogSize( dialogConfig );
+    KConfigGroup dialogConfig = KSharedConfig::openConfig()->group("ConfigDialog");
+    KWindowConfig::saveWindowSize(configDialog->windowHandle(), dialogConfig);
     BaseConfig::setActiveView( possibleViews.keys().at(viewSelection.checkedId()) );
     BaseConfig::setShowToolTips( configWidget.ChTooltips->isChecked() );
     activeView->saveConfiguration();
@@ -288,7 +292,7 @@ void SettingsBase::about()
     }
 
     if( about ) {
-        aboutDialog = new KAboutApplicationDialog(about, 0);
+        aboutDialog = new KAboutApplicationDialog(*about, 0);
         aboutDialog->show();
     }
 }
@@ -380,21 +384,21 @@ void SettingsBase::changeToolBar( BaseMode::ToolBarItems toolbar )
     }
 }
 
-void SettingsBase::changeAboutMenu( const KAboutData * menuAbout, KAction * menuItem, QString fallback )
+void SettingsBase::changeAboutMenu( const KAboutData * menuAbout, QAction * menuItem, QString fallback )
 {
     if( !menuItem ) {
         return;
     }
 
     if( menuAbout ) {
-        menuItem->setText( i18n( "About %1", menuAbout->programName() ) );
-        menuItem->setIcon( KIcon( menuAbout->programIconName() ) );
+        menuItem->setText( i18n( "About %1", menuAbout->displayName() ) );
+        menuItem->setIcon( QIcon::fromTheme( menuAbout->programIconName() ) );
         menuItem->setEnabled(true);
     } else {
         menuItem->setText( fallback );
-        menuItem->setIcon( KIcon( KGlobal::activeComponent().aboutData()->programIconName() ) );
+        menuItem->setIcon( QIcon::fromTheme( KAboutData::applicationData().programIconName() ) );
         menuItem->setEnabled(false);
     }
 }
 
-#include "moc_SettingsBase.cpp"
+#include "SettingsBase.moc"
